Package: gbm3
Version: 3.0.1
Date: 2026-05-07
Title: Generalized Boosted Regression Models
Authors@R: c(
    person("James", "Hickey",
           email="james.hickey@cantab.net",
           role="aut"),
    person("Paul", "Metcalfe",
           email="paul.metcalfe@gmail.com",
           role=c("aut")),
    person("Greg", "Ridgeway",
           email="gridge@upenn.edu",
           role=c("aut", "cre")),
    person("Stefan", "Schroedl",
           email="schroedl@a9.com",
           role="aut"),
    person("Harry", "Southworth",
           email="harry.southworth@gmail.com",
           role="aut"),
    person("Terry", "Therneau",
           email="therneau.terry@mayo.edu",
           role="aut")
    )
Depends: R (>= 3.5.0)
Imports: survival, lattice, splines, Rcpp (>= 1.0.0)
Suggests: testthat (>= 3.1.0), knitr, rmarkdown, MASS
VignetteBuilder: knitr
Description: Extensions to Freund and Schapire's AdaBoost algorithm, Y. Freund 
    and R. Schapire (1997) <doi:10.1006/jcss.1997.1504> and Friedman's gradient 
    boosting machine, J.H. Friedman (2001) <doi:10.1214/aos/1013203451>. 
    Includes regression methods for least squares, absolute loss, 
    t-distribution loss, quantile regression, logistic, Poisson,
    Cox proportional hazards partial likelihood, AdaBoost
    exponential loss, Huberized hinge loss,
    and Learning to Rank measures (LambdaMART).
License: GPL (>= 2)
URL: https://github.com/gbm-developers/gbm3
Encoding: UTF-8
LinkingTo: Rcpp
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-10 01:51:16 UTC; greg_
Author: James Hickey [aut],
  Paul Metcalfe [aut],
  Greg Ridgeway [aut, cre],
  Stefan Schroedl [aut],
  Harry Southworth [aut],
  Terry Therneau [aut]
Maintainer: Greg Ridgeway <gridge@upenn.edu>
Repository: CRAN
Date/Publication: 2026-05-13 20:00:09 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-13 20:10:36 UTC; unix
Archs: gbm3.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(adjust_pred_scale,AdaBoostGBMDist)
S3method(adjust_pred_scale,BernoulliGBMDist)
S3method(adjust_pred_scale,CoxPHGBMDist)
S3method(adjust_pred_scale,GammaGBMDist)
S3method(adjust_pred_scale,GaussianGBMDist)
S3method(adjust_pred_scale,HuberizedGBMDist)
S3method(adjust_pred_scale,LaplaceGBMDist)
S3method(adjust_pred_scale,PairwiseGBMDist)
S3method(adjust_pred_scale,PoissonGBMDist)
S3method(adjust_pred_scale,QuantileGBMDist)
S3method(adjust_pred_scale,TDistGBMDist)
S3method(adjust_pred_scale,TweedieGBMDist)
S3method(as.double,GBMTPerformance)
S3method(check_dist_params,AdaBoostGBMDist)
S3method(check_dist_params,BernoulliGBMDist)
S3method(check_dist_params,CoxPHGBMDist)
S3method(check_dist_params,GammaGBMDist)
S3method(check_dist_params,GaussianGBMDist)
S3method(check_dist_params,HuberizedGBMDist)
S3method(check_dist_params,LaplaceGBMDist)
S3method(check_dist_params,PairwiseGBMDist)
S3method(check_dist_params,PoissonGBMDist)
S3method(check_dist_params,QuantileGBMDist)
S3method(check_dist_params,TDistGBMDist)
S3method(check_dist_params,TweedieGBMDist)
S3method(check_response_values,AdaBoostGBMDist)
S3method(check_response_values,BernoulliGBMDist)
S3method(check_response_values,CoxPHGBMDist)
S3method(check_response_values,GammaGBMDist)
S3method(check_response_values,GaussianGBMDist)
S3method(check_response_values,HuberizedGBMDist)
S3method(check_response_values,LaplaceGBMDist)
S3method(check_response_values,PairwiseGBMDist)
S3method(check_response_values,PoissonGBMDist)
S3method(check_response_values,QuantileGBMDist)
S3method(check_response_values,TDistGBMDist)
S3method(check_response_values,TweedieGBMDist)
S3method(create_cv_groups,BernoulliGBMDist)
S3method(create_cv_groups,PairwiseGBMDist)
S3method(create_cv_groups,default)
S3method(create_dist,AdaBoostGBMDist)
S3method(create_dist,BernoulliGBMDist)
S3method(create_dist,CoxPHGBMDist)
S3method(create_dist,GammaGBMDist)
S3method(create_dist,GaussianGBMDist)
S3method(create_dist,HuberizedGBMDist)
S3method(create_dist,LaplaceGBMDist)
S3method(create_dist,PairwiseGBMDist)
S3method(create_dist,PoissonGBMDist)
S3method(create_dist,QuantileGBMDist)
S3method(create_dist,TDistGBMDist)
S3method(create_dist,TweedieGBMDist)
S3method(create_dist,default)
S3method(determine_groups,PairwiseGBMDist)
S3method(determine_groups,default)
S3method(distribution_name,GBMDist)
S3method(distribution_name,GBMFit)
S3method(distribution_name,default)
S3method(gbm_call_dist_name,PairwiseGBMDist)
S3method(gbm_call_dist_name,default)
S3method(gbm_cv_errors,GBMCVFit)
S3method(get_misc,AdaBoostGBMDist)
S3method(get_misc,BernoulliGBMDist)
S3method(get_misc,CoxPHGBMDist)
S3method(get_misc,GammaGBMDist)
S3method(get_misc,GaussianGBMDist)
S3method(get_misc,HuberizedGBMDist)
S3method(get_misc,LaplaceGBMDist)
S3method(get_misc,PairwiseGBMDist)
S3method(get_misc,PoissonGBMDist)
S3method(get_misc,QuantileGBMDist)
S3method(get_misc,TDistGBMDist)
S3method(get_misc,TweedieGBMDist)
S3method(get_misc,default)
S3method(get_ylabel,AdaBoostGBMDist)
S3method(get_ylabel,BernoulliGBMDist)
S3method(get_ylabel,CoxPHGBMDist)
S3method(get_ylabel,GammaGBMDist)
S3method(get_ylabel,GaussianGBMDist)
S3method(get_ylabel,HuberizedGBMDist)
S3method(get_ylabel,LaplaceGBMDist)
S3method(get_ylabel,PairwiseGBMDist)
S3method(get_ylabel,PoissonGBMDist)
S3method(get_ylabel,QuantileGBMDist)
S3method(get_ylabel,TDistGBMDist)
S3method(get_ylabel,TweedieGBMDist)
S3method(get_ylabel,default)
S3method(get_ylabel_one_var,BernoulliGBMDist)
S3method(get_ylabel_one_var,PairwiseGBMDist)
S3method(get_ylabel_one_var,PoissonGBMDist)
S3method(get_ylabel_one_var,default)
S3method(interact,GBMFit)
S3method(loss,AdaBoostGBMDist)
S3method(loss,BernoulliGBMDist)
S3method(loss,CoxPHGBMDist)
S3method(loss,GammaGBMDist)
S3method(loss,GaussianGBMDist)
S3method(loss,HuberizedGBMDist)
S3method(loss,LaplaceGBMDist)
S3method(loss,PairwiseGBMDist)
S3method(loss,PoissonGBMDist)
S3method(loss,QuantileGBMDist)
S3method(loss,TDistGBMDist)
S3method(loss,TweedieGBMDist)
S3method(loss,default)
S3method(plot,GBMFit)
S3method(plot,GBMTPerformance)
S3method(predict,GBMCVFit)
S3method(predict,GBMFit)
S3method(print,GBMFit)
S3method(print,GBMTPerformance)
S3method(print,gbmParallel)
S3method(reorder_fit,CoxPHGBMDist)
S3method(reorder_fit,PairwiseGBMDist)
S3method(reorder_fit,default)
S3method(response,BernoulliGBMDist)
S3method(response,GammaGBMDist)
S3method(response,PairwiseGBMDist)
S3method(response,PoissonGBMDist)
S3method(response,TweedieGBMDist)
S3method(response,default)
S3method(summary,GBMFit)
S3method(update_fold_dist_data,CoxPHGBMDist)
S3method(update_fold_dist_data,PairwiseGBMDist)
S3method(update_fold_dist_data,default)
S3method(update_num_train_groups,PairwiseGBMDist)
S3method(update_num_train_groups,default)
S3method(weight_group_consistency,GBMDist)
S3method(weight_group_consistency,PairwiseGBMDist)
S3method(weight_group_consistency,default)
export(available_distributions)
export(baseline_hazard)
export(calibrate_plot)
export(distribution_name)
export(gbm)
export(gbm.fit)
export(gbm.perf)
export(gbmParallel)
export(gbm_dist)
export(gbm_more)
export(gbm_roc_area)
export(gbmt)
export(gbmt_fit)
export(gbmt_performance)
export(interact)
export(iteration_error)
export(perf_pairwise)
export(permutation_relative_influence)
export(pretty_gbm_tree)
export(quantile_rug)
export(relative_influence)
export(to_old_gbm)
export(training_params)
export(trees)
import(lattice)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,rug)
importFrom(splines,ns)
importFrom(stats,approx)
importFrom(stats,binomial)
importFrom(stats,delete.response)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,loess)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.pass)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,runif)
importFrom(stats,supsmu)
importFrom(stats,terms)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
useDynLib(gbm3)

# gbm3: Generalized Boosted Models

<!-- badges: start -->
[![R-CMD-check](https://github.com/gbm-developers/gbm3/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/gbm-developers/gbm3/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

Originally written by Greg Ridgeway between 1999-2003, added to by various 
authors, extensively updated and polished by James Hickey in 2016, survival 
models greatly improved by Terry Therneau in 2016, and currently 
maintained by Greg Ridgeway.
Development is discussed at the
[gbm-dev Google Group](https://groups.google.com/forum/#!forum/gbm-dev).

`gbm3` provides generalized boosted regression models with a newer API than the
original `gbm` package. The package supports regression, classification,
survival models, and learning-to-rank methods, with optional OpenMP
parallelization in the core fitting code.

To install the development version from GitHub, first install `remotes`:

```R
install.packages("remotes")
```

Then install `gbm3`:

```R
remotes::install_github("gbm-developers/gbm3")

# or to build vignettes during installation
remotes::install_github("gbm-developers/gbm3", build_vignettes = TRUE, force = TRUE)
```

# RANKING EXAMPLE

cat("Running ranking (LambdaMart) example.\n")

# Create synthetic data that shows how pairwise training can be better
# Note: no claim to represent 'real world' data!

generate.data <- function(N) {
  
  # create query groups, with an average size of 25 items each
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data.frame(Y, query=query, X1, X2, X3)
}

cat('Generating data\n')
N=1000
data.train <- generate.data(N)

# Now we fit 3 different models to the same data:
# * Gaussian
# * Pairwise with NDCG ranking metric
# * Pairwise with CONC (fraction of concordant pairs) ranking metric

cat('Fitting a model with gaussian loss function\n')
train_params_gauss <- training_params(num_trees = 2000, shrinkage = 0.005, 
                                      interaction_depth = 3, bag_fraction = 0.5,
                                      num_train = N, id = seq_len(nrow(data)), num_features = 3,
                                      min_num_obs_in_node = 10)
gbm.gaussian <- gbmt(Y~X1+X2+X3,      # formula
                    data=data.train,         # dataset
                    distribution=gbm_dist('Gaussian'), # loss function: gaussian
                    train_params=train_params_gauss,
                    keep_gbm_data=TRUE,          # store copy of input data in model
                    cv_folds=5,              # number of cross validation folds
                    is_verbose = FALSE         # don't print progress
)             # use a single core (to prevent possible problems caused by wronly detecting cores)

# estimate number of trees
best.iter.gaussian <- gbmt_performance(gbm.gaussian, method="cv")
title('Training of gaussian model')

cat('Fitting a model with pairwise loss function (ranking metric: normalized discounted cumulative gain)\n')

gbm.ndcg <- gbmt(Y~X1+X2+X3,          # formula
                data=data.train,     # dataset
                distribution=gbm_dist(  # loss function:
                  name='Pairwise',   # pairwise
                  metric="ndcg",     # ranking metric: normalized discounted cumulative gain
                  group='query'),    # column indicating query groups
                train_params=train_params_gauss,
                keep_gbm_data=TRUE,      # store copy of input data in model
                cv_folds=5,          # number of cross validation folds
                is_verbose = FALSE     # don't print progress
)         # use a single core
# estimate number of trees
best.iter.ndcg <- gbmt_performance(gbm.ndcg, method='cv')
title('Training of pairwise model with ndcg metric')

cat('Fit a model with pairwise loss function (ranking metric: fraction of concordant pairs)\n')

gbm.conc <- gbmt(Y~X1+X2+X3,          # formula
                data=data.train,     # dataset
                distribution=gbm_dist(   # loss function:
                  name='Pairwise',   # pairwise
                  metric="conc",     # ranking metric: concordant pairs
                  group='query'),    # column indicating query groups
                train_params = train_params_gauss,
                keep_gbm_data=TRUE,      # store copy of input data in model
                cv_folds=5,          # number of cross validation folds
                is_verbose = FALSE     # don't print progress
)         # use a single core

# estimate number of trees
best.iter.conc <- gbmt_performance(gbm.conc, method='cv')
title('Training of pairwise model with conc metric')


# plot variable importance
par.old <- par(mfrow=c(1,3))
summary(gbm.gaussian, num_trees=best.iter.gaussian, main='gaussian')
summary(gbm.ndcg, num_trees=best.iter.ndcg, main='pairwise (ndcg)')
summary(gbm.conc, num_trees=best.iter.conc, main='pairwise (conc)')
par(par.old)

cat("Generating some new data\n")

data.test <- generate.data(N)

cat("Calculating predictions\n")

predictions <- data.frame(random=runif(N),
                          X2=data.test$X2,
                          gaussian=predict(gbm.gaussian, data.test, best.iter.gaussian),
                          pairwise.ndcg=predict(gbm.ndcg, data.test, best.iter.ndcg),
                          pairwise.conc=predict(gbm.conc, data.test, best.iter.conc))

cat("Computing loss metrics\n")
dist_1 <- gbm_dist("Gaussian")
dist_2 <- gbm_dist(name='Pairwise', metric="ndcg", 
                   group_index=data.test$query, max_rank=5)
dist_3 <- gbm_dist(name='Pairwise', metric="conc",
                   group_index=data.test$query, max_rank=0)

result.table <- data.frame(measure=c('random', 'X2 only', 'gaussian', 'pairwise (ndcg)', 'pairwise (conc)'),
                           squared.loss=sapply(1:length(predictions), FUN=function(i) {
                             loss(y=data.test$Y, predictions[[i]], w=rep(1,N), offset=rep(0, N), dist=gbm_dist(name="Gaussian")) }),
                           ndcg5.loss=sapply(1:length(predictions), FUN=function(i) {
                             loss(y=data.test$Y, predictions[[i]], w=rep(1,N), offset=rep(0 ,N), distribution_obj=dist_2)}),
                           concordant.pairs.loss=sapply(1:length(predictions), FUN=function(i) {
                             loss(y=data.test$Y, predictions[[i]], w=rep(1,N), offset=rep(0, N), distribution_obj= dist_3) }),
                           row.names=NULL)

cat('Performance measures for the different models on the test set (smaller is better):\n')
print(result.table,digits=2)

# Brief explanation: Variable X1 is not correlated with the order of items, only
# with queries. Variable X2 is the only one that is correlated with the order of
# items within queries. However, it has a high query-correlated variance.
# Therefore, the 'optimal' possible ranking is just by X2. Of course, the
# pairwise models don't know this and don't completely achieve the same
# accuracy, due to noise and data limitation.
#
# The Gaussian model uses mostly X1, due to the high variance of X2; on the
# contrary, the pairwise models rely mainly on X2. The loss table shows that
# both pairwise models are better in terms of the ranking metrics, but worse in
# terms of squared loss.

# LEAST SQUARES EXAMPLE

cat("Running least squares regression example.\n")
# create some data

N <- 1000
X1 <- runif(N)
X2 <- 2*runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
X5 <- factor(sample(letters[1:3],N,replace=TRUE))
X6 <- 3*runif(N)
mu <- c(-1,0,1,2)[as.numeric(X3)]

SNR <- 10 # signal-to-noise ratio
Y <- X1**1.5 + 2 * (X2**.5) + mu
sigma <- sqrt(var(Y)/SNR)
Y <- Y + rnorm(N,0,sigma)

# create a bunch of missing values
X1[sample(1:N,size=100)] <- NA
X3[sample(1:N,size=300)] <- NA

# random weights if you want to experiment with them
# w <- rexp(N)
# w <- N*w/sum(w)
w <- rep(1,N)

data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
train_params <- training_params(num_trees = 2000, 
                                bag_fraction = 0.5, 
                                num_train = N/2,
                                id = seq_len(nrow(data)),
                                min_num_obs_in_node = 10, 
                                num_features = 3, 
                                interaction_depth = 3)
# fit initial model
gbm1 <- gbmt(Y~X1+X2+X3+X4+X5+X6,         # formula
            data=data,                    # dataset
            var_monotone=c(0,0,0,0,0,0),  # -1: monotone decrease, 
                                          # +1: monotone increase, 0: no monotone restrictions
            distribution=gbm_dist("Gaussian"),    
            train_params=train_params,
            keep_gbm_data=TRUE,
            cv_folds=10,                  # do 10-fold cross-validation
            is_verbose = FALSE)           # don't print progress

str(gbm1,max.level=1)
# plot the performance
# returns out-of-bag estimated best number of trees
best.iter <- gbmt_performance(gbm1,method="OOB")  
# returns test set estimate of best number of trees
best.iter <- gbmt_performance(gbm1,method="test") 
# returns cv estimate of best number of trees
best.iter <- gbmt_performance(gbm1,method="cv")   
# plot variable influence
summary(gbm1,num_trees=1)         # based on the first tree
summary(gbm1,num_trees=best.iter) # based on the estimated best number of trees

# print the first and last trees
print(pretty_gbm_tree(gbm1,1))
print(pretty_gbm_tree(gbm1,gbm1$params$num_trees))

print(gbm1$c.splits[1:3])

# make some new data
N <- 1000
X1 <- runif(N)
X2 <- 2*runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
X5 <- factor(sample(letters[1:3],N,replace=TRUE))
X6 <- 3*runif(N)
mu <- c(-1,0,1,2)[as.numeric(X3)]

Y <- X1**1.5 + 2 * (X2**.5) + mu
Y <- Y + rnorm(N,0,sigma)

data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
print(data2[1:10,])

# predict on the new data using "best" number of trees
#   f.predict will be on the canonical scale (logit,log,etc.)
f.predict <- predict(gbm1,data2,best.iter) 

print(f.predict[1:10])
# least squares error
print(sum((data2$Y-f.predict)^2))

# create marginal plots
# plot variable X1,X2,X3 after "best" iterations
par.old <- par(mfrow=c(1,3))
plot(gbm1,1,best.iter)
plot(gbm1,2,best.iter)
plot(gbm1,3,best.iter)
par(mfrow=c(1,1))
plot(gbm1,1:2,best.iter) # contour plot of variables 1 and 2 after "best" number iterations
plot(gbm1,2:3,best.iter) # lattice plot of variables 2 and 3 after "best" number iterations
plot(gbm1,3:4,best.iter) # lattice plot of variables 2 and 3 after "best" number iterations

plot(gbm1,c(1,2,6),best.iter,cont=20) # 3-way plots
plot(gbm1,1:3,best.iter)
plot(gbm1,2:4,best.iter)
plot(gbm1,3:5,best.iter)
par(par.old) # reset graphics options to previous settings

# check interactions
interact(gbm1,data=data,var_indices=1:2, num_trees=best.iter)
# get all two way interactions
i.var <- subset(expand.grid(x1=1:6,x2=1:6), x1<x2)
rownames(i.var) <- apply(i.var,1,paste,collapse=":",sep="")
apply(i.var, 1,
      function(i.var) interact(gbm1,data=data,var_indices=i.var,num_trees=best.iter))

## ----message=FALSE, echo=FALSE------------------------------------------------
library("gbm3")

## -----------------------------------------------------------------------------
# create some binomial response data - N observations 
N <- 1000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=T))
mu <- c(-1,0,1,2)[as.numeric(X3)]

p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
Y <- rbinom(N,1,p) # response

# Normalized weights for each observation
w <- rexp(N)
w <- N*w/sum(w)

# Offset is set to 0
offset <- rep(0, N)

# Collate data
data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)

## -----------------------------------------------------------------------------
available_distributions()

## -----------------------------------------------------------------------------
# Create a default Bernoulli 
bern_dist <- gbm_dist("Bernoulli")

## -----------------------------------------------------------------------------
# Creating an appropriate training parameters object
train_params <- training_params(num_trees = 2000, interaction_depth = 3, 
                                num_train=nrow(data), num_features = 3)

## -----------------------------------------------------------------------------
# Example of a gbmParallel with more threads
par_detail <- gbmParallel(num_threads = 2) # Pass to par_details in gbmt

## ----message=FALSE------------------------------------------------------------
# Create a gbm fit
fit <- gbmt(Y ~ X1 + X2 + X3, distribution=bern_dist, data=data, weights = w, 
            offset=offset, train_params = train_params,
            cv_folds=5, keep_gbm_data=TRUE)

## ----message=FALSE------------------------------------------------------------
# A default fit to gaussian data
# Create data
N <- 1000
X1 <- runif(N)
X2 <- 2*runif(N)
X3 <- factor(sample(letters[1:4],N,replace=T))
X4 <- ordered(sample(letters[1:6],N,replace=T))
X5 <- factor(sample(letters[1:3],N,replace=T))
X6 <- 3*runif(N)
mu <- c(-1,0,1,2)[as.numeric(X3)]

SNR <- 10 # signal-to-noise ratio
Y <- X1**1.5 + 2 * (X2**.5) + mu
sigma <- sqrt(var(Y)/SNR)
Y <- Y + rnorm(N,0,sigma)

# create a bunch of missing values
X1[sample(1:N,size=100)] <- NA
X3[sample(1:N,size=300)] <- NA

# Gaussian data
gauss_data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

# Perform a cross-validated fit
gauss_fit <- gbmt(Y ~ X1 + X2 + X3 + X4 + X5 + X6, 
                  data=gauss_data, cv_folds =5, keep_gbm_data = TRUE)

## ----message=FALSE------------------------------------------------------------
# Use gaussian fit and determine the performance
# red line is testing error
# green line is cv error
# blue line is OOB error
best_iter_test <- gbmt_performance(gauss_fit, method='test')
best_iter_cv <- gbmt_performance(gauss_fit, method='cv')
best_iter_oob <- gbmt_performance(gauss_fit, method='OOB')

## ----message=FALSE------------------------------------------------------------
# Fit additional trees to gaussian fit and recalculate optimal number of iterations
gauss_fit_more <- gbm_more(gauss_fit, num_new_trees = 5000) # This is a large number of 
                                                            #extra trees!
best_iter_test_more <- gbmt_performance(gauss_fit_more, method='test')

## ----message=FALSE------------------------------------------------------------
# Given additional data - of the same shape as our gaussian example
X1 <- runif(N)
X2 <- 2*runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
X5 <- factor(sample(letters[1:3],N,replace=TRUE))
X6 <- 3*runif(N)
mu <- c(-1,0,1,2)[as.numeric(X3)]

Y <- X1**1.5 + 2 * (X2**.5) + mu
Y <- Y + rnorm(N,0,sigma)

data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

# Then predictions can be made
preds <- predict(gauss_fit_more, newdata=data2, n.trees=best_iter_test_more)

## -----------------------------------------------------------------------------
# Relative influence of predictors - both examples
print(relative_influence(gauss_fit_more, best_iter_test_more))
print(relative_influence(fit, gbmt_performance(fit, method='cv')))

## -----------------------------------------------------------------------------
# Examine the marginal effects of the first two variables - gaussian example
# Shows the fitted model predictions as a function of 1st two predictors
# with all the others integrated out
plot(gauss_fit_more, var_index = 1:2)

# Create some output to test the new print function and
# to be comparable with any future changes to gbmt.

library( MASS )

data( birthwt )
data( VA )
data( iris )
data( fgl )
data( cpus )
data( housing )

set.seed( 20090417 )

t_params <- training_params(num_trees = 1000, shrinkage = 0.01, num_features = ncol(birthwt)-1,
                            num_train = nrow(birthwt), id = seq_len(nrow(birthwt)))
bMod <- gbmt( low ~ ., data=birthwt, distribution=gbm_dist("Bernoulli"),
            train_params=t_params, cv_folds=5,
             is_verbose = FALSE)
bMod

bwt <- birthwt
bwt <- bwt[ sample( 1:nrow( bwt ) ),]
t_params <- training_params(num_trees = 1000, shrinkage = 0.01, num_features = ncol(bwt)-1,
                            num_train = round(0.9 * nrow(bwt)), id = seq_len(nrow(bwt)))
aMod <- gbmt( low ~ ., data=bwt, distribution=gbm_dist("AdaBoost"),
             train_params=t_params, cv_folds=10,
            is_verbose = FALSE )
aMod

t_params <- training_params(num_trees = 1000, shrinkage = 0.1, num_features = ncol(VA)-1,
                            num_train = nrow(VA), id = seq_len(nrow(VA)))
cMod <- gbmt( Surv( stime, status ) ~ treat + age + Karn + diag.time + cell + prior,
             data = VA, distribution=gbm_dist("CoxPH"), train_params=t_params, cv_folds = 5,
             is_verbose = FALSE)
cMod

t_params <- training_params(num_trees = 1000, shrinkage = 0.1, num_features = ncol(iris)-1,
                            num_train = round(0.9 * nrow(iris)), id = seq_len(nrow(iris)))
kMod <- gbmt( Species ~ . , data=iris , distribution = gbm_dist("Gaussian"), train_params=t_params,
             cv_folds=5)
kMod

t_params <- training_params(num_trees = 1000, shrinkage = 0.01, num_features = ncol(fgl)-1,
                            num_train = nrow(fgl), id = seq_len(nrow(fgl)))
kMod2 <- gbmt( type ~ ., data=fgl, distribution = gbm_dist("Gaussian"), train_params=t_params,
              cv_folds=5)
kMod2

mycpus <- cpus
mycpus <- mycpus[, -1 ]
t_params <- training_params(num_trees = 1000, shrinkage = 0.01, num_features = ncol(mycpus)-1,
                            num_train = nrow(mycpus), id = seq_len(nrow(mycpus)))
gMod <- gbmt( log( perf ) ~ ., data = mycpus, distribution=gbm_dist("Gaussian"),
             cv_folds=5, train_params = t_params,
             is_verbose = FALSE)
gMod

biMod <- gbmt( log(perf) ~ ., data=mycpus, distribution = gbm_dist("Gaussian"),
              cv_folds=5, train_params = t_params)
biMod

t_params <- training_params(num_trees = 1000, shrinkage = 0.01, num_features = ncol(mycpus)-1,
                            num_train = nrow(mycpus), id = seq_len(nrow(mycpus)),
                            interaction_depth = 3)
tMod <- gbmt( log(perf) ~ ., data=mycpus, distribution=gbm_dist("TDist"),
             cv_folds=5, train_params = t_params)
tMod

lMod <- gbmt( log(perf) ~ ., data=mycpus, distribution=gbm_dist("Laplace"),
             cv_folds=5, train_params = t_params)
lMod

qMod <- gbmt( log(perf) ~ ., data=mycpus,
             distribution=gbm_dist(name="Quantile", alpha=.7),
             cv_folds=5, train_params = t_params, is_verbose = FALSE)
qMod

t_params <- training_params(num_trees = 1000, shrinkage = 0.01, num_features = ncol(housing)-1,
                            num_train = nrow(housing), id = seq_len(nrow(housing)))
pMod <- gbmt( Freq ~ ., data=housing , distribution=gbm_dist("Poisson"),
              cv_folds=5 , train_params = t_params)
pMod

## ----message=FALSE, echo=FALSE------------------------------------------------
library("gbm3")

## -----------------------------------------------------------------------------
# Create strata
strats <- sample(1:5, 100, replace=TRUE)

# Create CoxPH dist object
cox_dist <- gbm_dist(name="CoxPH", ties="breslow", 
                     strata=strats, prior_node_coeff_var=100)


## -----------------------------------------------------------------------------
# Create pairwise grouped data
# create query groups, with an average size of 25 items each
N <- 1000
num.queries <- floor(N/25)
query <- sample(1:num.queries, N, replace=TRUE)

# X1 is a variable determined by query group only
query.level <- runif(num.queries)
X1 <- query.level[query]

# X2 varies with each item
X2 <- runif(N)

# X3 is uncorrelated with target
X3 <- runif(N)

# The target
Y <- X1 + X2

# Add some random noise to X2 that is correlated with
# queries, but uncorrelated with items

X2 <- X2 + scale(runif(num.queries))[query]

# Add some random noise to target
SNR <- 5 # signal-to-noise ratio
sigma <- sqrt(var(Y)/SNR)
Y <- Y + runif(N, 0, sigma)

data <- data.frame(Y, query=query, X1, X2, X3)

# Create appropriate Pairwise object
pair_dist <- gbm_dist(name="Pairwise", group="query", max_rank=1, metric="ndcg")

## -----------------------------------------------------------------------------
# Create a QuantileGBMDist object
quant_dist <- gbm_dist(name="Quantile", alpha=0.1)

## -----------------------------------------------------------------------------
# Creat a t-distribution object with 7 degrees of freedom
t_dist <- gbm_dist(name="TDist", df=7)

## -----------------------------------------------------------------------------
# Create a TweedieGBMDist object with a compound Poisson-Gamma power
tweedie_dist <- gbm_dist(name="Tweedie", power=1.5)

# Use the Gamma distribution for the Tweedie p = 2 endpoint
gamma_dist <- gbm_dist(name="Gamma")

## ----message=FALSE, echo=FALSE------------------------------------------------
library("gbm3")
set.seed(557165559)

## ----shrinkViters, fig.cap="**Figure 1**. Out-of-sample predictive performance by number of iterations and shrinkage. Smaller values of the shrinkage parameter offer improved predictive performance, but with decreasing marginal improvement", echo=FALSE, cache=TRUE, out.width="100%", dpi=300----
# N <- 10000
# X1 <- runif(N)
# X2 <- 2*runif(N)
# X3 <- ordered(sample(letters[1:4],N,replace=TRUE),levels=letters[4:1])
# X4 <- factor(sample(letters[1:6],N,replace=TRUE))
# X5 <- factor(sample(letters[1:3],N,replace=TRUE))
# X6 <- 3*runif(N)
# mu <- c(-1,0,1,2)[as.numeric(X3)]
# 
# SNR <- 10 # signal-to-noise ratio
# Y <- X1**1.5 + 2 * (X2**.5) + mu
# sigma <- sqrt(var(Y)/SNR)
# Y <- Y + rnorm(N,0,sigma)
# 
# ## introduce some missing values
# X1[sample(1:N,size=500)] <- NA
# X4[sample(1:N,size=300)] <- NA
# 
# data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
# 
# ## fit initial model
# shrink <- c(0.1,0.05,0.01,0.005,0.001)
# err <- vector("list",length(shrink))
# 
# for(i in 1:length(shrink))
# {
#     gbm1 <- gbm(Y~X1+X2+X3+X4+X5+X6,
#                 data=data,
#                 distribution="gaussian",
#                 n.trees=10000,
#                 shrinkage=shrink[i],
#                 interaction.depth=3,
#                 bag.fraction = 0.5,
#                 train.fraction = 0.2,
#                 n.minobsinnode = 10,
#                 keep.data=FALSE,
#                 verbose=FALSE)
#     err[[i]] <- gbm1$valid.error
# }
# 
# ylim <- range(unlist(lapply(err,range)))
# mins <- min(sapply(err,min))
# ylim <- c(0.185,0.21)
# 
# plot(0,0,ylim=ylim,xlim=c(0,10000),type="n",xlab="Iterations",ylab="Squared error")
# for(i in 1:length(shrink))
#     {
#         x <- which.min(err[[i]])
#         y <- err[[i]][x]
#         j <- round(seq(1,10000,length=500))
#         j <- sort(c(j,x))
# 
#         k <- 1:length(j)
#         lines(j[k],err[[i]][j][k],col=i)
#         rug(x, col=i)
#         text(x,y-0.0005,as.character(shrink[i]),adj=1)
#     }
# 
# abline(h=min(mins))

knitr::include_graphics("shrinkViters.png")

## ----oobperf2, echo = FALSE, message=FALSE, fig.align='center', fig.cap='**Figure 2**. Out-of-sample predictive performance of four methods of selecting the optimal number of iterations. The vertical axis plots performance relative the best. The boxplots indicate relative performance across thirteen real datasets from the UCI repository. See `demo(OOB-reps)`.', out.width="80%"----

knitr::include_graphics("oobperf2.png")

## ----message=FALSE, echo=FALSE------------------------------------------------
library("gbm3")

## ----message=FALSE------------------------------------------------------------
## Install package
require(survival)

# get datasets
right_cens <- cgd[cgd$enum==1, ]
start_stop <- cgd

# Set up GBMDist objects
right_cens_dist <- gbm_dist("CoxPH", strata=right_cens$hos.cat)
start_stop_dist <- gbm_dist("CoxPH", strata=start_stop$hos.cat)

## ----messagee=FALSE-----------------------------------------------------------
# Set-up training parameters
params_right_cens <- training_params(num_trees = 2000, interaction_depth = 3, 
                                     id=right_cens$id,
                                     num_train=round(0.5 * length(unique(right_cens$id))) )
params_start_stop <- training_params(num_trees = 2000, interaction_depth = 3, 
                                     id=start_stop$id,
                                     num_train=round(0.5 * length(unique(start_stop$id))) )

# Call to gbmt
fit_right_cens <- gbmt(Surv(tstop, status)~ age + sex + inherit +
                     steroids + propylac, data=right_cens, 
                     distribution = right_cens_dist,
                     train_params = params_right_cens, cv_folds=10,
                     keep_gbm_data = TRUE)
fit_start_stop <- gbmt(Surv(tstart, tstop, status)~ age + sex + inherit +
                     steroids + propylac, data=start_stop, 
                     distribution = start_stop_dist,
                     train_params = params_start_stop, cv_folds=10, 
                     keep_gbm_data = TRUE)

# Plot performance
best_iter_right <- gbmt_performance(fit_right_cens, method='test')
best_iter_stop_start <- gbmt_performance(fit_start_stop, method='test')

## ----message=FALSE, fig.height=6, fig.width=6---------------------------------
# Example using Breslow and Efron tie-breaking

# Create data
require(survival)

set.seed(1)
N <- 3000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=T))
mu <- c(-1,0,1,2)[as.numeric(X3)]

f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
tt.surv <- rexp(N,exp(f))
tt.cens <- rexp(N,0.5)
delta <- as.numeric(tt.surv <= tt.cens)
tt <- apply(cbind(tt.surv,tt.cens),1,min)

# throw in some missing values
X1[sample(1:N,size=100)] <- NA
X3[sample(1:N,size=300)] <- NA

# random weights if you want to experiment with them
w <- rep(1,N)
data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)

# Set up distribution objects
cox_breslow <- gbm_dist("CoxPH", ties="breslow", prior_node_coeff_var=100)
cox_efron <- gbm_dist("CoxPH", ties="efron", prior_node_coeff_var=100)

# Define training parameters
params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                          shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), 
                          num_train=N/2, num_features=3)

# Fit gbm 
fit_breslow <- gbmt(Surv(tt, delta)~X1+X2+X3, data=data, distribution=cox_breslow, 
                    weights=w, train_params=params, var_monotone=c(0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)

fit_efron <- gbmt(Surv(tt, delta)~X1+X2+X3, data=data, distribution=cox_efron,
                  weights=w, train_params=params, var_monotone=c(0, 0, 0), 
                  keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)


# Evaluate fit 
plot(gbmt_performance(fit_breslow, method='test'))
legend("topleft", c("training error", "test error", "optimal iteration"),
       lty=c(1, 1, 2), col=c("black", "red", "blue"))
plot(gbmt_performance(fit_efron, method='test'))
legend("topleft", c("training error", "test error", "optimal iteration"),
       lty=c(1, 1, 2), col=c("black", "red", "blue"))

# LOGISTIC REGRESSION EXAMPLE

cat("Running logistic regression example.\n")
# create some data
N <- 1000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
mu <- c(-1,0,1,2)[as.numeric(X3)]

p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
Y <- rbinom(N,1,p)

# random weights if you want to experiment with them
w <- rexp(N)
w <- N*w/sum(w)

data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)

# fit initial model
gbm1 <- gbm(Y~X1+X2+X3,                # formula
            data=data,                 # dataset
            weights=w,
            var.monotone=c(0,0,0),     # -1: monotone decrease, 
                                       # +1: monotone increase, 
                                       # 0: no monotone restrictions
            distribution="bernoulli",
            n.trees=3000,              # number of trees
            shrinkage=0.001,           # shrinkage/learning rate, 0.001 to 0.1 usually work
            interaction.depth=3,       # 1: additive model, 2: two-way interactions, etc
            bag.fraction = 0.5,        # subsampling fraction, 0.5 is probably best
            train.fraction = 0.5,      # fraction of data for training, 
                                       #   first train.fraction*N used for training
            cv.folds=5,                # do 5-fold cross-validation
            n.minobsinnode = 10,       # minimum total weight needed in each node
            verbose = FALSE)           # don't print progress

# plot the performance
# returns out-of-bag estimated best number of trees
best.iter.oob <- gbmt_performance(gbm1,method="OOB")  
print(best.iter.oob)

# returns 5-fold cv estimate of best number of trees
best.iter.cv <- gbmt_performance(gbm1,method="cv")   
print(best.iter.cv)

# returns test set estimate of best number of trees
best.iter.test <- gbmt_performance(gbm1,method="test") 
print(best.iter.test)

best.iter <- best.iter.test

# plot variable influence
summary(gbm1,num_trees=1)         # based on the first tree
summary(gbm1,num_trees=best.iter) # based on the estimated best number of trees

# create marginal plots
# plot variable X1,X2,X3 after "best" iterations
par.old <- par(mfrow=c(1,3))
plot(gbm1,1,best.iter)
plot(gbm1,2,best.iter)
plot(gbm1,3,best.iter)
par(mfrow=c(1,1))
plot(gbm1,1:2,best.iter) # contour plot of variables 1 and 2 after "best" number iterations
plot(gbm1,2:3,best.iter) # lattice plot of variables 2 and 3 after "best" number iterations

# 3-way plot
plot(gbm1,1:3,best.iter)

# print the first and last trees
print(pretty_gbm_tree(gbm1,1))
print(pretty_gbm_tree(gbm1, gbm1$params$num_trees))

# make some new data
N <- 1000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
mu <- c(-1,0,1,2)[as.numeric(X3)]

p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
Y <- rbinom(N,1,p)
data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)

# predict on the new data using "best" number of trees
# f.predict will be on the canonical scale (logit,log,etc.)
f.predict <- predict(gbm1,data2,
                         num_trees=c(best.iter.oob,best.iter.cv,best.iter.test))
# transform to probability scale for logistic regression
p.pred <- 1/(1+exp(-f.predict))

# calibration plot for logistic regression - well calibrated means a 45 degree line
par(mfrow=c(1,1))
calibrate_plot(Y,p.pred[,3])
par(par.old) # reset graphics options to previous settings

# logistic error
sum(data2$Y*f.predict[,1] - log(1+exp(f.predict[,1])))
sum(data2$Y*f.predict[,2] - log(1+exp(f.predict[,2])))
sum(data2$Y*f.predict[,3] - log(1+exp(f.predict[,3])))

# COX PROPORTIONAL HAZARDS REGRESSION EXAMPLE
require(survival)
cat("Running cox proportional hazards regression example.\n")
# create some data
set.seed(1)
N <- 3000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
mu <- c(-1,0,1,2)[as.numeric(X3)]

f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
tt.surv <- rexp(N,exp(f))
tt.cens <- rexp(N,0.5)
delta <- as.numeric(tt.surv <= tt.cens)
tt <- apply(cbind(tt.surv,tt.cens),1,min)

# throw in some missing values
X1[sample(1:N,size=100)] <- NA
X3[sample(1:N,size=300)] <- NA

# random weights if you want to experiment with them
w <- rep(1,N)

data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)
train_params <- training_params(num_trees = 3000, 
                                shrinkage = 0.001, 
                                interaction_depth = 3,
                                min_num_obs_in_node = 10, 
                                num_train = N/2, 
                                id = seq_len(nrow(data)),
                                bag_fraction = 0.5, 
                                num_features = 3)

# fit initial model
gbm1 <- gbmt(Surv(tt,delta)~X1+X2+X3,  # formula
            data=data,                 # dataset
            weights=w,
            var_monotone=c(0,0,0),     # -1: monotone decrease, 
                                       # +1: monotone increase, 
                                       #  0: no monotone restrictions
            distribution=gbm_dist("CoxPH"),
            train_params = train_params,
            cv_folds = 5,              # do 5-fold cross-validation
            keep_gbm_data = TRUE,
            is_verbose = FALSE)        # don't print progress

# plot the performance
# returns out-of-bag estimated best number of trees
best.iter <- gbmt_performance(gbm1,method="OOB")  
print(best.iter)
# returns test set estimate of best number of trees
best.iter <- gbmt_performance(gbm1,method="cv") 
print(best.iter)
# returns test set estimate of best number of trees
best.iter <- gbmt_performance(gbm1,method="test") 
print(best.iter)

# plot variable influence
summary(gbm1,num_trees=1)         # based on the first tree
summary(gbm1,num_trees=best.iter) # based on the estimated best number of trees

# create marginal plots
# plot variable X1,X2,X3 after "best" iterations
par.old <- par(mfrow=c(1,3))
plot(gbm1,1,best.iter)
plot(gbm1,2,best.iter)
plot(gbm1,3,best.iter)
par(mfrow=c(1,1))
plot(gbm1,1:2,best.iter) # contour plot of variables 1 and 2 after "best" number iterations

# 3-way plots
plot(gbm1,1:3,best.iter)
par(par.old) # reset graphics options to previous settings

# print the first and last trees... just for curiosity
pretty_gbm_tree(gbm1,1)
pretty_gbm_tree(gbm1,gbm1$params$num_trees)

# make some new data
N <- 1000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
mu <- c(-1,0,1,2)[as.numeric(X3)]

f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
tt.surv <- rexp(N,exp(f))
tt.cens <- rexp(N,0.5)

data2 <- data.frame(tt=apply(cbind(tt.surv,tt.cens),1,min),
                    delta=as.numeric(tt.surv <= tt.cens),
                    X1=X1,X2=X2,X3=X3)

# predict on the new data using "best" number of trees
# f.predict will be on the canonical scale (logit,log,etc.)
f.predict <- predict(gbm1,data2,best.iter)

# Cox PH error
# boosting
risk <- rep(0,N)
for(i in 1:N)
{
  risk[i] <- sum( (data2$tt>=data2$tt[i])*exp(f.predict) )
}
cat("Boosting:",sum( data2$delta*( f.predict - log(risk) ) ),"\n")

# linear model
coxph1 <- coxph(Surv(tt,delta)~X1+X2+X3,data=data)
f.predict <- predict(coxph1,newdata=data2)
risk <- rep(0,N)
for(i in 1:N)
{
  risk[i] <- sum( (data2$tt>=data2$tt[i])*exp(f.predict) )
}
cat("Linear model:",sum( data2$delta*( f.predict - log(risk) ) ),"\n")

set.seed(06182001)

# number of replicates
n.reps <- 20
n.datasets <- 11 # needs to match the number of datasets
i.data <- 0

squared.error.loss <- function(y,f.x)
{
    mean((y-f.x)^2)
}

neg.bernoulli.loglikelihood <- function(y,f.x)
{
    -mean(y*f.x - log(1+exp(f.x)))
}

dataset <- vector("list",n.datasets)

# abalone
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Abalone",
       distribution="gaussian",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/abalone/",
       filename="abalone.data",
       var.names=c("sex","length","diameter","height","whole.weight",
                   "shucked.weight","viscera.weight","shell.weight",
                   "Rings"),
       outcome="Rings",
       factors="sex",
       na.strings="",
       sep=",",
       shrinkage=0.02)

# Adult
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Adult",
       distribution="bernoulli",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/adult/",
       filename="adult.data",
       var.names=c("age","workclass","w","education","education.num",
                   "marital.status","occupation","relationship","race",
                   "male","capital.gain","capital.loss",
                   "hours.per.week","native.country","income"),
       outcome="income",
       factors=c("workclass","education","marital.status","occupation",
                 "relationship","race","native.country","male"),
       na.strings="?",
       sep=",",
       shrinkage=0.04)

# Housing
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Boston housing",
       distribution="gaussian",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/housing/",
       filename="housing.data",
       var.names=c("CRIM","ZN","INDUS","CHAS","NOX","RM","AGE",
                   "DIS","RAD","TAX","PTRATIO","B","LSTAT","MEDV"),
       factors=NULL,
       outcome="MEDV",
       na.strings="",
       sep="",
       shrinkage=0.005)

# mushrooms
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Mushrooms",
       distribution="bernoulli",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/mushroom/",
       filename="agaricus-lepiota.data",
       var.names=c("poisonous","cap-shape","cap-surface","cap-color",
                   "bruises","odor","gill-attachment",
                   "gill-spacing","gill-size","gill-color",
                   "stalk-shape","stalk-root","stalk-surface-above-ring",
                   "stalk-surface-below-ring","stalk-color-above-ring",
                   "stalk-color-below-ring","veil-type","veil-color",
                   "ring-number","ring-type","spore-print-color",
                   "population","habitat"),
       factors=c("cap-shape","cap-surface","cap-color",
                 "bruises","odor","gill-attachment",
                 "gill-spacing","gill-size","gill-color",
                 "stalk-shape","stalk-root","stalk-surface-above-ring",
                 "stalk-surface-below-ring","stalk-color-above-ring",
                 "stalk-color-below-ring","veil-type","veil-color",
                 "ring-number","ring-type","spore-print-color",
                 "population","habitat"),
       outcome="poisonous",
       drop.vars=c("veil-type"),
       na.strings="?",
       sep=",",
       shrinkage=0.05)

# autoprices 1
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Auto Prices",
       distribution="gaussian",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/autos/",
       filename="imports-85.data",
       var.names=c("symboling","normalizedlosses","make","fueltype",
                   "aspiration","ndoors","bodystyle",
                   "drivewheels","enginelocation", "wheelbase", "length",
                   "width", "height", "curbweight", "enginetype",
                   "numerofcylinders", "enginesize", "fuelsystem", "bore",
                   "stroke", "compressionratio", "horsepower", "peakrpm",
                   "citympg", "highwatmpg", "price"),
       factors=c("symboling","make","fueltype","aspiration","ndoors",
                 "bodystyle","drivewheels","enginelocation", "enginetype",
                 "numerofcylinders", "fuelsystem"),
       outcome="price",
       na.strings="?",
       sep=",",
       shrinkage=0.002)

# auto MPG
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Auto MPG",
       distribution="gaussian",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/auto-mpg/",
       filename="auto-mpg.data",
       var.names=c("mpg","cylinders","displacement","horsepower","weight",
                   "acceleration","modelyear","origin","carname"),
       factors=c("cylinders", "modelyear", "origin"),
       outcome="mpg",
       drop.vars=c("carname"),
       na.strings="?",
       sep="",
       shrinkage=0.005)

# CPU
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="CPU Performance",
       distribution="gaussian",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/cpu-performance/",
       filename="machine.data",
       var.names=c("vendorname","modelname","myct","mmin","mmax",
                   "cach","chmin","chmax","prp","ERP"),
       factors=c("vendorname","modelname"),
       outcome="prp",
       na.strings="",
       drop.vars=c("vendorname","modelname"),
       sep=",",
       shrinkage=0.01)

# credit
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Credit rating",
       distribution="bernoulli",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/credit-screening/",
       filename="crx.data",
       var.names=c("A1","A2","A3","A4","A5","A6","A7","A8","A9","A10","A11",
                   "A12", "A13", "A14", "A15","CLASS"),
       factors=c("A1","A4", "A5", "A6", "A7", "A9", "A10", "A12", "A13","CLASS"),
       outcome="CLASS",
       na.strings="?",
       sep=",",
       shrinkage=0.005)

# Haberman
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Haberman",
       distribution="bernoulli",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/haberman/",
       filename="haberman.data",
       var.names=c("age","year","nodes","CLASS"),
       outcome="CLASS",
       factors=c("CLASS"),
       na.strings="",
       sep=",",
       shrinkage=0.001)

# Diabetes - seems removed from UCI repository
# i.data <- i.data + 1
# dataset[[i.data]] <-
#    list(name="Diabetes",
#          distribution="bernoulli",
#          urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/pima-indians-diabetes/",
#          filename="pima-indians-diabetes.data",
#          var.names=c("n_preg","plasma","blood-pre","triceps","serum",
#                   "mass-index","pedigree","age","CLASS"),
#          factors=c("CLASS"),
#          outcome="CLASS",
#          na.strings="?",
#          sep=",",
#          shrinkage=0.005)

# Ionosphere
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="Ionosphere",
       distribution="bernoulli",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/ionosphere/",
       filename="ionosphere.data",
       var.names=c("A1","A2","A3","A4","A5","A6","A7","A8","A9","A10","A11",
                   "A12","A13","A14","A15","A16","A17","A18","A19","A20",
                   "A21","A22","A23","A24","A25","A26","A27","A28","A29",
                   "A30","A31","A32","A33","A34","CLASS"),
       factors=c("CLASS"),
       outcome="CLASS",
       na.strings="",
       sep=",",
       shrinkage=0.005)

# Breast cancer
i.data <- i.data + 1
dataset[[i.data]] <-
  list(name="breast cancer",
       distribution="bernoulli",
       urlpath="https://archive.ics.uci.edu/ml/machine-learning-databases/breast-cancer-wisconsin/",
       filename="breast-cancer-wisconsin.data",
       var.names=c("CODE","thickness","cellsize","cellshape","adhension",
                   "singleecell","bnuclei","chromatin","nnucleo","mitoses",
                   "CLASS"),
       factors=c("CODE","CLASS"),
       outcome="CLASS",
       drop.vars=c("CODE"),
       na.strings="?",
       sep=",",
       shrinkage=0.005)

# Load datasets
for(i.data in 1:n.datasets)
  # for(i.data in which(sapply(dataset,function(x){is.null(x$oob.iter)})))
{
  # Progress
  cat("Dataset ",i.data,":",dataset[[i.data]]$name," N = ")
  filename <- paste0(dataset[[i.data]]$url,
                     dataset[[i.data]]$filename)
  dataset[[i.data]]$data <-
    read.table(file=filename,
               na.strings=dataset[[i.data]]$na.strings,
               sep=dataset[[i.data]]$sep,
               header=is.null(dataset[[i.data]]$var.names))
  if(!is.null(dataset[[i.data]]$var.names))
  {
    names(dataset[[i.data]]$data) <- dataset[[i.data]]$var.names
  }
  
  # take care of nominal predictors
  for(j in dataset[[i.data]]$factors)
  {
    dataset[[i.data]]$data[,j] <- factor(dataset[[i.data]]$data[,j])
  }
  
  # take care of factor binary outcomes
  if( with(dataset[[i.data]],
           (distribution=="bernoulli") && is.character(data[,outcome])) )
  {
    dataset[[i.data]]$data[,dataset[[i.data]]$outcome] <-
      with(dataset[[i.data]], as.factor(data[,outcome]))
  }
  if( with(dataset[[i.data]],
           (distribution=="bernoulli") && is.factor(data[,outcome])) )
  {
    dataset[[i.data]]$data[,dataset[[i.data]]$outcome] <-
      with(dataset[[i.data]], as.numeric(data[,outcome])-1)
  }
  
  # drop observations with missing outcomes
  i <- with(dataset[[i.data]], !is.na(data[,outcome]))
  dataset[[i.data]]$data <- dataset[[i.data]]$data[i,]
  
  # drop selected predictor variables
  if(!is.null(dataset[[i.data]]$drop.vars))
  {
    j <- match(dataset[[i.data]]$drop.vars,names(dataset[[i.data]]$data))
    dataset[[i.data]]$data <- dataset[[i.data]]$data[,-j]
  }
  
  cat(nrow(dataset[[i.data]]$data),"\n")
}


# loop over all the datasets
i.datasets <- which(sapply(dataset,function(x){is.null(x$oob.loss)}))
for(i.data in i.datasets)
{
   N <- nrow(dataset[[i.data]]$data)
   # Progress
   cat("Dataset ",i.data,":",dataset[[i.data]]$name," N = ",N,"\n",sep="")

   # construct model formula for this dataset
   formula.fit <- formula(paste(dataset[[i.data]]$outcome,"~ ."))

   # initialize prediction
   pred.oob <- pred.base <- pred.test33 <- pred.test20 <- pred.cv5 <- rep(0,N)

   # track iteration estimates
   dataset[[i.data]]$oob.iter    <- rep(NA,n.reps)
   dataset[[i.data]]$test33.iter <- rep(NA,n.reps)
   dataset[[i.data]]$test20.iter <- rep(NA,n.reps)
   dataset[[i.data]]$cv5.iter    <- rep(NA,n.reps)

   # do replicates
   for(i.rep in 1:n.reps)
   {
      cat("rep:",i.rep,"")
      i.train <- sample(1:N,size=0.75*N,replace=FALSE)
      i.valid <- (1:N)[-i.train]

      # use out-of-bag method
      cat("OOB, ")
      gbm1 <- gbm(formula.fit,
                  data=dataset[[i.data]]$data[i.train,],
                  distribution=dataset[[i.data]]$distribution,
                  train.fraction=1.0,
                  bag.fraction=0.5,
                  shrinkage=dataset[[i.data]]$shrinkage,
                  n.trees=1000,
                  verbose = FALSE)

      best.iter.oob <- suppressWarnings(gbmt_performance(gbm1,method="OOB"))
      while((gbm1$params$num_trees-best.iter.oob < 1000) &&
            !all(gbm1$oobag.improve[(gbm1$params$num_trees-100):gbm1$params$num_trees] < 1e-6))
      {
         gbm1 <- gbm_more(gbm1,1000)
         best.iter.oob <- suppressWarnings(gbmt_performance(gbm1,method="OOB"))
      }
      pred.oob[i.valid] <- predict(gbm1,
                                   newdata=dataset[[i.data]]$data[i.valid,],
                                   n.trees=as.numeric(best.iter.oob))
      dataset[[i.data]]$oob.iter[i.rep] <- as.numeric(best.iter.oob)

      # use a 1/3 test set
      cat("33% test data, ")
      gbm1 <- gbm(formula.fit,
                  data=dataset[[i.data]]$data[i.train,],
                  distribution=dataset[[i.data]]$distribution,
                  train.fraction=2/3,
                  bag.fraction=0.5,
                  shrinkage=dataset[[i.data]]$shrinkage,
                  n.trees=1000,
                  verbose = FALSE)
      best.iter.test <- gbmt_performance(gbm1, method="test")
      while((gbm1$params$num_trees-best.iter.test < 1000) &&
            !all(abs(gbm1$valid.error[(gbm1$params$num_trees-100):gbm1$params$num_trees]) < 1e-6))
      {
         gbm1 <- gbm_more(gbm1,1000)
         best.iter.test <- gbmt_performance(gbm1,method="test")
      }
      pred.test33[i.valid] <- predict(gbm1,
                                      newdata=dataset[[i.data]]$data[i.valid,],
                                      n.trees=as.numeric(best.iter.test))
      dataset[[i.data]]$test33.iter[i.rep] <- as.numeric(best.iter.test)

      # use a 20% test set
      cat("20% test data, ")
      gbm1 <- gbm(formula.fit,
                  data=dataset[[i.data]]$data[i.train,],
                  distribution=dataset[[i.data]]$distribution,
                  train.fraction=0.80,
                  bag.fraction=0.5,
                  shrinkage=dataset[[i.data]]$shrinkage,
                  n.trees=1000,
                  verbose = FALSE)
      best.iter.test <- gbmt_performance(gbm1, method="test")
      while((gbm1$params$num_trees-best.iter.test < 1000) &&
            !all(abs(gbm1$valid.error[(gbm1$params$num_trees-100):gbm1$params$num_trees]) < 1e-6))
      {
         gbm1 <- gbm_more(gbm1,1000)
         best.iter.test <- gbmt_performance(gbm1, method="test")
      }
      pred.test20[i.valid] <-
         predict(gbm1,
                 newdata=dataset[[i.data]]$data[i.valid,],
                 n.trees=as.numeric(best.iter.test))
      dataset[[i.data]]$test20.iter[i.rep] <- as.numeric(best.iter.test)

      # use 5-fold cross-validation
      cat("5-fold CV")
      n.cv <- 5
      cv.group  <- sample(rep(1:n.cv,length=length(i.train)))
      max.iters <- round(best.iter.test*1.2)
      cv.loss   <- matrix(0,ncol=n.cv,nrow=max.iters)
      for(i.cv in 1:n.cv)
      {
         cat(".")
         i <- order(cv.group==i.cv) # used to put the held out obs last
         gbm1 <- gbm(formula.fit,
                     data=dataset[[i.data]]$data[i.train[i],],
                     distribution=dataset[[i.data]]$distribution,
                     train.fraction=mean(cv.group!=i.cv),
                     bag.fraction=0.5,
                     shrinkage=dataset[[i.data]]$shrinkage,
                     n.trees=max.iters,
                     verbose = FALSE)
         cv.loss[,i.cv] <- gbm1$valid.error
      }
      cat("\n")

      best.iter.cv <- which.min(apply(cv.loss,1,weighted.mean,w=table(cv.group)))
      gbm1 <- gbm(formula.fit,
                  data=dataset[[i.data]]$data[i.train,],
                  distribution=dataset[[i.data]]$distribution,
                  train.fraction=1.0,
                  bag.fraction=0.5,
                  shrinkage=dataset[[i.data]]$shrinkage,
                  n.trees=best.iter.cv,
                  verbose = FALSE)
      pred.cv5[i.valid] <-
         predict(gbm1,
                 newdata=dataset[[i.data]]$data[i.valid,],
                 n.trees=best.iter.cv)
      dataset[[i.data]]$cv5.iter[i.rep] <- best.iter.cv

      # baseline prediction
      pred.base[i.valid] <- gbm1$initF

      # evaluate the methods
      if(dataset[[i.data]]$distribution=="bernoulli")
      {
        stored.loss <- neg.bernoulli.loglikelihood
      } else
      {
        stored.loss <- squared.error.loss
      }
      
      dataset[[i.data]]$base.loss[i.rep] <-
         with(dataset[[i.data]], stored.loss(data[i.valid,outcome],pred.base[i.valid]))
      dataset[[i.data]]$oob.loss[i.rep]  <-
         with(dataset[[i.data]], stored.loss(data[i.valid,outcome],pred.oob[i.valid]))
      dataset[[i.data]]$test33.loss[i.rep] <-
         with(dataset[[i.data]], stored.loss(data[i.valid,outcome],pred.test33[i.valid]))
      dataset[[i.data]]$test20.loss[i.rep] <-
         with(dataset[[i.data]], stored.loss(data[i.valid,outcome],pred.test20[i.valid]))
      dataset[[i.data]]$cv5.loss[i.rep] <-
         with(dataset[[i.data]], stored.loss(data[i.valid,outcome],pred.cv5[i.valid]))

      with(dataset[[i.data]],
          cat(oob.iter[i.rep],test33.iter[i.rep],test20.iter[i.rep],
              cv5.iter[i.rep],"\n"))
   }
}

results <- data.frame(problem=sapply(dataset,function(x){x$name}),
                      N=sapply(dataset,function(x){nrow(x$data)}),
                      d=sapply(dataset,function(x){ncol(x$data)-1}),
                      loss=sapply(dataset,function(x){x$distribution}),
                      base=sapply(dataset,function(x){mean(x$base.loss)}),
                      oob=sapply(dataset,function(x){mean(x$oob.loss)}),
                      test33=sapply(dataset,function(x){mean(x$test33.loss)}),
                      test20=sapply(dataset,function(x){mean(x$test20.loss)}),
                      cv5=sapply(dataset,function(x){mean(x$cv5.loss)}))
j <- match(c("base","oob","test33","test20","cv5"),names(results))
results$win <- c("base","oob","test33","test20","cv5")[apply(results[,j],1,which.min)]
results$oob.rank <- apply(results[,j],1,rank)[2,]
results$perf <- (results$base-results$oob)/apply(results$base-results[,j],1,max)

plot(0,0,ylim=c(0,14000),xlim=c(0,n.datasets+1),
     xlab="Dataset",ylab="Number of iterations",
     type="n",axes=FALSE)
lines(sapply(dataset,function(x){mean(x$oob.iter)}),    col="blue")
lines(sapply(dataset,function(x){mean(x$test33.iter)}), col="red")
lines(sapply(dataset,function(x){mean(x$test20.iter)}), col="green")
lines(sapply(dataset,function(x){mean(x$cv5.iter)}),    col="purple")
axis(1,at=1:n.datasets,labels=as.character(results$problem))
axis(2)

library(MASS)

set.seed(20090415)


x <- mvrnorm(100, mu=rep(0,5), Sigma=diag(rep(1,5)))
r <- rnorm(100)
r <- ifelse(runif(100) < 0.25, r*4, r)
y <- apply(x, 1, sum) + r

d <- data.frame(y=y, x)

gmod <- gbm(y ~ ., data=d, distribution="gaussian",
            n.tree = 2000, shrinkage = 0.01, cv.folds=5,
            verbose = FALSE)
tmod4 <- gbm(y ~ ., data=d, distribution="tdist", # defaults to 4 df
             n.tree=2000, shrinkage = 0.01, cv.folds=5,
             verbose = FALSE)
tmod6 <- gbm(y ~ ., data=d, distribution=list(name="tdist", df=6),
             n.tree=2000, shrinkage = 0.01, cv.folds=5,
             verbose = FALSE)
tmod100 <- gbm(y ~ ., data=d, distribution=list(name="tdist", df=100),
               n.tree=2000, shrinkage = 0.01, cv.folds=5,
               verbose = FALSE)

par.old <- par(mfrow=c(2,2))
gbest <- gbmt_performance(gmod, method="cv")
t4best <- gbmt_performance(tmod4, method="cv")
t6best <- gbmt_performance(tmod6, method="cv")
t100best <- gbmt_performance(tmod100, method="cv")
par(par.old)

library(MASS)

set.seed(20090415)


x <- mvrnorm(100, mu=rep(0,5) , Sigma=diag(rep(1,5)))
r <- rnorm(100)
r <- ifelse(runif(100) < 0.25, r*4, r)
y <- apply(x, 1, sum) + r

d <- data.frame(y=y, x)
train_params <- training_params(num_trees = 2000, 
                                interaction_depth = 1, 
                                bag_fraction = 0.5,
                                num_train = nrow(d), 
                                id = seq_len(nrow(d)), 
                                num_features = ncol(x),
                                shrinkage = 0.01)

gmod <- gbmt(y ~ ., data=d, distribution=gbm_dist("Gaussian"),
             train_params = train_params, cv_folds=5, is_verbose = FALSE)
tmod4 <- gbmt(y ~ ., data=d, distribution=gbm_dist("TDist"), # defaults to 4 df
             train_params = train_params, cv_folds=5, is_verbose = FALSE)
tmod6 <- gbmt(y ~ ., data=d, distribution=gbm_dist(name="TDist", df=6),
              train_params = train_params,cv_folds=5, is_verbose = FALSE)
tmod100 <- gbmt(y ~ ., data=d, distribution=gbm_dist(name="TDist", df=100),
               train_params = train_params, cv_folds=5, is_verbose = FALSE)

par.old <- par(mfrow=c(2,2))
gbest <- gbmt_performance(gmod , method="cv")
t4best <- gbmt_performance(tmod4 , method="cv")
t6best <- gbmt_performance(tmod6 , method="cv")
t100best <- gbmt_performance(tmod100 , method="cv")
par(par.old)

# Extract parameters from gbm to create \code{GBMDist}
#
# Internal function which creates the appropriate \code{GBMDist} from 
# the arguments of a call to \code{gbm} or \code{gbm.fit}.
#
# @usage create_dist_obj_for_gbmt_fit(distribution, tied.times.method="efron", strata=NA, prior.node.coeff.var=1000)
#
# @param distribution the list of parameters defining the distribution for gbmt_fit - see \code{gbm}
#
# @param tied.times.method This is an optional string used with
# \code{CoxPH} distribution specifying what method to employ when dealing with tied times. 
# Currently only "efron" and "breslow" are available; the default value is "efron". 
# Setting the string to any other value reverts the method to the original CoxPH model
# implementation where ties are not explicitly dealt with.
#
# @param strata Optional vector of integers (or factors) only used with the \code{CoxPH} distributions.
# Each integer in this vector represents the stratum the corresponding row in the data belongs to,
# e. g. if the 10th element is 3 then the 10th data row belongs to the 3rd strata.
#
# @param prior.node.coeff.var Optional double only used with the \code{CoxPH} distribution.
# It is a prior on the coefficient of variation associated with the hazard 
# rate assigned to each terminal node when fitting a tree.Increasing its value emphasises 
# the importance of the training data in the node when assigning a prediction to said node.
#
# @return \code{GBMDist} object
# 
# @author James Hickey

create_dist_obj_for_gbmt_fit <- function(distribution, tied.times.method="efron", strata=NA, prior.node.coeff.var=1000) {
  # Check inputs
  if(is.null(distribution$name)) stop("distribution parameter name not defined.")
  if(!(tolower(distribution$name) %in% tolower(available_distributions()))) {
    stop("The distribution ", distribution$name, " is not available in the gbm package.")
  }
  
  # Create distribution
  if(tolower(distribution$name) == "coxph") {
    dist <- create_coxph_for_gbmt_fit(distribution, tied.times.method, strata, prior.node.coeff.var)
  } else if(tolower(distribution$name) == "pairwise") {
    dist <- create_pairwise_for_gbmt_fit(distribution)
  } else if(tolower(distribution$name) == "quantile") {
    dist <- create_quantile_for_gbmt_fit(distribution)
  } else if(tolower(distribution$name) == "tdist") {
    dist <- create_tdist_for_gbmt_fit(distribution)
  } else if(tolower(distribution$name) == "tweedie") {
    dist <- create_tweedie_for_gbmt_fit(distribution)
  } else {
    # Find distribution in list of available distributions
    match_dist <- match(tolower(distribution$name), tolower(available_distributions()))
    dist <- create_dist(empty_distribution(available_distributions()[match_dist]))
  }
  
  return(dist)
}

create_coxph_for_gbmt_fit <- function(distribution, tied.times.method, strata, prior.node.coeff.var) {
  return(create_dist(empty_distribution("CoxPH"), strata, sorted=NA, 
                     ties=tied.times.method, prior_node_coeff_var=prior.node.coeff.var))
}

create_pairwise_for_gbmt_fit <- function(distribution) {
  if(is.null(distribution$group)) distribution$group <- "query"
  if(is.null(distribution$metric)) distribution$metric <- "ndcg"
  if(is.null(distribution$max.rank)) distribution$max.rank <- 0
  return(create_dist(empty_distribution("Pairwise"), 
                     distribution$group, distribution$metric, distribution$max.rank, distribution$group_index))
}

create_quantile_for_gbmt_fit <- function(distribution) {
  return(create_dist(empty_distribution("Quantile"), distribution$alpha))
}

create_tdist_for_gbmt_fit <- function(distribution) {
  return(create_dist(empty_distribution("TDist"), distribution$df))
}

create_tweedie_for_gbmt_fit <- function(distribution) {
  return(create_dist(empty_distribution("Tweedie"), distribution$power))
}

####################
# Author: James Hickey
#
# Series of tests to check the creation of distribution objects in gbm
#
####################

context("Testing creation of dist objects in gbm")
test_that("Error thrown if name in distribution is null", {
  dist <- list()
  expect_error(create_dist_obj_for_gbmt_fit(dist))
})
test_that("Error thrown if name not recognised", {
  dist <- list(name="Wrong")
  expect_error(create_dist_obj_for_gbmt_fit(dist))
})
test_that("Can create default CoxPH object", {
  def_cox_ph <- gbm_dist("CoxPH")
  dist <- create_dist_obj_for_gbmt_fit(list(name="CoxPH"))
  expect_equal(def_cox_ph, dist)
})
test_that("Can create default TDist object", {
  def_tdist_ph <- gbm_dist("TDist")
  dist <- create_dist_obj_for_gbmt_fit(list(name="TDist"))
  expect_equal(def_tdist_ph, dist)
})
test_that("Can create default Pairwise object", {
  def_pairwise_ph <- gbm_dist("Pairwise")
  dist <- create_dist_obj_for_gbmt_fit(list(name="Pairwise"))
  expect_equal(def_pairwise_ph, dist)
})
test_that("Can create default Quantile object", {
  def_quantile_ph <- gbm_dist("Quantile")
  dist <- create_dist_obj_for_gbmt_fit(list(name="Quantile"))
  expect_equal(def_quantile_ph, dist)
})
test_that("Can create default Tweedie object", {
  def_tweedie_ph <- gbm_dist("Tweedie")
  dist <- create_dist_obj_for_gbmt_fit(list(name="Tweedie"))
  expect_equal(def_tweedie_ph, dist)
})
test_that("Can create other distributions", {
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="AdaBoost")), gbm_dist("AdaBoost"))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Bernoulli")), gbm_dist("Bernoulli"))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Gamma")), gbm_dist("Gamma"))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Gaussian")), gbm_dist("Gaussian"))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Laplace")), gbm_dist("Laplace"))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Huberized")), gbm_dist("Huberized"))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Poisson")), gbm_dist("Poisson"))
})
test_that("Can set misc", {
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Quantile", alpha=1.0)), gbm_dist("Quantile", alpha=1.0))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="TDist", df=4.5)), gbm_dist("TDist", df=4.5))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Tweedie", power=3.0)), gbm_dist("Tweedie", power=3.0))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="Pairwise", metric="ndcg", max.rank=1, group="g")),
               gbm_dist("Pairwise", metric="ndcg", max_rank=1, group="g"))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="CoxPH"), tied.times.method="breslow", prior.node.coeff.var=10, strata=c(1, 2)),
               gbm_dist("CoxPH", ties="breslow", strata=c(1,2), prior_node_coeff_var=10))
})



  

####################
# Author: James Hickey
#
# Series of tests to check creation of GBMDist object
# for old gbm API.
#
####################
context("Test the creation of GBMDist objects for old API")
test_that("Error thrown if distribution not recognised", {
  # Given a made up distribution
  dist <- list(name="woooooo! Error!")
  
  # When create_dist_obj_for_gbmt_fit is called
  # Then an error is thrown
  expect_error(create_dist_obj_for_gbmt_fit(dist), 
               paste("The distribution",dist$name,"is not available in the gbm package."))
})
test_that("Can create default distributions", {
  # Given default distributions
  # When creating similar defaults from create_dist_obj_for_gbmt_fit
  # Then successfully create those objects
  expect_equal(gbm_dist("AdaBoost"), create_dist_obj_for_gbmt_fit(list(name="AdaBoost")))
  expect_equal(gbm_dist("Bernoulli"), create_dist_obj_for_gbmt_fit(list(name="Bernoulli")))
  expect_equal(gbm_dist("CoxPH"), create_dist_obj_for_gbmt_fit(list(name="CoxPH")))
  expect_equal(gbm_dist("Gamma"), create_dist_obj_for_gbmt_fit(list(name="Gamma")))
  expect_equal(gbm_dist("Gaussian"), create_dist_obj_for_gbmt_fit(list(name="Gaussian")))
  expect_equal(gbm_dist("Huberized"), create_dist_obj_for_gbmt_fit(list(name="Huberized")))
  expect_equal(gbm_dist("Laplace"), create_dist_obj_for_gbmt_fit(list(name="Laplace")))
  expect_equal(gbm_dist("Pairwise"), create_dist_obj_for_gbmt_fit(list(name="Pairwise")))
  expect_equal(gbm_dist("Poisson"), create_dist_obj_for_gbmt_fit(list(name="Poisson")))
  expect_equal(gbm_dist("Quantile"), create_dist_obj_for_gbmt_fit(list(name="Quantile")))
  expect_equal(gbm_dist("TDist"), create_dist_obj_for_gbmt_fit(list(name="TDist")))
  expect_equal(gbm_dist("Tweedie"), create_dist_obj_for_gbmt_fit(list(name="Tweedie")))
})
test_that("Can specify model dependent data - CoxPH", {
  # Given a distribution list with strata, tied.times.method
  # and prior.node.coeff.var
  dist <- list(name="CoxPH")
  strata <- c(1, 2, 1)
  tied.times.method <- "breslow"
  prior.node.coeff.var <- 150.4
  
  # When distribution object is created
  dist_obj <- create_dist_obj_for_gbmt_fit(dist, tied.times.method, 
                                           strata, prior.node.coeff.var)
  
  # Then it has the correct model dependent data
  expect_equal(dist_obj$original_strata_id, strata)
  expect_equal(dist_obj$ties, tied.times.method)
  expect_equal(dist_obj$prior_node_coeff_var, prior.node.coeff.var)
})
test_that("Can specify model dependent data - Pairwise", {
  # Given a distribution list with metric, max.rank and group
  dist <- list(name="Pairwise", metric="mrr", max.rank=1, group="query")
  
  # When distribution object is created
  dist_obj <- create_dist_obj_for_gbmt_fit(dist)
  
  # Then it has the correct model dependent data
  expect_equal(dist_obj$metric, dist$metric)
  expect_equal(dist_obj$group, dist$group)
  expect_equal(dist_obj$max_rank, dist$max.rank)
})
test_that("Can specify model dependent data - Quantile", {
  # Given a distribution list with alpha specified
  dist <- list(name="Quantile", alpha=0.5)
  
  # When distribution object is created
  dist_obj <- create_dist_obj_for_gbmt_fit(dist)
  
  # Then it has the correct alpha
  expect_equal(dist_obj$alpha, dist$alpha)
})
test_that("Can specify model dependent data - TDist", {
  # Given a distribution list with df specified
  dist <- list(name="TDist", df=10)
  
  # When distribution object is created
  dist_obj <- create_dist_obj_for_gbmt_fit(dist)
  
  # Then it has the correct df
  expect_equal(dist_obj$df, dist$df)
})
test_that("Can specify model dependent data - Tweedie", {
  # Given a distribution list with power specified
  dist <- list(name="Tweedie", power=100)
  
  # When distribution object is created
  dist_obj <- create_dist_obj_for_gbmt_fit(dist)
  
  # Then it has the correct power
  expect_equal(dist_obj$power, dist$power)
})
test_that("Case of distribution name characters is irrelevant", {
  # Creating distributions - then case of letters in name doesn't 
  # matter
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="adaboost")), 
               create_dist_obj_for_gbmt_fit(list(name="AdaBoost")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="bernoulli")),
               create_dist_obj_for_gbmt_fit(list(name="Bernoulli")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="coxph")),
               create_dist_obj_for_gbmt_fit(list(name="CoxPH")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="gamma")),
               create_dist_obj_for_gbmt_fit(list(name="Gamma")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="gaussian")),
               create_dist_obj_for_gbmt_fit(list(name="Gaussian")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="huberized")),
               create_dist_obj_for_gbmt_fit(list(name="Huberized")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="laplace")),
               create_dist_obj_for_gbmt_fit(list(name="Laplace")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="pairwise")),
               create_dist_obj_for_gbmt_fit(list(name="Pairwise")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="poisson")),
               create_dist_obj_for_gbmt_fit(list(name="Poisson")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="quantile")),
               create_dist_obj_for_gbmt_fit(list(name="Quantile")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="tdist")), 
               create_dist_obj_for_gbmt_fit(list(name="TDist")))
  expect_equal(create_dist_obj_for_gbmt_fit(list(name="tweedie")),
               create_dist_obj_for_gbmt_fit(list(name="Tweedie")))
})

skip_on_valgrind <- function() {
  use_valgrind <- tolower(Sys.getenv("_R_CHECK_USE_VALGRIND_", "false"))
  valgrind_env <- nzchar(Sys.getenv("R_VALGRIND_OPTS")) ||
    nzchar(Sys.getenv("VALGRIND_OPTS"))

  if (use_valgrind %in% c("true", "yes", "1") || valgrind_env) {
    testthat::skip("Skipping long-running integration tests under valgrind")
  }
}

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of tests to check if group determination 
# and weight updates based on groups
#
####################

context("Testing determine groups")
test_that("determine_groups throws error if not passed a GBMDist object", {
  # Given data and distribution obj
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  
  # When dist is stripped of class
  class(dist) <- "Wrong"
  
  # Then determine_groups throws an error
  expect_error(determine_groups(data, Y, dist))
})
test_that("determine_groups returns original distribution_obj if NOT Pairwise", {
  # Given data and distribution objs - NOT Pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Poisson")
  dist_9 <- gbm_dist("Quantile")
  dist_10 <- gbm_dist("TDist")
  dist_11 <- gbm_dist("Tweedie")

  # When determine_groups called
  # Then returns original distribution
  expect_equal(determine_groups(data, Y, dist_1), dist_1)
  expect_equal(determine_groups(data, Y, dist_2), dist_2)
  expect_equal(determine_groups(data, Y, dist_3), dist_3)
  expect_equal(determine_groups(data, Y, dist_4), dist_4)
  expect_equal(determine_groups(data, Y, dist_5), dist_5)
  expect_equal(determine_groups(data, Y, dist_6), dist_6)
  expect_equal(determine_groups(data, Y, dist_7), dist_7)
  expect_equal(determine_groups(data, Y, dist_8), dist_8)
  expect_equal(determine_groups(data, Y, dist_9), dist_9)
  expect_equal(determine_groups(data, Y, dist_10), dist_10)
  expect_equal(determine_groups(data, Y, dist_11), dist_11)
})
test_that("determine_groups throws error if group field in distribution_obj is NULL - Pairwise", {
  # Given data and distribution obj - Pairwise (group set to NULL)
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  dist$group <- NULL
  
  # When determine_groups called
  # Then determine_groups throws an error
  expect_error(determine_groups(data, Y, dist))
})
test_that("determine_groups throws an error if group does not occur as column in original_data - Pairwise", {
  # Given data and distribution obj - Pairwise with group not in original_data
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="WRONG")
  
  # When determine_groups is called
  # Then determine_groups throws an error
  expect_error(determine_groups(data, Y, dist))
})
test_that("determine_groups can run and populates group_index & group_order fields and updates group field - Pairwise", {
  # Given data and distribution obj - Pairwise and group correct
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  
  # When determine_groups called
  updated_dist <- determine_groups(data, Y, dist)
  
  # Then group_index and group_order fields created and group updated
  expect_false(is.null(updated_dist$group_index))
  expect_false(is.null(updated_dist$group_order))
  expect_false(any(updated_dist$group == dist$group))
})

context("Testing weight_group_consistency")
test_that("weight_group_consistency throws an error if distribution object is not GBMDist class", {
  # Given weights and a distribution_object
  w <- rep(1, 10)
  dist <- gbm_dist()
  
  # When dist obj has class removed
  class(dist) <- "Wrong"
  
  # Then weight_group_consistency throws error
  expect_error(weight_group_consistency(dist, w))
})
test_that("weight_group_consistency does not alter weights if NOT Pairwise", {
  # Given weights and distribution objects NOT Pairwise
  w <- rep(1, 10)
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Poisson")
  dist_9 <- gbm_dist("Quantile")
  dist_10 <- gbm_dist("TDist")
  dist_11 <- gbm_dist("Tweedie")
  
  # When weight_group_consistency is called
  # Then the original weights are returned
  expect_equal(weight_group_consistency(dist_1, w), w)
  expect_equal(weight_group_consistency(dist_2, w), w)
  expect_equal(weight_group_consistency(dist_3, w), w)
  expect_equal(weight_group_consistency(dist_4, w), w)
  expect_equal(weight_group_consistency(dist_5, w), w)
  expect_equal(weight_group_consistency(dist_6, w), w)
  expect_equal(weight_group_consistency(dist_7, w), w)
  expect_equal(weight_group_consistency(dist_8, w), w)
  expect_equal(weight_group_consistency(dist_9, w), w)
  expect_equal(weight_group_consistency(dist_10, w), w)
  expect_equal(weight_group_consistency(dist_11, w), w)
})
test_that("weight_group_consistency throws error if weights across groups are different - Pairwise", {
  # Given weights are different across groups and a Pairwise distribution
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  dist <- determine_groups(data, Y, dist)
  
  # When weights are not same across groups
  w <- rep(1, N)
  w[dist$group == 1] <- abs(rnorm(length(dist$group[dist$group == 1])))
  
  # Then error is thrown
  expect_error(weight_group_consistency(dist, w))
})

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of test to check the behaviour of the predict S3 method.
#
####################

context("Test input checking - predict.GBMFit")
test_that("Error thrown if type is not 'link' or 'response'", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict with type not 'link' or 'response'
  # Then an error will be thrown
  expect_error(predict(fit, data2, length(fit$trees), type='wrong'), "type must be either 'link' or 'response'")
})
test_that("Error thrown if data is missing", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict without data
  # Then an error will be thrown
  expect_error(predict(fit, n.trees=length(fit$trees)), "newdata must be provided as a data frame")
})
test_that("Error thrown if data is not in data.frame", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict with data not in data.frame
  # Then an error will be thrown
  expect_error(predict(fit, newdata=as.list(data2),
                       n.trees=length(fit$trees)), 
               "newdata must be provided as a data frame")
})
test_that("Error thrown if n.trees not provided", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict without number of trees specified
  # Then an error will be thrown
  expect_error(predict(fit, data2), "Number of trees to be used in prediction must be provided.")
})
test_that("Error thrown if n.trees is NULL or vector of 0 length", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict with number of trees NULL or length 0
  # Then an error will be thrown
  expect_error(predict(fit, data2, NULL), "n.trees cannot be NULL or a vector of zero length")
  expect_error(predict(fit, data2, c()), "n.trees cannot be NULL or a vector of zero length")
})
test_that("Error thrown if n.trees has element which is not an integer", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict with number of trees containing an element which isn't an integer
  # Then an error will be thrown
  expect_error(predict(fit, data2, n.trees=c(1, -1.2)),
               "n.trees must be a vector of non-negative integers")
})

test_that("Warning thrown if offset was in original fit", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6 + offset(offset), data=data, distribution=dist, weights=w,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict with original fit containing the offset
  # Then a warning will be thrown
  expect_warning(predict(fit, data2, length(fit$trees)), 
                 "predict.GBMFit does not add the offset to the predicted values.")
})
test_that("Warning thrown if n.trees exceeds number in original fit", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict with number of trees exceeding number in original fit
  # Then a warning will be thrown
  expect_warning(
    predict(fit, data2, length(fit$trees)+1),
    paste0("Number of trees exceeded number fit so far. Using ",
           paste(length(fit$trees), collapse=" "), "\\.")
  )
})

context("Test basic functionality of predict.GBMFit")
test_that("predict works if Terms defined in original fit", {
  # Given a fit and data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling predict correctly
  # Then no error is thrown
  expect_error(predict(fit, data2, length(fit$trees)), NA)
})
test_that("type='response' scales predictions", {
  # Given a fit and data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=300, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)

  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # When calling predict with type = response
  preds <- predict(fit, data2, n.trees=length(fit$trees), type='response')
  preds_link <- predict(fit, data2, n.trees=length(fit$trees), type='link')
  
  # Then type=response predictions are scaled
  expect_equal(preds, adjust_pred_scale(preds_link, fit$distribution))
})
test_that("Output is matrix if length(n.trees) > 1", {
  # Given a fit and data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=300, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # When calling predict with length n.trees > 1
  preds <- predict(fit, data2, n.trees=c(50, 100))

  # Then output is a matrix
  expect_true(is.matrix(preds))
})
test_that("Output is vector if length(n.trees) == 1", {
  # Given a fit and data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=300, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # When calling predict with length n.trees = 1
  preds <- predict(fit, data2, n.trees=length(fit$trees), type='response')

  # Then output is a vector
  expect_true(is.vector(preds))
})
test_that("When n.trees specified exceeds total number in fit then number in fit used", {
  # Given a fit and data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=300, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # When calling predict with n.trees > length(fit$trees)
  # Then predictions are evaluated at n.trees = length(fit$trees)
  expect_equal(predict(fit, data2, n.trees=length(fit$trees) + 1),
               predict(fit, data2, n.trees=length(fit$trees)))
})

skip_on_valgrind()

context("test parallelization")

test_that("gaussian works in parallel", {
  skip_on_cran()
  skip_on_travis()
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Set up training parameters and 
  params <- training_params(num_trees=2000, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2, num_features=6)
  
  # fit initial model
  gbm1 <- gbmt(Y~X1+X2+X3+X4+X5+X6,         # formula
              data=data,                   # dataset
              var_monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              train_params = params,
              keep_gbm_data=TRUE,
              cv_folds=10 # do 10-fold cross-validation
  )                 
  
  # Get best model
  best_iter <- gbmt_performance(gbm1,method="cv")   # returns cv estimate of best number of trees
  
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  
  # Want to see how close predictions are to the underlying signal; noise would just interfere with this
  # Y <- Y + rnorm(N,0,sigma)
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # predict on the new data using "best" number of trees
  f.predict <- predict(gbm1, data2, best_iter) # f.predict will be on the canonical scale (logit,log,etc.)
  
  # Base the validation tests on observed discrepancies
  expect_true(cor(data2$Y, f.predict) > 0.990)
  expect_true(sd(data2$Y-f.predict) < sigma)
})

test_that("gaussian works in parallel- old api", {
    skip_on_cran()
    skip_on_travis()
    ## Based on example in R package
    
    ## test Gaussian distribution gbm model
    set.seed(1)

    # create some data
    N <- 1000
    X1 <- runif(N)
    X2 <- 2*runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=T))
    X4 <- ordered(sample(letters[1:6],N,replace=T))
    X5 <- factor(sample(letters[1:3],N,replace=T))
    X6 <- 3*runif(N)
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    SNR <- 10 # signal-to-noise ratio
    Y <- X1**1.5 + 2 * (X2**.5) + mu
    sigma <- sqrt(var(Y)/SNR)
    Y <- Y + rnorm(N,0,sigma)

    # create a bunch of missing values
    X1[sample(1:N,size=100)] <- NA
    X3[sample(1:N,size=300)] <- NA

    w <- rep(1,N)

    data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

    # fit initial model
    gbm1 <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                data=data,                   # dataset
                var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                                            # list(name="quantile",alpha=0.05) for quantile regression
                n.trees=2000,                 # number of trees
                shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
                bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                n.minobsinnode = 10,         # minimum number of obs needed in each node
                keep.data=TRUE,
                cv.folds=10 # do 10-fold cross-validation
                )                 

    # Get best model
    best_iter <- gbmt_performance(gbm1,method="cv")   # returns cv estimate of best number of trees
    
    set.seed(2)
    # make some new data
    N <- 1000
    X1 <- runif(N)
    X2 <- 2*runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=TRUE))
    X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
    X5 <- factor(sample(letters[1:3],N,replace=TRUE))
    X6 <- 3*runif(N)
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    # Actual underlying signal
    Y <- X1**1.5 + 2 * (X2**.5) + mu

    # Want to see how close predictions are to the underlying signal; noise would just interfere with this
    # Y <- Y + rnorm(N,0,sigma)
    data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

    # predict on the new data using "best" number of trees
    f.predict <- predict(gbm1, data2, best_iter) # f.predict will be on the canonical scale (logit,log,etc.)

    # Base the validation tests on observed discrepancies
    expect_true(cor(data2$Y, f.predict) > 0.990)
    expect_true(sd(data2$Y-f.predict) < sigma)
})

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of tests to check gbm_perf
#
####################

context("gbm.perf input checking")
test_that("Error thrown if not passed GBMFit object", {
  # Given a fit object
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20,
                            interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When gbm.perf is called on fit object not of class GBMFit
  class(fit) <- "wrong"
  
  # Then an error is thrown
  expect_error(gbm.perf(fit, method="cv"))
})

test_that("Error thrown if plot.it is not a logical", {
  # Given a fit object
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When calling gbm_perf with plot_it not a logical
  # Then an error is thrown
  expect_error(gbm.perf(fit, method="cv", plot.it=c(TRUE, FALSE)))
  expect_error(gbm.perf(fit, method="cv", plot.it=1.5))
  expect_error(gbm.perf(fit, method="cv", plot.it=""))
  expect_error(gbm.perf(fit, method="cv", plot.it=NaN))
  expect_error(gbm.perf(fit, method="cv", plot.it=Inf))
})

test_that("Error thrown if plot.it is NA", {
  # Given a fit object
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When calling gbm_perf with plot_it=NA
  # Then error is thrown
  expect_error(gbm.perf(fit, method="cv", plot.it=NA))
})

test_that("Error thrown if method not element of c('OOB', 'cv', 'test')", {
  # Given a fit object
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When gbm_perf called with method that is not 'cv', 'test' or 'OOB'
  # Then an error is thrown
  expect_error(gbm.perf(fit, method="weird_metric"))
})

test_that("Warning given if method is 'OOB'", {
  # Given a fit object
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When gbm_perf is called with method 'OOB'
  # Then a warning is thrown
  expect_warning(gbm.perf(fit, method="OOB"))
})

context("gbm_perf return")
test_that("gbm_perf returns correct best iteration for each method", {
  # Given a fit object and perf evaluated with each method
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  best_iter_t <- which.min(fit$valid.error)
  best_iter_c <- which.min(fit$cv_error)
  x <- seq_len(fit$params$num_trees)
  smoother <- loess(fit$oobag.improve~x,
                    enp.target=min(max(4,length(x)/10),50))
  smoother$y <- smoother$fitted
  smoother$x <- x
  best_iter_oo <- smoother$x[which.min(-cumsum(smoother$y))]
  
  # When calling gbm_perf with 3 methods
  iter_t <- gbm.perf(fit, method="test")
  iter_c <- gbm.perf(fit, method="cv")
  iter_oo <- expect_warning(gbm.perf(fit, method="OOB"))
  
  # Then correctly calculates best iterations
  expect_equal(iter_t, best_iter_t)
  expect_equal(iter_c, best_iter_c)
  expect_equal(iter_oo, best_iter_oo)
})

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of test to validate the GBMDist objects.
#
####################


########## Definition ###############

context("Testing GBMDist Object Definition:")

test_that("Default distribution is Gaussian", {
  gbm_dist_obj <- gbm_dist()
  expect_equal(gbm_dist_obj$name, "Gaussian")
})
test_that("Check AdaBoost Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="AdaBoost")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("AdaBoostGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Bernoulli Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Bernoulli")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("BernoulliGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check CoxPH Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="CoxPH")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("CoxPHGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Gamma Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Gamma")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("GammaGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Gaussian Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Gaussian")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("GaussianGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Huberized Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Huberized")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("HuberizedGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Laplace Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Laplace")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("LaplaceGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Pairwise Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Pairwise")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("PairwiseGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Poisson Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Poisson")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("PoissonGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Quantile Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Quantile")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("QuantileGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check TDist Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="TDist")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("TDistGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check Tweedie Distribution Object has correct class attributes", {
  gbm_dist_obj <- gbm_dist(name="Tweedie")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("TweedieGBMDist" %in% class(gbm_dist_obj))
})
test_that("Check AdaBoost distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist()
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check AdaBoost distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist()
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check Bernoulli distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist()
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check CoxPH distribution has the right elements - order", {
  gbm_dist_obj <- gbm_dist()
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check Gamma distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist()
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check Gaussian distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="Gaussian")
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check Huberized distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="Huberized")
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check Laplace distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="Laplace")
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check Pairwise distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="Pairwise")
  expect_equal(names(gbm_dist_obj), c("name", "reorder", "metric", "group", "max_rank"))
})
test_that("Check Poisson distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="Poisson")
  expect_equal(names(gbm_dist_obj), c("name", "reorder"))
})
test_that("Check Quantile distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="Quantile")
  expect_equal(names(gbm_dist_obj), c("name", "reorder", "alpha"))
})
test_that("Check TDist distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="TDist")
  expect_equal(names(gbm_dist_obj), c("name", "reorder", "df"))
})
test_that("Check Tweedie distribution has the right elements - in right order", {
  gbm_dist_obj <- gbm_dist(name="Tweedie")
  expect_equal(names(gbm_dist_obj), c("name", "reorder", "power"))
})

###### Warnings ######
context("Check warnings are thrown when too many arguments on construction of object")
test_that("Check warning for too many arguments - AdaBoost", {
  expect_warning(gbm_dist(name="AdaBoost", extra=1.0))
})
test_that("Check warning for too many arguments - Bernoulli", {
  expect_warning(gbm_dist(name="Bernoulli", extra=1.0))
})
test_that("Check warning for too many arguments - CoxPH", {
  expect_warning(gbm_dist(name="CoxPH", extra=1.0))
})
test_that("Check warning for too many arguments - Gamma", {
  expect_warning(gbm_dist(name="Gamma", extra=1.0))
})
test_that("Check warning for too many arguments - Gaussian", {
  expect_warning(gbm_dist(name="Gaussian", extra=1.0))
})

test_that("Check warning for too many arguments - Huberized", {
  expect_warning(gbm_dist(name="Huberized", extra=1.0))
})

test_that("Check warning for too many arguments - Laplace", {
  expect_warning(gbm_dist(name="Laplace", extra=1.0))
})

test_that("Check warning for too many arguments - Pairwise", {
  expect_warning(gbm_dist(name="Pairwise", extra=1.0))
})

test_that("Check warning for too many arguments - Poisson", {
  expect_warning(gbm_dist(name="Poisson", extra=1.0))
})

test_that("Check warning for too many arguments - Quantile", {
  expect_warning(gbm_dist(name="Quantile", extra=1.0))
})

test_that("Check warning for too many arguments - TDist", {
  expect_warning(gbm_dist(name="TDist", extra=1.0))
})

test_that("Check warning for too many arguments - Tweedie", {
  expect_warning(gbm_dist(name="Tweedie", extra=1.0))
})

##### Error checking ##### 
context("Check expect errors on construction if incorrect parameters provided")
test_that("Error thrown if unsupported distribution selected", {
  expect_error(gbm_dist("No sense in believing this will construct"))
})

test_that("Error thrown if 'ties' parameter is not a string- CoxPH", {
  expect_error(gbm_dist(name="CoxPH", ties=1.0))
  expect_error(gbm_dist(name="CoxPH", ties=Inf))
  expect_error(gbm_dist(name="CoxPH", ties=NA))
  expect_error(gbm_dist(name="CoxPH", ties=NULL))
})

test_that("Error thrown if strata not a vector of integers or factors- CoxPH", {
  expect_error(gbm_dist(name="CoxPH", strata=c(1.2, 1.4, 1.5)))
  expect_error(gbm_dist(name="CoxPH", strata=NULL))
  expect_error(gbm_dist(name="CoxPH", strata="String"))
  expect_error(gbm_dist(name="CoxPH", strata=-0.1))
  expect_error(gbm_dist(name="CoxPH", strata=Inf))
})

test_that("Error thrown if sorted not a vector of integers - CoxPH", {
  expect_error(gbm_dist(name="CoxPH", sorted=c(1.2, 1.4, 1.5)))
  expect_error(gbm_dist(name="CoxPH", sorted=NULL))
  expect_error(gbm_dist(name="CoxPH", sorted="String"))
  expect_error(gbm_dist(name="CoxPH", sorted=-0.1))
  expect_error(gbm_dist(name="CoxPH", sorted=Inf))
})

test_that("Error thrown if prior coefficient of variation if not a finite double - CoxPH", {
  expect_error(gbm_dist(name="CoxPH", prior_node_coeff=Inf))
  expect_error(gbm_dist(name="CoxPH", prior_node_coeff="Nope"))
  expect_error(gbm_dist(name="CoxPH", prior_node_coeff=c(1.2, 3.4)))
  expect_error(gbm_dist(name="CoxPH", prior_node_coeff=0))
  expect_error(gbm_dist(name="CoxPH", prior_node_coeff=-1))
})

test_that("Error thrown if max_rank is not a finite double greater than 0.0 - Pairwise", {
  expect_error(gbm_dist(name="Pairwise", metric="ndcg", max_rank=-0.1))
  expect_error(gbm_dist(name="Pairwise", metric="ndcg", max_rank=-0.1))
  expect_error(gbm_dist(name="Pairwise", metric="ndcg", max_rank="Stuff"))
  expect_error(gbm_dist(name="Pairwise", metric="ndcg", max_rank=c(1.0, 2.0)))
  expect_error(gbm_dist(name="Pairwise", metric="ndcg", max_rank=Inf))
  expect_error(gbm_dist(name="Pairwise", metric="ndcg", max_rank=NA))
  expect_error(gbm_dist(name="Pairwise", metric="ndcg", max_rank=NULL))
  
  expect_error(gbm_dist(name="Pairwise", metric="mrr", max_rank=-0.1))
  expect_error(gbm_dist(name="Pairwise", metric="mrr", max_rank=-0.1))
  expect_error(gbm_dist(name="Pairwise", metric="mrr", max_rank="Stuff"))
  expect_error(gbm_dist(name="Pairwise", metric="mrr", max_rank=c(1.0, 2.0)))
  expect_error(gbm_dist(name="Pairwise", metric="mrr", max_rank=Inf))
  expect_error(gbm_dist(name="Pairwise", metric="mrr", max_rank=NA))
  expect_error(gbm_dist(name="Pairwise", metric="mrr", max_rank=NULL))
})

test_that("Error thrown if max_rank is non-zero for conc or map - Pairwise", {
  expect_error(gbm_dist(name="Pairwise", metric="conc", max_rank=1.0))
  expect_error(gbm_dist(name="Pairwise", metric="map", max_rank=1.0))
})

test_that("Error thrown if group is not a string - Pairwise", {
  expect_error(gbm_dist(name="Pairwise", group=1.0))
  expect_error(gbm_dist(name="Pairwise", group=Inf))
  expect_error(gbm_dist(name="Pairwise", group=NA))
  expect_error(gbm_dist(name="Pairwise", group=NULL))
  expect_error(gbm_dist(name="Pairwise", group=c("Group1", "Group2")))
})

test_that("Error thrown if group index is not a vector of integers - Pairwise", {
  expect_error(gbm_dist(name="Pairwise", group_index=c("a", 2)))
  expect_error(gbm_dist(name="Pairwise", group_index=matrix(1, 2)))
  expect_error(gbm_dist(name="Pairwise", group_index=Inf))
  expect_error(gbm_dist(name="Pairwise", group_index=NA))
})

test_that("Error thrown if metric is not: ndcg, map, mrr or conc - Pairwise", {
  expect_error(gbm_dist(name="Pairwise", metric="Made-up"))
})

test_that("Error thrown if alpha specified is not a finite double between 0.0 and 1.0 - Quantile", {
  expect_error(gbm_dist(name="Quantile", alpha=2.0))
  expect_error(gbm_dist(name="Quantile", alpha=-0.01))
  expect_error(gbm_dist(name="Quantile", alpha="Character"))
  expect_error(gbm_dist(name="Quantile", alpha=Inf))
  expect_error(gbm_dist(name="Quantile", alpha=c(0.5, 0.1)))
  expect_error(gbm_dist(name="Quantile", alpha=NA))
})

test_that("Error thrown if degrees of freedom specified is not a finite double > 0.0 - TDist", {
  expect_error(gbm_dist(name="TDist", df=-0.01))
  expect_error(gbm_dist(name="TDist", df="Character"))
  expect_error(gbm_dist(name="TDist", df=Inf))
  expect_error(gbm_dist(name="TDist", df=c(0.5, 0.1)))
  expect_error(gbm_dist(name="TDist", df=NA))
})

test_that("Error thrown if power specified is not a valid Tweedie power", {
  expect_error(gbm_dist(name="Tweedie", power="Character"))
  expect_error(gbm_dist(name="Tweedie", power=Inf))
  expect_error(gbm_dist(name="Tweedie", power=c(0.5, 0.1)))
  expect_error(gbm_dist(name="Tweedie", power=NA))
  expect_error(gbm_dist(name="Tweedie", power=0.5))
  expect_error(gbm_dist(name="Tweedie", power=0), "Gaussian")
  expect_error(gbm_dist(name="Tweedie", power=1), "Poisson")
  expect_error(gbm_dist(name="Tweedie", power=2), "Gamma")
  expect_error(gbm_dist(name="Tweedie", power=-0.01), NA)
  expect_error(gbm_dist(name="Tweedie", power=3.0), NA)
})

##### Default Parameters #####
context("Check default values of fields")
test_that("AdaBoost has reorder is FALSE", {
  expect_false(gbm_dist(name="AdaBoost")$reorder)
})

test_that("Bernoulli has reorder is FALSE", {
  expect_false(gbm_dist(name="Bernoulli")$reorder)
})

test_that("CoxPH has reorder is TRUE", {
  expect_true(gbm_dist(name="CoxPH")$reorder)
})

test_that("Gamma has reorder is FALSE", {
  expect_false(gbm_dist(name="Gamma")$reorder)
})

test_that("Gaussian has reorder is FALSE", {
  expect_false(gbm_dist(name="Gaussian")$reorder)
})

test_that("Laplace has reorder is FALSE", {
  expect_false(gbm_dist(name="Laplace")$reorder)
})

test_that("Pairwise has reorder is TRUE", {
  expect_true(gbm_dist(name="Pairwise")$reorder)
})

test_that("Poisson has reorder is FALSE", {
  expect_false(gbm_dist(name="Poisson")$reorder)
})

test_that("Quantile has reorder is FALSE", {
  expect_false(gbm_dist(name="Quantile")$reorder)
})

test_that("TDist has reorder is FALSE", {
  expect_false(gbm_dist(name="TDist")$reorder)
})

test_that("Tweedie has reorder is FALSE", {
  expect_false(gbm_dist(name="Tweedie")$reorder)
})

test_that("CoxPH - defaults to 'efron', a prior coeff var of 1000, with NAs for sorted and strata", {
  expect_true(is.na(gbm_dist(name="CoxPH")$original_strata_id))
  expect_true(is.na(gbm_dist(name="CoxPH")$sorted))
  expect_equal(gbm_dist(name="CoxPH")$prior_node_coeff, 1000)
  expect_equal(gbm_dist(name="CoxPH")$ties, "efron")
})


test_that("Pairwise params default to - 'ndcg', max.rank=0 and group='query'", {
  expect_equal(gbm_dist(name="Pairwise")$metric, "ndcg")
  expect_equal(gbm_dist(name="Pairwise")$max_rank, 0)
  expect_equal(gbm_dist(name="Pairwise")$group, "query")
})

test_that("Quantile alpha defaults to 0.25", {
  expect_equal(gbm_dist(name="Quantile")$alpha, 0.25)
})

test_that("TDist df defaults to 4", {
  expect_equal(gbm_dist(name="TDist")$df, 4)
})

test_that("Tweedie defaults to dist with power = 1.5", {
  expect_equal(gbm_dist(name="Tweedie")$power, 1.5)
})

##### Creation #####
context("Testing creation methods")
test_that("Can't create empty distribution object without passing a distribution name", {
  expect_error(empty_distribution())
})

test_that("Can create empty distribution - AdaBoost", {
  gbm_dist_obj <- empty_distribution(name="AdaBoost")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("AdaBoostGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Bernoulli", {
  gbm_dist_obj <- empty_distribution(name="Bernoulli")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("BernoulliGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - CoxPH", {
  gbm_dist_obj <- empty_distribution(name="CoxPH")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("CoxPHGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Gamma", {
  gbm_dist_obj <- empty_distribution(name="Gamma")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("GammaGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Gaussian", {
  gbm_dist_obj <- empty_distribution(name="Gaussian")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("GaussianGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Huberized", {
  gbm_dist_obj <- empty_distribution(name="Huberized")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("HuberizedGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Laplace", {
  gbm_dist_obj <- empty_distribution(name="Laplace")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("LaplaceGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Pairwise", {
  gbm_dist_obj <- empty_distribution(name="Pairwise")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("PairwiseGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Poisson", {
  gbm_dist_obj <- empty_distribution(name="Poisson")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("PoissonGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Quantile", {
  gbm_dist_obj <- empty_distribution(name="Quantile")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("QuantileGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - TDist", {
  gbm_dist_obj <- empty_distribution(name="TDist")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("TDistGBMDist" %in% class(gbm_dist_obj))
})

test_that("Can create empty distribution - Tweedie", {
  gbm_dist_obj <- empty_distribution(name="Tweedie")
  expect_true("GBMDist" %in% class(gbm_dist_obj))
  expect_true("TweedieGBMDist" %in% class(gbm_dist_obj))
})

test_that("Empty distributions only have reorder and name fields", {
  dist1 <- empty_distribution("AdaBoost")
  dist2 <- empty_distribution("Bernoulli")
  dist3 <- empty_distribution("CoxPH")
  dist4 <- empty_distribution("Gamma")
  dist5 <- empty_distribution("Gaussian")
  dist6 <- empty_distribution("Huberized")
  dist7 <- empty_distribution("Laplace")
  dist8 <- empty_distribution("Pairwise")
  dist9 <- empty_distribution("Poisson")
  dist10 <- empty_distribution("Quantile")
  dist11 <- empty_distribution("TDist")
  dist12 <- empty_distribution("Tweedie")
  
  expect_equal(names(dist1), c("name", "reorder"))
  expect_equal(names(dist2), c("name", "reorder"))
  expect_equal(names(dist3), c("name", "reorder"))
  expect_equal(names(dist4), c("name", "reorder"))
  expect_equal(names(dist5), c("name", "reorder"))
  expect_equal(names(dist6), c("name", "reorder"))
  expect_equal(names(dist7), c("name", "reorder"))
  expect_equal(names(dist8), c("name", "reorder"))
  expect_equal(names(dist9), c("name", "reorder"))
  expect_equal(names(dist10), c("name", "reorder"))
  expect_equal(names(dist11), c("name", "reorder"))
  expect_equal(names(dist12), c("name", "reorder"))
})

test_that("Create distribution method breaks if not given a GBMDist object", {
  # Given two identical empty GBMDist objects
  dist_a <- empty_distribution("Gaussian")
  dist_b <- dist_a
  
  # When one of the objects has its class removed
  class(dist_b) <- ""
  
  # Then error thrown when trying to create a distribution from the empty 
  # object whose class has been removed
  expect_error(create_dist(dist_b))
  expect_error(create_dist(dist_a), NA)
})

#### CoxPH ####
test_that("CoxPH - stores the original strata observations ids (positive integers) in original_strata_id field", {
  orig_strat <- c(1, 1, 2, 3, 5, 5)
  
  # When a CoxPH is created
  dist <- gbm_dist("CoxPH", strata=orig_strat)
  
  # Then original strata stored in dist
  expect_equal(dist$original_strata_id, orig_strat)
})

test_that("CoxPH - convert and store original strata observations (factors) in original_strata_id field", {
  # Given original strata of factors
  orig_strat <- as.factor(c("a", "b"))
  
  # When a CoxPH is created
  dist <- gbm_dist("CoxPH", strata=orig_strat)
  
  # Then original strata is converted to integers and stored in dist
  expect_equal(dist$original_strata_id, as.integer(orig_strat))
})

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of tests for gbm_more
#
####################

context("Checking inputs to gbm_more")
test_that("Error thrown if gbm_fit_obj is not of class GBMFit", {
  # Given a GBMFit from data
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When stripped of GBMFit class and gbm_more is called
  class(fit) <- "wrong"
  
  # Then an error is thrown
  expect_error(gbm_more(fit))
})
test_that("Error thrown if num_new_trees is not a natural number > 1", {
  # Given an appropriate GBMFit object
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=500, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When gbm_more is called with num_new_trees not a natural number > 1
  # Then an error is thrown
  expect_error(gbm_more(fit, num_new_trees = c(1, 200)))
  expect_error(gbm_more(fit, num_new_trees = FALSE))
  expect_error(gbm_more(fit, num_new_trees = NA))
  expect_error(gbm_more(fit, num_new_trees = Inf))
  expect_error(gbm_more(fit, num_new_trees = NaN))
  expect_error(gbm_more(fit, num_new_trees = "Eh?"))
  expect_error(gbm_more(fit, num_new_trees = 10.3))
})
test_that("Error thrown if is_verbose is not a logical", {
  # Given an appropriate GBMFit object
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=500, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When gbm_more is called with is_verbose not a logical
  # Then an error is thrown
  expect_error(gbm_more(fit, is_verbose = c(1, 200)))
  expect_error(gbm_more(fit, is_verbose = 100))
  expect_error(gbm_more(fit, is_verbose =  NA))
  expect_error(gbm_more(fit, is_verbose =  Inf))
  expect_error(gbm_more(fit, is_verbose =  NaN))
  expect_error(gbm_more(fit, is_verbose = "Eh?"))
  expect_error(gbm_more(fit, is_verbose = 10.3))
})
test_that("Error thrown if data not provided or gbm_data_obj not stored in gbm_fit_obj", {
  # Given an appropriate GBMFit object - DATA NOT KEPT
  keep_data <- FALSE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  # When gbm_more is called without providing new data
  # Then an error is thrown
  expect_error(gbm_more(fit))
})
test_that("Error thrown if data has different number of rows than the original number of rows in response", {
  # Given an appropriate GBMFit object (DATA NOT KEPT) 
  # and new data with more rows than before
  keep_data <- FALSE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  # make some new data - more than last time
  N <- N + 1
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  new_data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When gbm_more is called with this new_data
  # Then an error is thrown
  expect_error(gbm_more(fit, data = new_data))
})

context("Check gbm_more properties")
test_that("Output of gbm_more is GBMFit object", {
  # Given an appropriate GBMFit object (DATA KEPT) 
  keep_data <- TRUE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
 
  
  # When gbm_more is called 
  fit_2 <- gbm_more(fit)
  
  # Then returned object is a GBMFit object
  expect_true(class(fit_2) %in% "GBMFit")
})

test_that("gbm_more removes cv properties of GBMFit output", {
  # Given an appropriate GBMFit object (DATA KEPT) 
  # produced from cv
  keep_data <- TRUE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  # When gbm_more is called 
  fit_2 <- gbm_more(fit)
  
  # Then cross validation has been stripped

  expect_true(has_cross_validation(fit))
  expect_false(has_cross_validation(fit_2))
})

test_that("gbm_more fits additional trees - reflected in length of fit fields", {
  # Given an appropriate GBMFit object (DATA KEPT) 
  # with number of new trees
  keep_data <- TRUE
  num_trees <- 150
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  
  
  # When gbm_more is called 
  fit_2 <- gbm_more(fit, num_new_trees = num_trees)
  
  # Then errors and trees have these additional fitted trees to the object
  expect_equal(length(fit_2$train.error) - length(fit$train.error), num_trees)
  expect_equal(length(fit_2$valid.error) - length(fit$valid.error), num_trees)
  expect_equal(length(fit_2$oobag.improve) - length(fit$oobag.improve), num_trees)
  expect_equal(length(fit_2$trees) - length(fit$trees), num_trees)
})
test_that("Can run gbm_more with new data", {
  # Given an appropriate GBMFit object (DATA NOT KEPT) 
  # and new data same number of rows as before
  keep_data <- FALSE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  # make some new data - more than last time
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  new_data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling gbm_more with new data
  # Then runs successfully
  expect_error(gbm_more(fit, data=new_data), NA)
})

test_that("Output of gbm_more DOES NOT contain constructed gbm_data_obj if run with new data", {
  # Given an appropriate GBMFit object (DATA NOT KEPT) 
  # and new data same number of rows as before
  keep_data <- FALSE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  # make some new data - more than last time
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  new_data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling gbm_more with new data
  fit_more <- gbm_more(fit, data=new_data)
  
  # Then output dpes not store new_data object
  expect_true(is.null(fit_more$gbm_data_obj))
})
test_that("If run with new data and weights are length 0 then they are set to 1 for each data row", {
  # Given an appropriate GBMFit object (DATA KEPT) 
  # and new data same number of rows as before
  keep_data <- TRUE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  # make some new data - more than last time
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  new_data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling gbm_more with new data and weights length 0
  fit_more <- gbm_more(fit, data=new_data, weights = NULL)
  
  # Then output weights are vector of 1s
  expect_equal(fit_more$gbm_data_obj$weights, rep(1, N))
})
test_that("If run with new data and offset is length 0 then they are set to 0 for each data row", {
  # Given an appropriate GBMFit object (DATA KEPT) 
  # and new data same number of rows as before
  keep_data <- TRUE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  # make some new data - more than last time
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  new_data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # When calling gbm_more with new data and offset length 0
  fit_more <- gbm_more(fit, data=new_data, offset = NULL)
  
  # Then output offset are vector of 0s
  expect_equal(fit_more$gbm_data_obj$offset, rep(0, N))
})

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of tests to check that the old gbm
# API 
#
####################

#### GBM ####
context("Test old API works on basic examples - gbm")

test_that("Gaussian works - gbm", {
  skip_on_cran()
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm1 <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
              data=data,                   # dataset
              var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              distribution="gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
              # list(name="quantile",alpha=0.05) for quantile regression
              n.trees=2000,                 # number of trees
              shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
              interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
              bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
              train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
              n.minobsinnode = 10,         # minimum number of obs needed in each node
              keep.data=TRUE,
              cv.folds=10 # do 10-fold cross-validation
              )                 
  
  # Get best model
  best.iter <- gbm.perf(gbm1,method="cv")   # returns cv estimate of best number of trees
  
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  
  # Want to see how close predictions are to the underlying signal; noise would just interfere with this
  # Y <- Y + rnorm(N,0,sigma)
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # predict on the new data using "best" number of trees
  f.predict <- predict(gbm1,data2,best.iter) # f.predict will be on the canonical scale (logit,log,etc.)
  
  # Base the validation tests on observed discrepancies
  expect_true(cor(data2$Y, f.predict) > 0.990)
  expect_true(sd(data2$Y-f.predict) < sigma)
})

test_that("CoxPH works - efron - gbm", {
  skip_on_cran()
  ## Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)
  
  # fit initial model
  gbm1 <- gbm(Surv(tt,delta)~X1+X2+X3,       # formula
              data=data,                 # dataset
              weights=w,
              var.monotone=c(0,0,0),     # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              distribution="coxph",
              n.trees=3000,              # number of trees
              shrinkage=0.001,           # shrinkage or learning rate, 0.001 to 0.1 usually work
              interaction.depth=3,       # 1: additive model, 2: two-way interactions, etc.
              bag.fraction = 0.5,        # subsampling fraction, 0.5 is probably best
              train.fraction = 0.5,      # fraction of data for training, first train.fraction*N used for training
              cv.folds = 5,              # do 5-fold cross-validation
              n.minobsinnode = 10,       # minimum total weight needed in each node
              keep.data = TRUE, tied.times.method = "efron", prior.node.coeff.var = 10)
  
  best.iter <- gbm.perf(gbm1, method="test") # returns test set estimate of best number of trees
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)  # -0.5 <= f <= 0.5 via sin fn.
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  
  data2 <- data.frame(tt=apply(cbind(tt.surv,tt.cens),1,min),
                      delta=as.numeric(tt.surv <= tt.cens),
                      f=f,
                      X1=X1,X2=X2,X3=X3)
  
  # predict on the new data using "best" number of trees
  # f.predict will be on the canonical scale (logit,log,etc.)
  f.predict <- predict(gbm1,data2,best.iter)
  
  #plot(data2$f,f.predict)
  # Use observed sd
  expect_true(sd(data2$f - f.predict) < 0.4)
})

test_that("CoxPH works - breslow - gbm", {
  skip_on_cran()
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)
  
  # fit initial model
  gbm1 <- gbm(Surv(tt,delta)~X1+X2+X3,       # formula
              data=data,                 # dataset
              weights=w,
              var.monotone=c(0,0,0),     # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              distribution="coxph",
              n.trees=3000,              # number of trees
              shrinkage=0.001,           # shrinkage or learning rate, 0.001 to 0.1 usually work
              interaction.depth=3,       # 1: additive model, 2: two-way interactions, etc.
              bag.fraction = 0.5,        # subsampling fraction, 0.5 is probably best
              train.fraction = 0.5,      # fraction of data for training, first train.fraction*N used for training
              cv.folds = 5,              # do 5-fold cross-validation
              n.minobsinnode = 10,       # minimum total weight needed in each node
              keep.data = TRUE, tied.times.method="breslow", prior.node.coeff.var = 10)
  
  best.iter <- gbm.perf(gbm1,method="test") # returns test set estimate of best number of trees
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)  # -0.5 <= f <= 0.5 via sin fn.
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  
  data2 <- data.frame(tt=apply(cbind(tt.surv,tt.cens),1,min),
                      delta=as.numeric(tt.surv <= tt.cens),
                      f=f,
                      X1=X1,X2=X2,X3=X3)
  
  # predict on the new data using "best" number of trees
  # f.predict will be on the canonical scale (logit,log,etc.)
  f.predict <- predict(gbm1,data2, best.iter)
  
  #plot(data2$f,f.predict)
  # Use observed sd
  expect_true(sd(data2$f - f.predict) < 0.4)
})

test_that("coxph - runs to completion with train.fraction of 1.0", {
  ## Needed packages
  require(survival)
  
  # Given data from the survival data
  # keep only certain baseline variables, subjects with longitudinal data
  temp <- subset(pbc, id%in%pbcseq$id, select=c(id:sex, stage))
  pbc1 <- tmerge(data1=temp, data2=temp, id=id, death = event(time, status))
  
  pbc2 <- tmerge(pbc1, pbcseq, id=id, ascites = tdc(day, ascites),
                 bili = tdc(day, bili), albumin = tdc(day, albumin),
                 protime = tdc(day, protime), alk.phos = tdc(day, alk.phos))
  
  # Then expect no errors when performing gbm fit with train fraction = 1.0
  # GBM fit baseline data
  expect_error(gbm(Surv(time, status==2) ~ bili + protime + albumin + alk.phos, data=pbc,
                   train.fraction=1.0, distribution="coxph", n.trees=500, shrinkage=.01,  interaction.depth=3), NA)
  
  # GBM fit using start/stop times to get time-dependent covariates
  expect_error(gbm(Surv(tstart, tstop, death==2) ~ bili + protime + albumin + alk.phos, 
                   data=pbc2, distribution="coxph", train.fraction=1.0, n.trees=500, shrinkage=.01, interaction.depth=3), NA)
})

test_that("coxph - runs to completion with train.fraction < 1.0 and cv.folds > 1", {
  ## Needed packages
  require(survival)
  
  # Given data from the survival data
  # keep only certain baseline variables, subjects with longitudinal data
  temp <- subset(pbc, id%in%pbcseq$id, select=c(id:sex, stage))
  pbc1 <- tmerge(data1=temp, data2=temp, id=id, death = event(time, status))
  
  pbc2 <- tmerge(pbc1, pbcseq, id=id, ascites = tdc(day, ascites),
                 bili = tdc(day, bili), albumin = tdc(day, albumin),
                 protime = tdc(day, protime), alk.phos = tdc(day, alk.phos))
  
  # Then expect no errors when performing gbm fit with train fraction < 1.0 and cv.folds > 1
  # GBM fit baseline data
  expect_error(gbm(Surv(time, status==2) ~ bili + protime + albumin + alk.phos, data=pbc,
                   train.fraction=0.8, distribution="coxph", n.trees=500, shrinkage=.01,  interaction.depth=3), NA)
  expect_error(gbm(Surv(time, status==2) ~ bili + protime + albumin + alk.phos, data=pbc,
                   train.fraction=0.8, distribution="coxph", n.trees=500, shrinkage=.01,  cv.folds=5, interaction.depth=3), NA)
  
  # GBM fit using start/stop times to get time-dependent covariates
  expect_error(gbm(Surv(tstart, tstop, death==2) ~ bili + protime + albumin + alk.phos, 
                   data=pbc2, distribution="coxph", train.fraction=0.8, n.trees=500, shrinkage=.01, interaction.depth=3), NA)
  expect_error(gbm(Surv(tstart, tstop, death==2) ~ bili + protime + albumin + alk.phos, 
                   data=pbc2, distribution="coxph", train.fraction=0.8, n.trees=500, shrinkage=.01, cv.folds=5, interaction.depth=3), NA)
})

test_that("coxph cv.folds - runs to completion with start-stop, id'ed and stratified dataset", {
  ## Needed packages
  require(survival)
  
  # Given data from the survival package
  cgd2 <- cgd[cgd$enum==1,]
  
  # Then fitting a gbm model should throw no errors - with cv.folds > 1
  expect_error(gbm(Surv(tstop, status) ~ age + sex + inherit +
                     steroids + propylac + hos.cat, data=cgd2, 
                   n.trees=500, shrinkage=.01, distribution = "coxph", interaction.depth=1, train.fraction=1.0, cv.folds=10), NA)
  expect_error(gbm(Surv(tstop, status) ~ age + sex + inherit +
                     steroids + propylac + hos.cat, data=cgd2, 
                   n.trees=500, shrinkage=.01, distribution = "coxph", interaction.depth=1, train.fraction=0.8, cv.folds=5), NA)
  
  expect_error(gbm(Surv(tstart, tstop, status) ~ age + sex + inherit +
                     steroids + propylac, data=cgd, obs.id=cgd$id,
                   train.fraction=1.0, n.trees=500, strata= cgd$hos.cat, distribution = "coxph", shrinkage=.01, interaction.depth=3, cv.folds=10), NA)
  expect_error(gbm(Surv(tstart, tstop, status) ~ age + sex + inherit +
                     steroids + propylac, data=cgd, obs.id=cgd$id,
                   train.fraction=0.8, n.trees=500, strata= cgd$hos.cat, distribution = "coxph", shrinkage=.01, interaction.depth=3, cv.folds=10), NA)
})

test_that("Bernoulli works - gbm", {
  skip_on_cran()  
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # fit initial model
  gbm1 <- gbm(Y~X1+X2+X3,                # formula
              data=data,                 # dataset
              weights=w,
              var.monotone=c(0,0,0),     # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              distribution="bernoulli",
              n.trees=3000,              # number of trees
              shrinkage=0.001,           # shrinkage or learning rate, 0.001 to 0.1 usually work
              interaction.depth=3,       # 1: additive model, 2: two-way interactions, etc.
              bag.fraction = 0.5,        # subsampling fraction, 0.5 is probably best
              train.fraction = 0.5,      # fraction of data for training, first train.fraction*N used for training
              cv.folds=5,                # do 5-fold cross-validation
              n.minobsinnode = 10      # minimum total weight needed in each node
              )
  
  best.iter.test <- gbm.perf(gbm1,method="test") # returns test set estimate of best number of trees
  
  best.iter <- best.iter.test
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # predict on the new data using "best" number of trees
  # f.predict will be on the canonical scale (logit,log,etc.)
  f.1.predict <- predict(gbm1, data2, n.trees=best.iter.test)
  
  # compute quantity prior to transformation
  f.new = sin(3*X1) - 4*X2 + mu
  
  # Base the validation tests on observed discrepancies
  expect_true(sd(f.new - f.1.predict) < 1.0)
})

test_that("relative influence picks out true predictors", {
  set.seed(1234)
  X1 <- matrix(nrow=1000, ncol=50)
  X1 <- apply(X1, 2, function(x) rnorm(1000)) # Random noise
  X2 <- matrix(nrow=1000, ncol=5)
  X2 <- apply(X2, 2, function(x) c(rnorm(500), rnorm(500, 3))) # Real predictors
  cls <- rep(c(0, 1), ea=500) # Class
  X <- data.frame(cbind(X1, X2, cls))
  mod <- gbm(cls ~ ., data= X, n.trees=1000, cv.folds=5,
             shrinkage=.01, interaction.depth=2
             ,distribution = 'bernoulli')
  ri <- relative_influence(mod, sort_it=TRUE, rescale=TRUE)
  
  wh <- names(ri)[1:5]
  res <- sum(wh %in% paste("V", 51:55, sep = ""))
  expect_equal(res, 5)
})
test_that("Conversion of 2 factor Y is successful", {
  
  NumY <- sample(c(0, 1), size=1000, replace=TRUE)
  FactY = factor(ifelse(NumY==1, "Yes", "No"), levels=c("No", "Yes"))
  
  PredX <-
    data.frame(
      x1 = runif(1000)
      ,x2 = runif(1000)
    )
  
  set.seed(32479)
  g1 <- gbm(y ~ ., data = data.frame(y = NumY, PredX)
            , distribution = 'bernoulli', verbose = FALSE
            , n.trees = 50)
  rig1 <- relative_influence(g1, num_trees=50)
  
  set.seed(32479)
  g2 <- gbm(y ~ ., data = data.frame(y = FactY, PredX)
            , distribution = 'bernoulli', verbose = FALSE
            , n.trees = 50)
  rig2 <- relative_influence(g2, num_trees=50)
  
  expect_equal(rig1, rig2)
})

#### GBM.FIT ####
context("Test old API works on basic examples - gbm.fit")
test_that("Gaussian works - gbm.fit", {
  skip_on_cran()
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm1 <- gbm.fit(X,
                  Y,                   # dataset
                  var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                  distribution="gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                  # list(name="quantile",alpha=0.05) for quantile regression
                  n.trees=2000,                 # number of trees
                  shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                  interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
                  bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                  train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                  n.minobsinnode = 10,         # minimum number of obs needed in each node
                  keep.data=TRUE)                 
  
  # Get best model
  best.iter <- gbm.perf(gbm1, method="test")   # returns cv estimate of best number of trees
  
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  
  # Want to see how close predictions are to the underlying signal; noise would just interfere with this
  # Y <- Y + rnorm(N,0,sigma)
  data2 <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # predict on the new data using "best" number of trees
  f.predict <- predict(gbm1, data2, best.iter) # f.predict will be on the canonical scale (logit,log,etc.)
  
  # Base the validation tests on observed discrepancies
  expect_true(cor(Y, f.predict) > 0.990)
  expect_true(sd(Y-f.predict) < sigma)
})

test_that("CoxPH works - efron - gbm", {
  skip_on_cran()
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  X <- data.frame(X1=X1,X2=X2,X3=X3)

  # fit initial model
  gbm1 <- gbm.fit(X,  
              Surv(tt, delta),                 # dataset
              w=w,
              var.monotone=c(0,0,0),     # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              distribution="coxph",
              n.trees=3000,              # number of trees
              shrinkage=0.001,           # shrinkage or learning rate, 0.001 to 0.1 usually work
              interaction.depth=3,       # 1: additive model, 2: two-way interactions, etc.
              bag.fraction = 0.5,        # subsampling fraction, 0.5 is probably best
              train.fraction = 0.5,      # fraction of data for training, first train.fraction*N used for training
              n.minobsinnode = 10,       # minimum total weight needed in each node
              keep.data = TRUE,tied.times.method = "efron", prior.node.coeff.var = 10)
  
  best.iter <- gbm.perf(gbm1, method="test") # returns test set estimate of best number of trees
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)  # -0.5 <= f <= 0.5 via sin fn.
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  
  data2 <- data.frame(X1=X1,X2=X2,X3=X3)
  
  # predict on the new data using "best" number of trees
  # f.predict will be on the canonical scale (logit,log,etc.)
  f.predict <- predict(gbm1, data2, best.iter)
  
  #plot(data2$f,f.predict)
  # Use observed sd
  expect_true(sd(f - f.predict) < 0.4)
})

test_that("CoxPH works - breslow - gbm", {
  skip_on_cran()
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  X <- data.frame(X1=X1,X2=X2,X3=X3)
  
  # fit initial model
  gbm1 <- gbm.fit(X,       # formula
                  Surv(tt, delta),              # dataset
                  w=w,
                  var.monotone=c(0,0,0),     # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                  distribution="coxph",
                  n.trees=3000,              # number of trees
                  shrinkage=0.001,           # shrinkage or learning rate, 0.001 to 0.1 usually work
                  interaction.depth=3,       # 1: additive model, 2: two-way interactions, etc.
                  bag.fraction = 0.5,        # subsampling fraction, 0.5 is probably best
                  train.fraction = 0.5,      # fraction of data for training, first train.fraction*N used for training
                  n.minobsinnode = 10,       # minimum total weight needed in each node
                  keep.data = TRUE, tied.times.method = "breslow", prior.node.coeff.var = 10)
  
  best.iter <- gbm.perf(gbm1, method="test") # returns test set estimate of best number of trees
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)  # -0.5 <= f <= 0.5 via sin fn.
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  
  data2 <- data.frame(X1=X1,X2=X2,X3=X3)
  
  # predict on the new data using "best" number of trees
  # f.predict will be on the canonical scale (logit,log,etc.)
  f.predict <- predict(gbm1, data2, best.iter)
  
  #plot(data2$f,f.predict)
  # Use observed sd
  expect_true(sd(f - f.predict) < 0.4)
})

test_that("Bernoulli works - gbm", {
  skip_on_cran()
  
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  X <- data.frame(X1=X1,X2=X2,X3=X3)
  
  # fit initial model
  gbm1 <- gbm.fit(X,                
              Y,
              w=w,
              var.monotone=c(0,0,0),     # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              distribution="bernoulli",
              n.trees=3000,              # number of trees
              shrinkage=0.001,           # shrinkage or learning rate, 0.001 to 0.1 usually work
              interaction.depth=3,       # 1: additive model, 2: two-way interactions, etc.
              bag.fraction = 0.5,        # subsampling fraction, 0.5 is probably best
              train.fraction = 0.5,      # fraction of data for training, first train.fraction*N used for training
              n.minobsinnode = 10)
  
  best.iter.test <- gbm.perf(gbm1, method="test") # returns test set estimate of best number of trees
  best.iter <- best.iter.test
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  data2 <- data.frame(X1=X1,X2=X2,X3=X3)
  
  # predict on the new data using "best" number of trees
  # f.predict will be on the canonical scale (logit,log,etc.)
  f.1.predict <- predict(gbm1, data2, n.trees=best.iter.test)
  
  # compute quantity prior to transformation
  f.new = sin(3*X1) - 4*X2 + mu
  
  # Base the validation tests on observed discrepancies
  expect_true(sd(f.new - f.1.predict) < 1.0)
})

skip_on_valgrind()

##############
# Author: James Hickey
# 
# Series of tests on the effect of the offset on the output.
# Help identify if the refactoring is changing the higher level behaviour.
#
##############
context("Testing 0 offset - old API")
test_that("Setting the offset to 0 does not alter the initial value - Adaboost",{
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # Offset 
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3,
                  data=data,
                  weights=w,
                  var.monotone=c(0,0,0),
                  distribution="adaboost",
                  n.trees=100,
                  shrinkage=0.001,
                  interaction.depth=3,
                  bag.fraction = 0.5,
                  train.fraction = 0.5,
                  cv.folds=1,
                  n.minobsinnode = 10,
                  verbose = FALSE)
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3,
                       data=data,
                       weights=w,
                       offset = offset,
                       var.monotone=c(0,0,0),
                       distribution="adaboost",
                       n.trees=100,
                       shrinkage=0.001,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       cv.folds=1,
                       n.minobsinnode = 10,
                       verbose = FALSE)
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Bernoulli",{
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3,
                  data=data,
                  weights=w,
                  var.monotone=c(0,0,0),
                  distribution="bernoulli",
                  n.trees=100,
                  shrinkage=0.001,
                  interaction.depth=3,
                  bag.fraction = 0.5,
                  train.fraction = 0.5,
                  cv.folds=1,
                  n.minobsinnode = 10,
                  verbose = FALSE)
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3,
                       data=data,
                       weights=w,
                       offset=offset,
                       var.monotone=c(0,0,0),
                       distribution="bernoulli",
                       n.trees=100,
                       shrinkage=0.001,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       cv.folds=1,
                       n.minobsinnode = 10,
                       verbose = FALSE)
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - CoxPH",{
  require(survival)
  set.seed(1)
  ## GIVEN
  # create some data
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(y=Surv(tt,delta),X1=X1,X2=X2,X3=X3)
  
  #Offset 
  offset <- rep(0, N)
  
  ## WHEN
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(y~X1+X2+X3,
                  data=data,
                  weights=w,
                  var.monotone=c(0,0,0),
                  distribution="coxph",
                  n.trees=100,
                  shrinkage=0.001,
                  interaction.depth=3,
                  bag.fraction = 0.5,
                  train.fraction = 0.5,
                  cv.folds = 1,
                  n.minobsinnode = 10,
                  keep.data = TRUE,
                  verbose = FALSE)
  set.seed(15)
  gbm.zero.offset <- gbm(y~X1+X2+X3,
                       data=data,
                       weights=w,
                       var.monotone=c(0,0,0),
                       distribution="CoxPH",
                       offset=offset,
                       n.trees=100,
                       shrinkage=0.001,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       cv.folds = 1,
                       n.minobsinnode = 10,
                       keep.data = TRUE,
                       verbose = FALSE)
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Gamma",{
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(0,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + abs(rnorm(N,0,sigma))
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                  data=data,
                  var.monotone=c(0,0,0,0,0,0),
                  distribution="gamma",
                  n.trees=100,
                  shrinkage=0.005,
                  interaction.depth=3,
                  bag.fraction = 0.5,
                  train.fraction = 0.5,
                  mFeatures = 3,
                  n.minobsinnode = 10,
                  keep.data=TRUE,
                  cv.folds=1,
                  verbose = FALSE)
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                       data=data,
                       var.monotone=c(0,0,0,0,0,0),
                       distribution="Gamma",
                       n.trees=100,
                       offset=offset,
                       shrinkage=0.005,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       mFeatures = 3,
                       n.minobsinnode = 10,
                       keep.data=TRUE,
                       cv.folds=1,
                       verbose = FALSE)

    expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Gaussian",{
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                  data=data,
                  var.monotone=c(0,0,0,0,0,0),
                  distribution="gaussian",
                  n.trees=100,
                  shrinkage=0.005,
                  interaction.depth=3,
                  bag.fraction = 0.5,
                  train.fraction = 0.5,
                  mFeatures = 3,
                  n.minobsinnode = 10,
                  keep.data=TRUE,
                  cv.folds=1,
                  verbose = FALSE)
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                       data=data,
                       var.monotone=c(0,0,0,0,0,0),
                       distribution="gaussian",
                       n.trees=100,
                       offset=offset,
                       shrinkage=0.005,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       mFeatures = 3,
                       n.minobsinnode = 10,
                       keep.data=TRUE,
                       cv.folds=1,
                       verbose = FALSE)
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Laplace",{
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                  data=data,
                  var.monotone=c(0,0,0,0,0,0),
                  distribution="laplace",
                  n.trees=100,
                  shrinkage=0.005,
                  interaction.depth=3,
                  bag.fraction = 0.5,
                  train.fraction = 0.5,
                  mFeatures = 3,
                  n.minobsinnode = 10,
                  keep.data=TRUE,
                  cv.folds=1,
                  verbose = FALSE)
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                       data=data,
                       var.monotone=c(0,0,0,0,0,0),
                       distribution="laplace",
                       n.trees=100,
                       shrinkage=0.005,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       mFeatures = 3,
                       n.minobsinnode = 10,
                       keep.data=TRUE,
                       offset=offset,
                       cv.folds=1,
                       verbose = FALSE)
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Huberized Hinge Loss",{
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3,
                  data=data,
                  weights=w,
                  var.monotone=c(0,0,0),
                  distribution="huberized",
                  n.trees=100,
                  shrinkage=0.001,
                  interaction.depth=3,
                  bag.fraction = 0.5,
                  train.fraction = 0.5,
                  cv.folds=1,
                  n.minobsinnode = 10,
                  verbose = FALSE)
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3,
                       data=data,
                       weights=w,
                       offset=offset,
                       var.monotone=c(0,0,0),
                       distribution="huberized",
                       n.trees=100,
                       shrinkage=0.001,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       cv.folds=1,
                       n.minobsinnode = 10,
                       verbose = FALSE)
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})

test_that("Setting the offset to 0 does not alter the initial value - Pairwise",{
  skip("Skipping pairwise")
  # create query groups, with an average size of 25 items each
  set.seed(1)
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  Y.norm <- round(Y/(max(Y) + 0.001))
  
  data.train <- data.frame(Y, query=query, X1, X2, X3)
  data.train.mrr <- data.frame(Y.norm, query=query, X1, X2, X3)
  
  
  # Generate new gbm objects
  set.seed(15)
  gbm.no.offset.ndcg <- gbm(Y~X1+X2+X3,
                  data=data.train,     
                  distribution=list(   
                    name='pairwise',   
                    metric="ndcg",     
                    group="query"),    
                  n.trees=100,        
                  shrinkage=0.005,     
                  interaction.depth=3, 
                  bag.fraction = 0.5,  
                  train.fraction = 1,  
                  n.minobsinnode = 10, 
                  keep.data=TRUE,      
                  cv.folds=1,          
                  verbose = FALSE     
                  )         
  
  gbm.no.offset.mrr <- gbm(Y.norm~X1+X2+X3, 
                 data=data.train.mrr,     
                 distribution=list(  
                   name='pairwise',  
                   metric="mrr",
                   group='query'),   
                 n.trees=100,        
                 shrinkage=0.005,    
                 interaction.depth=3,
                 bag.fraction = 0.5, 
                 train.fraction = 1, 
                 n.minobsinnode = 10,
                 keep.data=TRUE,     
                 cv.folds=1,         
                 verbose = FALSE     
                 )         
  
  gbm.no.offset.map <- gbm(Y.norm~X1+X2+X3,
                 data=data.train.mrr,
                 distribution=list(  
                   name='pairwise',  
                   metric="map",     
                   group='query'),   
                 n.trees=100,        
                 shrinkage=0.005,    
                 interaction.depth=3,
                 bag.fraction = 0.5, 
                 train.fraction = 1, 
                 n.minobsinnode = 10,
                 keep.data=TRUE,     
                 cv.folds=1,         
                 verbose = FALSE     
                 )         
  gbm.no.offset.conc <- gbm(Y~X1+X2+X3,  
                  data=data.train,    
                  distribution=list(  
                    name='pairwise',  
                    metric="conc",    
                    group='query'),   
                  n.trees=100,        
                  shrinkage=0.005,    
                  interaction.depth=3,
                  bag.fraction = 0.5, 
                  train.fraction = 1, 
                  n.minobsinnode = 10,
                  keep.data=TRUE,     
                  cv.folds=1,         
                  verbose = FALSE     
                  )
  set.seed(15)
  gbm.zero.offset.ndcg <- gbm(Y~X1+X2+X3,       
                            data=data.train,    
                            distribution=list(  
                              name='pairwise',  
                              metric="ndcg",     
                              group='query'),   
                            n.trees=100,        
                            shrinkage=0.005,    
                            interaction.depth=3,
                            bag.fraction = 0.5, 
                            train.fraction = 1, 
                            n.minobsinnode = 10,
                            keep.data=TRUE,     
                            cv.folds=1,         
                            verbose = FALSE,    
                            offset=rep(0, N)
                            )
  
  gbm.zero.offset.mrr <- gbm(Y.norm~X1+X2+X3,  
                           data=data.train.mrr,
                           distribution=list(  
                             name='pairwise',  
                             metric="mrr",     
                             group='query'),   
                           n.trees=100,        
                           shrinkage=0.005,    
                           interaction.depth=3,
                           bag.fraction = 0.5, 
                           train.fraction = 1, 
                           n.minobsinnode = 10,
                           keep.data=TRUE,     
                           cv.folds=1,         
                           verbose = FALSE,    
                           offset=rep(0,N)
                           )         
  
  gbm.zero.offset.map <- gbm(Y.norm~X1+X2+X3,  
                           data=data.train.mrr,
                           distribution=list(  
                             name='pairwise',  
                             metric="map",     
                             group='query'),   
                           n.trees=100,        
                           shrinkage=0.005,    
                           interaction.depth=3,
                           bag.fraction = 0.5, 
                           train.fraction = 1, 
                           n.minobsinnode = 10,
                           keep.data=TRUE,     
                           cv.folds=1,         
                           verbose = FALSE,    
                           offset=rep(0,N)
                           )         
  gbm.zero.offset.conc <- gbm(Y~X1+X2+X3,       
                            data=data.train,    
                            distribution=list(  
                              name='pairwise',  
                              metric="conc",    
                              group='query'),   
                            n.trees=100,        
                            shrinkage=0.005,    
                            interaction.depth=3,
                            bag.fraction = 0.5, 
                            train.fraction = 1, 
                            n.minobsinnode = 10,
                            keep.data=TRUE,     
                            cv.folds=1,         
                            verbose = FALSE,    
                            offset = rep(0,N))
  
  expect_equal(gbm.no.offset.conc$initF, gbm.zero.offset.conc$initF)
  expect_equal(gbm.no.offset.mrr$initF, gbm.zero.offset.mrr$initF)
  expect_equal(gbm.no.offset.map$initF, gbm.zero.offset.map$initF)
  expect_equal(gbm.no.offset.ndcg$initF, gbm.zero.offset.ndcg$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Poisson",{
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(0,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- round(Y + abs(rnorm(N,0,sigma))) # Ensure it is normal
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,   
                  data=data,                  
                  var.monotone=c(0,0,0,0,0,0),
                  distribution="poisson",     
                  n.trees=100,                
                  shrinkage=0.005,            
                  interaction.depth=3,        
                  bag.fraction = 0.5,         
                  train.fraction = 0.5,       
                  mFeatures = 3,              
                  n.minobsinnode = 10,        
                  keep.data=TRUE,
                  cv.folds=1,                 
                  verbose = FALSE)            
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,      
                       data=data,                  
                       var.monotone=c(0,0,0,0,0,0),
                       distribution="poisson",     
                       n.trees=100,                
                       offset=offset,
                       shrinkage=0.005,            
                       interaction.depth=3,        
                       bag.fraction = 0.5,         
                       train.fraction = 0.5,       
                       mFeatures = 3,              
                       n.minobsinnode = 10,        
                       keep.data=TRUE,
                       cv.folds=1,                 
                       verbose = FALSE)            
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Quantile Reg",{
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  #Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                  data=data,               
                  var.monotone=c(0,0,0,0,0,0),
                  distribution= list(name="quantile",alpha=0.95),
                  n.trees=100,                 
                  shrinkage=0.005,             
                  interaction.depth=3,         
                  bag.fraction = 0.5,          
                  train.fraction = 0.5,        
                  mFeatures = 3,               
                  n.minobsinnode = 10,         
                  keep.data=TRUE,
                  cv.folds=1,                 
                  verbose = FALSE)            
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3+X4+X5+X6, 
                       data=data,             
                       var.monotone=c(0,0,0,0,0,0),
                       distribution= list(name="quantile",alpha=0.95), 
                       n.trees=100,                 
                       shrinkage=0.005,
                       interaction.depth=3,
                       bag.fraction = 0.5,
                       train.fraction = 0.5,
                       mFeatures = 3, 
                       n.minobsinnode = 10,
                       keep.data=TRUE,
                       offset=offset,
                       cv.folds=1,
                       verbose = FALSE)
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - T dist",{
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,    
                  data=data,                   
                  var.monotone=c(0,0,0,0,0,0), 
                  distribution="tdist",     
                  n.trees=100,              
                  shrinkage=0.005,          
                  interaction.depth=3,      
                  bag.fraction = 0.5,       
                  train.fraction = 0.5,     
                  mFeatures = 3,            
                  n.minobsinnode = 10,      
                  keep.data=TRUE,
                  cv.folds=1,               
                  verbose = FALSE)          
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                       data=data,            
                       var.monotone=c(0,0,0,0,0,0),
                       distribution="tdist",     
                       n.trees=100,              
                       shrinkage=0.005,          
                       interaction.depth=3,      
                       bag.fraction = 0.5,       
                       train.fraction = 0.5,     
                       mFeatures = 3,            
                       n.minobsinnode = 10,      
                       keep.data=TRUE,
                       offset=offset,
                       cv.folds=1,               
                       verbose = FALSE)          
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Setting the offset to 0 does not alter the initial value - Tweedie",{
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(0,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + abs(rnorm(N,0,sigma))
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Offset
  offset <- rep(0, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.no.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                  data=data,               
                  var.monotone=c(0,0,0,0,0,0),
                  distribution="tweedie",
                  n.trees=100,           
                  shrinkage=0.005,       
                  interaction.depth=3,   
                  bag.fraction = 0.5,    
                  train.fraction = 0.5,  
                  mFeatures = 3,         
                  n.minobsinnode = 10,   
                  keep.data=TRUE,
                  cv.folds=1,            
                  verbose = FALSE)       
  set.seed(15)
  gbm.zero.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                       data=data,            
                       var.monotone=c(0,0,0,0,0,0),
                       distribution="tweedie",    
                       n.trees=100,               
                       shrinkage=0.005,           
                       interaction.depth=3,       
                       bag.fraction = 0.5,        
                       train.fraction = 0.5,      
                       mFeatures = 3,             
                       n.minobsinnode = 10,       
                       keep.data=TRUE,
                       offset=offset,
                       cv.folds=1,                
                       verbose = FALSE)           
  
  expect_equal(gbm.no.offset$initF, gbm.zero.offset$initF)
})
test_that("Increasing the offset reduces the initial value - Adaboost", {
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  
  # Offset 
  offset.smaller <- rep(1, N)
  offset.larger <- rep(20, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.smaller.offset <- gbm(Y~X1+X2+X3,        
                       data=data,              
                       weights=w,
                       offset=offset.smaller,
                       var.monotone=c(0,0,0),  
                       distribution="adaboost",
                       n.trees=100,            
                       shrinkage=0.001,        
                       interaction.depth=3,    
                       bag.fraction = 0.5,     
                       train.fraction = 0.5,   
                       cv.folds=1,             
                       n.minobsinnode = 10,    
                       verbose = FALSE)        
  set.seed(15)  
  gbm.larger.offset <- gbm(Y~X1+X2+X3,
                         data=data,                 
                         weights=w,
                         offset = offset.larger,
                         var.monotone=c(0,0,0),
                         distribution="adaboost",
                         n.trees=100, 
                         shrinkage=0.001,
                         interaction.depth=3,
                         bag.fraction = 0.5,        
                         train.fraction = 0.5,
                         cv.folds=1,                
                         n.minobsinnode = 10,       
                         verbose = FALSE)           
  
  expect_true(gbm.smaller.offset$initF - gbm.larger.offset$initF > 0)
})
test_that("Increasing the offset reduces the initial value - Poisson", {
  # Create data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(0,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- round(Y + abs(rnorm(N,0,sigma))) # Ensure it is normal
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Offset 
  offset.smaller <- rep(1, N)
  offset.larger <- rep(20, N)
  
  # Generate new gbm object
  set.seed(15)
  gbm.smaller.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                            data=data,          
                            var.monotone=c(0,0,0,0,0,0),
                            distribution="poisson",
                            n.trees=100,        
                            offset=offset.smaller,
                            shrinkage=0.005,
                            interaction.depth=3,
                            bag.fraction = 0.5,
                            train.fraction = 0.5,
                            mFeatures = 3,
                            n.minobsinnode = 10,
                            keep.data=TRUE,
                            cv.folds=1,         
                            verbose = FALSE)    
  set.seed(15)
  gbm.larger.offset <- gbm(Y~X1+X2+X3+X4+X5+X6,
                         data=data,            
                         var.monotone=c(0,0,0,0,0,0),
                         distribution="Poisson",
                         n.trees=100,           
                         offset=offset.larger,
                         shrinkage=0.005,       
                         interaction.depth=3,   
                         bag.fraction = 0.5,    
                         train.fraction = 0.5,  
                         mFeatures = 3,         
                         n.minobsinnode = 10,   
                         keep.data=TRUE,
                         cv.folds=1,            
                         verbose = FALSE)       
    
  
  expect_true(gbm.smaller.offset$initF - gbm.larger.offset$initF > 0)
})

skip_on_valgrind()

context("testing openmp parallelization")
test_that("gbm refuses to work with insane numbers of threads", {
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N, 0, sigma)
  
  ## create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  expect_error(gbmt(Y~X1+X2+X3+X4+X5+X6,         # formula
                   data=data,                   # dataset
                   var_monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                   keep_gbm_data=TRUE,
                   cv_folds=10, # do 10-fold cross-validation
                   par_details=gbmParallel(num_threads=-1)),
               "number of threads must be strictly positive",
               fixed=TRUE)
})
test_that("gbm refuses to work with insane array chunk size - old api", {
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  ## create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  expect_error(gbmt(Y~X1+X2+X3+X4+X5+X6,         # formula
                   data=data,                   # dataset
                   var_monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                   keep_gbm_data=TRUE,
                   cv_folds=10, # do 10-fold cross-validation
                   par_details=gbmParallel(num_threads=1, array_chunk_size=0)),
               "array chunk size must be strictly positive", fixed=TRUE)
})
test_that("gbm refuses to work with insane numbers of threads - old API", {
    N <- 1000
    X1 <- runif(N)
    X2 <- 2*runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=T))
    X4 <- ordered(sample(letters[1:6],N,replace=T))
    X5 <- factor(sample(letters[1:3],N,replace=T))
    X6 <- 3*runif(N)
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    SNR <- 10 # signal-to-noise ratio
    Y <- X1**1.5 + 2 * (X2**.5) + mu
    sigma <- sqrt(var(Y)/SNR)
    Y <- Y + rnorm(N,0,sigma)

    ## create a bunch of missing values
    X1[sample(1:N,size=100)] <- NA
    X3[sample(1:N,size=300)] <- NA

    w <- rep(1,N)

    data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

    # fit initial model
    expect_error(gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                                        # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=2000,                 # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=10, # do 10-fold cross-validation
                     par.details=gbmParallel(num_threads=-1)),
                 "number of threads must be strictly positive",
                 fixed=TRUE)
})
test_that("gbm refuses to work with insane array chunk size - old api", {
    N <- 1000
    X1 <- runif(N)
    X2 <- 2*runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=T))
    X4 <- ordered(sample(letters[1:6],N,replace=T))
    X5 <- factor(sample(letters[1:3],N,replace=T))
    X6 <- 3*runif(N)
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    SNR <- 10 # signal-to-noise ratio
    Y <- X1**1.5 + 2 * (X2**.5) + mu
    sigma <- sqrt(var(Y)/SNR)
    Y <- Y + rnorm(N,0,sigma)

    ## create a bunch of missing values
    X1[sample(1:N,size=100)] <- NA
    X3[sample(1:N,size=300)] <- NA

    w <- rep(1,N)

    data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

    # fit initial model
    expect_error(gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                                        # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=2000,                 # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=10, # do 10-fold cross-validation
                     par.details=gbmParallel(num_threads=2, array_chunk_size=0)),
                 "array chunk size must be strictly positive", fixed=TRUE)
})

# GBM data object
# Object storing the data used in training a gbm model. 
# 
# @usage gbm_data(x, y, weights, offset)
# 
# @param x matrix or data-frame of the predictor variables 
# 
# @param y matrix or vector of response variables
# 
# @param weights vector containing the weights applied to each data row
# 
# @param offset vector containing the response variables offsets
# 
# @author James Hickey
# 
# @return a gbm_data object 
# 
# @export 
# 
gbm_data <- function(x, y, weights, offset) {
    
    # Check inputs
    verify_data(x, y)
    
    # Convert y if 2-level factor
    y <- convertY(y)
  
    # Check weights and offsets are doubles  
    weights <- check_weights(weights, length(y))
    if((length(weights)==0) || 
       any(is.infinite(weights)) || 
       !is.atomic(weights) ||
       !is.double(weights)) {
      stop("Weights must be a vector of finite doubles")
    }
    
    if((length(offset)<=1) || 
       any(is.infinite(offset)) || 
       !is.atomic(offset) ||
       !is.double(offset)) {
      stop("Offsets must be a vector of finite doubles")
    }
    
    # Store original data - before any formatting
    data <- cbind(as.data.frame(y), as.data.frame(x))
    
    return(structure(list(x=x, y=y, original_data=data, 
                          weights=weights, offset=offset),
                     class="GBMData"))
}

####################
# Author: James Hickey
#
# Series of tests to check that the conversion of factors 
# and that data is data is appropriate for distribution
#
####################

#### Factor Conversion #### 
context("Testing factor conversion")
test_that("Test factor conversion requires GBMData object", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # When data object has class removed
  data <- gbm_data(matrix(x), y, w, offset)
  attr(data, "class") <- "FAKE"
  
  # Then error thrown when data is not gbm data 
  expect_error(convert_factors(data))
})

#### Validate data given distribution ####
context("Testing data validation and conversion")
test_that("Validation fails if not passed gbm_data object", {
  # Given data (removed class) and a default distribution object
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  data <- gbm_data(matrix(x), y, w, offset)
  attr(data, "class") <- "FAKE"
  dist <- gbm_dist()
  
  # Then error thrown
  expect_error(validate_gbm_data(data, dist))
})
test_that("Validation fails if not given distribution obj", {
  # Given data but not a distribution (remove class)
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  data <- gbm_data(matrix(x), y, w, offset)
  dist <- gbm_dist()
  attr(dist, "class") <- "FAKE"
  
  # Then error thrown
  expect_error(validate_gbm_data(data, dist))
})
test_that("Weights will be normalized to N if not Pairwise distribution", {
  # Given data and a distribution - not pairwise
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  data <- gbm_data(matrix(x), y, w, offset)
  dist <- gbm_dist()
  
  # When data is validated
  data <- validate_gbm_data(data, dist)
  
  # Then weights are normalised to N
  expect_equal(w*length(w)/sum(w), data$weights)
})
test_that("Offset vector must contain same number of points as response - if not CoxPH", {
  # Given data and a distribution - not CoxPH
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  data <- gbm_data(matrix(x), y, w, offset)
  dist <- gbm_dist()
  
  # When offset does not have the same number of points as the response
  data$offset <- data$offset[1:N-1]
  
  # Then error expected on validation
  expect_error(validate_gbm_data(data, dist))
  
})
test_that("Offset must contain same number of points as response - if CoxPH", {
  # Given data (not valid but irrelevant here) and a distribution - CoxPH
  # Offset does not have the same number of points as response
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N/2 + 1)
  
  data <- gbm_data(matrix(x), y, w, offset)
  dist <- gbm_dist("CoxPH")
  
  # Then error expected on validation
  expect_error(validate_gbm_data(data, dist))
})
test_that("Log-link offsets must stay within numerical range", {
  N <- 10
  x <- matrix(runif(N), ncol=1)
  y <- rep(1, N)
  w <- rep(1, N)
  data <- gbm_data(x, y, w, rep(651, N))
  
  expect_error(validate_gbm_data(data, gbm_dist("Poisson")),
               "between -650 and 650")
  expect_error(validate_gbm_data(data, gbm_dist("Gamma")),
               "between -650 and 650")
  expect_error(validate_gbm_data(data, gbm_dist("Tweedie")),
               "between -650 and 650")
  expect_error(validate_gbm_data(data, gbm_dist("Gaussian")), NA)
})
test_that("Responses check requires GBMDist object", {
  # Given data and a distribution - not CoxPH
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  data <- gbm_data(matrix(x), y, w, offset)
  dist <- gbm_dist()
  
  # When checking responses without GBMDist object
  attr(dist, "class") <- "FAKE"
  
  # Then error thrown on checking response
  expect_error(check_response_values(dist, data$y))
})
test_that("Responses must be either a data-frame, matrix or vector", {
  # Given data and a distribution - not CoxPH
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  data <- gbm_data(matrix(x), y, w, offset)
  dist <- gbm_dist()
  
  # When responses are not a vector/matrix or data-frame
  data$y <- list()
  
  # Then error thrown on checking response
  expect_error(check_response_values(dist, data$y))
})
test_that("AdaBoost responses must be in {0, 1}", {
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  dist <- gbm_dist("AdaBoost")

  # When responses not in {0, 1}
  Y[1] <- -1
  
  # Then error thrown on validation
  expect_error(check_response_values(dist, Y))
})
test_that("Bernoulli responses must be in {0, 1}", {
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  dist <- gbm_dist("Bernoulli")
  
  # When responses not in {0, 1}
  Y[1] <- -1
  
  # Then error thrown on validation
  expect_error(check_response_values(dist, Y))
})
test_that("CoxPH responses must be a survival object", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  Resp <- Surv(tt, delta)
  dist <- gbm_dist("CoxPH")
  
  # When response is not Surv object
  attr(Resp, "class") <- "NOTSurv"
  
  # Then error thrown on checking response
  expect_error(check_response_values(dist, Resp))
})
test_that("Gamma responses must be positive", {
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  dist <- gbm_dist("Gamma")
  
  # When responses not positive
  Y[1] <- -1
  
  # Then error thrown on validation
  expect_error(check_response_values(dist, Y))
  Y[1] <- 0
  expect_error(check_response_values(dist, Y))
  
})
test_that("Huberized hinge loss requires responses in {0, 1}", {
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  dist <- gbm_dist("Huberized")
  
  # When responses not in {0, 1}
  Y[1] <- -1
  
  # Then error thrown on validation
  expect_error(check_response_values(dist, Y))
  
})
test_that("Pairwise requires non-negative response - all metrics", {
  # create query groups, with an average size of 25 items each
  set.seed(1)
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  Y.norm <- round(Y/(max(Y) + 0.001))
  
  dist <- gbm_dist("Pairwise", metric="ndcg")
  dist_2 <- gbm_dist("Pairwise", metric="conc")
  dist_3 <- gbm_dist("Pairwise", metric="mrr")
  dist_4 <- gbm_dist("Pairwise", metric="map")
  
  # When response is negative
  Y.norm[1] <- -0.01
  
  # Then error thrown on checking responses
  expect_error(check_response_values(dist, Y.norm))
  expect_error(check_response_values(dist_2, Y.norm))
  expect_error(check_response_values(dist_3, Y.norm))
  expect_error(check_response_values(dist_4, Y.norm))
})
test_that("Pairwise map and mrr metrics require response in {0, 1}", {
  # create query groups, with an average size of 25 items each
  set.seed(1)
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  Y.norm <- round(Y/(max(Y) + 0.001))
  
  dist <- gbm_dist("Pairwise", metric="mrr")
  dist_2 <- gbm_dist("Pairwise", metric="map")
  
  # When response is not in {0, 1}
  Y.norm[1] <- 2
  
  # Then error thrown on checking responses
  expect_error(check_response_values(dist, Y.norm))
  expect_error(check_response_values(dist_2, Y.norm))
})
test_that("Poisson requires positive integer response", {
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rpois(N,p)
  dist <- gbm_dist("Poisson")
  
  # When responses not positive integer
  Y[1] <- 0.2
  
  # Then error thrown on validation
  expect_error(check_response_values(dist, Y))
  
})
test_that("Tweedie requires response to be positive", {
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  dist <- gbm_dist("Tweedie")
  
  # When responses not positive
  Y[1] <- -1
  
  # Then error thrown on validation
  expect_error(check_response_values(dist, Y))
})

####################
# Author: James Hickey
#
# Series of tests to check the functionality that creates strata
#
####################

context("Testing Strata creation:")
test_that("Strata creation function requires GBMData and GBMDist objects", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=c(rep(1, N/4), rep(2, N/4), rep(3, N/4), rep(4, N/4)), num_train = 4, 
                             num_features = 3, bag_fraction = 1, min_num_obs_in_node = 1)
  dist <- gbm_dist("CoxPH")
  
  # When not a GBMData object or GBMDist
  copy_data <- data
  copy_dist <- dist
  attr(data, "class") <- "NOTData"
  attr(dist, "class") <- "NOTDist"
  
  # Then an error is thrown
  expect_error(create_strata(data, train_p, copy_dist))
  expect_error(create_strata(copy_data, train_p, dist))
})
test_that("Strata are NA if distribution is not CoxPH", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=c(rep(1, N/4), rep(2, N/4), rep(3, N/4), rep(4, N/4)), num_train = 4, 
                             num_features = 3, bag_fraction = 1, min_num_obs_in_node = 1)
  
  # GIVEN Dist Not COXPH
  dist <- gbm_dist("AdaBoost")
  
  # When strata created
  dist <- create_strata(data, train_p, dist)
  
  # Then dist object is unchanged
  expect_true(is.na(dist$strata))
  expect_true(is.na(dist$sorted))
})
test_that("Creating strata fills strata, time_order and sorted fields - CoxPH", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N/2)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=c(rep(1, N/4), rep(2, N/4), rep(3, N/4), rep(4, N/4)), num_train = 4, num_features = 3,
                             bag_fraction = 1, min_num_obs_in_node = 1)
  
  # GIVEN Dist - COXPH
  dist <- gbm_dist("CoxPH")
  
  # When strata created
  dist <- create_strata(data, train_p, dist)
  
  # Then dist object is changed
  expect_equal(length(dist$strata), N)
  expect_equal(length(dist$time_order), N)
  expect_equal(nrow(dist$sorted), N)
})
test_that("If response is a matrix with more than 3 columns strata cannot be created - CoxPH", {
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N)
  
  
  # GIVEN Dist - COXPH
  dist <- gbm_dist("CoxPH")
  
  # When response has too many columns - set to 4
  Resp <- data.frame(tt, delta)
  Resp <- cbind(cbind(Resp, rnorm(N)), rnorm(N))
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=c(rep(1, N/4), rep(2, N/4), rep(3, N/4), rep(4, N/4)), num_train = 4, num_features = 3,
                             bag_fraction = 1, min_num_obs_in_node = 1)
  
  # Then error thrown when creating strata
  expect_error(create_strata(data, train_p, dist))
})
test_that("If strata field in distribution object is NULL, all data are put in same strata - CoxPH", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=c(rep(1, N/4), rep(2, N/4), rep(3, N/4), rep(4, N/4)), num_train = 4, num_features = 3, 
                             bag_fraction = 1, min_num_obs_in_node = 1)
  
  # GIVEN Dist - COXPH
  dist <- gbm_dist("CoxPH")
  
  # When creating strata with strata initialized to NULL
  dist$strata <- NULL
  dist <- create_strata(data, train_p, dist)
  
  # Then all examples put in same strata
  expect_equal(dist$strata[1], N)
})
test_that("The training responses are sorted according to strata and this order is stored in time_order", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=c(rep(1, N/5), rep(2, N/5), rep(3, N/5), rep(4, N/5)), num_train = 4, num_features = 3, 
                             bag_fraction = 1, min_num_obs_in_node = 1)
  
  # GIVEN Dist - COXPH
  dist <- gbm_dist("CoxPH")
  
  # When strata are created 
  dist <- create_strata(data, train_p, dist)
    
  # Then the training responses are sorted according to strata
  expect_equal(order(-data$y[seq_len(4*N/5), 1]), dist$time_order[seq_len(4*N/5)])
  expect_equal(order(-data$y[(4*N/5 + 1):N, 1])+(4*N/5), dist$time_order[(4*N/5 + 1):N])
})
test_that("Strata not NULL then observations put in different strata - CoxPH", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N/2)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=c(rep(1, N/4), rep(2, N/4), rep(3, N/4), rep(4, N/4)), num_train = 4, num_features = 3,
                             bag_fraction = 1, min_num_obs_in_node = 1)
  
  # GIVEN Dist - COXPH
  dist <- gbm_dist("CoxPH")
  Num_Strata <- 5
  strata <- sample(seq_len(Num_Strata), N, replace=TRUE)
  dist$original_strata_id <- strata
  
  # When strata are created
  dist <- create_strata(data, train_p, dist)
  
  # Then ordered by id
  expect_equal(dist$strata[seq_len(Num_Strata)], as.vector(cumsum(table(strata))))
})

####################
# Author: James Hickey
#
# Series of test to validate the GBMVarCont objects
#
####################


##### Definition #####
context("Testing Variable Properties container definition:")
test_that("Constructed object has correct class", {
  # Given correct gbm-data object
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  data <- gbm_data(matrix(x), y, w, offset)
  
  # Constructed container has correct class
  expect_true("GBMVarCont" %in% class(var_container(data)))
})

test_that("Constructed object has correct fields", {
  # Given correct gbm-data object
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  data <- gbm_data(matrix(x), y, w, offset)
  
  # Constructed container has correct fields
  expect_equal(names(var_container(data)), c("var_monotone", "var_names", "var_levels", "var_type"))
})

test_that("If variable names are not provided they and retrieved from data", {
  # Given correct gbm-data object - variables have names
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  x <- matrix(x)
  colnames(x) <- "Test"
  data <- gbm_data(x, y, w, offset)
  
  # Then constructed container extracts colnames
  expect_equal(var_container(data)$var_names, "Test")
})

test_that("If monotonicity is not provided it defaults to 0", {
  # Given correct gbm-data object
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  data <- gbm_data(matrix(x), y, w, offset)
  
  # Constructed container will have monotonicity of 0 for
  # each predictor
  expect_equal(var_container(data)$var_monotone, rep(0, ncol(matrix(x))) )
})

##### Errors ####
context("Testing incorrect inputs will throw errors:")
test_that("Names of data must be vector of strings", {
  # Given correct gbm-data object
  N <- 1000
  x <- matrix(c(runif(N), runif(N)), nrow=N, ncol=2)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  data <- gbm_data(x, y, w, offset)
  
  # When names are not vector of strings
  names <- list("1", "2")
  names_2 <- c(NULL, "2")
  names_3 <- c(NA, "2")
  
  # Then error is thrown on construction
  expect_error(var_container(data, var_names = names))
  expect_error(var_container(data, var_names = names_2))
  expect_error(var_container(data, var_names = names_3))
})

test_that("Monotonicity must be specified for each predictor", {
  # Given correct gbm-data object
  N <- 1000
  x <- matrix(c(runif(N), runif(N)), nrow=N, ncol=2)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  data <- gbm_data(x, y, w, offset)
  
  # When monotonicity not specified for each predictor
  monotone <- c(-1)
  
  # Then expect error thrown
  expect_error(var_container(data, var_monotone = monotone))
})

test_that("Monotonicity must be -1, 0 or 1", {
  # Given correct gbm-data object
  N <- 1000
  x <- matrix(c(runif(N), runif(N)), nrow=N, ncol=2)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  data <- gbm_data(x, y, w, offset)
  
  # When monotonicity is not 0, -1, +1
  monotone <- c(-1, NA)
  monotone_2 <- c(0.5, 1)
  monotone_3 <- c(-2, 4)
  monotone_4 <- c(0, Inf)
  
  # Then expect error thrown
  expect_error(var_container(data, var_monotone = monotone))
  expect_error(var_container(data, var_monotone = monotone_2))
  expect_error(var_container(data, var_monotone = monotone_3))
  expect_error(var_container(data, var_monotone = monotone_4))
})

test_that("The number of variable names MUST match the number of predictors", {
  # Given correct gbm-data object
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  data <- gbm_data(matrix(x), y, w, offset)
  
  # When number of var_names does not match the number of predictors
  var_names <- c("First", "OneTooMany")
  
  # Then error on construction
  expect_error(var_container(data, var_names = var_names))
})

####################
# Author: James Hickey
#
# Series of tests to check the creation of cv groups
#
####################

context("Testing input checking - create_cv_groups")
test_that("create_cv_groups throws an error if not passed a GBMData object", {
  # Given inputs
  # Dist_Obj
  dist <- gbm_dist()
  
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_class_stratify <- FALSE
  cv_folds <- 2
  fold_id <- NULL
  
  # When data is stripped of GBMData class
  class(data) <- "wrong"
  
  # Then error thrown when trying to calculate CV groups
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify, fold_id))
})
test_that("create_cv_groups throws an error if not passed a GBMDist object", {
  # Given inputs
  # Dist_Obj
  dist <- gbm_dist()
  
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_class_stratify <- FALSE
  cv_folds <- 2
  fold_id <- NULL
  
  # When dist is stripped of GBMDist class
  class(dist) <- ""
  
  # Then error thrown when trying to calculate CV groups
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify, fold_id))
})
test_that("create_cv_groups throws an error if not passed a GBMTrainParams object", {
  # Given inputs
  # Dist_Obj
  dist <- gbm_dist()
  
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_class_stratify <- FALSE
  cv_folds <- 2
  fold_id <- NULL
  
  # When params is stripped of GBMTrainParams class
  class(params) <- ""
  
  # Then error thrown when trying to calculate CV groups
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify, fold_id))
})
test_that("create_cv_groups throws an error if not cv_folds is not a whole number >= 1", {
  # Given inputs - cv_folds not a whole number >= 1
  # Dist_Obj
  dist <- gbm_dist()
  
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_class_stratify <- FALSE
  fold_id <- NULL
  cv_folds_1 <- 2.1
  cv_folds_2 <- -1
  cv_folds_3 <- "What?"
  cv_folds_4 <- Inf
  cv_folds_5 <- NaN
  cv_folds_6 <- 0
  
  # When creating cv groups
  # Then error thrown when trying to calculate CV groups
  expect_error(create_cv_groups(data, dist, params, cv_folds_1, cv_class_stratify, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds_2, cv_class_stratify, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds_3, cv_class_stratify, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds_4, cv_class_stratify, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds_5, cv_class_stratify, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds_6, cv_class_stratify, fold_id))
})
test_that("create_cv_groups throws an error if not cv_class_stratify is not a logical", {
  # Given inputs - cv_class_stratify is not a logical!
  # Dist_Obj
  dist <- gbm_dist()
  
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  fold_id <- NULL
  cv_folds <- 2
  cv_class_stratify_1 <- -1
  cv_class_stratify_2 <- "What?"
  cv_class_stratify_3 <- Inf
  cv_class_stratify_4 <- NaN
  cv_class_stratify_5 <- 0
  cv_class_stratify_6 <- c(TRUE, FALSE)
  
  # When creating cv groups
  # Then error thrown when trying to calculate CV groups
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_1, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_2, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_3, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_4, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_5, fold_id))
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_6, fold_id))
})

context("Testing cv groupings")
test_that("cv groups is an atomic vector of integers whose length is equal to the number of training observations", {
  # Given inputs
  # Dist_Obj
  dist <- gbm_dist()
  
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_class_stratify <- FALSE
  cv_folds <- 2
  fold_id <- NULL
  
  # When creating cv_groups
  cv_groups <- create_cv_groups(data, dist, params, cv_folds, cv_class_stratify, fold_id)
  
  # Then cv_groups is an atomic vector of integers with length equal to N
  expect_true(any(cv_groups == as.integer(cv_groups)))
  expect_true(is.atomic(cv_groups))
  expect_true(length(cv_groups)== params$num_train)  
})
test_that("max integer in cv groups produced is = cv_folds parameter", {
  # Given inputs
  # Dist_Obj
  dist <- gbm_dist()
  
  # create some data
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_class_stratify <- FALSE
  cv_folds <- 2
  fold_id <- NULL
  
  # When creating cv_groups
  cv_groups <- create_cv_groups(data, dist, params, cv_folds, cv_class_stratify, fold_id)
  
  # Then cv_groups max fold number == cv_folds
  expect_true(max(cv_groups) == cv_folds)
})
test_that("cv groups is equal to fold_id if not NULL - distributions other than Bernoulli or Pairwise", {
  # Given inputs - default distributions other than Bernoulli
  # Dist_Objs
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("CoxPH")
  dist_3 <- gbm_dist("Gamma")
  dist_4 <- gbm_dist("Gaussian")
  dist_5 <- gbm_dist("Huberized")
  dist_6 <- gbm_dist("Laplace")
  dist_7 <- gbm_dist("Poisson")
  dist_8 <- gbm_dist("Quantile")
  dist_9 <- gbm_dist("TDist")
  dist_10 <- gbm_dist("Tweedie")
  
  # create some data - DOESN'T NEED TO REFLECT DIST
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters - NB: fold_id not NULL
  cv_class_stratify <- FALSE
  cv_folds <- 2
  fold_id <- seq_len(params$num_train)
  
  # When creating cv_groups for distributions other than Bernoulli
  cv_groups_1 <- create_cv_groups(data, dist_1, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_2 <- create_cv_groups(data, dist_2, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_3 <- create_cv_groups(data, dist_3, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_4 <- create_cv_groups(data, dist_4, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_5 <- create_cv_groups(data, dist_5, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_6 <- create_cv_groups(data, dist_6, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_7 <- create_cv_groups(data, dist_7, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_8 <- create_cv_groups(data, dist_8, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_9 <- create_cv_groups(data, dist_9, params, cv_folds, cv_class_stratify, fold_id)
  cv_groups_10 <- create_cv_groups(data, dist_10, params, cv_folds, cv_class_stratify, fold_id)

    
  # Then cv_groups == fold_id
  expect_equal(cv_groups_1, fold_id)
  expect_equal(cv_groups_2, fold_id)
  expect_equal(cv_groups_3, fold_id)
  expect_equal(cv_groups_4, fold_id)
  expect_equal(cv_groups_5, fold_id)
  expect_equal(cv_groups_6, fold_id)
  expect_equal(cv_groups_7, fold_id)
  expect_equal(cv_groups_8, fold_id)
  expect_equal(cv_groups_9, fold_id)
  expect_equal(cv_groups_10, fold_id)
})
test_that("cv groups is equal to fold_id if not NULL - Bernoulli with cv_class_stratify=FALSE", {
  # Given inputs - default distributions for Bernoulli
  # Dist_Objs
  dist <- gbm_dist("Bernoulli")
  
  # create some data - DOESN'T NEED TO REFLECT DIST
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_folds <- 2
  fold_id <- seq_len(params$num_train)
  
  # When creating cv_groups with cv_class_stratify=FALSE and fold_id != NULL
  cv_class_stratify <- FALSE
  cv_groups <- create_cv_groups(data, dist, params, cv_folds, cv_class_stratify, fold_id)
  
  # Then cv_groups == fold_id
  expect_equal(cv_groups, fold_id)
})
test_that("cv_class_stratify does nothing for distributions other than Bernoulli - fold_id != NULL", {
  # Given inputs - default distributions other than Bernoulli
  # Dist_Objs
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("CoxPH")
  dist_3 <- gbm_dist("Gamma")
  dist_4 <- gbm_dist("Gaussian")
  dist_5 <- gbm_dist("Huberized")
  dist_6 <- gbm_dist("Laplace")
  dist_7 <- gbm_dist("Pairwise")
  dist_8 <- gbm_dist("Poisson")
  dist_9 <- gbm_dist("Quantile")
  dist_10 <- gbm_dist("TDist")
  dist_11 <- gbm_dist("Tweedie")
  
  # create some data - DOESN'T NEED TO REFLECT DIST
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters - NB: fold_id not NULL
  cv_folds <- 2
  fold_id <- seq_len(params$num_train)
  
  # Set up groups for Pairwise
  dist_7$group <- sample(1:5, N, replace=TRUE)
  
  # When creating cv_groups for both cv_class_stratify==TRUE and FALSE for distributions other than Bernoulli
  cv_groups_1_T <- create_cv_groups(data, dist_1, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_2_T <- create_cv_groups(data, dist_2, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_3_T <- create_cv_groups(data, dist_3, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_4_T <- create_cv_groups(data, dist_4, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_5_T <- create_cv_groups(data, dist_5, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_6_T <- create_cv_groups(data, dist_6, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_7_T <- create_cv_groups(data, dist_7, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_8_T <- create_cv_groups(data, dist_8, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_9_T <- create_cv_groups(data, dist_9, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_10_T <- create_cv_groups(data, dist_10, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_11_T <- create_cv_groups(data, dist_11, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  
  cv_groups_1_F <- create_cv_groups(data, dist_1, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_2_F <- create_cv_groups(data, dist_2, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_3_F <- create_cv_groups(data, dist_3, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_4_F <- create_cv_groups(data, dist_4, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_5_F <- create_cv_groups(data, dist_5, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_6_F <- create_cv_groups(data, dist_6, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_7_F <- create_cv_groups(data, dist_7, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_8_F <- create_cv_groups(data, dist_8, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_9_F <- create_cv_groups(data, dist_9, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_10_F <- create_cv_groups(data, dist_10, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_11_F <- create_cv_groups(data, dist_11, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  
  # These groups don't depend on cv_class_stratify
  expect_equal(cv_groups_1_T, cv_groups_1_F)
  expect_equal(cv_groups_2_T, cv_groups_2_F)
  expect_equal(cv_groups_3_T, cv_groups_3_F)
  expect_equal(cv_groups_4_T, cv_groups_4_F)
  expect_equal(cv_groups_5_T, cv_groups_5_F)
  expect_equal(cv_groups_6_T, cv_groups_6_F)
  expect_equal(cv_groups_7_T, cv_groups_7_F)
  expect_equal(cv_groups_8_T, cv_groups_8_F)
  expect_equal(cv_groups_9_T, cv_groups_9_F)
  expect_equal(cv_groups_10_T, cv_groups_10_F)
  expect_equal(cv_groups_11_T, cv_groups_11_F)
})
test_that("cv_class_stratify does nothing for distributions other than Bernoulli - fold_id == NULL", {
  # Given inputs - default distributions other than Bernoulli
  # Dist_Objs
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("CoxPH")
  dist_3 <- gbm_dist("Gamma")
  dist_4 <- gbm_dist("Gaussian")
  dist_5 <- gbm_dist("Huberized")
  dist_6 <- gbm_dist("Laplace")
  dist_7 <- gbm_dist("Pairwise")
  dist_8 <- gbm_dist("Poisson")
  dist_9 <- gbm_dist("Quantile")
  dist_10 <- gbm_dist("TDist")
  dist_11 <- gbm_dist("Tweedie")
  
  # create some data - DOESN'T NEED TO REFLECT DIST
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters - NB: fold_id is NULL
  cv_folds <- 2
  fold_id <- NULL
  
  # Set up groups for Pairwise
  dist_7$group <- sample(1:5, N, replace=TRUE)
  
  # When creating cv_groups for both cv_class_stratify==TRUE and FALSE for distributions other than Bernoulli
  set.seed(1)
  cv_groups_1_T <- create_cv_groups(data, dist_1, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_2_T <- create_cv_groups(data, dist_2, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_3_T <- create_cv_groups(data, dist_3, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_4_T <- create_cv_groups(data, dist_4, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_5_T <- create_cv_groups(data, dist_5, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_6_T <- create_cv_groups(data, dist_6, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_7_T <- create_cv_groups(data, dist_7, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_8_T <- create_cv_groups(data, dist_8, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_9_T <- create_cv_groups(data, dist_9, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_10_T <- create_cv_groups(data, dist_10, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  cv_groups_11_T <- create_cv_groups(data, dist_11, params, cv_folds, cv_class_stratify=TRUE, fold_id)
  
  set.seed(1)
  cv_groups_1_F <- create_cv_groups(data, dist_1, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_2_F <- create_cv_groups(data, dist_2, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_3_F <- create_cv_groups(data, dist_3, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_4_F <- create_cv_groups(data, dist_4, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_5_F <- create_cv_groups(data, dist_5, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_6_F <- create_cv_groups(data, dist_6, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_7_F <- create_cv_groups(data, dist_7, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_8_F <- create_cv_groups(data, dist_8, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_9_F <- create_cv_groups(data, dist_9, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_10_F <- create_cv_groups(data, dist_10, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  cv_groups_11_F <- create_cv_groups(data, dist_11, params, cv_folds, cv_class_stratify=FALSE, fold_id)
  
  
  # These groups don't depend on cv_class_stratify
  expect_equal(cv_groups_1_T, cv_groups_1_F)
  expect_equal(cv_groups_2_T, cv_groups_2_F)
  expect_equal(cv_groups_3_T, cv_groups_3_F)
  expect_equal(cv_groups_4_T, cv_groups_4_F)
  expect_equal(cv_groups_5_T, cv_groups_5_F)
  expect_equal(cv_groups_6_T, cv_groups_6_F)
  expect_equal(cv_groups_7_T, cv_groups_7_F)
  expect_equal(cv_groups_8_T, cv_groups_8_F)
  expect_equal(cv_groups_9_T, cv_groups_9_F)
  expect_equal(cv_groups_10_T, cv_groups_10_F)
  expect_equal(cv_groups_11_T, cv_groups_11_F)
  
})
test_that("cv_class_stratify changes grouping for Bernoulli distribution", {
  # Given inputs - default distributions for Bernoulli, data
  # now representative of distribution
  # Dist_Objs
  dist <- gbm_dist("Bernoulli")
  
  # create some data - DOES NEED TO REFLECT DIST
  set.seed(1)
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  offset <- rep(0, N)
  
  X <- data.frame(X1=X1,X2=X2,X3=X3)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  cv_folds <- 2
  fold_id <- seq_len(params$num_train)
  
  # When creating cv_groups with cv_class_stratify=TRUE and FALSE
  cv_class_stratify_T <- TRUE
  cv_class_stratify_F <- FALSE
  
  set.seed(1)
  cv_groups_T <- create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_T, fold_id)
  
  set.seed(1)
  cv_groups_F <- create_cv_groups(data, dist, params, cv_folds, cv_class_stratify_F, fold_id)
  
  # Then cv_groups are not identical
  expect_true(any(cv_groups_T != cv_groups_F))
})
test_that("Error thrown when number in a certain class is < cv_folds on stratification - Bernoulli", {
  # Given inputs - default distributions for Bernoulli, data
  # now representative of distribution
  # Dist_Objs
  dist <- gbm_dist("Bernoulli")
  
  # create some data - DOES NEED TO REFLECT DIST
  set.seed(1)
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  offset <- rep(0, N)
  
  X <- data.frame(X1=X1,X2=X2,X3=X3)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters
  fold_id <- seq_len(params$num_train)
  
  # When creating cv_groups with cv_class_stratify=TRUE and cv_folds > min number of a certain class
  cv_class_stratify <- TRUE
  Ones <- tabulate(data$y[seq_len(params$num_train)])
  Zeros <- length(data$y[seq_len(params$num_train)])-Ones
  cv_folds <- min(c(Ones, Zeros)) + 1
  
  # Then error will be thrown
  expect_error(create_cv_groups(data, dist, params, cv_folds, cv_class_stratify, fold_id))
})
test_that("cv groups are such that folds are split at group boundaries - Pairwise", {
  # Given a Pairwise distribution object and appropriate
  # groupings etc.
  dist <- gbm_dist("Pairwise")

  
  # create some data - DOESN'T NEED TO REFLECT DIST
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Params Obj
  params <- training_params(num_train = N)
  
  # Other CV Parameters - NB: fold_id not NULL
  cv_folds <- 2
  fold_id <- seq_len(params$num_train)
  
  # DEFINE GROUPINGS
  dist$group <- as.factor(sample(seq_len(5), N, replace=TRUE))
  
  # When cv_groups is called and compared to 
  set.seed(1)
  cv_groups <- create_cv_groups(data, dist, params, cv_folds, FALSE, fold_id)
  
  # Then the splits are at group boundaries
  set.seed(1)
  samp <- sample(rep(seq_len(cv_folds), length=nlevels(dist$group)))
  expect_equal(cv_groups, samp[as.integer(dist$group[seq_len(params$num_train)])])
})

####################
# Author: James Hickey
#
# Series of test to validate the GBMData objects
#
####################

##### Definition #####
context("Testing GBMData object definition:")
test_that("Constructed object has correct class", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Constructed object
  expect_true("GBMData" %in% class(gbm_data(matrix(x), y, w, offset)))
})

test_that("Constructed object has correct fields", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Constructed objetc
  expect_equal(names(gbm_data(matrix(x), y, w, offset)), c("x", "y", "original_data", "weights", "offset"))
})

#### Error Checking #####
context("Testing variable check")
test_that("Predictors must be in a data-frame or matrix", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Error thrown when predictors not put in matrix or data-frame
  expect_error(gbm_data(x, y, w, offset))
  expect_error(gbm_data(matrix(x), y, w, offset), NA)
  expect_error(gbm_data(data.frame(x), y, w, offset), NA)
})

test_that("Responses must be a data-frame, matrix or vector", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Error thrown when responses not put in matrix or vector
  expect_error(gbm_data(matrix(x), list(y), w, offset))
  expect_error(gbm_data(matrix(x), y, w, offset), NA)
  expect_error(gbm_data(matrix(x), matrix(y), w, offset), NA)
  expect_error(gbm_data(matrix(x), data.frame(y), w, offset), NA)
})

test_that("Each predictor row must have a response", {
  # Given correct data
  N <- 1000
  x <- runif(N-1)
  x2 <- runif(N+1)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Error thrown as more responses than predictors and vice-versa
  expect_error(gbm_data(matrix(x), y, w, offset))
  expect_error(gbm_data(matrix(x2), y, w, offset))
})

test_that("Predictor variables must be ordered, numeric or categorical", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Error thrown when x is not categorical, ordered or numeric
  expect_error(gbm_data(matrix(rep("A", N)), y, w, offset))
  expect_error(gbm_data(data.frame(ordered(x)), y, w, offset), NA)
  expect_error(gbm_data(data.frame(factor(x)), y, w, offset), NA)
  expect_error(gbm_data(matrix(as.numeric(x)), y, w, offset), NA)
})

test_that("Missing values not allowed in response", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Set subset of responses to be Missing
  y[sample(1:N, 100)] <- NA
  
  # Expect error
  expect_error(gbm_data(matrix(x), y, w, offset))
})

test_that("Can't have predictor with all missing values", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  w <- rexp(N) 
  offset <- rexp(N)
  
  # Add in variable which is all NAs
  x2 <- rep(NA, N)
  
  # Expect error to be thrown
  expect_error(gbm_data(data.frame(x, x2), y, w, offset))
})

test_that("Weights must be a vector of positive doubles", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N,1,p)
  offset <- rexp(N)
  
  expect_error(gbm_data(matrix(x), y, w=-rexp(N), offset))
})

test_that("Offset must be a vector of doubles", {
  # Given correct data
  N <- 1000
  x <- runif(N)
  p <- 0.5
  y <- rbinom(N, 1, p)
  w <- rexp(N)
  
  expect_error(gbm_data(matrix(x), y, w, c(1, "A")))
  expect_error(gbm_data(matrix(x), y, w, 1))
  expect_error(gbm_data(matrix(x), y, w, Inf))
  expect_error(gbm_data(matrix(x), y, w, NULL))
})

####################
# Author: James Hickey
#
# Series of tests to check that the basic gbm behaviour
#
####################

context("some basic checks")

test_that("gaussian works", {
  skip_on_cran()
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  best_iter <- gbmt_performance(fit, method="cv") # returns test set estimate of best number of trees
  
  # Make prediction
  set.seed(2)
  # make some new data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=TRUE))
  X4 <- ordered(sample(letters[1:6],N,replace=TRUE))
  X5 <- factor(sample(letters[1:3],N,replace=TRUE))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  # Actual underlying signal - Check how close to this we are
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # predict on the new data using "best" number of trees
  f.predict <- predict(fit, data2, best_iter) # f.predict will be on the canonical scale (logit,log,etc.)
  
  # Base the validation tests on observed discrepancies
  expect_true(cor(data2$Y, f.predict) > 0.990)
  expect_true(sd(data2$Y-f.predict) < sigma)
})

test_that("coxph works - efron", {
  skip_on_cran()
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)
  
  # Put into new API
  dist <- gbm_dist("CoxPH", prior_node_coeff_var=10)
  params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  
  # fit initial model
  gbm1 <- gbmt(Surv(tt,delta)~X1+X2+X3, data=data, distribution=dist, weights=w, offset=rep(0, N),
               train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose=FALSE)
  
  best_iter <- gbmt_performance(gbm1, method="test") # returns test set estimate of best number of trees
  
  # make some new data
  set.seed(2)
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)  # -0.5 <= f <= 0.5 via sin fn.
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  
  data2 <- data.frame(tt=apply(cbind(tt.surv,tt.cens),1,min),
                      delta=as.numeric(tt.surv <= tt.cens),
                      f=f,
                      X1=X1,X2=X2,X3=X3)
  
  # predict on the new data using "best" number of trees
  # f.predict will be on the canonical scale (logit,log,etc.)
  f.predict <- predict(gbm1, data2, best_iter)
  
  #plot(data2$f,f.predict)
  # Use observed sd
  expect_true(sd(data2$f - f.predict) < 0.4)
})

test_that("coxph works - breslow", {
    skip_on_cran()
    # Require Surv to be available
    require(survival)
  
    # create some data
    set.seed(1)
    N <- 3000
    X1 <- runif(N)
    X2 <- runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=T))
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
    tt.surv <- rexp(N,exp(f))
    tt.cens <- rexp(N,0.5)
    delta <- as.numeric(tt.surv <= tt.cens)
    tt <- apply(cbind(tt.surv,tt.cens),1,min)

    # throw in some missing values
    X1[sample(1:N,size=100)] <- NA
    X3[sample(1:N,size=300)] <- NA

    # random weights if you want to experiment with them
    w <- rep(1,N)

    data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)

    # Put into new API
    dist <- gbm_dist("CoxPH", prior_node_coeff_var=10, ties="breslow")
    params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                              shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
    
    # fit initial model
    gbm1 <- gbmt(Surv(tt,delta)~X1+X2+X3, data=data, distribution=dist, weights=w, offset=rep(0, N),
                 train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose=FALSE)
    
    best_iter <- gbmt_performance(gbm1, method="test") # returns test set estimate of best number of trees
  
    # make some new data
    set.seed(2)
    N <- 1000
    X1 <- runif(N)
    X2 <- runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=T))
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)  # -0.5 <= f <= 0.5 via sin fn.
    tt.surv <- rexp(N,exp(f))
    tt.cens <- rexp(N,0.5)

    data2 <- data.frame(tt=apply(cbind(tt.surv,tt.cens),1,min),
                        delta=as.numeric(tt.surv <= tt.cens),
                        f=f,
                        X1=X1,X2=X2,X3=X3)

    # predict on the new data using "best" number of trees
    # f.predict will be on the canonical scale (logit,log,etc.)
    f.predict <- predict(gbm1, data2, best_iter)

    #plot(data2$f,f.predict)
    # Use observed sd
    expect_true(sd(data2$f - f.predict) < 0.4)
})
test_that("coxph - runs to completion with train_fraction of 1.0", {
  ## Needed packages
  require(survival)
  
  # Given data from the survival data
  # keep only certain baseline variables, subjects with longitudinal data
  temp <- subset(pbc, id%in%pbcseq$id, select=c(id:sex, stage))
  pbc1 <- tmerge(data1=temp, data2=temp, id=id, death = event(time, status))
  
  pbc2 <- tmerge(pbc1, pbcseq, id=id, ascites = tdc(day, ascites),
                 bili = tdc(day, bili), albumin = tdc(day, albumin),
                 protime = tdc(day, protime), alk.phos = tdc(day, alk.phos))
  # Training params
  params_1 <- training_params(interaction_depth = 3, num_train=round(1.0 * nrow(pbc)), num_trees = 500, id=seq(nrow(pbc)),
                              shrinkage=.01)
  params_2 <- training_params(interaction_depth = 3, num_train=round(1.0 * nrow(pbc2)), num_trees = 500, id=seq(nrow(pbc2)),
                              shrinkage=.01)
  
  # Then expect no errors when performing gbm fit with train_fraction = 1.0
  # GBM fit baseline data
  expect_error(gbmt(Surv(time, status==2) ~ bili + protime + albumin + alk.phos, data=pbc,
                    distribution=gbm_dist("CoxPH"), train_params=params_1), NA)
  
  # GBM fit using start/stop times to get time-dependent covariates
  expect_error(gbmt(Surv(tstart, tstop, death==2) ~ bili + protime + albumin + alk.phos, 
                   data=pbc2, distribution=gbm_dist("CoxPH"), train_params=params_2), NA)
})

test_that("coxph - runs to completion with train_fraction < 1.0 and cv_folds > 1", {
  ## Needed packages
  require(survival)
  
  # Given data from the survival data
  # keep only certain baseline variables, subjects with longitudinal data
  temp <- subset(pbc, id%in%pbcseq$id, select=c(id:sex, stage))
  pbc1 <- tmerge(data1=temp, data2=temp, id=id, death = event(time, status))
  
  pbc2 <- tmerge(pbc1, pbcseq, id=id, ascites = tdc(day, ascites),
                 bili = tdc(day, bili), albumin = tdc(day, albumin),
                 protime = tdc(day, protime), alk.phos = tdc(day, alk.phos))
  
  # Training params
  params_1 <- training_params(interaction_depth = 3, num_train=round(0.8 * nrow(pbc)), num_trees = 500, id=seq(nrow(pbc)),
                              shrinkage=.01)
  params_2 <- training_params(interaction_depth = 3, num_train=round(0.8 * nrow(pbc2)), num_trees = 500, id=seq(nrow(pbc2)),
                              shrinkage=.01)

  
  # Then expect no errors when performing gbm fit with train_fraction < 1.0 and cv_folds > 1
  # GBM fit baseline data
  expect_error(gbmt(Surv(time, status==2) ~ bili + protime + albumin + alk.phos, data=pbc,  distribution=gbm_dist("CoxPH"),
                   train_params=params_1), NA)
  expect_error(gbmt(Surv(time, status==2) ~ bili + protime + albumin + alk.phos, data=pbc,  distribution=gbm_dist("CoxPH"),
                   train_params=params_1, cv_folds=5), NA)
  
  # GBM fit using start/stop times to get time-dependent covariates
  expect_error(gbmt(Surv(tstart, tstop, death==2) ~ bili + protime + albumin + alk.phos, 
                   data=pbc2, distribution=gbm_dist("CoxPH"), train_params=params_2), NA)
  expect_error(gbmt(Surv(tstart, tstop, death==2) ~ bili + protime + albumin + alk.phos, 
                    data=pbc2, distribution=gbm_dist("CoxPH"), train_params=params_2, cv_folds=5), NA)
})

test_that("coxph cv_folds - runs to completion with start-stop, id'ed and stratified dataset", {
  ## Needed packages
  require(survival)
  
  # Given data from the survival package
  cgd2 <- cgd[cgd$enum==1,]
  
  # Training params
  params_1 <- training_params(interaction_depth = 1, num_train=nrow(cgd2), num_trees = 500, id=seq(nrow(cgd2)),
                            shrinkage=.01)
  params_2 <- training_params(interaction_depth = 1, num_train=round(0.8 * nrow(cgd2)), num_trees = 500, id=seq(nrow(cgd2)),
                              shrinkage=.01)
  params_3 <- training_params(interaction_depth = 3, num_train=round(1.0 * length(unique(cgd$id))), num_trees = 500, id=cgd$id,
                              shrinkage=.01)
  params_4 <- training_params(interaction_depth = 3, num_train=round(0.8 * length(unique(cgd$id))), num_trees = 500, id=cgd$id,
                              shrinkage=.01)
  
  # Then fitting a gbm model should throw no errors - with cv_folds > 1
  expect_error(gbmt(Surv(tstop, status) ~ age + sex + inherit +
                     steroids + propylac + hos.cat, data=cgd2, 
                   distribution = gbm_dist("CoxPH"), train_params=params_1, cv_folds=10), NA)
  expect_error(gbmt(Surv(tstop, status) ~ age + sex + inherit +
                     steroids + propylac + hos.cat, data=cgd2, 
                   distribution = gbm_dist("CoxPH"),  train_params=params_2, cv_folds=5), NA)
  
  expect_error(gbmt(Surv(tstart, tstop, status) ~ age + sex + inherit +
                     steroids + propylac, data=cgd,
                   distribution = gbm_dist("CoxPH", strata=cgd$hos.cat), train_params=params_3, cv_folds=10), NA)
  expect_error(gbmt(Surv(tstart, tstop, status) ~ age + sex + inherit +
                     steroids + propylac, data=cgd, 
                    distribution = gbm_dist("CoxPH", strata=cgd$hos.cat), train_params=params_4, cv_folds=10), NA)
})

test_that("bernoulli works", {
    skip_on_cran()
    set.seed(1)

    # create some data
    N <- 1000
    X1 <- runif(N)
    X2 <- runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=T))
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
    Y <- rbinom(N,1,p)

    # random weights if you want to experiment with them
    w <- rexp(N)
    w <- N*w/sum(w)

    data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
    offset <- rep(0, N)
    
    # Set up for new API
    params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                              shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
    dist <- gbm_dist("Bernoulli")
    fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
                train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
    
    best_iter <- gbmt_performance(fit, method="test") # returns test set estimate of best number of trees

    # make some new data
    set.seed(2)
    N <- 1000
    X1 <- runif(N)
    X2 <- runif(N)
    X3 <- factor(sample(letters[1:4],N,replace=T))
    mu <- c(-1,0,1,2)[as.numeric(X3)]

    p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
    Y <- rbinom(N,1,p)
    data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)

    # predict on the new data using "best" number of trees
    # f.predict will be on the canonical scale (logit,log,etc.)
    f.1.predict <- predict(fit, data2, n.trees=best_iter)

    # compute quantity prior to transformation
    f.new = sin(3*X1) - 4*X2 + mu

    # Base the validation tests on observed discrepancies
    expect_true(sd(f.new - f.1.predict) < 1.0)
})

test_that("relative influence picks out true predictors", {
    skip_on_cran()
    set.seed(1234)
    X1 <- matrix(nrow=1000, ncol=50)
    X1 <- apply(X1, 2, function(x) rnorm(1000)) # Random noise
    X2 <- matrix(nrow=1000, ncol=5)
    X2 <- apply(X2, 2, function(x) c(rnorm(500), rnorm(500, 3))) # Real predictors
    cls <- rep(c(0, 1), ea=500) # Class
    X <- data.frame(cbind(X1, X2, cls))
    
    # Construct model
    params <- training_params(num_trees=1000, interaction_depth=2, min_num_obs_in_node=10, 
                              shrinkage=0.01, bag_fraction=0.5, id=seq(1000), num_train=1000, num_features=55)
    dist <- gbm_dist("Bernoulli")
    fit <- gbmt(cls~ ., data=X, distribution=dist, weights=rep(1, 1000), offset=rep(0, 1000),
                train_params=params, keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
    
    # Check can get out real predictors
    ri <- relative_influence(fit, rescale=TRUE, sort_it=TRUE)
    wh <- names(ri)[1:5]
    res <- sum(wh %in% paste("V", 51:55, sep = ""))
    expect_equal(res, 5)
})

test_that("Conversion of 2 factor Y is successful", {
  
  NumY <- sample(c(0, 1), size=1000, replace=TRUE)
  FactY = factor(ifelse(NumY==1, "Yes", "No"), levels=c("No", "Yes"))

  PredX <-
  data.frame(
    x1 = runif(1000)
    ,x2 = runif(1000)
  )

  # Fit 1 - Rel Influence
  set.seed(32479)
  params <- training_params(num_trees=50, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(1000), num_train=1000, num_features=2)
  g1 <- gbmt(y ~ ., data = data.frame(y = NumY, PredX)
            , distribution = gbm_dist("Bernoulli"), train_params=params, is_verbose = FALSE)
  rig1 <- relative_influence(g1, num_trees=50)
  
  # Fit 2 - Rel Influence
   set.seed(32479)
  g2 <- gbmt(y ~ ., data = data.frame(y = FactY, PredX)
           ,  distribution = gbm_dist("Bernoulli"), train_params=params, is_verbose = FALSE)
   rig2 <- relative_influence(g2, num_trees=50)
  
  expect_equal(rig1, rig2)
})

test_that("Cross Validations group generation - Bernoulli", {
  
  NumY <- sample(rep(c(0,1), 500), size=1000)
  NumX <- matrix(rep(0, size=1000), nrow=1000, ncol=1)
  dist <- gbm_dist("Bernoulli")
  data <- gbm_data(NumX, NumY, weights=rep(1, length=1000), offset=rep(0, length=1000))
  
  cv_groups <- create_cv_groups(data, dist, training_params(num_train=1000, id=1:1000, num_features=1), 
                                cv_folds=4, cv_class_stratify=FALSE, fold_id=NULL)
  expect_true(all(table(cv_groups)==250))
  
  cv_stratified <- create_cv_groups(data, dist, training_params(num_train=1000, id=1:1000, num_features=1), 
                                    cv_folds=4, cv_class_stratify=TRUE, fold_id=NULL)
  
  Strats <- sapply(1:4, function(x){table(NumY[cv_stratified == 1])})
  
  expect_true(all(Strats == 125))
  
})

#' Print model summary
#' 
#' Display basic information about a \code{GBMFit} object.
#' 
#' Prints some information about the model object. In particular, this method
#' prints the call to \code{gbmt}, the type of loss function that was used,
#' and the total number of iterations.
#' 
#' If cross-validation was performed, the 'best' number of trees as estimated
#' by cross-validation error is displayed. If a test set was used, the 'best'
#' number of trees as estimated by the test set error is displayed.
#' 
#' The number of available predictors, and the number of those having non-zero
#' influence on predictions is given (which might be interesting in data mining
#' applications).
#' 
#' If bernoulli or adaboost was used, the confusion matrix and
#' prediction accuracy are printed (objects being allocated to the class with
#' highest probability for bernoulli). These classifications
#' are performed using the cross-validation fitted values.
#' 
#' If the 'distribution' was specified as gaussian, laplace, quantile or
#' t-distribution, a summary of the residuals is displayed.  The residuals are
#' the cross-validation residuals. Also, a pseudo R-squared value is displayed.
#' For Gaussian response, this is 1 - sum(r*r) / sum(z*z) where z = y -
#' mean(y). For the other distributions, this is 1 - (median(abs(r)) /
#' mad(y))^2, following the suggestion of Rousseeuw and Leroy (equation 3.11).
#' Note that this definition of a robust R-squared is contentious.
#' 
#' @param x is a fitted generalized boosting object of class \code{GBMFit}.
#' 
#' @param \dots arguments passed to \code{print.default}.
#' 
#' @return Returns \code{x} invisibly
#' 
#' @author James Hickey, Harry Southworth, Daniel Edwards
#' @seealso \code{\link{gbmt}}
#' @references P. J. Rousseeuw and A. M. Leroy, Robust Regression and Outlier
#' Detection, Wiley, 1987 (2003).
#' @keywords models nonlinear survival nonparametric
#' @aliases print
#' @export 
print.GBMFit <- function(x, ... ){
  # Print call
  if(!is.null(x$call)) print(x$call)
  
  #  Print out number of iterations and distribution used
  print_iters_and_dist(x)
  
  # Print out performance measures
  best_iter <- print_perf_measures(x)
  
  # Print out relative influence of variables
  ri <- relative_influence(x, num_trees=best_iter)
  cat( "There were", length(x$variables$var_names), "predictors of which",
       sum(ri > 0), "had non-zero influence.\n" )
  
  # CV confusion matrix and pseudo-R-squared
  print_confusion_matrix(x)
  
  return(invisible(x))
}


#### Helper Functions ####
print_iters_and_dist <- function(x) {
  # If Pairwise dist extract metric and max_rank
  # else distribution details is just the name
  check_if_gbm_fit(x)
  if(distribution_name(x) == "Pairwise") {
    if (!is.null(x$distribution$max.rank) && (x$distribution$max.rank > 0)) {
      distribution_details <- sprintf("pairwise (metric=%s, max.rank=%d)", x$distribution$metric,
                                      x$distribution$max.rank)
    } else {
      distribution_details <- sprintf("pairwise (metric=%s)", x$distribution$metric)
    }
  } else {
    distribution_details <- distribution_name(x)
  }
  cat("A gradient boosted model with", distribution_details, "loss function.\n",
      length(iteration_error(x, 'train')), "iterations were performed.\n")
}

print_perf_measures <- function(x) {
  # Calculate the best number of iterations - returns test set if 
  # possible
  check_if_gbm_fit(x)
  
  # Set default answer - final iteration
  best_iter <- length(iteration_error(x, 'train'))
  
  # CV best iteration 
  if (has_cross_validation(x)) {
      best_iter <- print(gbmt_performance(x, method="cv"))
  }
  
  # Test set best iteration
  if (has_train_test_split(x)) {
      best_iter <- print(gbmt_performance(x, method="test"))
  }
  
  return(best_iter)
}

print_confusion_matrix <- function(x) {
  # This prints the confusion matrix based on the
  # cross validated fit - if no cv fit is present - break
  if (!has_cross_validation(x)) return(invisible())
  
  # If data was not kept than can't calculate confusion matrix
  check_if_gbm_fit(x)
  if(is.null(x$gbm_data_obj)) return(invisible())
  
  # Print off confusion matrix or pseudo R^2
  if (distribution_name(x) %in% c("Bernoulli", "AdaBoost", "Huberized")) {
    binary_response_conf_matrix(x$gbm_data_obj$y, x$cv_fitted)
  } else if (distribution_name(x) %in% c("Gaussian", "Laplace", "Quantile", "TDist")) {
    pseudo_r_squared(x$gbm_data_obj$y, x$cv_fitted, distribution_name(x))
  }
}

binary_response_conf_matrix <- function(response, cv_fit) {
  # Function to print out the confusion matrix for a binary response 
  # cross-validated generalized boosted model
  p <- 1 / (1 + exp(-cv_fit))
  p <- ifelse(p < .5, 0, 1)
  
  conf_mat <- matrix(table(c(response + 2 * p , 0:3)), ncol=2)
  conf_mat <- conf_mat - 1
  pred_acc <- round(100 * sum(diag(conf_mat)) / sum(conf_mat),2)
  
  conf_mat <- cbind(conf_mat,  round(100*diag(conf_mat)/rowSums(conf_mat),2))
  dimnames(conf_mat) <- list(c("0","1"), c("0", "1", "Pred. Acc."))
  
  cat("\nCross-validation confusion matrix:\n")
  print(conf_mat)
  cat("\nCross-validation prediction Accuracy = ", pred_acc, "%\n", sep = "")
}

pseudo_r_squared <- function(response, cv_fit, dist_name) {
  # Calculate residuals
  ifelse(nlevels(response)>0, response <- as.integer(response), response <- response)
  cv_resids <- response - cv_fit
  
  cat("\nSummary of cross-validation residuals:\n" )
  print(quantile(cv_resids))
  cat("\n")
  
  # Do pseudo R^2
  if (dist_name == "Gaussian") {
    yadj <- response - mean(response)
    R2 <- 1 - sum(cv_resids^2)/sum(yadj^2)
    cat("Cross-validation pseudo R-squared: ", signif(R2, 3), "\n")
  }
  else { # Rousseeuw & Leroy, page 44
    R2 <- 1 - (median(abs(cv_resids)) / mad(response))^2
    cat("Cross-validation robust pseudo R-squared: ", signif(R2, 3), "\n")
  }
  
  return(R2)
}

####################
# Author: James Hickey
#
# Series of tests to check print and summary methods
#
####################

context("Testing print.GBMFit helper functions")
test_that("print_perf_measures defaults to total number of iterations if train_fraction = 1 and fit is not cross-validated", {
  # Given a "correct" GBMFit object - train_fraction=1
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=1, is_verbose = FALSE)
  
  # When calculating print_perf_measures
  best_iter <- print_perf_measures(fit)
  
  # Then results equal to num_trees
  expect_equal(best_iter, params$num_trees)
  
})

test_that("print_perf_measures calculates the performance using test if train_fraction < 1", {
  # Given a "correct" GBMFit object - train_fraction < 1
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When calculating print_perf_measures
  best_iter <- print_perf_measures(fit)
  
  # Then results equal to gbm_perf with "test" method
  expect_equal(best_iter, gbmt_performance(fit, method="test"))
})
test_that("print_perf_measures returns the performance using cv if fit is cross-validated and train_fraction=1", {
  # Given a "correct" GBMFit object - train_fraction=1 and cv_folds > 1
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When calculating print_perf_measures
  best_iter <- print_perf_measures(fit)
  
  # Then results equal to gbm_perf with method="cv"
  expect_equal(best_iter, gbmt_performance(fit, method="cv"))
})
test_that("print_iters_and_dist does not throw error when passed a GBMFit object", {
  # Given a "correct" GBMFit object
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When calculating print_iters_and_dist 
  # Then no error is thrown
  expect_error(print_iters_and_dist(fit), NA)
})
test_that("print_confusion_matrix does not throw error when passed a GBMFit object", {
  # Given a "correct" GBMFit object
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When calculating print_confusion_matrix
  # Then no error is thrown
  expect_error(print_confusion_matrix(fit), NA)
})
test_that("binary_response_conf_matrix does not throw error when called correctly", {
  # Given a "correct" GBMFit object - Bernoulli distribution
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When calculating binary_response_conf_matrix
  # Then no error is thrown
  expect_error(binary_response_conf_matrix(fit$gbm_data_obj$y, fit$cv_fitted), NA)
})
test_that("pseudo_r_squared does not throw error when passed correct inputs", {
  # Given a "correct" GBMFit object - using Gauss
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When calculating pseudo_r_squared 
  # Then no error is thrown
  expect_error(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, fit$distribution$name), NA)
})
test_that("pseudo_r_squared is same for all dists except Gaussian", {
  # Given a "correct" GBMFit object - NOT Gaussian
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When calculating pseudo_r_squared for Bernoulli
  r_sq <- pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, fit$distribution$name)
  
  # Then this value is same for all dists except Gaussian
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "AdaBoost"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Bernoulli"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "CoxPH"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Gamma"), r_sq)
  expect_true(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Gaussian") != r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Huberized"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Laplace"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Pairwise"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Poisson"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Quantile"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "TDist"), r_sq)
  expect_equal(pseudo_r_squared(fit$gbm_data_obj$y, fit$cv_fitted, "Tweedie"), r_sq)
})


context("Test Input Error checking - printing")
test_that("print_perf_measures throws error if passed an object other than GBMFit class", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  
  # When  print_perf_measures is called on fit which is missing class GBMFit
  class(fit) <- ""
  
  # Then an error is thrown
  expect_error(print_perf_measures(fit))
})

test_that("print_iters_and_dist throws error if passed an object other than GBMFit class", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  
  # When  print_iters_and_dist is called on fit which is missing class GBMFit
  class(fit) <- ""
  
  # Then an error is thrown
  expect_error(print_iters_and_dist(fit))
})

test_that("print_confusion_matrix throws error if passed an object other than GBMFit class", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  
  # When  print_confustion_matrix is called on fit which is missing class GBMFit
  class(fit) <- ""
  
  # Then an error is thrown
  expect_error(print_confusion_matrix(fit))
})

test_that("summary.GBMFit throws error if cBars is not a whole number > 1", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  
  # When summary method is called with cBars is not a whole number > 1
  # Then an error is thrown
  expect_error(summary(fit, cBars=c(TRUE, FALSE)))
  expect_error(summary(fit, cBars=1.3))
  expect_error(summary(fit, cBars="What"))
  expect_error(summary(fit, cBars=Inf))
  expect_error(summary(fit, cBars=NULL))
})

test_that("summary.GBMFit throws error if num_trees is not a whole number > 1", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  
  # When summary method is called with num_trees is not a whole number > 1
  # Then an error is thrown
  expect_error(summary(fit, num_trees=c(TRUE, FALSE)))
  expect_error(summary(fit, num_trees=1.3))
  expect_error(summary(fit, num_trees="What"))
  expect_error(summary(fit, num_trees=Inf))
  expect_error(summary(fit, num_trees=NULL))
})

test_that("summary.GBMFit throws error if plot_it is not a logical", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  
  # When summary method is called with plot_it not a logical
  # Then an error is thrown
  expect_error(summary(fit, plot_it=c(TRUE, FALSE)))
  expect_error(summary(fit, plot_it=1))
  expect_error(summary(fit, plot_it="What"))
  expect_error(summary(fit, plot_it=Inf))
  expect_error(summary(fit, plot_it=NULL))
})

test_that("summary.GBMFit throws error if normalize is not a logical", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
 
  
  # When summary method is called with normalize not a logical
  # Then an error is thrown
  expect_error(summary(fit, normalize=c(TRUE, FALSE)))
  expect_error(summary(fit, normalize=1))
  expect_error(summary(fit, normalize="What"))
  expect_error(summary(fit, normalize=Inf))
  expect_error(summary(fit, normalize=NULL))
})

context("Test print.GBMFit")
test_that("Print method on GBMFit object runs without error", {
  # Given some "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  fit2 <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
               train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=1, is_verbose = FALSE)
  
  # When print method is called
  # Then runs without error
  expect_error(print(fit), NA)
  expect_error(print(fit2), NA)
})
context("Test summary.GBMFit")
test_that("Summary method on GBMFit object runs without error", {
  # Given some "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  fit2 <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=1, is_verbose = FALSE)

  # When summary method is called
  # Then runs without error
  expect_error(summary(fit), NA)
  expect_error(summary(fit2), NA)
})

test_that("Summary method returns data.frame of variables and relative influences", {
  # Given some "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  fit2 <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
               train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=1, is_verbose = FALSE)
  
  # When summary method is called
  summary_fit_1 <- summary(fit)
  summary_fit_2 <- summary(fit2)
  
  # Then summary method returns data.frame of variables and relative influences
  # with variables ordered 
  expect_true(is.data.frame(summary_fit_1))
  expect_true(is.data.frame(summary_fit_2))
  expect_equal(names(summary_fit_1), c("var", "rel_inf"))
  expect_equal(names(summary_fit_2), c("var", "rel_inf"))
})

test_that("Summary method returns variables and relative influence ordered by relative influence in descending order", {
  # Given a "correct" GBMFit objects
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=30, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  dist <- gbm_dist("Bernoulli")
  fit <- gbmt(Y~X1+X2+X3, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)
  
  # When summary method is called
  summary_fit_1 <- summary(fit)
  
  # Then the variables are ordered in terms of descending relative influence
  rel_inf <- relative_influence(fit, gbmt_performance(fit, method="cv"))
  expect_equal(fit$variables$var_names[order(-rel_inf)], summary_fit_1$var)
})

# GBM Cross Validation
# 
# Create fitted object - Cross validation is performed if \code{cv_folds > 1}, otherwise only the full model
# is fitted. This function is called by \code{\link{gbmt}} to perform the model fitting.
# 
# @param gbm_data_obj a GBMData object containing all of the data used to fit a gbm model. 
# 
# @param gbm_dist_obj a GBMDist object specifying the distribution and any additional parameters needed.
# 
# @param train_params a GBMTrainParams object containing generic parameters defining how the model should be
# trained.
# 
# @param var_container a GBMVarCont object which defines the properties of the predictor variables in the data.
# 
# @param cv_folds a positive integer specifying the number of folds to be used in cross-validation of the gbm
# fit. If cv_folds > 1 then cross-validation is performed; the default of cv_folds is 1.
# 
# @param cv_groups vector of integers specifying which row of data belongs to which cv_folds.
# 
# @param par_details Details of the parallelization to use in the
#     core algorithm.
#     
# @param is_verbose if TRUE, will print out progress and performance of the fitting.
# 
# @author James Hickey
#
# @return a \code{GBMFit} object which contains appropriate CV info if requested.


gbm_cross_val <- function(gbm_data_obj, gbm_dist_obj, train_params, var_container, 
                          cv_folds, cv_groups, par_details, is_verbose) {
  # Create output list
  gbm_results <- list() 
  
  # Full model fit
  if(is_verbose) message("Fitting Final Model \n")
  gbm_results[[length(gbm_results)+1]] <- gbm_call(gbm_data_obj, gbm_dist_obj, train_params,
                                                  var_container, par_details, is_verbose)

  # Check if only need to fit full model
  if(cv_folds == 1) {
    gbm_results[[1]]$cv_folds <- 1
    return(gbm_results[[1]])
  }

  # Loop over folds
  for(fold_num in seq_len(cv_folds)) {
    if(is_verbose) message("CV:", fold_num, "\n")
    
    # Extract observations in cv fold
    gbm_object_list <- extract_obs_in_fold(gbm_data_obj, gbm_dist_obj, train_params, cv_groups, fold_num)
    
    # Fit to fold
    gbm_results[[length(gbm_results)+1]] <- gbm_call(gbm_object_list$data, gbm_object_list$dist, 
                                                    gbm_object_list$params, var_container, par_details, is_verbose)
  }
  
  # If have multiple folds then total results object is a different class
  class(gbm_results) <- "GBMCVFit"  
  
  # Calculate errors
  cv_errors <- gbm_cv_errors(gbm_results, cv_folds, cv_groups)
  
  # Best number of trees
  best_iter_cv <- which.min(cv_errors)
  
  # Calculate predictions
  predictions <- predict(gbm_results, gbm_data_obj, cv_folds, cv_groups, best_iter_cv)
  
  # Extract relevant parts - all data model
  gbm_results <- gbm_results[[1]]
  gbm_results$cv_folds <- cv_folds
  gbm_results$cv_error <- cv_errors
  gbm_results$cv_fitted <- predictions
  
  return(gbm_results)
}

#' Create distributions
#' 
#' Creation methods that take an empty distribution object
#' and builds on top of the skeleton.
#' 
#' @usage create_dist(empty_obj, ...)
#' 
#' @param empty_obj  An empty distribution object of the correct class.
#' 
#' @param ...  extra parameters used to define the distribution object,
#'  see \code{\link{gbm_dist}}.
#'  
#' @author James Hickey
#'  
#' @return an appropriated gbm distribution object
#' 

create_dist <- function(empty_obj, ...) {
  check_if_gbm_dist(empty_obj)
  UseMethod("create_dist", empty_obj)
}

#' @export
create_dist.default <- function(empty_obj, ...) {
  stop("couldn't create distribution object as creation method for selected distribution not found.")
}

#' @export
create_dist.AdaBoostGBMDist <- function(empty_obj, ...) {
  check_dist_params(empty_obj, ...)
  return(empty_obj)
}

#' @export
create_dist.BernoulliGBMDist <- function(empty_obj, ...) {
  check_dist_params(empty_obj, ...)
  return(empty_obj)
}

#' @export
create_dist.CoxPHGBMDist <- function(empty_obj, strata=NA, sorted=NA, ties="efron"
                                     , prior_node_coeff_var=1000, ...) {
  check_dist_params(empty_obj, strata, sorted, ties, prior_node_coeff_var, ...)
  if(!(ties %in% c("breslow", "efron"))) {
    message("Ties method not recognised - defaulting to efron")
    ties <- "efron"
  }
  empty_obj$ties <- ties
  empty_obj$original_strata_id <- convert_strata(strata)
  empty_obj$sorted <- sorted
  empty_obj$prior_node_coeff_var <- prior_node_coeff_var
  empty_obj$reorder <- TRUE
  return(empty_obj)
}

#' @export
create_dist.GammaGBMDist <- function(empty_obj, ...) {
  check_dist_params(empty_obj, ...)
  return(empty_obj)
}

#' @export
create_dist.GaussianGBMDist <- function(empty_obj, ...) {
  check_dist_params(empty_obj, ...)
  return(empty_obj)
}

#' @export
create_dist.HuberizedGBMDist <- function(empty_obj, ...) {
  check_dist_params(empty_obj, ...)
  return(empty_obj)
}

#' @export
create_dist.LaplaceGBMDist <- function(empty_obj, ...) {
  check_dist_params(empty_obj, ...)
  return(empty_obj)
}

#' @export
create_dist.PairwiseGBMDist <- function(empty_obj, group="query", metric="ndcg", max_rank=0,
                                        group_index=NULL, ...) {
  check_dist_params(empty_obj, group, metric, max_rank, group_index, ...)
  empty_obj$metric <- metric
  empty_obj$group <- group
  empty_obj$group_index <- group_index
  empty_obj$max_rank <- max_rank
  empty_obj$reorder <- TRUE
  return(empty_obj)
}

#' @export
create_dist.PoissonGBMDist <- function(empty_obj, ...) {
  check_dist_params(empty_obj, ...)
  return(empty_obj)
}

#' @export
create_dist.QuantileGBMDist <- function(empty_obj, alpha=0.25, ...) {
  if(is.null(alpha)) alpha <- 0.25
  check_dist_params(empty_obj, alpha, ...)
  empty_obj$alpha <- alpha 
  return(empty_obj)
}

#' @export
create_dist.TDistGBMDist <- function(empty_obj, df=4, ...) {
  if(is.null(df)) df <- 4
  check_dist_params(empty_obj, df, ...)
  empty_obj$df <- df
  return(empty_obj)
}

#' @export
create_dist.TweedieGBMDist <- function(empty_obj, power=1.5, ...) {
  if(is.null(power)) power <- 1.5
  check_dist_params(empty_obj, power, ...)
  empty_obj$power <- power
  return(empty_obj)
}

#' GBMT
#' 
#' Fits generalized boosted regression models - new API. This
#' prepares the inputs, performing tasks such as creating cv folds,
#' before calling \code{gbmt_fit} to call the underlying C++ and fit
#' a generalized boosting model.
#' 
#' @param formula a symbolic description of the model to be fit.  The
#' formula may include an offset term (e.g. y~offset(n) + x).
#' 
#' @param distribution a \code{GBMDist} object specifying the
#' distribution and any additional parameters needed. If not
#' specified then the distribution will be guessed.
#' 
#' @param data a data frame containing the variables in the model.
#' By default, the variables are taken from the environment.
#' 
#' @param weights optional vector of weights used in the fitting
#' process.  These weights must be positive but need not be
#' normalized. By default they are set to 1 for each data row.
#' 
#' @param offset optional vector specifying the model offset; must be
#' positive.  This defaults to a vector of 0's, the length of which
#' is equal to the number rows of data.
#' 
#' @param train_params a GBMTrainParams object which specifies the
#' parameters used in growing decision trees.
#' 
#' @param var_monotone optional vector, the same length as the number
#' of predictors, indicating the relationship each variable has with
#' the outcome.  It have a monotone increasing (+1) or decreasing
#' (-1) or an arbitrary relationship.
#' 
#' @param var_names a vector of strings of containing the names of
#' the predictor variables.
#' 
#' @param cv_folds a positive integer specifying the number of folds
#' to be used in cross-validation of the gbm fit. If cv_folds > 1
#' then cross-validation is performed; the default of cv_folds is 1.
#' 
#' @param cv_class_stratify a bool specifying whether or not to
#' stratify via response outcome. Currently only applies to
#' "Bernoulli" distribution and defaults to false.
#' 
#' @param fold_id An optional vector of values identifying what fold
#' each observation is in. If supplied, cv_folds can be
#' missing. Note: Multiple rows of the same observation must have the
#' same fold_id.
#' 
#' @param keep_gbm_data a bool specifying whether or not the gbm_data
#' object created in this method should be stored in the results.
#' 
#' @param par_details Details of the parallelization to use in the
#' core algorithm (\code{\link{gbmParallel}}).
#'     
#' @param is_verbose if TRUE, gbmt will print out progress and
#' performance of the fit.
#' 
#' @return a \code{GBMFit} object.
#'
#' @examples
#' ## create some data
#' N <- 1000
#' X1 <- runif(N)
#' X2 <- runif(N)
#' X3 <- factor(sample(letters[1:4],N,replace=TRUE))
#' mu <- c(-1,0,1,2)[as.numeric(X3)]
#' 
#' p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
#' Y <- rbinom(N,1,p)
#' 
#' # random weights if you want to experiment with them
#' w <- rexp(N)
#' w <- N*w/sum(w)
#' 
#' data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
#' 
#' \donttest{
#' # takes longer, but num_trees=3000 preferable
#' train_params <-
#'      training_params(num_trees = 3000,
#'                      shrinkage = 0.001,
#'                      bag_fraction = 0.5,
#'                      num_train = N/2,
#'                      id=seq_len(nrow(data)),
#'                      min_num_obs_in_node = 10,
#'                      interaction_depth = 3,
#'                      num_features = 3)
#' }
#'
#' # for the example to run quickly, num_trees=100
#' train_params <-
#'      training_params(num_trees = 100,
#'                      shrinkage = 0.001,
#'                      bag_fraction = 0.5,
#'                      num_train = N/2,
#'                      id=seq_len(nrow(data)),
#'                      min_num_obs_in_node = 10,
#'                      interaction_depth = 3,
#'                      num_features = 3)
#'  
#' # fit initial model
#' gbm1 <- gbmt(Y~X1+X2+X3,                # formula
#'              data=data,                 # dataset
#'              weights=w,
#'              var_monotone=c(0,0,0),     # -1: monotone decrease,
#'                                         # +1: monotone increase, 
#'                                         #  0: no monotone restrictions
#'              distribution=gbm_dist("Bernoulli"),
#'              train_params = train_params,
#'              cv_folds=5,                # do 5-fold cross-validation
#'              is_verbose = FALSE)           # don't print progress
#' 
#' # plot the performance
#' #   returns out-of-bag estimated best number of trees
#' best.iter.oob <- gbmt_performance(gbm1,method="OOB")  
#' plot(best.iter.oob)
#' print(best.iter.oob)
#' 
#' # returns 5-fold cv estimate of best number of trees
#' best.iter.cv <- gbmt_performance(gbm1,method="cv")   
#' plot(best.iter.cv)
#' print(best.iter.cv)
#' 
#' # returns test set estimate of best number of trees
#' best.iter.test <- gbmt_performance(gbm1,method="test") 
#' plot(best.iter.cv)
#' print(best.iter.test)
#' 
#' best.iter <- best.iter.test
#' 
#' # plot variable influence
#' summary(gbm1,num_trees=1)         # based on first tree
#' summary(gbm1,num_trees=best.iter) # based on  estimated best number of trees
#' 
#' # create marginal plots
#' # plot variable X1,X2,X3 after "best" iterations
#' oldpar <- par(no.readonly = TRUE)
#' par(mfrow=c(1,3))
#' plot(gbm1,1,best.iter)
#' plot(gbm1,2,best.iter)
#' plot(gbm1,3,best.iter)
#' par(mfrow=c(1,1))
#' plot(gbm1,1:2,best.iter) # contour plot vars 1 & 2 after "best" num iterations
#' plot(gbm1,2:3,best.iter) # lattice plot vars 2 & 3 after "best" num iterations
#' 
#' # 3-way plot
#' plot(gbm1,1:3,best.iter)
#' 
#' # print the first and last trees
#' print(pretty_gbm_tree(gbm1,1))
#' print(pretty_gbm_tree(gbm1, gbm1$params$num_trees))
#' par(oldpar) # reset graphics options to previous settings
#' @export
gbmt <- function(formula,
                 distribution=gbm_dist("Gaussian"),
                 data,
                 weights=rep(1, nrow(data)),
                 offset=rep(0, nrow(data)),
                 train_params=training_params(num_trees=2000,
                     interaction_depth=3,
                     min_num_obs_in_node=10,
                     shrinkage=0.001,
                     bag_fraction=0.5,
                     id=seq_len(nrow(data)),
                     num_train=round(0.5 * nrow(data)),
                     num_features=ncol(data)-1),
                 var_monotone=NULL,
                 var_names=NULL,
                 cv_folds=1,
                 cv_class_stratify=FALSE,
                 fold_id=NULL,
                 keep_gbm_data=FALSE,
                 par_details=getOption('gbm.parallel'),
                 is_verbose=FALSE) {
  # Extract the model
  the_call <- match.call()
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("formula", "data", "weights", "offset"), names(mf), 0)
  mf <- mf[c(1, m)]
  mf$drop.unused.levels <- TRUE
  mf$na.action <- na.pass
  mf[[1]] <- as.name("model.frame")
  m <- mf
  mf <- eval(mf, parent.frame())
  Terms <- attr(mf, "terms")
  y <- model.response(mf)
  w <- model.weights(mf)
  offset_mf <- model.offset(mf)
  
  # Set offset/ weights off defaults if specified
  if(!is.null(w))
    weights <- w
  if(!is.null(offset_mf))
    offset <- offset_mf
  
  # get the character name of the response variable
  var_names <- attributes(Terms)$term.labels
  x <- model.frame(terms(reformulate(var_names)),
                   data,
                   na.action=na.pass)
  
  # Check and infer folds if necessary
  check_cv_parameters(cv_folds, cv_class_stratify, fold_id, train_params)
  if (!is.null(fold_id)) {
    if (length(fold_id) != nrow(x)){
      stop("length(fold_id) inequal to number of rows.")
    }
    num_inferred_folds <- length(unique(fold_id))
    if (cv_folds != num_inferred_folds) {
      # Warn if cv_folds and fold_id disagree, but take fold_id.
      warning("CV folds changed from ", cv_folds, " to ", num_inferred_folds,
              " because of levels in fold_id.")
    } 
    cv_folds <- num_inferred_folds
    
    # Set fold_id from whatever it is to an integer ascending from 1. Lazy way.
    fold_id <- as.numeric(as.factor(fold_id))
  }
  
  # If missing guess distribution
  if(missing(distribution)) {
    distribution <- gbm_dist(guess_distribution(y))
  }
  # Update distribution according to groups
  distribution <- determine_groups(data, y, distribution)
  
  # Update weights according to groupings
  weights <- weight_group_consistency(distribution, weights)
  
  # Update number of training rows based off of groups
  train_params <- update_num_train_groups(train_params, distribution)
  
  # Call gbmt.fit 
  gbm_fit_obj <- gbmt_fit(x, y, distribution, weights, offset,
                         train_params, as.character(formula[[2]]),
                         var_monotone, var_names, keep_gbm_data, cv_folds,
                         cv_class_stratify, fold_id, par_details, is_verbose)

  # Wrap up extra pieces 
  gbm_fit_obj$model <- m
  gbm_fit_obj$Terms <- Terms
  gbm_fit_obj$call <- the_call
  gbm_fit_obj$is_verbose <- is_verbose
 
  return(gbm_fit_obj)
}

context("Test error messages - new api")
test_that("Columns with all missing stop", {
  
  iris2 <- iris
  iris2$Sepal.Width = NA
  expect_error(
    gbmt(Species == 'setosa' ~ ., data = iris2, distribution = gbm_dist('Bernoulli'), train_params = training_params(num_trees=2))
    ,'variable 2: Sepal.Width is not of type - numeric, ordered or factor.'
  )
  
  iris2$Petal.Width = NA
  expect_error(
    gbmt(Species == 'setosa' ~ ., data = iris2, distribution = gbm_dist('Bernoulli'), train_params = training_params(num_trees=2))
    ,'variable 2, 4: Sepal.Width, Petal.Width is not of type - numeric, ordered or factor.'
  )
  
})

test_that("x longer than y", {
  
  expect_error(
    gbmt_fit(x = iris[,c('Sepal.Length', 'Petal.Length')]
            , y = (iris$Species[1:145] == 'setosa')
            , distribution = gbm_dist('Bernoulli'), train_params=training_params(num_trees = 2))
    ,'The number of rows in x does not equal the length of y\\.')
  
})

test_that("Excessive levels in x", {
  
  testExcess <- data.frame(
    y = sample(c(0,1), 1025, replace = TRUE)
    ,x1 = runif(1025)
    ,x2 = factor(1:1025)
  )
  
  expect_error(
    gbmt_fit(x = testExcess[,c('x1', 'x2')]
            , y = testExcess$y
            , distribution = gbm_dist("Bernoulli"), train_params=training_params(num_trees = 2))
    ,'gbm does not currently handle categorical variables with more than 1024 levels\\. Variable 2\\: x2 has 1025 levels\\.')
  
})

test_that("Inacceptable classes", {
  
  testClasses <- data.frame(
    y = sample(c(0,1), 15, replace = TRUE)
    ,x1 = runif(15)
    ,x2 = seq(as.Date('2015-01-01'), as.Date('2015-01-15'), 'days')
  )
  
  expect_error(
    gbmt_fit(x = testClasses[,c('x1', 'x2')]
            , y = testClasses$y
            , distribution = gbm_dist('Bernoulli'), train_params=training_params(num_trees = 2, min_num_obs_in_node = 1))
    ,'variable 2\\: x2 is not of type - numeric, ordered or factor\\.')
  
})

context("Test error messages - old api")
test_that("Columns with all missing stop", {
  
  iris2 <- iris
  iris2$Sepal.Width = NA
  expect_error(
    gbm(Species == 'setosa' ~ ., data = iris2, distribution = 'Bernoulli', n.trees = 2)
    ,'variable 2: Sepal.Width is not of type - numeric, ordered or factor.'
    )

  iris2$Petal.Width = NA
  expect_error(
    gbm(Species == 'setosa' ~ ., data = iris2, distribution = 'Bernoulli', n.trees = 2)
    ,'variable 2, 4: Sepal.Width, Petal.Width is not of type - numeric, ordered or factor.'
    )
  
})

test_that("x longer than y", {
  
  expect_error(
    gbm.fit(x = iris[,c('Sepal.Length', 'Petal.Length')]
            , y = (iris$Species[1:145] == 'setosa')
            , distribution = 'Bernoulli', n.trees = 2)
    ,'The number of rows in x does not equal the length of y\\.')

})

test_that("Excessive levels in x", {
  
  testExcess <- data.frame(
    y = sample(c(0,1), 1025, replace = TRUE)
    ,x1 = runif(1025)
    ,x2 = factor(1:1025)
  )
  
  expect_error(
    gbm.fit(x = testExcess[,c('x1', 'x2')]
            , y = testExcess$y
            , distribution = 'Bernoulli', n.trees = 2)
    ,'gbm does not currently handle categorical variables with more than 1024 levels\\. Variable 2\\: x2 has 1025 levels\\.')

})

test_that("Inacceptable classes", {
  
  testClasses <- data.frame(
    y = sample(c(0,1), 15, replace = TRUE)
    ,x1 = runif(15)
    ,x2 = seq(as.Date('2015-01-01'), as.Date('2015-01-15'), 'days')
  )
  
  expect_error(
    gbm.fit(x = testClasses[,c('x1', 'x2')]
            , y = testClasses$y
            , distribution = 'Bernoulli', n.trees = 2, n.minobsinnode = 1)
    ,'variable 2\\: x2 is not of type - numeric, ordered or factor\\.')

})

context("reproducibility")
test_that("Setting the seed causes result to be reproducible (1 core)", {
  set.seed(18900217)
  mod <- gbmt(Species == 'setosa' ~ ., data=iris, distribution=gbm_dist("Bernoulli"),
             cv_folds=3)
  nt1 <- gbmt_performance(mod, method="cv")
  ri1 <- relative_influence(mod, num_trees=nt1)
  
  set.seed(18900217)
  mod <- gbmt(Species == 'setosa' ~ ., data=iris, distribution=gbm_dist("Bernoulli"),
             cv_folds=3)
  nt2 <- gbmt_performance(mod, method="cv")
  ri2 <- relative_influence(mod, num_trees=nt2)
  
  expect_equal(nt1, nt2,
               label="Number of trees match when same seed is used")
  expect_equal(ri1, ri2,
               label="Relative influences match when same seed is used")
})

test_that("Setting different seeds causes result to be different (1 core)", {
  set.seed(18900217)
  mod <- gbmt(Species == 'setosa' ~ ., data=iris, distribution=gbm_dist("Bernoulli"),
             cv_folds=3)
  nt1 <- gbmt_performance(mod, method="cv")
  ri1 <- relative_influence(mod, num_trees=nt1)
  
  set.seed(19620729)
  mod <- gbmt(Species == 'setosa' ~ ., data=iris, distribution=gbm_dist("Bernoulli"),
             cv_folds=3)
  nt2 <- gbmt_performance(mod, method="cv")
  ri2 <- relative_influence(mod, num_trees=nt2)
  
  expect_false(isTRUE(all.equal(ri1, ri2)),
               label="Relative influences don't match when different seeds are used")
})

context("reproducibility - old api")

test_that("Setting the seed causes result to be reproducible (1 core)", {
  set.seed(18900217)
  mod <- gbm(Species == 'setosa' ~ ., data=iris, distribution="Bernoulli",
             cv.folds=3, n.trees=100, shrinkage=.1)
  nt1 <- gbmt_performance(mod, method="cv")
  ri1 <- relative_influence(mod, num_trees=nt1)

  set.seed(18900217)
  mod <- gbm(Species == 'setosa' ~ ., data=iris, distribution="Bernoulli",
             cv.folds=3, n.trees=100, shrinkage=.1)
  nt2 <- gbmt_performance(mod, method="cv")
  ri2 <- relative_influence(mod, num_trees=nt2)
  
  expect_equal(nt1, nt2,
               label="Number of trees match when same seed is used")
  expect_equal(ri1, ri2,
               label="Relative influences match when same seed is used")
})

test_that("Setting different seeds causes result to be different (1 core)", {
  set.seed(18900217)
  mod <- gbm(Species == 'setosa' ~ ., data=iris, distribution="Bernoulli",
             cv.folds=3, n.trees=100, shrinkage=.1)
  nt1 <- gbmt_performance(mod, method="cv")
  ri1 <- relative_influence(mod, num_trees=nt1)

  set.seed(19620729)
  mod <- gbm(Species == 'setosa' ~ ., data=iris, distribution="Bernoulli",
             cv.folds=3, n.trees=100, shrinkage=.1)
  nt2 <- gbmt_performance(mod, method="cv")
  ri2 <- relative_influence(mod, num_trees=nt2)

  expect_false(isTRUE(all.equal(ri1, ri2)),
              label="Relative influences don't match when different seeds are used")
})

####################
# Author: James Hickey
#
# Series of tests for gbm loss functionality
#
####################

context("Error checking for GBM loss methods")
test_that("Error thrown responses not vector of doubles", {
  # Given offset, weights, predictions and default dist
  N <- 100
  dist <- gbm_dist()
  offset <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  
  # When responses are not a vector of doubles
  resps <- rep("1", N)
  resps_2 <- NA
  resps_3 <- Inf
  resps_4 <- rep(NaN, N)
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
  expect_error(loss(resps_2, preds, weights, offset, dist))
  expect_error(loss(resps_3, preds, weights, offset, dist))
  expect_error(loss(resps_4, preds, weights, offset, dist))
})
test_that("Error thrown when predictions not vector of doubles", {
  # Given offset, weights, responses and default dist
  N <- 100
  dist <- gbm_dist()
  offset <- runif(N)
  weights <- runif(N)
  resps <- runif(N)
  
  # When predictions are not a vector of doubles
  preds <- rep("1", N)
  preds_2 <- NA
  preds_3 <- Inf
  preds_4 <- rep(NaN, N)
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
  expect_error(loss(resps, preds_2, weights, offset, dist))
  expect_error(loss(resps, preds_3, weights, offset, dist))
  expect_error(loss(resps, preds_4, weights, offset, dist))
})
test_that("Error thrown when weights not vector of doubles", {
  # Given offset, responses, predictions and default dist
  N <- 100
  dist <- gbm_dist()
  offset <- runif(N)
  resps <- runif(N)
  preds <- runif(N)
  
  # When weights are not a vector of doubles
  weights <- rep("1", N)
  weights_2 <- NA
  weights_3 <- Inf
  weights_4 <- rep(NaN, N)
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
  expect_error(loss(resps, preds, weights_2, offset, dist))
  expect_error(loss(resps, preds, weights_3, offset, dist))
  expect_error(loss(resps, preds, weights_4, offset, dist))
})
test_that("Error thrown when offset not vector of doubles", {
  # Given responses, weights, predictions and default dist
  N <- 100
  dist <- gbm_dist()
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  
  # When offsets are not a vector of doubles
  offset <- rep("1", N)
  offset_2 <- NA
  offset_3 <- Inf
  offset_4 <- rep(NaN, N)
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
  expect_error(loss(resps, preds, weights, offset_2, dist))
  expect_error(loss(resps, preds, weights, offset_3, dist))
  expect_error(loss(resps, preds, weights, offset_4, dist))
})
test_that("Error thrown when baseline not vector of doubles", {
  # Given responses, weights, offset, predictions and default dist
  N <- 100
  dist <- gbm_dist()
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  
  # When baseline are not a vector of doubles
  baseline <- rep("1", N)
  baseline_2 <- NA
  baseline_3 <- Inf
  baseline_4 <- rep(NaN, N)
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist, baseline))
  expect_error(loss(resps, preds, weights, offset, dist, baseline_2))
  expect_error(loss(resps, preds, weights, offset, dist, baseline_3))
  expect_error(loss(resps, preds, weights, offset, dist, baseline_4))
})
test_that("Error thrown when distribution is not a GBMDist", {
  # Given responses, weights, offset, predictions and default dist
  N <- 100
  dist <- gbm_dist()
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  
  # When dist obj is not GBMDist
  class(dist) <- ""
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when length of responses, predictions, weights and baseline not the same", {
  # Given responses, weights, offset, predictions and a baselines
  # of varying lengths
  N <- 100
  N2 <- 102
  dist <- gbm_dist()
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  baseline <- runif(N)
  
  resps_2 <- runif(N2)
  weights_2 <- runif(N2)
  preds_2 <- runif(N2)
  offset_2 <- runif(N2)
  baseline_2 <- runif(N2)
  
  # Then error thrown when trying to calculate the loss if 
  # length of responses, predictions, weights and baselines not the same
  expect_error(loss(resps_2, preds, weights, offset, dist, baseline))
  expect_error(loss(resps, preds_2, weights, offset, dist, baseline))
  expect_error(loss(resps, preds, weights_2, offset, dist, baseline))
  expect_error(loss(resps, preds_2, weights, offset_2, dist, baseline))
  expect_error(loss(resps, preds, weights, offset, dist, baseline_2))
})
test_that("Error thrown when length of offset is not the same as predictions", {
  # Given responses, weights, predictions and default dist
  N <- 100
  dist <- gbm_dist()
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  
  # When offset has different number of elements to preds
  offset <- runif(N+1)
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when distribution object not recognised - default method", {
  # Given responses, weights, predictions and offset
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  
  # When distribution not recognised
  dist <- gbm_dist()
  class(dist) <- c("WeirdGBMDist", "GBMDist")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when distribution is CoxPH", {
  # Given responses, weights, predictions, offset and
  # CoxPH dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  dist <- gbm_dist("CoxPH")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when distribution is Gamma", {
  # Given responses, weights, predictions, offset and
  # Gamma dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  dist <- gbm_dist("Gamma")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when distribution is Huberized", {
  # Given responses, weights, predictions, offset and
  # Huberized dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  dist <- gbm_dist("Huberized")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when distribution is Quantile", {
  # Given responses, weights, predictions, offset and
  # Quantile dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  dist <- gbm_dist("Quantile")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when distribution is TDist", {
  # Given responses, weights, predictions, offset and
  # TDist dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  dist <- gbm_dist("TDist")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})
test_that("Error thrown when distribution is Tweedie", {
  # Given responses, weights, predictions, offset and
  # Tweedie dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  dist <- gbm_dist("Tweedie")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})

test_that("Error thrown if Pairwise but group_index is NULL", {
  # Given responses, weights, predictions, offset and
  # Pairwise dist but group_index not specified
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  dist <- gbm_dist("Pairwise", group="query")
  
  # Then error thrown when trying to calculate the loss
  expect_error(loss(resps, preds, weights, offset, dist))
})

context("Check loss calculation correct for various distributions")
test_that("Correctly calculates AdaBoost loss", {
  # Given responses, weights, predictions, offset, baseline and
  # AdaBoost dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  baseline <- runif(N)
  dist <- gbm_dist("AdaBoost")
  
  # When calculting loss
  calc_loss <- loss(resps, preds, weights, offset, dist, baseline)
  
  # Then it is correct
  preds <- preds + offset
  loss_true <- weighted.mean(exp(-(2*resps-1)*preds), weights) - baseline
  expect_equal(calc_loss, loss_true)
})
test_that("Correctly calculates Bernoulli loss", {
  # Given responses, weights, predictions, offset, baseline and
  # Bernoulli dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  baseline <- runif(N)
  dist <- gbm_dist("Bernoulli")
  
  # When calculting loss
  calc_loss <- loss(resps, preds, weights, offset, dist, baseline)
  
  # Then it is correct
  preds <- preds + offset
  loss_true <- -2*weighted.mean(resps*preds - log(1+exp(preds)), weights) - baseline
  expect_equal(calc_loss, loss_true)
})

test_that("Bernoulli loss is stable for large logits", {
  resps <- c(1, 0)
  preds <- c(1000, -1000)
  weights <- c(1, 1)
  offset <- c(0, 0)
  baseline <- c(0, 0)
  dist <- gbm_dist("Bernoulli")

  calc_loss <- loss(resps, preds, weights, offset, dist, baseline)

  expect_true(all(is.finite(calc_loss)))
  expect_equal(calc_loss, c(0, 0))
})
test_that("Correctly calculates Gaussian loss", {
  # Given responses, weights, predictions, offset, baseline and
  # Gaussian dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  baseline <- runif(N)
  dist <- gbm_dist("Gaussian")
  
  # When calculting loss
  calc_loss <- loss(resps, preds, weights, offset, dist, baseline)
  
  # Then it is correct
  preds <- preds + offset
  loss_true <- weighted.mean((resps - preds)^2, weights) - baseline
  expect_equal(calc_loss, loss_true)
})
test_that("Correctly calculates Laplace loss", {
  # Given responses, weights, predictions, offset, baseline and
  # Laplace dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  baseline <- runif(N)
  dist <- gbm_dist("Laplace")
  
  # When calculting loss
  calc_loss <- loss(resps, preds, weights, offset, dist, baseline)
  
  # Then it is correct
  preds <- preds + offset
  loss_true <- weighted.mean(abs(resps-preds), weights) - baseline
  expect_equal(calc_loss, loss_true)
})

test_that("Correctly calculates Pairwise loss - ndcg", {
  skip("Skipping pairwise")
  # Given responses, weights, predictions, offset, baseline and
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  params <- training_params(num_trees = 2000, num_train = nrow(data),
                            id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  baseline <- runif(length(fit$fit))
  
  # When calculting loss
  set.seed(1)
  calc_loss <- loss(Y, fit$fit, fit$gbm_data_obj$weights, fit$gbm_data_obj$offset, fit$distribution, baseline)
  
  # Then it is correct
  preds <- fit$fit + fit$gbm_data_obj$offset
  set.seed(1)
  loss_true <- (1 - perf_pairwise(Y, preds, fit$distribution$group_index, dist$metric, 
                                  fit$gbm_data_obj$weights, fit$distribution$max_rank)) - baseline
  expect_equal(calc_loss, loss_true) # tolerance for random tie breaking
})

test_that("Correctly calculates Pairwise loss - conc", {
  skip("Skipping pairwise")
  # Given responses, weights, predictions, offset, baseline and
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="conc", group="query")
  params <- training_params(num_trees = 2000, num_train = nrow(data), id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  baseline <- runif(length(fit$fit))
  
  # When calculting loss
  set.seed(1)
  calc_loss <- loss(Y, fit$fit, fit$gbm_data_obj$weights, fit$gbm_data_obj$offset, fit$distribution, baseline)
  
  # Then it is correct
  preds <- fit$fit + fit$gbm_data_obj$offset
  set.seed(1)
  loss_true <- (1 - perf_pairwise(Y, preds, fit$distribution$group_index, dist$metric, 
                                  fit$gbm_data_obj$weights, fit$distribution$max_rank)) - baseline
  expect_equal(calc_loss, loss_true) # tolerance for random tie breaking
})

test_that("Correctly calculates Pairwise loss - map", {
  skip("Skipping pairwise")
  # Given responses, weights, predictions, offset, baseline and
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  Y[Y >= 1] <- 1
  Y[Y < 1] <- 0
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="map", group="query")
  params <- training_params(num_trees = 2000, num_train = nrow(data),
                            id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  baseline <- runif(length(fit$fit))
  
  # When calculting loss
  set.seed(1)
  calc_loss <- loss(Y, fit$fit, fit$gbm_data_obj$weights, fit$gbm_data_obj$offset, fit$distribution, baseline)
  
  # Then it is correct
  preds <- fit$fit + fit$gbm_data_obj$offset
  set.seed(1)
  loss_true <- (1 - perf_pairwise(Y, preds, fit$distribution$group_index, dist$metric, 
                                  fit$gbm_data_obj$weights, fit$distribution$max_rank)) - baseline
  expect_equal(calc_loss, loss_true) # tolerance for random tie breaking
})

test_that("Correctly calculates Pairwise loss - mrr", {
  skip("Skipping pairwise")
  # Given responses, weights, predictions, offset, baseline and
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  Y[Y >= 1] <- 1
  Y[Y < 1] <- 0
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="mrr", group="query")
  params <- training_params(num_trees = 2000, num_train = nrow(data),
                            id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  baseline <- runif(length(fit$fit))
  
  # When calculting loss
  set.seed(1)
  calc_loss <- loss(Y, fit$fit, fit$gbm_data_obj$weights, fit$gbm_data_obj$offset, fit$distribution, baseline)
  
  # Then it is correct
  preds <- fit$fit + fit$gbm_data_obj$offset
  set.seed(1)
  loss_true <- (1 - perf_pairwise(Y, preds, fit$distribution$group_index, dist$metric, 
                                  fit$gbm_data_obj$weights, fit$distribution$max_rank)) - baseline
  expect_equal(calc_loss, loss_true) # tolerance for random tie breaking
})

test_that("Correctly calculates Poisson loss", {
  # Given responses, weights, predictions, offset, baseline and
  # AdaBoost dist
  N <- 100
  resps <- runif(N)
  weights <- runif(N)
  preds <- runif(N)
  offset <- runif(N)
  baseline <- runif(N)
  dist <- gbm_dist("Poisson")
  
  # When calculting loss
  calc_loss <- loss(resps, preds, weights, offset, dist, baseline)
  
  # Then it is correct
  preds <- preds + offset
  loss_true <- -2*weighted.mean(resps*preds-exp(preds), weights) - baseline
  expect_equal(calc_loss, loss_true)
})

context("miscellaneous tests - previous issues")
test_that("predicts correctly on unknown levels (issue #18)", {
  d <- data.frame(x=as.factor(1:20), y=1:20)
  
  train <- d[1:10,]
  test  <- d[11:20,]
  
  ## try this 10 times
  ## this isn't a deterministic bug
  params <- training_params(bag_fraction=1, num_trees = 1, min_num_obs_in_node = 1,
                            num_train = round(0.5 * nrow(train)), shrinkage=1, id=seq_along(train[,1]))
  for (ind in seq_len(10)) {
    g <- gbmt(y ~ x,
             data=train, 
             train_params=params
             )
    p <- predict(g, newdata=test, n.trees=1) - g$initF
    if (sum(abs(p)) != 0) break
  }
  
  expect_equal(sum(abs(p)), 0)
})

test_that("print.GBMFit works without cross-validation (issue #5)", {
  df <- data.frame(
    x = runif(100),
    y = runif(100),
    z = sample(0:1, 100, replace = TRUE)
  )
  
  trained_gbm <- gbmt_fit(df[, c("x", "y")], df$z)
  
  expect_error(print(trained_gbm), regexp=NA)
})


test_that("predicts correctly on unknown levels (issue #18) - old api", {
    d <- data.frame(x=as.factor(1:20), y=1:20)

    train <- d[1:10,]
    test  <- d[11:20,]

    ## try this 10 times
    ## this isn't a deterministic bug
    for (ind in seq_len(10)) {
        g <- gbm(y ~ x,
                 distribution="Gaussian",
                 bag.fraction=1,
                 data=train, 
                 n.trees=1,
                 shrinkage=1,
                 n.minobsinnode=1)
        p <- predict(g, newdata=test, n.trees=1) - g$initF
        if (sum(abs(p)) != 0) break
    }

    expect_equal(sum(abs(p)), 0)
})

test_that("print.GBMFit works without cross-validation (issue #5) - old api", {
    df <- data.frame(
        x = runif(100),
        y = runif(100),
        z = sample(0:1, 100, replace = TRUE)
    )

    trained_gbm <- gbm.fit(df[, c("x", "y")], df$z)
    
    expect_error(print(trained_gbm), regexp=NA)
})

####################
# Author: James Hickey
#
# Series of tests for perf_pairwise functionality
#
####################

context("Testing perf_pairwise")
test_that("Metric ndcg  fit passed to perf_pairwise runs", {
  skip("Skipping pairwise")
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  params <- training_params(num_trees = 200, num_train = nrow(data), id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
             data=data,     # dataset
             distribution=dist,
             train_params=params,
             keep_gbm_data=TRUE,      # store copy of input data in model
             cv_folds=5,          # number of cross validation folds
             is_verbose = FALSE ,    # don't print progress
             par_details=gbmParallel())  
  
  # When perf_pairwise is called
  # Then runs without error
  expect_error(perf_pairwise(Y, fit$fit, fit$distribution$group_index, 
                             fit$distribution$metric, max_rank=fit$distribution$max_rank), NA)
  
})

test_that("Metric map fit passed to perf_pairwise runs", {
  skip("Skipping pairwise")
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  Y[Y >= 1] <- 1
  Y[Y < 1] <- 0
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="map", group="query")
  params <- training_params(num_trees = 200, num_train = nrow(data),
                            id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  # When perf_pairwise is called
  # Then runs without error
  expect_error(perf_pairwise(Y, fit$fit, fit$distribution$group_index, 
                             fit$distribution$metric, max_rank=fit$distribution$max_rank), NA)
  
})

test_that("Metric mrr fit passed to perf_pairwise runs", {
  skip("Skipping pairwise")
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  Y[Y >= 1] <- 1
  Y[Y < 1] <- 0
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  params <- training_params(num_trees = 200, num_train = nrow(data), id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  # When perf_pairwise is called
  # Then runs without error
  expect_error(perf_pairwise(Y, fit$fit, fit$distribution$group_index, 
                             fit$distribution$metric, max_rank=fit$distribution$max_rank), NA)
  
})

test_that("Metric conc fit passed to perf_pairwise runs", {
  skip("Skipping pairwise")
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="conc", group="query")
  params <- training_params(num_trees = 200, num_train = nrow(data), id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  # When perf_pairwise is called
  # Then runs without error
  expect_error(perf_pairwise(Y, fit$fit, fit$distribution$group_index, 
                             fit$distribution$metric, max_rank=fit$distribution$max_rank), NA)
  
})

test_that("Error thrown if metric not recognised", {
  skip("Skipping pairwise")
  # Given a fitted Pairwise distribution
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="conc", group="query")
  params <- training_params(num_trees = 200, num_train = nrow(data),
                            id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  # When perf_pairwise is called but with an unrecognised metric
  # Then an error is thrown
  expect_error(perf_pairwise(Y, fit$fit, fit$distribution$group_index, 
                             "wrong", max_rank=fit$distribution$max_rank))
})

####################
# Author: James Hickey
#
# Series of tests to check other, less used, distributions can be run to 
# completion
#
####################

context("Testing runs of other distributions")

test_that("Can fit GBM with - AdaBoost", {
  # Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=3)
  
  # When GBM fit with AdaBoost
  dist <- gbm_dist("AdaBoost")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3, data=data,
                    distribution=dist, weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE),
               NA)
})

test_that("Can fit GBM with - Gamma", {
  # Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- abs(Y + rnorm(N,0,sigma))
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  # When GBM fit with Gamma
  dist <- gbm_dist("Gamma")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3+X4+X5+X6, data=data,
                    distribution=dist, weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE),
               NA)
})

test_that("Can fit GBM with - Huberized", {
  # Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p)
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=3)
  
  # When GBM fit with Huberized
  dist <- gbm_dist("Huberized")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3, data=data, distribution=dist,
                    weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE),
               NA)
})

test_that("Can fit GBM with - Laplace", {
  # Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  # When GBM fit with Laplace
  dist <- gbm_dist("Laplace")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
                    weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE),
               NA)
})

test_that("Can fit GBM with - Pairwise", {
  skip("Skipping pairwise")
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  Y_M <- Y
  Y_M[Y_M >= 1] <- 1
  Y_M[Y_M < 1] <- 0
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  data_M <- data.frame(Y_M, query=query, X1, X2, X3)
  
  params <- training_params(num_trees = 20, num_train = nrow(data),
                            id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  # When fitting all Pairwise distributions
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  dist_2 <- gbm_dist("Pairwise", metric="conc", group="query")
  dist_3 <- gbm_dist("Pairwise", metric="mrr", group="query")
  dist_4 <- gbm_dist("Pairwise", metric="map", group="query")

  # Then runs without error
  expect_error(gbmt(Y~X1+X2+X3,          
              data=data,    
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,     
              cv_folds=5,   
              is_verbose = FALSE ,    
              par_details=gbmParallel()), NA)
  expect_error(gbmt(Y~X1+X2+X3,          
                    data=data,    
                    distribution=dist_2,
                    train_params=params,
                    keep_gbm_data=TRUE,     
                    cv_folds=5,   
                    is_verbose = FALSE ,    
                    par_details=gbmParallel()), NA)
  expect_error(gbmt(Y_M~X1+X2+X3,          
                    data=data_M,    
                    distribution=dist_3,
                    train_params=params,
                    keep_gbm_data=TRUE,     
                    cv_folds=5,   
                    is_verbose = FALSE ,    
                    par_details=gbmParallel()), NA)
  expect_error(gbmt(Y_M~X1+X2+X3,          
                    data=data_M,    
                    distribution=dist_4,
                    train_params=params,
                    keep_gbm_data=TRUE,     
                    cv_folds=5,   
                    is_verbose = FALSE ,    
                    par_details=gbmParallel()), NA)
  
})

test_that("Can fit GBM with - Poisson", {
  # Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
  Y <- rbinom(N,1,p) + 1
  
  # random weights if you want to experiment with them
  w <- rexp(N)
  w <- N*w/sum(w)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)
  offset <- rep(0, N)
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2, num_features=3)
  
  # When GBM fit with Poisson
  dist <- gbm_dist("Poisson")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3, data=data, distribution=dist,
                    weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE),
               NA)
})

test_that("Can fit GBM with - Quantile", {
  ## Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  # When GBM fit with Quantile
  dist <- gbm_dist("Quantile")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
                    weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE),
               NA)
})

test_that("Can fit GBM with - TDist", {
  # Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  # When GBM fit with TDist
  dist <- gbm_dist("TDist")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
                    weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE),
               NA)
})

test_that("Can fit GBM with - Tweedie", {
  # Given appropriate data
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- abs(Y + rnorm(N,0,sigma))
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2,
                            num_features=6)
  # When GBM fit with Gamma
  dist <- gbm_dist("Tweedie")
  
  # Then no error is thrown
  expect_error(gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
                    weights=w, offset=offset,
                    train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE),
               NA)
})

####################
# Author: James Hickey
#
# Series of tests to check the relative_influence and 
#
####################

context("Testing relative_influence")
test_that("Error thrown if not GBMFit object", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5,
                            id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist,
              weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0),
              keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When class is not GBMFit
  class(fit) <- "wrong"
  
  # Then relative_influence throws an error
  expect_error(relative_influence(fit))
})
test_that("Error thrown if rescale not logical", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When rescale is not logical
  # Then relative_influence throws an error
  expect_error(relative_influence(fit, rescale = 1.5))
  expect_error(relative_influence(fit, rescale = NA))
  expect_error(relative_influence(fit, rescale = c(TRUE, FALSE)))
})
test_that("Error thrown if sort_it not logical", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When sort_it is not logical
  # Then relative_influence throws an error
  expect_error(relative_influence(fit, sort_it = 1.5))
  expect_error(relative_influence(fit, sort_it = NA))
  expect_error(relative_influence(fit, sort_it = c(TRUE, FALSE)))
})

test_that("Error thrown if num_trees exceeds number in fit", {
## Based on example in R package

## test Gaussian distribution gbm model
set.seed(1)

# create some data
N <- 1000
X1 <- runif(N)
X2 <- 2*runif(N)
X3 <- factor(sample(letters[1:4],N,replace=T))
X4 <- ordered(sample(letters[1:6],N,replace=T))
X5 <- factor(sample(letters[1:3],N,replace=T))
X6 <- 3*runif(N)
mu <- c(-1,0,1,2)[as.numeric(X3)]

SNR <- 10 # signal-to-noise ratio
Y <- X1**1.5 + 2 * (X2**.5) + mu
sigma <- sqrt(var(Y)/SNR)
Y <- Y + rnorm(N,0,sigma)

# create a bunch of missing values
X1[sample(1:N,size=100)] <- NA
X3[sample(1:N,size=300)] <- NA

w <- rep(1,N)
offset <- rep(0, N)
data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)


# Set up for new API
params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                          shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
dist <- gbm_dist("Gaussian")

fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
            train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)

# When num trees is too large
# Then relative_influence throws an error
expect_error(relative_influence(fit, num_trees=length(fit$trees)+100))
})
test_that("Message given if num_trees not provided", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When num_trees not given
  # Then message given
  expect_message(relative_influence(fit))
})
test_that("Relative influence can run with a specified num_trees", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When number of trees given
  # Then relative_influence can run
  expect_error(relative_influence(fit, num_trees = 100), NA)
})

test_that("num_trees set to gbm_perf with test method if train_fraction < 1 - missing num_trees", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When num_trees not given
  r1 <- relative_influence(fit)
  r2 <- relative_influence(fit,
                           num_trees=gbmt_performance(fit, method="test"))
  
  # Then relative influence num trees is given by test method in perf
  expect_equal(r1, r2)
})
test_that("num_trees set to gbm_perf with cv method if train_fraction = 1 and cv_error not null - missing num_trees", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model - TRAINFRACTION = 1
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When num_trees not given
  r1 <- relative_influence(fit)
  r2 <- relative_influence(fit,
                           num_trees=gbmt_performance(fit, method="cv"))
  
  # Then relative influence num trees given by cv method in perf
  expect_equal(r1, r2)
})
test_that("num_trees set to max number of trees in fit otherwise - missing num_trees", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model - TRAINFRACTION = 1 and CV = 1
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=1, is_verbose=FALSE)
  
  # When num_trees not given
  r1 <- relative_influence(fit)
  r2 <- relative_influence(fit, num_trees=length(fit$trees))
  
  # Then relative influence num trees is max 
  expect_equal(r1, r2)
})
test_that("can run with rescale and sorting", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model - TRAINFRACTION = 1 and CV = 1
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=1, is_verbose=FALSE)
  
  # When run with rescaling and sorting
  # Then no error is thrown
  expect_error(relative_influence(fit, rescale=TRUE, sort_it=TRUE), NA)
})

context("Testing permutation_relative_influence")
test_that("Error thrown if not GBMFit object", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When class is not GBMFit
  class(fit) <- "wrong"
  
  # Then permutation_relative_influence throws an error
  expect_error(permutation_relative_influence(fit, 100))
})

test_that("Error thrown if rescale not logical", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When rescale is not logical
  # Then relative_influence throws an error
  expect_error(permutation_relative_influence(fit, 100, rescale = 1.5))
  expect_error(permutation_relative_influence(fit, 100, rescale = NA))
  expect_error(permutation_relative_influence(fit, 100, rescale = c(TRUE, FALSE)))
})
test_that("Error thrown if sort_it not logical", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When sort_it is not logical
  # Then permutation_relative_influence throws an error
  expect_error(permutation_relative_influence(fit, 100, sort_it = 1.5))
  expect_error(permutation_relative_influence(fit, 100, sort_it = NA))
  expect_error(permutation_relative_influence(fit, 100, sort_it = c(TRUE, FALSE)))
})

test_that("Error thrown if num_trees exceeds number in fit", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When num trees is too large
  # Then relative_influence throws an error
  expect_error(permutation_relative_influence(fit, num_trees=length(fit$trees)+100))
})

test_that("Error thrown if num_trees not a positive integer", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When num_trees not a positive integer
  # Then error thrown
  expect_error(permutation_relative_influence(fit, num_trees = -1))
  expect_error(permutation_relative_influence(fit, num_trees = c(1.3, 4.5)))
  expect_error(permutation_relative_influence(fit, num_trees = TRUE))
  expect_error(permutation_relative_influence(fit, num_trees = NA))
})
test_that("Perm Relative influence can run with a specified num_trees", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When number of trees given
  # Then permutation_relative_influence can run
  expect_error(permutation_relative_influence(fit, num_trees = 100), NA)
})
test_that("can run with rescale and sorting", {
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model - TRAINFRACTION = 1 and CV = 1
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=1, is_verbose=FALSE)
  
  # When run with rescaling and sorting
  # Then no error is thrown
  expect_error(permutation_relative_influence(fit, length(fit$trees), rescale=TRUE, sort_it=TRUE), NA)
})

####################
# Author: James Hickey
#
# Series of tests to check prettifying of trees
#
####################

context("Testing input checking for prettifying")
test_that("error thrown when tree_index is not a positive integer", {
  # Given a fitted gbm object
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When calling pretty_gbm_tree on the object and the tree_index is not a positive integer
  # Then an error is thrown
  expect_error(pretty_gbm_tree(fit, tree_index=NA))
  expect_error(pretty_gbm_tree(fit, tree_index=c(1, 2)))
  expect_error(pretty_gbm_tree(fit, tree_index=0))
  expect_error(pretty_gbm_tree(fit, tree_index=NA))
  expect_error(pretty_gbm_tree(fit, tree_index=FALSE))
})
test_that("error thrown when tree_index exceeds the number of trees fitted", {
  # Given a fitted gbm object
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When calling pretty_gbm_tree on the object and the tree_index is larger than number of fitted trees
  tree_index <- params$num_trees + 1
  
  # Then an error is thrown
  expect_error(pretty_gbm_tree(fit, tree_index))
})

context("Test output of pretty_gbm_tree")
test_that("Tree is prettified correctly", {
  # Given a fitted gbm object and a sensible tree_index
  n_trees <- 200
  tree_index <- 100
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=n_trees, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When calling pretty_gbm_tree on the object and the tree_index 
  pretty_gbm_tree_tree <- pretty_gbm_tree(fit, tree_index)
  tree <- data.frame(fit$trees[[tree_index]])
  names(tree) <- c("SplitVar","SplitCodePred","LeftNode",
                   "RightNode","MissingNode","ErrorReduction",
                   "Weight","Prediction")
  row.names(tree) <- 0:(nrow(tree)-1)
  
  # Then correctly prettifies tree
  expect_equal(pretty_gbm_tree_tree, tree)
})
test_that("default prettified tree is the first one", {
  # Given a fitted gbm object and the default tree_index
  n_trees <- 200
  tree_index <- 1
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=n_trees, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When calling pretty_gbm_tree on the object and with 
  pretty_gbm_tree_tree <- pretty_gbm_tree(fit)
  tree <- data.frame(fit$trees[[1]])
  names(tree) <- c("SplitVar","SplitCodePred","LeftNode",
                   "RightNode","MissingNode","ErrorReduction",
                   "Weight","Prediction")
  row.names(tree) <- 0:(nrow(tree)-1)
  
  # Then correctly prettifies the first trees
  expect_equal(pretty_gbm_tree_tree, tree)
})

context("fold_id")

test_that("gaussian works in parallel - api", {
  skip_on_cran()
  skip_on_travis()
  
  ## Based on example in R package
  # Data generation from gbm manual ------------------------------------------
  set.seed(1)
  N     <- 1000
  X1    <- runif(N)
  X2    <- 2*runif(N)
  X3    <- ordered(sample(letters[1:4],N,replace=TRUE),levels=letters[4:1])
  X4    <- factor(sample(letters[1:6],N,replace=TRUE))
  X5    <- factor(sample(letters[1:3],N,replace=TRUE))
  X6    <- 3*runif(N)
  mu    <- c(-1,0,1,2)[as.numeric(X3)]
  SNR   <- 10
  Y     <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y     <- Y + rnorm(N,0,sigma)
  X1[sample(1:N,size=500)] <- NA
  X4[sample(1:N,size=300)] <- NA
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # Fit some models with fold.id ---------------------------------------------
  set.seed(1)
  folds <- sample(seq(1,3), nrow(data), replace = TRUE)
  
  set.seed(123456)
  gid1 <- gbmt(Y ~ X1 + X2 + X3 + X4 + X5 + X6,
               data              = data,
               distribution      = gbm_dist("Gaussian"),
               train_params = training_params(num_trees = 5, 
                                              interaction_depth=3, 
                                              bag_fraction = 1.0,
                                              id=seq(nrow(data)), 
                                              num_train=nrow(data), 
                                              shrinkage=0.01),
               fold_id           = folds)
  
  set.seed(260591)
  gid2 <- gbmt(Y ~ X1 + X2 + X3 + X4 + X5 + X6,
               data              = data,
               distribution      = gbm_dist("Gaussian"),
               train_params = training_params(num_trees = 5, 
                                              interaction_depth=3, 
                                              bag_fraction = 1.0,
                                              id=seq(nrow(data)), 
                                              num_train=nrow(data), 
                                              shrinkage=0.01),
               fold_id           = folds)
  
  
  expect_true(all(gid1$fold.id == gid2$fold.id))
})
test_that("gaussian works in parallel - old api", {
    skip_on_cran()
    skip_on_travis()
    
    ## Based on example in R package
    # Data generation from gbm manual ------------------------------------------
    set.seed(1)
    N     <- 1000
    X1    <- runif(N)
    X2    <- 2*runif(N)
    X3    <- ordered(sample(letters[1:4],N,replace=TRUE),levels=letters[4:1])
    X4    <- factor(sample(letters[1:6],N,replace=TRUE))
    X5    <- factor(sample(letters[1:3],N,replace=TRUE))
    X6    <- 3*runif(N)
    mu    <- c(-1,0,1,2)[as.numeric(X3)]
    SNR   <- 10
    Y     <- X1**1.5 + 2 * (X2**.5) + mu
    sigma <- sqrt(var(Y)/SNR)
    Y     <- Y + rnorm(N,0,sigma)
    X1[sample(1:N,size=500)] <- NA
    X4[sample(1:N,size=300)] <- NA
    data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
     
    # Fit some models with fold.id ---------------------------------------------
    set.seed(1)
    folds <- sample(seq(1,3), nrow(data), replace = TRUE)
     
    set.seed(123456)
    gid1 <- gbm(Y ~ X1 + X2 + X3 + X4 + X5 + X6,
                data              = data,
                distribution      = "Gaussian",
                n.trees           = 5,
                bag.fraction      = 1.0,
                shrinkage         = 0.01,
                interaction.depth = 3,
                fold.id           = folds)
     
    set.seed(260591)
    gid2 <- gbm(Y ~ X1 + X2 + X3 + X4 + X5 + X6,
                data              = data,
                distribution      = "Gaussian",
                n.trees           = 5,
                bag.fraction      = 1.0,
                shrinkage         = 0.01,
                interaction.depth = 3,
                fold.id           = folds)
     

    expect_true(all(gid1$fold.id == gid2$fold.id))
})

####################
# Author: James Hickey
#
# Series of test to validate conversion to old gbm class
#
####################

context("Testing conversion of GBMFit to gbm object")

test_that("to_old_gbm produces an object with the right class", {
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When converted to old form
  # Then has right class
  expect_equal(class(to_old_gbm(fit)), "gbm")
})
test_that("to_old_gbm output has correct fields", {
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When converted to old form
                                        # Then has right fields
  expected <-
      sort(c("initF", "train.error", "valid.error", "trees",
             "c.splits", "oobag.improve", "fit", "bag.fraction",
             "distribution", "interaction.depth", "n.minobsinnode",
             "n.trees", "nTrain", "nTrainPats", "patient.id",
             "mFeatures", "train.fraction", "response.name", "shrinkage",
             "var.levels", "var.monotone", "var.names", "var.type",
             "verbose",
             "strata", "sorted", "prior.node.coeff.var", "data",
             "cv.folds", "cv.error", "cv.fitted", "Terms",
             "call", "m"))
  
  expect_equal(sort(names(to_old_gbm(fit))), expected)
})

test_that("to_old_gbm gives correct message on calling", {
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When converted to old form
  # Then correct message given
  expect_message(to_old_gbm(fit), "Converted to old gbm object - this will not work with new packages functions")
})

test_that("to_old_gbm has data if keep_gbm_data was specified and it is correct", {
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When converted to old form
  # Then data stored is correct
  expect_equal(to_old_gbm(fit)$data$y, fit$gbm_data_obj$y)
  expect_equal(to_old_gbm(fit)$data$x, fit$gbm_data_obj$x)
  expect_equal(to_old_gbm(fit)$data$w, fit$gbm_data_obj$w)
  expect_equal(to_old_gbm(fit)$data$offset, fit$gbm_data_obj$offset)
  expect_true(is.na(to_old_gbm(fit)$data$Misc))
})

test_that("to_old_gbm has correct default prior.node.coeff.var if not CoxPH", {
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  # When to_old_gbm called
  # prior.node.coeff.var has correct default value
  expect_equal(to_old_gbm(fit)$prior.node.coeff.var, 1000)
})

test_that("to_old_gbm has correct data and time ordering for CoxPH fit", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)
  
  # Put into new API
  dist <- gbm_dist("CoxPH", prior_node_coeff_var=10)
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  
  # fit initial model
  gbm1 <- gbmt(Surv(tt,delta)~X1+X2+X3, data=data, distribution=dist, weights=w, offset=rep(0, N),
               train_params=params, var_monotone=c(0, 0, 0), keep_gbm_data=TRUE, cv_folds=5, is_verbose=FALSE)
  # When to old called
  # Then tied.times.method, prior.node.coeff.var and i.order is specified
  expect_equal(to_old_gbm(gbm1)$tied.times.method, "efron")
  expect_equal(to_old_gbm(gbm1)$data$i.order, gbm1$distribution$time_order)
  expect_equal(to_old_gbm(gbm1)$prior.node.coeff.var, 10)
})

test_that("to_old_gbm has group ordering for Pairwise fit", {
  skip("Skipping pairwise")
  # Given data and a fitted pairwise
  # create query groups, with an average size of 25 items each
  N <- 1000
  num.queries <- floor(N/25)
  query <- sample(1:num.queries, N, replace=TRUE)
  
  # X1 is a variable determined by query group only
  query.level <- runif(num.queries)
  X1 <- query.level[query]
  
  # X2 varies with each item
  X2 <- runif(N)
  
  # X3 is uncorrelated with target
  X3 <- runif(N)
  
  # The target
  Y <- X1 + X2
  
  # Add some random noise to X2 that is correlated with
  # queries, but uncorrelated with items
  
  X2 <- X2 + scale(runif(num.queries))[query]
  
  # Add some random noise to target
  SNR <- 5 # signal-to-noise ratio
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + runif(N, 0, sigma)
  
  data <- data.frame(Y, query=query, X1, X2, X3)
  dist <- gbm_dist("Pairwise", metric="ndcg", group="query")
  params <- training_params(num_trees = 20, num_train = nrow(data),
                            id=seq_len(nrow(data)),
                            interaction_depth = 3)
  
  fit <- gbmt(Y~X1+X2+X3,          # formula
              data=data,     # dataset
              distribution=dist,
              train_params=params,
              keep_gbm_data=TRUE,      # store copy of input data in model
              cv_folds=5,          # number of cross validation folds
              is_verbose = FALSE ,    # don't print progress
              par_details=gbmParallel())  
  
  # When fit converted to old form
  # Then it has ord.group field
  expect_equal(to_old_gbm(fit)$ord.group, fit$distribution$group_order)
})

##############
# Author: James Hickey
# 
# Series of tests to check compatibility with 1.6 objects is
# present.
#
##############

context("Testing num.classes compatibility")
test_that("num.classes field passed to output - gbm_more", {
  # Given a fit with num.classes set
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  fit$num.classes <- 2
  
  # When gbm_more called
  fit_more <- gbm_more(fit)
  
  # Then new object has same num.classes
  expect_equal(fit$num.classes, fit_more$num.classes)
  
})
test_that("num.classes field passed to output - to_old_gbm", {
  # Given a fit with num.classes set
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  fit$num.classes <- 2
  
  # When to_old_gbm api
  fit_old <- to_old_gbm(fit)
  
  # Then old gbm object to have num.classes
  expect_equal(fit$num.classes, fit_old$num.classes)
})
test_that("if num.classes > 1 then output of predict is a matrix", {
  # Given a fit with num.classes set
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  fit$num.classes <- 2
  
  # When predict called
  # Then predictions will be a matrix
  expect_true(is.matrix(predict(fit, data, 10)))
})

# Verify data
# 
# Functions used to determine whether the data input is valid - checks that
# the predictor and response vectors are of the right form
# 
# @usage verify-data(x, y)
# 
# @param x the matrix or data-frame containing the predictive variables
# 
# @param y the response vector
#
# @author James Hickey
#
# @return NA - Throws an error if data is of incorrect type
# 

verify_data <- function(x, y) {
  # Check predictor variables are either data-frame or matrix
  if(!is.matrix(x) && !is.data.frame(x)) {
    stop("Predictor variables, x, must be contained in a dataframe or matrix")
  }
  
  # Check same for y
  if(!is.matrix(y) && !is.data.frame(y) && !is.atomic(y)) {
    stop("Response, y, must be contained in a dataframe, matrix or vector")
  }
  
  # Check same length
  check_sanity(x, y)
  
  # Check variables are either numeric, ordered or factor
  check_var_type(x, y)
  
  # Check missing values are of correct form
  checkMissing(x, y)
  
}

# Ordering methods
# 
# Series of functions that are used to re-order the fitted gbm model,
# depending on what distribution was used in the initial call to \code{\link{gbmt_fit}}.
# Currently reordering only takes place for generalized boosted models produced
# using the CoxPH or Pairwise distributions.  These methods are called internally
# on completion of a call to \code{\link{gbmt_fit}}.
# 
# @usage reorder_fit(gbm_fit_obj_obj, distribution_obj)
# 
# @param gbm_fit_obj_obj a \code{GBMFit} object produced by a previous call to \code{gbmt} or \code{gbmt_fit}.
# 
# @param distributin_obj the \code{GBMDist} object used to produce the gbm_fit_obj_obj.
# 
# @return a \code{GBMFit} object with an appropriately order fit.
#  
# @author James Hickey
#

#### Fit reordering methods ####
reorder_fit <- function(gbm_fit_obj, distribution_obj) {
  check_if_gbm_fit(gbm_fit_obj)
  UseMethod("reorder_fit", distribution_obj)
}

#' @export
reorder_fit.default <- function(gbm_fit_obj, distribution_obj) {
  return(gbm_fit_obj)
}


#' @export
reorder_fit.CoxPHGBMDist <- function(gbm_fit_obj, distribution_obj) {
  gbm_fit_obj$fit[distribution_obj$time_order] <- gbm_fit_obj$fit
  return(gbm_fit_obj)
}

#' @export
reorder_fit.PairwiseGBMDist <- function(gbm_fit_obj, distribution_obj) {
  gbm_fit_obj$fit <- gbm_fit_obj$fit[order(distribution_obj$group_order)]
  return(gbm_fit_obj)
}

#### Data ordering - internal methods ####
order_data <- function(gbm_data_obj, distribution_obj, train_params) {
  gbm_data_obj <- order_by_groupings(gbm_data_obj, distribution_obj)
  gbm_data_obj <- order_by_id(gbm_data_obj, train_params)
  gbm_data_obj <- predictor_order(gbm_data_obj, train_params)
  return(gbm_data_obj)
}

order_by_id <- function(gbm_data_obj, train_params) {
  # Check if gbm_data_obj
  check_if_gbm_data(gbm_data_obj)
  gbm_data_obj$x <- gbm_data_obj$x[train_params$id_order, , drop=FALSE]
  gbm_data_obj$y <- gbm_data_obj$y[train_params$id_order]
  
  return(gbm_data_obj)
}

order_by_groupings <- function(gbm_data_obj, distribution_obj) {
  # Check if gbm_data_obj
  check_if_gbm_data(gbm_data_obj)
  
  # Check if GBMDist obj
  check_if_gbm_dist(distribution_obj)
  if(!is.null(distribution_obj$group_order)) {
    gbm_data_obj$y            <- gbm_data_obj$y[distribution_obj$group_order]
    gbm_data_obj$x            <- gbm_data_obj$x[distribution_obj$group_order,,drop=FALSE]
    gbm_data_obj$weights      <- gbm_data_obj$weights[distribution_obj$group_order]
  }
  return(gbm_data_obj)
}

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of test to check the data ordering functionality
#
####################

context("Test data ordering functionality")
test_that("Reordering method does nothing to fit unless CoxPH or Pairwise selected", {
  # Given mock gbm_fit
  N <- 1500
  gbm_fit <- structure(list("fit"=1:N), class="GBMFit")
  
  # When ordering according to any distribution other than CoxPH or Pairwise
  dist <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("Gamma")
  dist_4 <- gbm_dist("Gaussian")
  dist_5 <- gbm_dist("Huberized")
  dist_6 <- gbm_dist("Laplace")
  dist_7 <- gbm_dist("Poisson")
  dist_8 <- gbm_dist("Quantile")
  dist_9 <- gbm_dist("TDist")
  dist_10 <- gbm_dist("Tweedie")
  
  # Then fit order does not change
  expect_equal(reorder_fit(gbm_fit, dist), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_2), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_3), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_4), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_5), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_6), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_7), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_8), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_9), gbm_fit)
  expect_equal(reorder_fit(gbm_fit, dist_10), gbm_fit)
})
test_that("CoxPH- reorders fit according to time and strata", {
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset=rep(0, N))
  train_p <- training_params(id=c(rep(1, N/5), rep(2, N/5), rep(3, N/5), rep(4, N/5), rep(5, N/5)), num_train = 5, num_features = 3,
                             bag_fraction=1, min_num_obs_in_node=1)
  dist <- gbm_dist("CoxPH")
  
  # Reorder Fit
  dist <- create_strata(data, train_p, dist)
  gbm_fit <- structure(list("fit"=1:N), class="GBMFit")
  gbm_fit_ordered <- reorder_fit(gbm_fit, dist)
  
  # Expect order according to time_order
  gbm_fit$fit[dist$time_order] <- gbm_fit$fit
  expect_equal(gbm_fit_ordered$fit, gbm_fit$fit)
})

test_that("Pairwise - reorders fit according to group order", {
  skip("Skipping pairwise")
  # Given Pairwise data and fit
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Put into new API
  params <- training_params(num_trees=3000, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5,
                            id=seq_len(nrow(X)), num_train=N/2,
                            num_features=3)
  
  
  # Set up for new API
  dist <- gbm_dist("Pairwise")
  dist$group <- as.factor(sample(seq_len(5), N, replace=TRUE))
  dist$group_order <-order(dist$group, -Y)
  fit <- structure(list("fit"=1:N), class="GBMFit")
  
  # When fit is reordered
  reordered_fit <- reorder_fit(fit, dist)
  
  # Then correctly reorders fit
  expect_equal(reordered_fit$fit, fit$fit[order(dist$group_order)])
})

test_that("Order data requires a gbm_data object and distribution object", {
  # Given gbm_data and dist
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N/2)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=rep(1, N), num_train = N, num_features = 3)
  dist <- gbm_dist("CoxPH")
  
  # When classes is removed
  data_2 <- data
  dist_2 <- dist
  attr(data, "class") <- "NOTData"
  attr(dist, "class") <- "NOTDist"
  
  # Then error thrown when trying to order data
  expect_error(order_data(data, train_p, dist_2))
  expect_error(order_data(data_2, train_p, dist))
})
test_that("Can order data by id", {
  # Given GBM data and training parameters
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N/2)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=sample(1:5, N, replace=TRUE), num_train = 5, bag_fraction=1, min_num_obs_in_node = 1, num_features = 3)
  
  # When ordered by id
  data_copy <- data
  data <- order_by_id(data, train_p)
  
  # Then data is ordered by id
  expect_equal(data$x, data_copy$x[order(train_p$id), , drop=FALSE])
  expect_equal(data$y, data_copy$y[order(train_p$id)])
})

test_that("Can order data by groupings", {
  # Given pairwise data and dist
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Put into new API
  params <- training_params(num_trees=3000, interaction_depth=3,
                            min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5,
                            id=seq_len(nrow(X)), num_train=N/2, num_features=3)
  
  
  # Set up for new API
  dist <- gbm_dist("Pairwise")
  dist$group <- as.factor(sample(seq_len(5), N, replace=TRUE))
  dist$group_order <-order(dist$group, -Y)
  
  
  # When order_by_groupings is called
  data_ordered <- order_by_groupings(data, dist)
  
  # Then correctly orders by groupings
  expect_equal(data_ordered$x, data$x[dist$group_order,,drop=FALSE])
  expect_equal(data_ordered$y, data$y[dist$group_order])
  expect_equal(data_ordered$weights, data$weights[dist$group_order])
  
})

test_that("Ordering by groupings does nothing if not Pairwise", {
  # Given data and not pairwise (using CoxPH data for example)
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N/2)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=sample(1:5, N, replace=TRUE),
                             num_train = 5, bag_fraction=1,
                             min_num_obs_in_node = 1,
                             num_features = 3)
  
  # When ordered by grouping - NOT Pairwise dist
  # Then nothing changes
  expect_equal(data, order_by_groupings(data, gbm_dist("AdaBoost")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Bernoulli")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Gamma")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Gaussian")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Huberized")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Laplace")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Poisson")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Quantile")))
  expect_equal(data, order_by_groupings(data, gbm_dist("TDist")))
  expect_equal(data, order_by_groupings(data, gbm_dist("Tweedie")))
})

test_that("Can order by predictors - check predictor_order", {
  # Given gbm data and training parameters
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  offset <- rep(0, N/2)
  
  Resp <- Surv(tt, delta)
  data <- gbm_data(data.frame(X1, X2, X3), Resp, w, offset)
  train_p <- training_params(id=sample(1:5, N, replace=TRUE),
                             num_train = 5, bag_fraction = 1,
                             min_num_obs_in_node = 1, num_features = 3)
  
  # Then can order by predictors
  data <- predictor_order(data, train_p)
  expect_equal(data$x_order,
               apply(data$x[seq_len(train_p$num_train_rows),,drop=FALSE],
                     2, order,na.last=FALSE)-1)
})

# Validate data
# 
# Validates that the data is consistent with the distribution selected.
# 
# @usage validata_gbm_data(gbm_data_obj, distribution_obj)
# 
# @param gbm_data_obj A GBMData object created using the function "gbm_data".
# 
# @param distribution_obj A GBMDist object storing all the parameters which define the model
# distribution selected.
# 
# @author James Hickey
#
# @return a validated gbm_data_obj
# 

validate_gbm_data <- function(gbm_data_obj, distribution_obj) {
  check_if_gbm_data(gbm_data_obj)
  check_if_gbm_dist(distribution_obj)
  
  # Normalise Weights
  if(!(distribution_name(distribution_obj) %in% available_distributions())) {
    stop("Distribution not recognised - see available_distributions for
           supported distributions")
  }

  if(distribution_name(distribution_obj) != "Pairwise") {
    gbm_data_obj$weights <- gbm_data_obj$weights / mean(gbm_data_obj$weights)
  } 
  
  # Check offset
  gbm_data_obj$offset <- check_offset(gbm_data_obj$offset, gbm_data_obj$y, distribution_obj)
  
  # Check responses
  check_response_values(distribution_obj, gbm_data_obj$y)
  
  return(gbm_data_obj)
}

# Function that updates the number of training rows and observations
# based off of groups
# @author James Hickey

update_num_train_groups <- function(train_params, dist_obj) {
  check_if_gbm_train_params(train_params)
  check_if_gbm_dist(dist_obj)
  UseMethod("update_num_train_groups", dist_obj)
}

#' @export
update_num_train_groups.default <- function(train_params, dist_obj) {
  return(train_params)
}

#' @export
update_num_train_groups.PairwiseGBMDist <- function(train_params, dist_obj) {
  # Check if more than one row per observation
  if(any(duplicated(train_params$id))) {
    stop("Pairwise does not currently support multiple rows per observation.")
  }
  
  # Calculate number of training groups
  num_train_groups <- max(1, round(train_params$train_fraction * nlevels(dist_obj$group)))
  
  # Get number of training rows - include those up to max group
  train_params$num_train_rows <- max(which(dist_obj$group == levels(dist_obj$group)[num_train_groups]))
  train_params$num_train <- train_params$num_train_rows
  
  return(train_params)
}

####################
# Author: James Hickey
#
# Series of tests to check if how training params are updated if groups are present
#
####################

context("Input checking update_num_train_groups")
test_that("update_num_train_groups throws an error if not given a GBMTrainParams object", {
  # Given a default train params and dist object
  params <- training_params()
  dist <- gbm_dist()
  
  # When training params stripped of class and update_num_train_groups called
  class(params) <- "wrong"
  
  # Then error thrown
  expect_error(update_num_train_groups(params, dist))
})
test_that("update_num_train_groups throws an error if not given a GBMDist object", {
  # Given a default train params and dist object
  params <- training_params()
  dist <- gbm_dist()
  
  # When dist is stripped of class and update_num_train_groups called
  class(params) <- "wrong"
  
  # Then error thrown
  expect_error(update_num_train_groups(params, dist))
})

context("Testing update_num_train_groups behaviour")
test_that("update_num_train_groups returns original params object when given an AdaBoostGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("AdaBoost")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an BernoulliGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("Bernoulli")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an CoxPHGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("CoxPH")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an GammaGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("Gamma")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an GaussianGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("Gaussian")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an HuberizedGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("Huberized")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an LaplaceGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("Laplace")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an PoissonGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("Poisson")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an QuantileGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("Quantile")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an TDistGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("TDist")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups returns original params object when given an TweedieGBMDist obj", {
  # Given some training parameters and a default dist
  params <- training_params()
  dist <- gbm_dist("AdaBoost")
  
  # When training parameters are updated using update_num_train_groups
  # Then no update happens
  expect_equal(update_num_train_groups(params, dist), params)
})
test_that("update_num_train_groups throws error if training_params id indicates multiple rows per observation - Pairwise", {
  # Given a training params object and an appropriately set-up pairwise object
  # such that observations have multiple rows
  N <- 100
  dist <- gbm_dist("Pairwise")
  params <- training_params(num_train=N, id=sample(seq_len(N/5), N, replace=TRUE))
  
  # When attempt to update_num_train_groups
  # Then error is thrown
  expect_error(update_num_train_groups(params, dist))
})
test_that("update_num_train_groups returns correctly updated training_params - Pairwise", {
  # Given some training parameters object and a Pairwise object with groups
  N <- 100
  params <- training_params(num_train=N/2, id=seq_len(N))
  dist <- gbm_dist("Pairwise")
  dist$groups <- sample(seq_len(5), N, replace=TRUE)
  
  # When update_num_train_groups
  params_update <- update_num_train_groups(params, dist)
  
  # Then correctly calculates update
  num_train_groups <- max(1, round(params$train_fraction * nlevels(dist$group)))
  params$num_train_rows <- max(which(dist$group == levels(dist$group)[num_train_groups]))
  params$num_train <- params$num_train_rows
  expect_equal(params_update, params)
})

#' Print gbm tree components
#' 
#' \code{GBMFit} stores the collection of trees used to construct the model in a
#' compact matrix structure. This function extracts the information from a
#' single tree and displays it in a slightly more readable form. This function
#' is mostly for debugging purposes and to satisfy some users' curiosity.
#' 
#' @param gbm_fit_obj a \code{GBMFit} object initially fit using
#' \code{\link{gbmt}}.
#' 
#' @param tree_index the index of the tree component to extract from \code{gbm_fit_obj}
#' and display.
#' 
#' @return \code{pretty_gbm_tree} returns a data frame. Each row corresponds to
#' a node in the tree. Columns indicate \item{SplitVar}{index of which variable
#' is used to split. -1 indicates a terminal node.} \item{SplitCodePred}{if the
#' split variable is continuous then this component is the split point. If the
#' split variable is categorical then this component contains the index of
#' \code{gbm_fit_obj$c.split} that describes the categorical split. If the node is a
#' terminal node then this is the prediction.} \item{LeftNode}{the index of the
#' row corresponding to the left node.} \item{RightNode}{the index of the row
#' corresponding to the right node.} \item{ErrorReduction}{the reduction in the
#' loss function as a result of splitting this node.} \item{Weight}{the total
#' weight of observations in the node. If weights are all equal to 1 then this
#' is the number of observations in the node.}
#' 
#' @author James Hickey, Greg Ridgeway \email{gregridgeway@@gmail.com}
#' @seealso \code{\link{gbm}}, \code{\link{gbmt}}
#' @keywords print
#' @export 
#' 
pretty_gbm_tree <- function(gbm_fit_obj, tree_index=1)
{
  # Initial checks
  check_if_natural_number(tree_index)
  if(tree_index >length(trees(gbm_fit_obj))) {
      stop("tree_index is out of range. Must be less than ",
           length(trees(gbm_fit_obj)))
  }
  
  # Convert selected tree to data.frame
  tree <- data.frame(trees(gbm_fit_obj)[[tree_index]])
  names(tree) <- c("SplitVar","SplitCodePred","LeftNode",
                   "RightNode","MissingNode","ErrorReduction",
                   "Weight","Prediction")
  row.names(tree) <- 0:(nrow(tree)-1)
  return(tree)
}

# GBM CV errors
# 
# Calculate cross validation errors
# 
# @usage gbm_cv_errors(gbm_cv_fit, cv_folds, cv_group)
# 
# @param gbm_cv_fit a GBMCVFit object containing CV gbm models
# 
# 
# @param cv_folds a positive integer specifying the number of folds to be used in cross-validation of the gbm
# fit.
# 
# @param cv_group vector of integers specifying which row of data belongs to which cv_fold.
#
# @author James Hickey
#
# @return vector of cross validated model errors
#

gbm_cv_errors <- function(gbm_cv_fit, cv_folds, cv_group) {
  UseMethod("gbm_cv_errors", gbm_cv_fit)
}

#' @export
gbm_cv_errors.GBMCVFit <- function(gbm_cv_fit, cv_folds, cv_group) {
  in_group <- tabulate(cv_group, nbins=cv_folds)
  cv_error <- vapply(1:cv_folds,
                     function(index) {
                       model <- gbm_cv_fit[[index+1]]
                       model$valid.error * in_group[[index]]
                     }, double(gbm_cv_fit[[1]]$params$num_trees))
  ## this is now a (num_trees, cv_folds) matrix
  ## and now a n.trees vector
  return(rowSums(cv_error) / gbm_cv_fit[[1]]$params$num_train)
}

#' Convert GBMFit to previous gbm object
#'
#' Function that takes a \code{GBMFit} object produced from a call
#' to \code{\link{gbmt}} and converts it to the form of a gbm object from
#' Version 2.1 of the package.
#'
#' @usage  to_old_gbm(gbm_fit_obj)
#'
#' @param gbm_fit_obj a \code{GBMFit} object produced by a call to \code{\link{gbmt}}.
#'
#' @return a \code{gbm} object of the form from Version 2.1 of the package.
#'
#' @author James Hickey
#'
#' @export

to_old_gbm <- function(gbm_fit_obj) {
  # Check input
  check_if_gbm_fit(gbm_fit_obj)

  # Convert gbm_fit_obj to old API
  gbm_fit_old <- list()
  gbm_fit_old$initF <- gbm_fit_obj$initF
  gbm_fit_old$train.error <- iteration_error(gbm_fit_obj, 'train')
  gbm_fit_old$valid.error <- iteration_error(gbm_fit_obj, 'valid')
  gbm_fit_old$trees <- trees(gbm_fit_obj)
  gbm_fit_old$c.splits <- gbm_fit_obj$c.splits
  gbm_fit_old$oobag.improve <- gbm_fit_obj$oobag.improve
  gbm_fit_old$fit <- gbm_fit_obj$fit

  gbm_fit_old$bag.fraction <- gbm_fit_obj$params$bag_fraction
  gbm_fit_old$distribution <- list(name=tolower(distribution_name(gbm_fit_obj)))
  gbm_fit_old$interaction.depth <- gbm_fit_obj$params$interaction_depth
  gbm_fit_old$n.minobsinnode <- gbm_fit_obj$params$min_num_obs_in_node
  gbm_fit_old$n.trees <- length(gbm_fit_old$trees)
  gbm_fit_old$nTrain <- gbm_fit_obj$params$num_train_rows
  gbm_fit_old$nTrainPats <- gbm_fit_obj$params$num_train
  gbm_fit_old$patient.id <- gbm_fit_obj$params$id
  gbm_fit_old$mFeatures <- gbm_fit_obj$params$num_features
  gbm_fit_old$train.fraction <- gbm_fit_obj$params$train_fraction
  gbm_fit_old$response.name <- gbm_fit_obj$response_name
  gbm_fit_old$shrinkage <- gbm_fit_obj$params$shrinkage
  gbm_fit_old$var.levels <- gbm_fit_obj$variables$var_levels
  gbm_fit_old$var.monotone <- gbm_fit_obj$variables$var_monotone
  gbm_fit_old$var.names <- gbm_fit_obj$variables$var_names
  gbm_fit_old$var.type <- gbm_fit_obj$variables$var_type
  gbm_fit_old$verbose <- gbm_fit_obj$is_verbose
  gbm_fit_old$strata <- gbm_fit_obj$distribution$strata
  gbm_fit_old$sorted <- gbm_fit_obj$distribution$sorted
  gbm_fit_old$prior.node.coeff.var <- ifelse(is.null(gbm_fit_obj$distribution$prior_node_coeff_var), 1000,
                                         gbm_fit_obj$distribution$prior_node_coeff_var)

  if(!is.null(gbm_fit_obj$gbm_data_obj)) {
    # put the observations back in - these are ordered according to id and group
    data <- gbm_fit_obj$gbm_data_obj
    if(distribution_name(gbm_fit_obj) == "CoxPH") {
      gbm_fit_old$data <- list(y=data$y, x=data$x,x.order=data$x_order, offset=data$offset,
                               Misc=unlist(get_misc(gbm_fit_obj$distribution)), w=data$weights,
                               i.order=gbm_fit_obj$distribution$time_order)
    } else {
      gbm_fit_old$data <- list(y=data$y, x=data$x,x.order=data$x_order, offset=data$offset,
                               Misc=unlist(get_misc(gbm_fit_obj$distribution)), w=data$weights)
    }

  } else {
    gbm_fit_old$data <- NULL
  }
  gbm_fit_old$tied.times.methods <- gbm_fit_obj$distribution$ties
  gbm_fit_old$ord.group <- gbm_fit_obj$distribution$group_order
  gbm_fit_old$cv.folds <- gbm_fit_obj$cv_folds
  gbm_fit_old$cv.error <- iteration_error(gbm_fit_obj, 'cv')
  gbm_fit_old$cv.fitted <- gbm_fit_obj$cv_fitted
  gbm_fit_old$Terms <- gbm_fit_obj$Terms
  gbm_fit_old$call <- gbm_fit_obj$call
  gbm_fit_old$m <- gbm_fit_obj$m
  gbm_fit_old$num.classes <- gbm_fit_obj$num.classes


  class(gbm_fit_old) <- "gbm"

  message("Converted to old gbm object - this will not work with new packages functions")

  return(gbm_fit_old)

}

# Variable Container
# 
# Contains properties associated with predictor variables.
# 
# 
# @usage var_container(gbm_data_obj,  var_monotone=NULL, var_names=NULL)
# 
# @param gbm_data_obj an initialized GBMData object for the fit
#
# @param var_monotone  an optional vector, the same length as the
# number of predictors, indicating which variables have a monotone
# increasing (+1), decreasing (-1), or arbitrary (0) relationship
# with the outcome.
# 
# @param var_names a vector of strings of the same length as the 
# 1st dimension of the response.
# 
# @return a \code{GBMVarCont} object
# 
# @export 
# 

var_container <- function(gbm_data_obj, var_monotone=NULL, var_names=NULL) {
  check_if_gbm_data(gbm_data_obj)
  
  # Check var_monotone
  cCols <- ncol(gbm_data_obj$x)
  if(is.null(var_monotone)) {
    var_monotone <- rep(0, cCols)
  } else if(length(var_monotone)!=cCols) {
    stop("Length of var_monotone != number of predictors")
  } else if(!all(is.element(var_monotone,-1:1))) {
    stop("var_monotone must be -1, 0, or 1")
  }
  
  # Check names and get if necessary
  if(is.null(var_names)) var_names <- get_var_names(gbm_data_obj$x)
  if(!is.null(var_names) && (!is.atomic(var_names) || any(var_names != as.character(var_names))
     || is.null(var_names)) ){
    stop("Names of data must be a vector of strings.")
  }
  if(!is.null(var_names) && length(var_names)!=cCols) stop("Length of var_names != number of predictors")
  
  # setup variable types
  var_type <- rep(0, cCols)
  var_levels <- vector("list", cCols)
  
  for(i in seq_len(cCols)) {
    if(is.ordered(gbm_data_obj$x[,i])) {
      var_levels[[i]] <- levels(factor(gbm_data_obj$x[,i]))
      var_type[i] <- 0
      
    } else if(is.factor(gbm_data_obj$x[,i])) {
      var_levels[[i]] <- levels(factor(gbm_data_obj$x[,i]))
      temp_x <- as.numeric(factor(gbm_data_obj$x[,i]))-1
      var_type[i] <- max(temp_x, na.rm=TRUE)+1
      
    } else if(is.numeric(gbm_data_obj$x[,i])) {
      var_levels[[i]] <- quantile(gbm_data_obj$x[,i],prob=(0:10)/10,na.rm=TRUE)
    }
  }
  
  return(structure(list(var_monotone=var_monotone, var_names=var_names,
                        var_levels=var_levels, var_type=var_type),
                   class = "GBMVarCont"))
}

#' Performance for pairwise
#' 
#' Additional performance using appropriate metric for pairwise
#' distribution fit.
#' 
#' @param y responses used for fit.
#' 
#' @param f the predicted responses.
#' 
#' @param group the indices specifiying which response variables
#' belong to which groups.
#' 
#' @param metric What type of performance measure to compute in
#' \code{perf_pairwise}.  Can take values "ir_measure_conc",
#' "ir_measure_mrr", "ir_measure_map" or "ir_measure_ndgc".
#'
#' @param w is the weights of each observation.
#' 
#' @param max_rank the maximum rank considered in the groups measure. 
#' 
#' @return returns a numeric value of the selected performance metric
#' 
#' @author Greg Ridgeway \email{gregridgeway@@gmail.com}
#' 
#' @seealso \code{\link{gbmt}}
#' @keywords nonlinear survival nonparametric tree
#' @export
perf_pairwise <- function(y, f, group, metric="ndcg", w=NULL, max_rank=0){
  func.name <- switch(metric,
                      conc = "ir_measure_conc",
                      mrr  = "ir_measure_mrr",
                      map  = "ir_measure_map",
                      ndcg = "ir_measure_ndcg",
                      stop(paste("Metric",metric,"is not supported"))
  )
  
  # Optimization: for binary targets,
  # AUC is equivalent but faster than CONC
  if (metric == "conc" && all(is.element(y, 0:1))) {
    func.name <- "ir_measure_auc"
  }
  
  # Max rank = 0 means no cut off
  if (max_rank <= 0) {
    max_rank <- length(y)+1
  }
  
  # Random tie breaking in case of duplicate scores.
  # (Without tie breaking, we would overestimate if instances are
  # sorted descending on target)
  f <- f + 1E-10 * runif(length(f), min=-0.5, max=0.5)
  
  measure.by.group <- as.matrix(by(list(y, f), INDICES=group, FUN=get(func.name), max_rank=max_rank))
  
  # Exclude groups with single result or only negative or positive instances
  idx <- which((!is.null(measure.by.group)) & measure.by.group >= 0)
  
  if (is.null(w)) {
    return (mean(measure.by.group[idx]))
  } else {
    # Assumption: weights are constant per group
    w.by.group <- tapply(w, group, mean)
    return (weighted.mean(measure.by.group[idx], w=w.by.group[idx]))
  }
}


#### Helper Functions ####
ir_measure_conc <- function(y.f, max_rank=0) {
  # Note: max_rank is meaningless for CONC
  
  y           <- y.f[[1]]
  f           <- y.f[[2]]
  
  tab         <- table(y)
  csum        <- cumsum(tab)
  total.pairs <- sum(tab * (csum - tab))
  
  if (total.pairs == 0) {
    return (-1.0)
  } else {
    return (gbm_conc(y[order(-f)]) / total.pairs)
  }
}

ir_measure_auc <- function(y.f, max_rank=0){
  # Note: max_rank is meaningless for AUC
  y       <- y.f[[1]]
  f       <- y.f[[2]]
  num.pos <- sum(y>0)
  
  if (length(f) <= 1 || num.pos == 0 || num.pos == length(f))
  {
    return (-1.0)
  }
  else
  {
    return (gbm_roc_area(obs=y, pred=f))
  }
}

ir_measure_mrr <- function(y.f, max_rank) {
  y       <- y.f[[1]]
  f       <- y.f[[2]]
  num.pos <- sum(y>0)
  
  if (length(f) <= 1 || num.pos == 0 || num.pos == length(f))
  {
    return (-1.0)
  }
  
  ord         <- order(f, decreasing=TRUE)
  min.idx.pos <- min(which(y[ord]>0))
  
  if (min.idx.pos <= max_rank)
  {
    return (1.0 / min.idx.pos)
  }
  else
  {
    return (0.0)
  }
}

ir_measure_map <- function(y.f, max_rank=0) {
  # Note: max_rank is meaningless for MAP
  
  y         <- y.f[[1]]
  f         <- y.f[[2]]
  ord       <- order(f, decreasing=TRUE)
  idx.pos   <- which(y[ord]>0)
  num.pos   <- length(idx.pos)
  
  if (length(f) <= 1 || num.pos == 0 || num.pos == length(f))
  {
    return (-1.0)
  }
  
  # Above and including the rank of the i-th positive result,
  # there are exactly i positives and rank(i) total results
  return (sum((1:length(idx.pos))/idx.pos) / num.pos)
}

ir_measure_ndcg <- function(y.f, max_rank) {
  y         <- y.f[[1]]
  f         <- y.f[[2]]
  
  if (length(f) <= 1 || all(diff(y)==0)) return (-1.0)
  
  num.items <- min(length(f), max_rank)
  ord       <- order(f, decreasing=TRUE)
  
  dcg       <- sum(y[ord][1:num.items] / log2(2:(num.items+1)))
  
  # The best possible DCG: order by target
  ord.max   <- order(y, decreasing=TRUE)
  dcg.max   <- sum(y[ord.max][1:num.items] / log2(2:(num.items+1)))
  
  # Normalize
  return (dcg / dcg.max)
}

# Series of helper functions for plot.GBMFit

#### One variable helpers ####
get_ylabel_one_var <- function(dist_obj) {
  UseMethod("get_ylabel_one_var", dist_obj)
}

#' @export
get_ylabel_one_var.default <- function(dist_obj) {
  return("")
}

#' @export
get_ylabel_one_var.BernoulliGBMDist <- function(dist_obj) {
  return("Predicted Probability")
}

#' @export
get_ylabel_one_var.PairwiseGBMDist <- function(dist_obj) {
  return("Predicted Probability")
}

#' @export
get_ylabel_one_var.PoissonGBMDist <- function(dist_obj) {
  return("Predicted Count")
}

#### Two variable helpers ####
select_two_var_plot <- function(f.factor, X, gbm_fit_obj, var_index, ...) {
  # Set plot identified
  if(!any(f.factor)) {
    which_plot <- 1
  } else {
    which_plot <- sum(f.factor) + max(which(f.factor==TRUE))
    which_plot <- ifelse(length(which_plot)==0, 1, which_plot)
  }
  which_plot <- toString(which_plot)
  
  # Call
  switch(which_plot,
         "1"=two_var_plot_no_factor(X, gbm_fit_obj, var_index, ...),
         "2"=two_var_plot_first_factor(X, gbm_fit_obj, var_index, ...),
         "3"=two_var_plot_second_factor(X, gbm_fit_obj, var_index, ...),
         "4"=two_var_plot_both_factor(X, gbm_fit_obj, var_index, ...))
}

two_var_plot_no_factor <- function(X, gbm_fit_obj, var_index, ...) {
  print(levelplot(y~X1*X2,data=X,
                  xlab=gbm_fit_obj$variables$var_names[var_index[1]],
                  ylab=gbm_fit_obj$variables$var_names[var_index[2]],...))
}

two_var_plot_first_factor <- function(X, gbm_fit_obj, var_index, ...) {
  print(xyplot(y~X2|X1,data=X,
               xlab=gbm_fit_obj$variables$var_names[var_index[2]],
               ylab=paste("f(", gbm_fit_obj$variables$var_names[var_index[1]],",", gbm_fit_obj$variables$var_names[var_index[2]],")",sep=""),
               type="l",
               panel = panel.xyplot,
               ...))
}

two_var_plot_second_factor <- function(X, gbm_fit_obj, var_index, ...) {
  print(xyplot(y~X1|X2,data=X,
               xlab=gbm_fit_obj$variables$var_names[var_index[1]],
               ylab=paste("f(",gbm_fit_obj$variables$var_names[var_index[1]],",",gbm_fit_obj$variables$var_names[var_index[2]],")",sep=""),
               type="l",
               panel = panel.xyplot,
               ...))
}

two_var_plot_both_factor <- function(X, gbm_fit_obj, var_index, ...) {
  print(stripplot(X1~y|X2,data=X,
                  xlab=gbm_fit_obj$variables$var_names[var_index[2]],
                  ylab=paste("f(",gbm_fit_obj$variables$var_names[var_index[1]],",",gbm_fit_obj$variables$var_names[var_index[2]],")",sep=""),
                  ...))
}

#### Three variable helpers ####
select_three_var_plot <- function(f.factor, X, gbm_fit_obj, var_index, ...) {
  which_plot <- toString(sum(f.factor))
  
  i <- order(f.factor)
  X.new <- X[,i]
  X.new$y <- X$y
  names(X.new) <- names(X)
  
  switch(which_plot,
         "0"=three_var_plot_no_factor(X.new, gbm_fit_obj, var_index, i,  ...),
         "1"=three_var_plot_one_factor(X.new, gbm_fit_obj, var_index, i, ...),
         "2"=three_var_plot_two_factor(X.new, gbm_fit_obj, var_index, i, ...),
         "3"=three_var_plot_three_factor(X.new, gbm_fit_obj, var_index, i, ...))
}

three_var_plot_no_factor <- function(X, gbm_fit_obj, var_index, select_index, ...) {

  X$X3 <- equal.count(X$X3)
  print(levelplot(y~X1*X2|X3,data=X,
                  xlab=gbm_fit_obj$variables$var_names[var_index[select_index[1]]],
                  ylab=gbm_fit_obj$variables$var_names[var_index[select_index[2]]],...))
  
}

three_var_plot_one_factor <- function(X, gbm_fit_obj, var_index, select_index, ...) {
  print(levelplot(y~X1*X2|X3,data=X,
                  xlab=gbm_fit_obj$variables$var_names[var_index[select_index[1]]],
                  ylab=gbm_fit_obj$variables$var_names[var_index[select_index[2]]],...))
}

three_var_plot_two_factor <- function(X, gbm_fit_obj, var_index, select_index, ...) {
  print(xyplot(y~X1|X2*X3,data=X,
               type="l",
               xlab=gbm_fit_obj$variables$var_names[var_index[select_index[1]]],
               ylab=paste("f(",paste(gbm_fit_obj$variables$var_names[var_index[1:3]],collapse=","),")",sep=""),
               panel = panel.xyplot,
               ...))
}

three_var_plot_three_factor <- function(X, gbm_fit_obj, var_index, select_index, ...) {
  print(stripplot(X1~y|X2*X3,data=X,
                  xlab=gbm_fit_obj$variables$var_names[var_index[select_index[1]]],
                  ylab=paste("f(",paste(gbm_fit_obj$variables$var_names[var_index[1:3]],collapse=","),")",sep=""),
                  ...))
}

# Series of internal functions used 
# to check inputs and convert parameters

assertInherits <- function(object, class.name) {
    if (!isTRUE(inherits(object, class.name))) {
        stop("Function requires a ", class.name, " object.")
    }
}

##### Check gbm objects #####
check_if_gbm_dist <- function(distribution_obj) {
  # Check if GBM dist object
  if(!any(class(distribution_obj) %in% paste0(available_distributions(), "GBMDist"))) {
    stop("Function requires a specific GBMDist object.")
  }
}

check_if_gbm_data <- function(data_obj) {
    assertInherits(data_obj, "GBMData")
}

check_if_gbm_fit <- function(fit_obj) {
    assertInherits(fit_obj, "GBMFit")
}

check_if_gbm_train_params <- function(params_obj) {
    assertInherits(params_obj, "GBMTrainParams")
}

check_if_gbm_var_container <- function(var_obj) {
    assertInherits(var_obj, "GBMVarCont")
}

#### Check function inputs ####
check_cv_parameters <- function(cv_folds, cv_class_stratify, fold_id, train_params) {
  check_if_natural_number(cv_folds, "cv_folds")
  if(!is.logical(cv_class_stratify)) stop("cv_class_stratify must be a logical")
  check_if_gbm_train_params(train_params)

  # Check fold_id does not split observation data up
  if(!is.null(fold_id)) {
    for(id in train_params$id) {
      if(length(unique(fold_id[train_params$id == id])) > 1) {
        stop("Observations are split across multiple folds")
      }
    }
  }
}

check_if_natural_number <- function(value, name) {
  # value - the value of the parameter to check 
  if(is.null(value) || is.infinite(value)  || is.logical(value)
     || !(abs(value - round(value)) < .Machine$double.eps^0.5) ||
     (value <= 0) || (length(value) > 1)) {
    stop("The parameter  ", name,  " must be a positive whole number")
  }
}

checkMissing <- function(x, y){
   nms <- get_var_names(x)
   #### Check for NaNs in x and NAs in response
   j <- apply(x, 2, function(z) any(is.nan(z)))
   if(any(j)) {
      stop("Use NA for missing values. NaN found in predictor variables:",
           paste(nms[j],collapse=","))
   }
   
   if(any(is.na(y))) stop("Missing values are not allowed in the response")
   
   AllMiss <- apply(x, 2, function(X){all(is.na(X))})
   AllMissVarIndex <- paste(which(AllMiss), collapse = ', ')
   AllMissVar <- paste(nms[which(AllMiss)], collapse = ', ')
   
   if(any(AllMiss)) {
      stop("variable(s) ", AllMissVarIndex, ": ", AllMissVar, " contain only missing values.")
   }
   
   invisible(NULL)
 }

check_interaction_depth <- function(id){
   # Check for disallowed interaction.depth
   if(id < 1) {
      stop("interaction_depth must be at least 1.")
   }
   else if(id > 49) {
      stop("interaction_depth must be less than 50. You should also ask yourself why you want such large interaction terms. A value between 1 and 5 should be sufficient for most applications.")
   }
   invisible(id)
}

check_weights <- function(w, n){
   # Logical checks on weights
   if(length(w)==0) { w <- rep(1, n) }
   else if(any(w != as.double(w))) {stop("weights must be doubles")}
   else if(any(w < 0)) {stop("negative weights not allowed")}
   w
}

check_offset <- function(o, y, dist){
   # Check offset
  if(is.null(o))
      o <- rep(0,length(y))
   else if(length(o) != length(y))
      stop("The length of offset does not equal the length of y.")
   else if(!is.numeric(o))
     stop("offset must be numeric")
   else if(sum(is.na(o))>0)
     stop("offset can not contain NA's")
   else if(distribution_name(dist) %in% c("Poisson", "Gamma", "Tweedie") &&
           any(abs(o) > 650))
     stop("offsets for Poisson, Gamma, and Tweedie must be between -650 and 650 on the link scale")

   o
}

get_var_names <- function(x){
  if(is.matrix(x)) { var.names <- colnames(x) }
  else if(is.data.frame(x)) { var.names <- names(x) }
  else { var.names <- paste("X", 1:ncol(x),sep="") }
  var.names
}

check_sanity <- function(x, y){
  # x and y are not the same length
  if(nrow(x) != ifelse(!is.null(dim(y)), nrow(y), length(y))) {
    stop("The number of rows in x does not equal the length of y.")
  }
}

check_var_type <- function(x, y){
  
  nms <- get_var_names(x)
  
  # Excessive Factors
  Factors <- vapply(x, is.factor, TRUE)
  nLevels <- vapply(x, nlevels, 0L)
  
  excessLevels <- nLevels > 1024
  excessLevelsIndex <- paste(which(excessLevels), collapse = ', ')
  excessLevelsVars <- paste(nms[which(excessLevels)], collapse = ', ')
  
  if(any(excessLevels)) {
    stop("gbm does not currently handle categorical variables with more than 1024 levels. Variable ", excessLevelsIndex,": ", excessLevelsVars," has ", nLevels[which(excessLevels)]," levels.")
    
  }
  
  # Not an acceptable class
  inacceptClass <- vapply(x, function(X){! (is.ordered(X) | is.factor(X) | is.numeric(X)) }, TRUE)
  inacceptClassIndex <- paste(which(inacceptClass), collapse = ', ')
  inacceptClassVars <- paste(nms[which(inacceptClass)], collapse = ', ')
  
  if(any(inacceptClass)){
    stop("variable ", inacceptClassIndex,": ", inacceptClassVars, " is not of type - numeric, ordered or factor.")
  }
  
}

#### Data conversion functions ####
convertY <- function(y){

  FactorsY <- is.factor(y)
  nLevelsY <- nlevels(y)
  
  if(FactorsY & nLevelsY == 2){
    Y = as.numeric(y == levels(y)[2])
  } else {
    Y = y
  }
  
  return(Y)
  
}

## miscellaneous small internal helper functions.

## these are not exported and not formally documented in the manual

## Warn if a variable does not vary
##
## @param x a numeric variable to check
## @param ind an index to use in a potential warning
## @param name a name to include in a potential warning
## @return NULL, invisibly

warnNoVariation <- function(x, ind, name) {
  ## suppress warnings in here
  ## because min and max warn if the variable is completely NA
  suppressWarnings(variation <- range(x, na.rm=TRUE))
  
  ## I really mean ">=" here, which catches the all NA case
  ## and the standard case
  if (variation[[1]] >= variation[[2]]) {
    warning("variable ", ind, ": ", name, " has no variation.")
  }
  
  invisible(NULL)
}

convert_strata <- function(strata) {
  # If factor then convert to integer
  if(is.factor(strata)) {
    strata <- as.integer(strata)
  }
  
  # If it isn't default then check
  if(!is.na(strata[1])) {
    if(!is.vector(strata) || any(is.character(strata)) || any(is.infinite(strata)) || any(is.nan(strata)) ||
       !(all(strata == as.factor(strata)) || all(strata == as.integer(strata)))) {
      stop("strata must be an atomic vector of factors or integers")
    }
  }
  
  return(strata)
}

guess_distribution <- function(response) {
  # This function guesses the distribution if one is not provided
  if(length(unique(response)) == 2) {
    name <- "Bernoulli"
  } else if (inherits(response, "Surv")) {
    name <- "CoxPH"
  } else {
    name <- "Gaussian"
  }
  message("Distribution not specified, assuming ", name, " ...")
  return(list(name=name))
}

####################
# Author: James Hickey
#
# Series of tests to check the extraction of observations
# into different folds
#
####################

context("Error checking")
test_that("split_and_join throws an error if gbm_data_obj is not GBMData object", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to split_and_join not GBMData
  class(gdata) <- "wrong"
  
  # Then error is thrown
  expect_error(split_and_join(gdata, params, rows_in_training_set, rows_in_fold))
  
})
test_that("split_and_join throws an error if train_params is not GBMTrainParams object", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to split_and_join not GBMTrainParams
  class(params) <- "wrong"
  
  # Then error is thrown
  expect_error(split_and_join(gdata, params, rows_in_training_set, rows_in_fold))
})
test_that("split_and_join throws an error if rows_in_training is not an atomic of logicals", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to split_and_join with rows_in_training_set not an atomic of logicals
  # Then error is thrown
  expect_error(split_and_join(gdata, params, rows_in_training_set=c(1, 2), rows_in_fold))
  expect_error(split_and_join(gdata, params, rows_in_training_set="what", rows_in_fold))
  expect_error(split_and_join(gdata, params, rows_in_training_set= NaN, rows_in_fold))
  
})
test_that("split_and_join throws an error if rows_in_fold is not an atomic of logicals of length num_train", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to split_and_join with rows_in_fold not an atomic of logicals of length num_train
  # Then error is thrown
  expect_error(split_and_join(gdata, params, rows_in_training_set, rows_in_fold =sample(c(TRUE, FALSE), params$num_train+1, replace=TRUE)))
  expect_error(split_and_join(gdata, params, rows_in_training_set, rows_in_fold=c(1, 2)))
  expect_error(split_and_join(gdata, params, rows_in_training_set, rows_in_fold=NaN))
  expect_error(split_and_join(gdata, params, rows_in_training_set, rows_in_fold="Where"))  
})
test_that("update_fold_dist_data throws an error if gbm_data_obj is not GBMData object", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to update_fold_dist_data not GBMData
  class(gdata) <- "wrong"
  
  # Then error is thrown
  expect_error(update_fold_dist_data(dist, gdata, params, rows_in_training_set, rows_in_fold))
})
test_that("update_fold_dist_data throws an error if gbm_dist_obj is not GBMDist object", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to update_fold_dist_data not GBMDist
  class(dist) <- "wrong"
  
  # Then error is thrown
  expect_error(update_fold_dist_data(dist, gdata, params, rows_in_training_set, rows_in_fold))
})
test_that("update_fold_dist_data throws an error if train_params is not GBMTrainParams object", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to update_fold_dist_data not GBMTrainParams
  class(params) <- "wrong"
  
  # Then error is thrown
  expect_error(update_fold_dist_data(dist, gdata, params, rows_in_training_set, rows_in_fold))
})

context("Test splitting and joining functionality")
test_that("split_and_join returns a GBMData object", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]

  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When split_and_join called
  returned_obj <- split_and_join(gdata, params, rows_in_training_set, rows_in_fold)
  
  # Then returned object is a GBMData object
  expect_error(check_if_gbm_data(returned_obj), NA)
})
test_that("split_and_join modifies gbm_data_obj so data in validation fold is at the 'end' of each field", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When split_and_join called
  returned_obj <- split_and_join(gdata, params, rows_in_training_set, rows_in_fold)
  
  # Then returned object is a GBMData object with validation fold data at end of rows
  expect_equal(returned_obj$x[rows_in_training_set, ,drop=FALSE][(N/2 - length(which(cv_groups==1)) + 1):(N/2), ,drop=FALSE], 
               data[rows_in_training_set, ,drop=FALSE][rows_in_fold, ,drop=FALSE])
  expect_equal(returned_obj$y[rows_in_training_set, ,drop=FALSE][(N/2 - length(which(cv_groups==1)) + 1):(N/2), ,drop=FALSE]$V1, 
              Y[rows_in_training_set][rows_in_fold])
  expect_equal(returned_obj$weights[rows_in_training_set][(N/2 - length(which(cv_groups==1)) + 1):(N/2)], 
               w[rows_in_training_set][rows_in_fold])
  expect_equal(returned_obj$offset[rows_in_training_set][(N/2 - length(which(cv_groups==1)) + 1):(N/2)], 
               offset[rows_in_training_set][rows_in_fold])
})
test_that("split_and_join updates x_order so as to only use data in training folds", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When split_and_join called
  returned_obj <- split_and_join(gdata, params, rows_in_training_set, rows_in_fold)
  
  # Get gdata training folds data
  gdata_train <- gdata
  gdata_train$x <- as.data.frame(gdata_train$x[rows_in_training_set, ,drop=FALSE][!rows_in_fold, ,drop=FALSE])
  gdata_train$y <- as.data.frame(as.matrix(gdata_train$y)[rows_in_training_set, ,drop=FALSE][!rows_in_fold, ,drop=FALSE])
  gdata_train$offset <- gdata_train$offset[rows_in_training_set][!rows_in_fold]
  gdata_train$weights <- gdata_train$weights[rows_in_training_set][!rows_in_fold]
  x_order_with_training_folds <- predictor_order(gdata_train, params)$x_order
  
  # Then returned object is a GBMData object with x_order updated so as to only 
  expect_equal(as.matrix(returned_obj$x_order), as.matrix(x_order_with_training_folds))
})
test_that("update_fold_dist_data returns the original distribution object if NOT Pairwise or CoxPH", {
  # Given gbm_data, a distribution obj, rows_in_fold and rows_in_training
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  gdata <- gbm_data(data, Y, w, offset)
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to update_fold_dist_data with distributions which are not CoxPH or Pairwise
  # Not assuming no stratification in cv_groups or fold_id so same for all dists
  # Then nothing changes for those distributions
  expect_equal(update_fold_dist_data(gbm_dist("AdaBoost"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("AdaBoost"))
  expect_equal(update_fold_dist_data(gbm_dist("Bernoulli"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Bernoulli"))
  expect_equal(update_fold_dist_data(gbm_dist("Gamma"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Gamma"))
  expect_equal(update_fold_dist_data(gbm_dist("Gaussian"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Gaussian"))
  expect_equal(update_fold_dist_data(gbm_dist("Huberized"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Huberized"))
  expect_equal(update_fold_dist_data(gbm_dist("Laplace"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Laplace"))
  expect_equal(update_fold_dist_data(gbm_dist("Poisson"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Poisson"))
  expect_equal(update_fold_dist_data(gbm_dist("Quantile"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Quantile"))
  expect_equal(update_fold_dist_data(gbm_dist("TDist"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("TDist"))
  expect_equal(update_fold_dist_data(gbm_dist("Tweedie"), gdata, params, rows_in_training_set, rows_in_fold), gbm_dist("Tweedie"))
})
test_that("update_fold_dist_data updates the dist objects strata and sorted fields if CoxPH", {
  # Given gbm_data, a distribution obj (CoxPH!!), rows_in_fold and rows_in_training
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)
  
  # Put into new API
  params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq_len(nrow(data)), num_train=N/2, num_features=3)
  
  
  # Set up for new API
  dist <- gbm_dist("CoxPH")
  gdata <- gbm_data(data, Surv(tt, delta), w, offset=rep(0, N))
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to update_fold_dist_data with distributions 
  dist_update <- update_fold_dist_data(dist, gdata, params, rows_in_training_set, rows_in_fold)
  
  # Then strata and sorted are updated
  expect_true(length(dist_update$strata) != length(dist$strata))
  expect_false(all(is.na(dist_update$sorted)))
})
test_that("update_fold_dist_data updates the dist objects group if Pairwise", {
  # Given gbm_data, a distribution obj (Pairwise!! + groups), rows_in_fold and rows_in_training
  # create some data - DOESN'T NEED TO REFLECT DIST
  set.seed(1)
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  X <- data.frame(X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  data <- gbm_data(X, Y, w, offset)
  
  # Put into new API
  params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq_len(nrow(X)), num_train=N/2, num_features=3)
  
  
  # Set up for new API
  dist <- gbm_dist("Pairwise")
  dist$group <- as.factor(sample(seq_len(5), N, replace=TRUE))
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = data, dist, params, cv_folds, FALSE, NULL)
 
  
  # DEFINE GROUPINGS
  # Observations in the training set
  rows_in_training_set <- params$id %in% seq_len(params$num_train_rows)
  params$id <- params$id[rows_in_training_set]
  
  # Observations in cv_group - interested in first fold
  rows_in_fold <- params$id %in% seq_len(params$num_train_rows)[(cv_groups == 1)]
  
  # Extract relevent data - split into training and validation sets
  # Calculate new number of training rows
  params$num_train_rows <- length(which(cv_groups != 1))
  params$num_train <- length(unique(params$id[!rows_in_fold]))
  
  # When try to update_fold_dist_data with distributions 
  dist_update <- update_fold_dist_data(dist, data, params, rows_in_training_set, rows_in_fold)
  
  # Then groupings are now updated
  expect_equal(dist_update$group,  c(dist$group[rows_in_training_set][!rows_in_fold],
                                     dist$group[rows_in_training_set][rows_in_fold]))
})
test_that("extract_obs_in_fold updates the training parameters so as to take into account the number of observations in the validation fold", {
  # Given gbm_data, a distribution obj (CoxPH!!), rows_in_fold and rows_in_training
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)
  
  # Put into new API
  dist <- gbm_dist("CoxPH", prior_node_coeff_var=10)
  params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  
  
  # Set up for new API
  dist <- gbm_dist("CoxPH")
  gdata <- gbm_data(data, Surv(tt, delta), w, offset=rep(0, N))
  cv_folds <- 5
  cv_groups <- create_cv_groups(gbm_data_obj = gdata, dist, params, cv_folds, FALSE, NULL)
  
  # When extract_obs_in_fold is called
  returned_obj <- extract_obs_in_fold(gdata, dist, params, cv_groups, fold_num=1) 
  
  # Then updates the num of training observations in train_params
  expect_equal(returned_obj$params$num_train, params$num_train * (cv_folds-1)/cv_folds)
})
test_that("extract_obs_in_fold returns a list of the updated gbm_data_obj, dist obj and training params", {
  # Given gbm_data, a distribution obj (CoxPH!!), rows_in_fold and rows_in_training
  # Require Surv to be available
  require(survival)
  
  # create some data
  set.seed(1)
  N <- 3000
  X1 <- runif(N)
  X2 <- runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
  tt.surv <- rexp(N,exp(f))
  tt.cens <- rexp(N,0.5)
  delta <- as.numeric(tt.surv <= tt.cens)
  tt <- apply(cbind(tt.surv,tt.cens),1,min)
  
  # throw in some missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  data <- data.frame(X1=X1,X2=X2,X3=X3)
  
  # Put into new API
  params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=3)
  
  
  # Set up for new API
  dist <- gbm_dist("CoxPH")
  gdata <- gbm_data(data, Surv(tt, delta), w, rep(0, N))
  cv_folds <- 5
  cv_groups <- create_cv_groups(gdata, dist, params, cv_folds, FALSE, NULL)
  
  # When extract_obs_in_fold is called
  returned_obj <- extract_obs_in_fold(gdata, dist, params, cv_groups, fold_num=1) 
    
  # Then returns a list with correct fields
  expect_true(is.list(returned_obj))
  expect_equal(names(returned_obj), c("data", "dist", "params"))
  expect_error(check_if_gbm_data(returned_obj$data), NA)
  expect_error(check_if_gbm_dist(returned_obj$dist), NA)
  expect_error(check_if_gbm_train_params(returned_obj$params), NA)
})

####################
# Author: James Hickey
#
# Series of tests to check the helper functions
# within the package
#
####################

context("Testing checks on GBM S3 objects")
test_that("check_if_gbm_data throws an error if given an object not of class GBMData", {
  # Given an object not of class GBMData
  obj <- list()
  
  # When passed to check_if_gbm_data
  # Then an error is thrown
  expect_error(check_if_gbm_data(obj))
})
test_that("check_if_gbm_data does not throw an error if given an object of class GBMData", {
  # Given an object of class GBMData
  obj <- list()
  class(obj) <- "GBMData"
  
  # When passed to check_if_gbm_data
  # Then no error is thrown
  expect_error(check_if_gbm_data(obj), NA)
})
test_that("check_if_gbm_dist throws an error if given an object not of class GBMDist", {
  # Given an object not of class GBMDist
  obj <- list()
  
  # When passed to check_if_gbm_dist
  # Then an error is thrown
  expect_error(check_if_gbm_dist(obj))
})
test_that("check_if_gbm_dist does not throw an error if given an object of class GBMDist", { 
  # Given an object of class GBMDist
  obj <- list()
  class(obj) <- "AdaBoostGBMDist"
  
  # When passed to check_if_gbm_dist
  # Then no error is thrown
  expect_error(check_if_gbm_dist(obj), NA)  
})
test_that("check_if_gbm_train_params throws an error if given object not of class GBMTrainParams", {
  # Given an object not of class GBMTrainParams
  obj <- list()
  
  # When passed to check_if_gbm_train_params
  # Then an error is thrown
  expect_error(check_if_gbm_train_params(obj))
})
test_that("check_if_gbm_train_params does not throw an error if given object of class GBMTrainParams", {
  # Given an object of class GBMTrainParams
  obj <- list()
  class(obj) <- "GBMTrainParams"
  
  # When passed to check_if_gbm_train_params
  # Then no error is thrown
  expect_error(check_if_gbm_train_params(obj), NA)
})
test_that("check_if_gbm_fit throws an error if given an object not of class GBMFit", {
  # Given an object not of class GBMFit
  obj <- list()
  
  # When passed to check_if_gbm_fit
  # Then an error is thrown
  expect_error(check_if_gbm_fit(obj))
})
test_that("check_if_gbm_fit does not throw an error if given an object of class GBMFit", {
  # Given an object of class GBMFit
  obj <- list()
  class(obj) <- "GBMFit"
  
  # When passed to check_if_gbm_fit
  # Then no error is thrown
  expect_error(check_if_gbm_fit(obj), NA)
})
test_that("check_if_gbm_var_container throws an error if given an object not of class GBMVarCont", {
  # Given an object not of class GBMVarCont
  obj <- list()
  
  # When passed to check_if_gbm_var_container
  # Then an error is thrown
  expect_error(check_if_gbm_var_container(obj))
})
test_that("check_if_gbm_var_container does not throw an error if given an object of class GBMVarCont", {
  # Given an object of class GBMVarCont
  obj <- list()
  class(obj) <- "GBMVarCont"
  
  # When passed to check_if_gbm_var_container
  # Then no error is thrown
  expect_error(check_if_gbm_var_container(obj), NA)
})


context("Testing checks on inputs for creation of S3 objects")
test_that("check_weights returns a vector of 1s if passed an obj of length 0", {
  # Given an empty vector
  w <- c()
  n <- 100
  
  # When check_weights is called 
  w <- check_weights(w, n)
  
  # Then returns a vector of 1s
  expect_equal(w, rep(1, n))
})
test_that("check_weights throws an error if any not a double", {
  # Given a vector of weights
  n <- 100
  w <- rep(1, 100)
  
  # When one weight is not a double
  w[1] <- NA
  
  # Then check_weights will throw an error
  expect_error(check_weights(w, n))
})
test_that("check_weights throws an error if any are < 0", {
  # Given a vector of weights
  w <- rep(1, 100)
  
  # When one weight is set < 0
  w[1] <- -1
  
  # Then check_weights will throw an error
  expect_error(check_weights(w, 100))
})
test_that("check_interaction_depth throws error if < 1 or > 49", {
  # Given interaction depths of 0 and 50
  id1 <- 0
  id2 <- 50
  
  # When they're checked
  # Then error
  expect_error(check_interaction_depth(id1))
  expect_error(check_interaction_depth(id2))
})
test_that("checkMissing throws error if NaN in predictors", {
  # Given predictors and response
  N <- 100
  x <- runif(N)
  y <- runif(N)
  
  # When x has a NaN in it
  x[1] <- NaN
  
  # Then error thrown by checkMissing
  expect_error(checkMissing(x, y))
})
test_that("checkMissing throws error if missing value in response", {
  # Given predictors and response
  N <- 100
  x <- runif(N)
  y <- runif(N)
  
  # When y has a missing value
  y[1] <- NA
  
  # Then error thrown by checkMissing
  expect_error(checkMissing(x, y))
})
test_that("warnNoVariation warns if a variable has no variation", {
  x <- c(1.2, 1.2)
  
  expect_warning(warnNoVariation(x, 1, 'test'),
                 "variable 1: test has no variation",
                 fixed=TRUE)
})
test_that("warnNoVariation passes OK if variable does vary", {
  x <- c(0, 1)
  
  expect_warning(warnNoVariation(x, 1, 'test'), regexp=NA)
})
test_that("warnNoVariation passes OK if variable does vary (with NA)", {
  x <- c(0, 1, NA)
  
  expect_warning(warnNoVariation(x, 1, 'test'), regexp=NA)
})
test_that("get_var_names gets colnames if passed a matrix", {
  # Given a matrix with 3 columns and names
  x <- matrix(ncol=3)
  colnames(x) <- c("V1", "V2", "V3")
  
  # When passed to get_var_names
  # Then column names returned
  expect_equal(get_var_names(x), colnames(x))
})
test_that("get_var_names gets the names if passed a data.frame", {
  # Given a data.frame with 3 columns and names
  x <- data.frame(X1=NA, X2=NA, X3=NA)
  
  # When passed to get_var_names
  # Then names returned
  expect_equal(get_var_names(x), names(x))
})
test_that("check_sanity only throws an error if nrows(x) is not equal to the length of y", {
  # Given two sets of predictors one of length y, the other not
  N <- 100
  x1 <- data.frame(runif(N))
  x2 <- data.frame(runif(N-1))
  y <- runif(N)
  
  # When check_sanity is called
  # Then error thrown for call where lengths are not equal
  expect_error(check_sanity(x1, y), NA)
  expect_error(check_sanity(x2, y))
})
test_that("check_if_natural_number throws error if not passed a whole number >=1" , {
  # When check_if_natural_number is not passed a whole number >= 1
  # Then an error is thrown
  expect_error(check_if_natural_number(-1, "Arg Name"))
  expect_error(check_if_natural_number(c(1, 2), "Arg Name"))
  expect_error(check_if_natural_number(1.4, "Arg Name"))
  expect_error(check_if_natural_number(Inf, "Arg Name"))
  expect_error(check_if_natural_number(list(), "Arg Name"))
  expect_error(check_if_natural_number(NA, "Arg Name"))
  expect_error(check_if_natural_number(NaN, "Arg Name"))
})
test_that("convertY changes factors with 2 levels to numeric", {
  # Given a vector of 2-level factors
  N <- 100
  y <- as.factor(sample(c(0, 1), N, replace=TRUE))
  
  # When convertY is called
  # Then converted to numeric
  expect_equal(convertY(y), as.numeric(y==levels(y)[2]))
})

test_that("convertY does nothing to factors with levels != 2", {
  # Given of a vector of 3-level factors
  N <- 100
  y <- as.factor(sample(c(0, 1, 2), N, replace=TRUE))
  
  # When convertY is called
  # Then remains the same
  expect_equal(convertY(y), y)
})
test_that("check_var_type throws error when passed excessive levels in x", {
  
  testExcess <- data.frame(
    y = sample(c(0,1), 1025, replace = TRUE)
    ,x1 = runif(1025)
    ,x2 = factor(1:1025)
  )
  
  expect_error(
    check_var_type(x = testExcess[,c('x1', 'x2')]
             , y = testExcess$y)
    ,'gbm does not currently handle categorical variables with more than 1024 levels\\. Variable 2\\: x2 has 1025 levels\\.')
  
})

test_that("check_var_type throws an error when passed Inacceptable classes", {
  
  testClasses <- data.frame(
    y = sample(c(0,1), 15, replace = TRUE)
    ,x1 = runif(15)
    ,x2 = seq(as.Date('2015-01-01'), as.Date('2015-01-15'), 'days')
  )
  
  expect_error(
    check_var_type(x = testClasses[,c('x1', 'x2')]
             , y = testClasses$y)
    ,'variable 2\\: x2 is not of type - numeric, ordered or factor\\.')
  
})

test_that("check_offset default returns a vector of 0s when offset set to NULL - irrespective of distribution", {
  # Given an offset=NULL and responses (and all distributions)
  N <- 100
  y <- runif(N)
  offset <- NULL
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Pairwise")
  dist_9 <- gbm_dist("Poisson")
  dist_10 <- gbm_dist("Quantile")
  dist_11 <- gbm_dist("TDist")
  dist_12 <- gbm_dist("Tweedie")
  
  # Then check_offset returns a vector of zeros
  # equal to length of response
  expect_equal(check_offset(offset, y, dist_1), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_2), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_3), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_4), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_5), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_6), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_7), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_8), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_9), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_10), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_11), rep(0, length(y)))
  expect_equal(check_offset(offset, y, dist_12), rep(0, length(y)))
})

test_that("check_offset throws an error length of offset does not equal the length of the response", {
  # Given an offset and vector of responses
  # offset is different length to responses
  N <- 100
  y <- runif(N)
  offset <- runif(N-2)
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Pairwise")
  dist_9 <- gbm_dist("Poisson")
  dist_10 <- gbm_dist("Quantile")
  dist_11 <- gbm_dist("TDist")
  dist_12 <- gbm_dist("Tweedie")
  
  # Then check_offset throws an error
  expect_error(check_offset(offset, y, dist_1))
  expect_error(check_offset(offset, y, dist_2))
  expect_error(check_offset(offset, y, dist_3))
  expect_error(check_offset(offset, y, dist_4))
  expect_error(check_offset(offset, y, dist_5))
  expect_error(check_offset(offset, y, dist_6))
  expect_error(check_offset(offset, y, dist_7))
  expect_error(check_offset(offset, y, dist_8))
  expect_error(check_offset(offset, y, dist_9))
  expect_error(check_offset(offset, y, dist_10))
  expect_error(check_offset(offset, y, dist_11))
  expect_error(check_offset(offset, y, dist_12))
})

test_that("check_offset throws an error if the offset contains a NA", {
  # Given an offset and vector of responses - irrespective of distribution
  N <- 100
  y <- runif(N)
  offset <- runif(N)
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Pairwise")
  dist_9 <- gbm_dist("Poisson")
  dist_10 <- gbm_dist("Quantile")
  dist_11 <- gbm_dist("TDist")
  dist_12 <- gbm_dist("Tweedie")
  
  
  # When an elemenet of offset is NA
  offset[1] <- NA
  
  # Then check_offset throws an error
  expect_error(check_offset(offset, y, dist_1))
  expect_error(check_offset(offset, y, dist_2))
  expect_error(check_offset(offset, y, dist_3))
  expect_error(check_offset(offset, y, dist_4))
  expect_error(check_offset(offset, y, dist_5))
  expect_error(check_offset(offset, y, dist_6))
  expect_error(check_offset(offset, y, dist_7))
  expect_error(check_offset(offset, y, dist_8))
  expect_error(check_offset(offset, y, dist_9))
  expect_error(check_offset(offset, y, dist_10))
  expect_error(check_offset(offset, y, dist_11))
  expect_error(check_offset(offset, y, dist_12))
})

test_that("check_offset throws an error if the offset contains a non-numeric", {
  # Given an offset and vector of responses - irrespective of distribution
  N <- 100
  y <- runif(N)
  offset <- runif(N)
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Pairwise")
  dist_9 <- gbm_dist("Poisson")
  dist_10 <- gbm_dist("Quantile")
  dist_11 <- gbm_dist("TDist")
  dist_12 <- gbm_dist("Tweedie")
  
  
  # When an elemenet of offset is non-numeric
  offset[1] <- "string"
  
  # Then check_offset throws an error
  expect_error(check_offset(offset, y, dist_1))
  expect_error(check_offset(offset, y, dist_2))
  expect_error(check_offset(offset, y, dist_3))
  expect_error(check_offset(offset, y, dist_4))
  expect_error(check_offset(offset, y, dist_5))
  expect_error(check_offset(offset, y, dist_6))
  expect_error(check_offset(offset, y, dist_7))
  expect_error(check_offset(offset, y, dist_8))
  expect_error(check_offset(offset, y, dist_9))
  expect_error(check_offset(offset, y, dist_10))
  expect_error(check_offset(offset, y, dist_11))
  expect_error(check_offset(offset, y, dist_12))
})


test_that("check_cv_parameters throws an error if cv_folds is not a natural number >= 1", {
  # When cv_folds is not a natural number >=1
  # Then an error is thrown
  expect_error(check_cv_parameters(cv_folds=-1, cv_class_stratify=FALSE, fold_id=NULL,
                                 train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                              shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=FALSE, cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds="string", cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=c(1, 2), cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=2.1, cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=NA, cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=NaN, cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=Inf, cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
})

test_that("check_cv_parameters throws an error if cv_class_stratify is not a logical", {
  # When cv_class_stratify is not a logical
  # Then check_cv_parameters throws an error
  expect_error(check_cv_parameters(cv_folds=5, cv_class_stratify="Help", fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=5, cv_class_stratify=NULL, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=5, cv_class_stratify=NaN, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=5, cv_class_stratify=c(1, 2), fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
  expect_error(check_cv_parameters(cv_folds=5, cv_class_stratify=Inf, fold_id=NULL,
                                   train_params=training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                                                                shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)))
})

test_that("check_cv_parameters throws an error if train_params not of class GBMTrainParams", {
  # Given training parameters
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=1:1000, num_train=1000, num_features=6)
  # When class removed
  class(params) <- ""
  
  # Then check_cv_parameters throws an error
  expect_error(check_cv_parameters(cv_folds=5, cv_class_stratify=FALSE, fold_id=NULL,
                                   train_params=params))
})

test_that("check_cv_parameters throws an error if fold_id has observations across different folds", {
  # Given training parameters and obs id
  N <- 2000
  obs_id <- c(rep(1, N/2), rep(2, N/2))
  params <- training_params(num_trees=2000, interaction_depth=3, min_num_obs_in_node=10, 
         shrinkage=0.005, bag_fraction=0.5, id=obs_id, num_train=N/2, num_features=6)
  
  # When fold_id not NULL and has observations across folds
  cv_folds <- 5
  fold_id <- sample(seq_len(cv_folds), replace=TRUE)
  
  # Then check_cv_parameters throws an error
  expect_error(check_cv_parameters(cv_folds=cv_folds, cv_class_stratify=FALSE, fold_id=fold_id,
                                   train_params=params))
})

test_that("convert_strata does not alter a NA", {
  expect_true(is.na(convert_strata(NA)))
})

test_that("convert_strata converts a vector of factors to integers", {
  # Given a vector of factors
  test_strata <- as.factor(c("a", "b", "b", "a"))
  
  # When converted
  converted <- convert_strata(test_strata)
  
  # Then is a vector of integers
  expect_equal(converted, as.integer(test_strata))
})

test_that("convert_strata throws an error when not passed a vector of integers or strata", {
  expect_error(convert_strata(data.frame(FALSE)))
  expect_error(convert_strata(data.frame(c(1, 2))))
  expect_error(convert_strata(matrix(FALSE)))
  expect_error(convert_strata(matrix(c(1, 2))))
  expect_error(convert_strata(Inf))
  expect_error(convert_strata("Error"))
})

test_that("guess_distribution makes correct guesses and displays message", {
  require(survival)
  
  # Given responses
  b_resp <- c(0, 1, 0, 1, 0, 1)
  c_resp <- Surv(c(1.2, 2.3, 4.5, 6.7), c(0, 0, 0, 1))
  other_resp <- "Default"
  
  # When guessing
  # Then correct guess and message
  expect_equal(guess_distribution(b_resp)$name, "Bernoulli")
  expect_equal(guess_distribution(c_resp)$name, "CoxPH")
  expect_equal(guess_distribution(other_resp)$name, "Gaussian")
  expect_message(guess_distribution(b_resp), "Distribution not specified, assuming Bernoulli ...")
  expect_message(guess_distribution(c_resp), "Distribution not specified, assuming CoxPH ...")
  expect_message(guess_distribution(other_resp), "Distribution not specified, assuming Gaussian ...")
})

####################
# Author: James Hickey
#
# Series of test to validate the GBMFit object
#
####################

context("Test GBMFit definition")
test_that("Calling gbmt creates a GBMFit object", {
  # Given Data from examples
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  
  # When gbmt called
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  
  # Then fit has correct class
  expect_equal(class(fit), "GBMFit")
})
test_that("GBMFit object has correct fields", {
  # Given Data from examples
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  
  # When gbmt called
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  
  # Then fit has names
  expect_equal(names(fit), c("initF", "fit", "train.error", "valid.error", "oobag.improve", "trees", "c.splits", "distribution", "params"       
               ,"variables", "cv_folds", "cv_error", "cv_fitted", "gbm_data_obj",
               "par_details", "response_name", "model", "Terms", "call", "is_verbose"))
})
test_that("GBMFit contains correct GBM objects", {
  # Given Data from examples
  ## Based on example in R package
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  
  # When gbmt called
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  
  # Then fit has correct GBM objects
  expect_error(check_if_gbm_data(fit$gbm_data_obj), NA)
  expect_error(check_if_gbm_dist(fit$distribution), NA)
  expect_error(check_if_gbm_train_params(fit$params), NA)
  expect_error(check_if_gbm_var_container(fit$variables), NA)  
})
test_that("GBMFit does not contain gbm_data_obj if keep_gbm_data=FALSE", {
  # Given Data from examples
  ## Based on example in R package
  keep_data <- FALSE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  
  # When gbmt called
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  
  
  # Then fit does not store gbm data from gbmt
  expect_true(is.null(fit$gbm_data_obj))
})
test_that("call to gbm and gbmt produces object with same class and fields", {
  # Given Data from examples
  ## Based on example in R package
  keep_data <- TRUE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 1000
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=100)] <- NA
  X3[sample(1:N,size=300)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  
  # When gbmt called and gbm called
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=keep_data, cv_folds=10, is_verbose=FALSE)
  fit_2 <- gbm(Y~X1+X2+X3+X4+X5+X6, data=data, weights=w, offset=offset, n.trees = 2000, cv.folds = 10, keep.data = keep_data, verbose = FALSE)

  
  # Then same object and fields produced
  expect_equal(class(fit), class(fit_2))
  expect_equal(names(fit), names(fit_2))
})

#' Predict method for GBM Model Fits
#' 
#' Predicted values based on a generalized boosted model object - from
#' gbmt
#' 
#' \code{predict.GBMFit} produces predicted values for each
#' observation in a new dataset \code{newdata} using the first
#' \code{n.trees} iterations of the boosting sequence. If
#' \code{n.trees} is a vector than the result is a matrix with each
#' column representing the predictions from gbm models with
#' \code{n.trees[1]} iterations, \code{n.trees[2]} iterations, and
#' so on.
#' 
#' The predictions from \code{gbmt} do not include the offset
#' term. The user may add the value of the offset to the predicted
#' value if desired.
#' 
#' If \code{gbm_fit_obj} was fit using \code{\link{gbmt}}, there will
#' be no \code{Terms} component. Therefore, the user has greater
#' responsibility to make sure that \code{newdata} is of the same
#' format (order and number of variables) as the one originally used
#' to fit the model.
#' 
#' @param object Object of class inheriting from \code{GBMFit}.
#' @param newdata Data frame of observations for which to make
#' predictions
#' @param n.trees Number of trees used in the prediction. If
#' \code{n.trees} is a vector, predictions are returned for each
#' iteration specified.
#' @param type The scale on which gbm makes the predictions
#' @param single.tree If \code{single.tree=TRUE} then \code{gbm_predict}
#' returns only the predictions from tree(s) \code{n.trees}
#' @param \dots further arguments passed to or from other methods
#' @return Returns a vector of predictions. By default the predictions
#' are on the scale of f(x). For example, for the Bernoulli loss the
#' returned value is on the log odds scale, poisson loss on the log
#' scale, and coxph is on the log hazard scale.
#' 
#' If \code{type="response"} then \code{gbmt} converts back to the same scale as
#' the outcome. Currently the only effect this will have is returning
#' probabilities for bernoulli and expected counts for poisson. For the other
#' distributions "response" and "link" return the same.
#' @seealso \code{\link{gbmt}}
#' @keywords models regression
#' @importFrom stats predict
#' @export 
predict.GBMFit <- function(object, newdata, n.trees,
                           type="link", single.tree=FALSE,
                           ...)
{
  # Check inputs
  if(!is.element(type, c("link","response" ))) {
    stop("type must be either 'link' or 'response'")
  }
  
  if(missing(newdata) || !is.data.frame(newdata)) {
    stop("newdata must be provided as a data frame")
  }
  
  if(missing(n.trees)) {
    stop("Number of trees to be used in prediction must be provided.")
  }
  
  if (length(n.trees) == 0) {
    stop("n.trees cannot be NULL or a vector of zero length")
  }
  
  if(any(n.trees != as.integer(n.trees)) || is.na(all(n.trees == as.integer(n.trees)))
     || any(n.trees < 0)) {
    stop("n.trees must be a vector of non-negative integers")
  }
  
  if(!is.null(attr(object$Terms,"offset")))
  {
    warning("predict.GBMFit does not add the offset to the predicted values.")
  }
  
  # Get data
  if(!is.null(object$Terms)) {
    x <- model.frame(terms(reformulate(object$variables$var_names)),
                     newdata,
                     na.action=na.pass)
  } else {
    x <- newdata
  }
  
  # Convert predictor factors into appropriate numeric
  for(i in seq_len(ncol(x))) {
    if(is.factor(x[,i])) {
      if (length(levels(x[,i])) > length(object$variables$var_levels[[i]])) {
        new_compare <- levels(x[,i])[seq_len(length(object$variables$var_levels[[i]]))]
      } else {
        new_compare <- levels(x[,i])
      }
      
      if (!identical(object$variables$var_levels[[i]], new_compare)) {
        x[,i] <- factor(x[,i], union(object$variables$var_levels[[i]], levels(x[,i])))
      }
      
      x[,i] <- as.numeric(x[,i])-1
    }
  }
  
  if(any(n.trees > object$params$num_trees)) {
    n.trees[n.trees > object$params$num_trees] <- object$params$num_trees
    warning("Number of trees exceeded number fit so far. Using ", paste(n.trees,collapse=" "),".")
  }
  
  i.ntree.order <- order(n.trees)
  
  # Next if block for compatibility with objects created with 1.6
  if(is.null(object$num.classes)) {
    object$num.classes <- 1
  }
  
  predF <- .Call("gbm_pred",
                 X=as.matrix(as.data.frame(x)),
                 n.trees=as.integer(n.trees[order(n.trees)]),
                 initF=object$initF,
                 trees=trees(object),
                 c.split=object$c.split,
                 var.type=as.integer(object$variables$var_type),
                 single.tree = as.integer(single.tree),
                 PACKAGE = "gbm3")
  
  # Convert into matrix of predictions
  if((length(n.trees) > 1) || (!is.null(object$num.classes) && (object$num.classes > 1))) {
    predF <- matrix(predF, ncol=length(n.trees), byrow=FALSE)
    colnames(predF) <- n.trees
    predF[, order(n.trees)] <- predF
  }
  
  # Adjust scale of predictions
  if(type=="response") {
    predF <- adjust_pred_scale(predF, object$distribution)
    if(length(n.trees)==1) {
      predF <- as.vector(predF)
    }
  }
  return(predF)
}
  

#' Available Distributions
#' 
#' Returns a vector of the distributions implemented in the package.
#' 
#' @usage available_distributions()
#' 
#' @return a vector containing the names of the available distributions in the package.
#' 
#' @author James Hickey 
#' 
#' @export 

available_distributions <- function() {
  distributions <- c("AdaBoost", "Bernoulli", "CoxPH", "Gamma", "Gaussian",
                     "Huberized", "Laplace", "Pairwise", "Poisson", "Quantile",
                     "TDist", "Tweedie")
  return(distributions)
}

context("Test available distributions")
test_that("available_distributions function returns vector of names of implemented distributions in package", {
  implemented_distributions <- c("AdaBoost", "Bernoulli", "CoxPH", "Gamma", "Gaussian", "Huberized", "Laplace", "Pairwise",
                                 "Poisson", "Quantile", "TDist", "Tweedie")
  expect_equal(available_distributions(), implemented_distributions)
})

# GBMT API call to C++
# 
# Wrapper function for calling the C++ gbm function.
# 
# @usage gbm_call(gbm_data_obj, gbm_dist_obj, train_params, var_container, is_verbose)
# 
# @param gbm_data_obj a GBMData object containing all of the data used to fit a gbm model. 
#
# @param gbm_dist_obj a GBMDist object specifying the distribution and any additional parameters needed.
# 
# @param train_params a GBMTrainParams object containing generic parameters defining how the model should be
# trained.
# 
# @param var_container a GBMVarCont object which defines the properties of the predictor variables in the data.
# 
# @param par_details Details of the parallelization to use in the
#     core algorithm.
# 
# @param is_verbose if TRUE, will print out progress and performance of the fitting.
#
# @author James Hickey
#
# @return a fitted gbm object
# 

gbm_call <- function(gbm_data_obj, gbm_dist_obj, train_params, var_container, par_details, is_verbose) {
  # Check inputs
  check_if_gbm_data(gbm_data_obj)
  check_if_gbm_dist(gbm_dist_obj)
  check_if_gbm_train_params(train_params)
  check_if_gbm_var_container(var_container)

  # Create y_input - check for factors
  # specifically for Bernoulli
  y_levels <- nlevels(gbm_data_obj$y)
  if(y_levels > 0) {
    y_input <- as.integer(gbm_data_obj$y)
  } else { 
    y_input <-   gbm_data_obj$y
  }
  
  # If cross-val y could be matrix
  if(!is.null(dim(gbm_data_obj$y))) {
    y_levels <- nlevels(gbm_data_obj$y[,1])
    if(y_levels > 0)
      y_input <- as.integer(gbm_data_obj$y[,1])
  }

  
  fit <- .Call("gbm",
                Y=as.matrix(as.data.frame(y_input)),
                intResponse = as.matrix(cbind(gbm_dist_obj$strata, gbm_dist_obj$sorted)),
                Offset=as.double(gbm_data_obj$offset),
                X=as.matrix(as.data.frame(gbm_data_obj$x)),
                X.order=as.integer(gbm_data_obj$x_order),
                weights=as.double(gbm_data_obj$weights),
                Misc=get_misc(gbm_dist_obj),
                prior.node.coeff.var = ifelse(is.null(gbm_dist_obj$prior_node_coeff_var), as.double(0),
                                                         as.double(gbm_dist_obj$prior_node_coeff_var)),
                id = as.integer(train_params$id),
                var.type=as.integer(var_container$var_type),
                var.monotone=as.integer(var_container$var_monotone),
                distribution=gbm_call_dist_name(gbm_dist_obj),
                n.trees=as.integer(train_params$num_trees),
                interaction.depth=as.integer(train_params$interaction_depth),
                n.minobsinnode=as.integer(train_params$min_num_obs_in_node),
                shrinkage=as.double(train_params$shrinkage),
                bag.fraction=as.double(train_params$bag_fraction),
                nTrainRows=as.integer(train_params$num_train_rows),
                nTrainObs = as.integer(train_params$num_train),
                mFeatures=as.integer(train_params$num_features),
                fit.old=as.double(NA),
                n.cat.splits.old=as.integer(0),
                n.trees.old=as.integer(0),
                par_details,
                verbose=as.integer(is_verbose),
                PACKAGE = "gbm3")
  
  fit$distribution <- gbm_dist_obj
  fit$params <- train_params
  fit$variables <- var_container
  class(fit) <- "GBMFit"
  return(fit)
}

##
gbm_call_dist_name <- function(obj) {
    UseMethod("gbm_call_dist_name")
}

#' @export
gbm_call_dist_name.default <- function(obj) {
    tolower(distribution_name(obj))
}

#' @export
gbm_call_dist_name.PairwiseGBMDist <- function(obj) {
    paste(tolower(distribution_name(obj)), tolower(obj$metric), sep="_")
}

# Performance Plots
# 
# Plot the performance of a GBMFit object produced by calling
# \code{\link{gbmt}}. This function is used in
# \code{\link{plot.GBMTPerformance}}.
# 
# 
# @param gbm_fit_obj a \code{GBMFit} created from an initial call to
# \code{\link{gbmt}}.
# 
# @param best_iter iteration specifying the optimum number of
# iterations. This is determined in \code{\link{gbmt_performance}}.
# 
# @param out_of_bag_curve logical indicating whether to plot the
# out-of-bag performance measures in a second plot.
# 
# @param overlay if TRUE and out_of_bag_curve=TRUE then a right y-axis
# is added to the training and test error plot and the estimated
# cumulative improvement in the loss function is plotted versus the
# iteration number.
# 
# @param method indicate the method used to estimate the optimal
# number of boosting iterations. \code{method="OOB"} computes the
# out-of-bag estimate and \code{method="test"} uses the test (or
# validation) dataset to compute an out-of-sample
# estimate. \code{method="cv"} extracts the optimal number of
# iterations using cross-validation if \code{gbmt} was called with
# \code{cv_folds}>1.
# 
# @param main the main title for the plot.
perf_plot <- function(gbm_fit_obj, best_iter, out_of_bag_curve,
                      overlay, method, main) {
  # Check inputs
  check_if_gbm_fit(gbm_fit_obj)
  if(!is.logical(overlay) || (length(overlay)) > 1 || is.na(overlay))
    stop("overlay must be a logical - excluding NA")
  
  if(!is.logical(out_of_bag_curve) || (length(out_of_bag_curve)) > 1 || is.na(out_of_bag_curve))
    stop("out_of_bag_curve must be a logical - excluding NA")
  
  par.old <- par(mar=c(5,4,4,4)+.1)
  on.exit(par(par.old)) 
  
  # Get y-axis label and limits
  ylab <- get_ylabel(gbm_fit_obj$distribution)
  if(gbm_fit_obj$params$train_fraction==1) {
    ylim <- switch(method,
                   cv=range(iteration_error(gbm_fit_obj, 'train'),
                       iteration_error(gbm_fit_obj, 'cv')),
                   test=range(iteration_error(gbm_fit_obj, 'train'),
                       iteration_error(gbm_fit_obj, 'valid')),
                   OOB=range(iteration_error('train')))
  } else {
        ylim <- range(iteration_error(gbm_fit_obj, 'train'),
                      iteration_error(gbm_fit_obj, 'valid'))
  }
  
  # Initial plot
  plot(iteration_error(gbm_fit_obj, 'train'),
       ylim=ylim,
       type="l",
       xlab="Iteration", ylab=ylab, main=main)
  
  if(gbm_fit_obj$params$train_fraction != 1) {
    lines(iteration_error(gbm_fit_obj, 'valid'), col="red")
  }
  if(method=="cv") {
    lines(iteration_error(gbm_fit_obj, 'cv'), col="green")
  }
  if(!is.na(best_iter)) abline(v=best_iter,col="blue",lwd=2,lty=2)
  
  # Plot out of bag curve
  if(out_of_bag_curve)
    plot_oobag(gbm_fit_obj, best_iter, overlay, ylab)
}


########## HELPER FUNCTIONS FOR PLOTTING ############
get_ylabel <- function(distribution) {
  UseMethod("get_ylabel", distribution)
}

#' @export
get_ylabel.default <- function(distribution) {
  stop("distribution object not recognised - cannot get y label for plot")
}

#' @export
get_ylabel.AdaBoostGBMDist <-function(distribution) {
  return("AdaBoost exponential bound")
}
#' @export
get_ylabel.BernoulliGBMDist <- function(distribution) {
  return("Bernoulli deviance")
}
#' @export
get_ylabel.CoxPHGBMDist <- function(distribution) {
  return("Cox partial deviance")
}
#' @export
get_ylabel.GammaGBMDist <- function(distribution) {
  return("Gamma deviance")
}
#' @export
get_ylabel.GaussianGBMDist <- function(distribution) {
  return("Squared error loss")
}
#' @export
get_ylabel.HuberizedGBMDist <- function(distribution) {
  return("Hinged loss")
}
#' @export
get_ylabel.LaplaceGBMDist <- function(distribution) {
  return("Absolute loss")
}
#' @export
get_ylabel.PairwiseGBMDist <- function(distribution) {
  ylab <- switch(distribution$metric,
                 conc ="Fraction of concordant pairs",
                 ndcg="Normalized discounted cumulative gain",
                 map ="Mean average precision",
                 mrr ="Mean reciprocal rank")
  return(ylab)
}
#' @export
get_ylabel.PoissonGBMDist <- function(distribution) {
  return("Poisson deviance")
}
#' @export
get_ylabel.QuantileGBMDist <- function(distribution) {
  return("Quantile loss")
}
#' @export
get_ylabel.TDistGBMDist <- function(distribution) {
  return("t-distribution deviance")
}
#' @export
get_ylabel.TweedieGBMDist <- function(distribution) {
  return("Tweedie deviance")
}


plot_oobag <- function(gbm_fit_obj, best_iter, overlay, ylab) {
  # Get smoother
  smoother <- generate_smoother_oobag(gbm_fit_obj)
  
  # Plot smoothed out of bag improvement
  if(overlay) {
    par(new=TRUE)
    plot(smoother$x,
         cumsum(smoother$y),
         col="blue",
         type="l",
         xlab="",ylab="",
         axes=FALSE)
    axis(4,srt=0)
    at <- mean(range(smoother$y))
    mtext(paste("OOB improvement in",ylab),side=4,srt=270,line=2)
    abline(h=0,col="blue",lwd=2)
  }
  
  # Plot original out of bag improvement
  plot(gbm_fit_obj$oobag.improve,type="l",
       xlab="Iteration",
       ylab=paste("OOB change in",ylab))
  lines(smoother,col="red",lwd=2)
  abline(h=0,col="blue",lwd=1)
  abline(v=best_iter,col="blue",lwd=1)
}

#' Marginal plots of fitted gbm objects
#' 
#' Plots the marginal effect of the selected variables by
#' "integrating" out the other variables.
#' 
#' \code{plot.GBMFit} produces low dimensional projections of the
#' \code{GBMFit} object, see \code{\link{gbmt}}, by integrating out
#' the variables not included in the \code{var_index} argument. The
#' function selects a grid of points and uses the weighted tree
#' traversal method described in Friedman (2001) to do the
#' integration. Based on the variable types included in the
#' projection, \code{plot_gbmt} selects an appropriate display
#' choosing amongst line plots, contour plots, and
#' \code{\link[lattice]{lattice}} plots. If the default graphics are
#' not sufficient the user may set \code{return.grid=TRUE}, store the
#' result of the function, and develop another graphic display more
#' appropriate to the particular example.
#' 
#' @param x a \code{GBMFit} object fitted using a call to
#' \code{\link{gbmt}}
#' 
#' @param var_index a vector of indices or the names of the variables
#' to plot. If using indices, the variables are indexed in the same
#' order that they appear in the initial \code{gbmt} formula.  If
#' \code{length(var_index)} is between 1 and 3 then \code{plot.GBMFit}
#' produces the plots. Otherwise, \code{plot.GBMFit} returns only the
#' grid of evaluation points and their average predictions
#'
#' @param num_trees the number of trees used to generate the
#' plot. Only the first \code{num_trees} trees will be used
#'
#' @param continuous_resolution The number of equally space points at
#' which to evaluate continuous predictors
#'
#' @param grid_levels A list containing the points at which to
#' evaluate each predictor in \code{var_index} (in the same order as
#' \code{var_index}). For continuous predictors this is usually a
#' regular sequence of values within the range of the variable. For
#' categorical predictors, the points are the levels of the
#' factor. When \code{length(var_index)} is one, the values can be
#' provided directly, outside a list.  This is NULL by default and
#' generated automatically from the data, using
#' \code{continuous_resolution} for continuous predictors. Forcing the
#' values can be useful to evaluate two models on the same exact range
#'
#' @param return_grid if \code{TRUE} then \code{plot.GBMFit} produces no
#' graphics and only returns the grid of evaluation points and their
#' average predictions. This is useful for customizing the graphics
#' for special variable types or for dimensions greater than 3
#'
#' @param type the type of prediction to plot on the vertical
#' axis. See \code{predict_gmt}
#'
#' @param \dots other arguments passed to the plot function
#'
#' @return Nothing unless \code{return_grid} is true then
#' \code{plot.GBMFit} produces no graphics and only returns the grid of
#' evaluation points and their average predictions.
#'
#' @seealso \code{\link{gbmt}}, \code{\link[graphics]{plot}}
#' @references J.H. Friedman (2001). "Greedy Function Approximation: A Gradient
#' Boosting Machine," Annals of Statistics 29(4).
#' @keywords hplot
#' @export 
plot.GBMFit <- function(x,
                        var_index=1,
                        num_trees=gbm_fit_obj$params$num_trees,
                        continuous_resolution=100,
                        grid_levels=NULL,
                        return_grid=FALSE,
                        type="link",
                        ...)
{
  gbm_fit_obj <- x
  # Check inputs
  if (!is.element(type, c("link", "response"))){
    stop( "type must be either 'link' or 'response'")
  }
  
  if(all(is.character(var_index))) {
    i <- match(var_index, gbm_fit_obj$variables$var_names)
    if(any(is.na(i))) {
      stop("Plot variables not used in gbm model fit: ", var_index[is.na(i)])
    } else {
      var_index <- i
    }
  }
  
  if((min(var_index)<1) || (max(var_index) > length(gbm_fit_obj$variables$var_names))) {
    warning("var_index must be between 1 and ", length(gbm_fit_obj$variables$var_names))
  }
  if(num_trees > gbm_fit_obj$params$num_trees) {
    warning(paste("num_trees exceeds the number of trees in the model, ", 
                  gbm_fit_obj$params$num_trees,
                  ". Plotting using ", gbm_fit_obj$variables$num_trees," trees.",sep=""))
    num_trees <- gbm_fit_obj$params$num_trees
  }
  
  if(length(var_index) > 3) {
    warning("gbm.int.plot creates up to 3-way interaction plots.\nplot.GBMFit will only return the plotting data structure.")
    return_grid = TRUE
  }
  
  # generate grid to evaluate gbm model
  if (is.null(grid_levels)) {
   grid_levels <- get_default_grid_levels(gbm_fit_obj, var_index, continuous_resolution)
  } else {
    grid_levels <- generate_grid_levels(grid_levels, gbm_fit_obj, var_index)
  }
  
  # Expand the grid of variables
  X <- expand.grid(grid_levels)
  names(X) <- paste("X", seq_len(length(var_index)),sep="")

  # Next if block for compatibility with objects created with 1.6
  if(is.null(gbm_fit_obj$num.classes)) {
    gbm_fit_obj$num.classes <- 1
  }
  
  # evaluate at each data point
  y <- .Call("gbm_plot",
             X = data.matrix(X),
             i.var = as.integer(var_index-1),
             num_trees = as.integer(num_trees) ,
             initF = as.double(gbm_fit_obj$initF),
             trees = trees(gbm_fit_obj),
             c.splits = gbm_fit_obj$c.splits,
             var.type = as.integer(gbm_fit_obj$variables$var_type),
             PACKAGE = "gbm3")
  
  if(type=="response") {
   X$y <- response(y, gbm_fit_obj$distribution)
  } else { 
    X$y <- y 
  }
  
  # transform categorical variables back to factors
  f.factor <- rep(FALSE, length(var_index))
  for(i in seq_len(length(var_index))) {
    if(!is.numeric(gbm_fit_obj$variables$var_levels[[var_index[i]]])) {
      X[,i] <- factor(gbm_fit_obj$variables$var_levels[[var_index[i]]][X[,i]+1],
                      levels=gbm_fit_obj$variables$var_levels[[var_index[i]]])
      f.factor[i] <- TRUE
    }
  }
  
  # Stop if grid is returned
  if(return_grid) {
    names(X)[seq_len(length(var_index))] <- gbm_fit_obj$variables$var_names[var_index]
    return(X)
  }
  
  # create the plots
  if(length(var_index)==1) {
      ylabel <- paste("f(", gbm_fit_obj$variables$var_names[var_index],")",sep="")
      if(type=="response") ylabel <- get_ylabel_one_var(gbm_fit_obj$distribution)
      plot(X$X1,X$y,
          type="l",
          xlab=gbm_fit_obj$variables$var_names[var_index],
          ylab=ylabel, ...)
 
  } else if(length(var_index)==2) {
    select_two_var_plot(f.factor, X, gbm_fit_obj, var_index, ...)
  } else if(length(var_index)==3) {
    select_three_var_plot(f.factor, X, gbm_fit_obj, var_index, ...)
  }
}


##### Helper Functions #####
get_default_grid_levels <- function(gbm_fit_obj, var_index, continuous_resolution) {
  # Vector of lists is output
  grid_levels <- vector("list",length(var_index))
  
  # Loop over variables and get levels
  for(i in seq_len(length(var_index))) {
    # continuous
    if(is.numeric(gbm_fit_obj$variables$var_levels[[var_index[i]]])) {
      grid_levels[[i]] <- seq(min(gbm_fit_obj$variables$var_levels[[var_index[i]]]),
                              max(gbm_fit_obj$variables$var_levels[[var_index[i]]]),
                              length=continuous_resolution)
    }
    # categorical or ordered - convert to appropriate numerics
    else {
      grid_levels[[i]] <- as.numeric(factor(gbm_fit_obj$variables$var_levels[[var_index[i]]],
                                            levels=gbm_fit_obj$variables$var_levels[[var_index[i]]]))-1
    }
  }
  
  return(grid_levels)
}

generate_grid_levels <- function(grid_levels, gbm_fit_obj, var_index) {
  
  # allow grid.levels to not be a list when there is only one predictor
  if (length(var_index) == 1 & !is.list(grid_levels)) {
    grid_levels <- list(grid_levels)
  }
  # check compatibility in length, at least
  if (length(grid_levels) != length(var_index)) {
    stop("Need grid_levels for all variables in var_index")
  }
  # convert levels for categorical predictors into numbers
  for(i in seq_len(length(var_index))) {
    if(!is.numeric(gbm_fit_obj$variables$var_levels[[var_index[i]]])) {
      grid_levels[[i]] <- as.numeric(grid_levels[[i]]) - 1
    }
  }
  
  return(grid_levels)
}

response <- function(resp, dist_obj) {
  UseMethod("response", dist_obj)
}

#' @export
response.default <- function(resp, dist_obj) {
  warning("type 'response' only implemented for 'Bernoulli', 'Poisson', 'Gamma', 'Tweedie', and 'Pairwise'. Ignoring" )
  return(NULL)
}

#' @export
response.BernoulliGBMDist <- function(resp, dist_obj) {
  return(1/(1+exp(-resp)))
}

#' @export
response.GammaGBMDist <- function(resp, dist_obj) {
  return(exp(resp))
}

#' @export
response.PairwiseGBMDist <- function(resp, dist_obj) {
  return(1/(1+exp(-resp)))
}

#' @export
response.PoissonGBMDist <- function(resp, dist_obj) {
  return(exp(resp))
}

#' @export
response.TweedieGBMDist <- function(resp, dist_obj) {
  return(exp(resp))
}

skip_on_valgrind()

####################
# Author: James Hickey
#
# Series of tests to check if gbm plotting functions run correctly
#
####################

context("Testing performance plotting")
test_that("perf_plot runs with all perf methods", {
  # Given a fit object and perf evaluated with all methods
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  best_iter_t <- best_iter_test(fit)
  best_iter_c <- best_iter_cv(fit)
  best_iter_oo <- best_iter_out_of_bag(fit)
  
  # When calling perf-plot
  # Using all 3 methods
  # Then no error is thrown
  expect_error(perf_plot(fit, best_iter_t, FALSE, FALSE, "test", "main"), NA)
  expect_error(perf_plot(fit, best_iter_t, FALSE, TRUE, "test", "main"), NA)
  expect_error(perf_plot(fit, best_iter_t, TRUE, FALSE, "test", "main"), NA)
  expect_error(perf_plot(fit, best_iter_t, TRUE, TRUE, "test", "main"), NA)
  
  expect_error(perf_plot(fit, best_iter_c, FALSE, FALSE, "cv", "main"), NA)
  expect_error(perf_plot(fit, best_iter_c, FALSE, TRUE, "cv", "main"), NA)
  expect_error(perf_plot(fit, best_iter_c, TRUE, FALSE, "cv", "main"), NA)
  expect_error(perf_plot(fit, best_iter_c, TRUE, TRUE, "cv", "main"), NA)
    
  expect_error(perf_plot(fit, best_iter_oo, FALSE, FALSE, "OOB", "main"), NA)
  expect_error(perf_plot(fit, best_iter_oo, FALSE, TRUE, "OOB", "main"), NA)
  expect_error(perf_plot(fit, best_iter_oo, TRUE, FALSE, "OOB", "main"), NA)
  expect_error(perf_plot(fit, best_iter_oo, TRUE, TRUE, "OOB", "main"), NA)
})
test_that("perf_plot throws error if gbm_fit_obj is not of class GBMFit", {
  # Given a fit object and perf evaluated with all methods
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  best_iter_t <- best_iter_test(fit)

  # When calling perf_plot with fit stripped of class
  class(fit) <- "Wrong"
  
  # Then no error is thrown
  expect_error(perf_plot(fit, best_iter_t, FALSE, FALSE, "test", "main"))
})
test_that("perf_plot throws error if out_of_bag_curve is not logical or is na", {
  # Given a fit object and perf evaluated with cv method
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  best_iter_c <- best_iter_cv(fit)
  
  # When calling perf-plot with out_of_bag_curve NA or not logical
  # Then an error is thrown
  expect_error(perf_plot(fit, best_iter_c, out_of_bag_curve=NA, FALSE, "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, out_of_bag_curve=c(TRUE, FALSE), FALSE, "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, out_of_bag_curve=1.5, FALSE, "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, out_of_bag_curve="", FALSE, "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, out_of_bag_curve=NaN, FALSE, "test", "main"))
  
})
test_that("perf_plot throws error if overlay is not logical or is na", {
  # Given a fit object and perf evaluated with cv method
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  best_iter_c <- best_iter_cv(fit)
  
  # When calling perf-plot with out_of_bag_curve NA or not logical
  # Then an error is thrown
  expect_error(perf_plot(fit, best_iter_c, FALSE, overlay=NA, "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, FALSE, overlay=c(TRUE, FALSE), "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, FALSE, overlay=1.5, "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, FALSE, overlay="", "test", "main"))
  expect_error(perf_plot(fit, best_iter_c, FALSE, overlay=NaN, "test", "main"))
})
test_that("get_ylabel returns correct string for distribution objects", {
  # Given distribution objects
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8_a <- gbm_dist("Pairwise", metric="conc")
  dist_8_b <- gbm_dist("Pairwise", metric="map")
  dist_8_c <- gbm_dist("Pairwise", metric="mrr")
  dist_8_d <- gbm_dist("Pairwise", metric="ndcg")
  dist_9 <- gbm_dist("Poisson")
  dist_10 <- gbm_dist("Quantile")
  dist_11 <- gbm_dist("TDist")
  dist_12 <- gbm_dist("Tweedie")
  
  # When we get the appropriate y_labels for perf_plot
  label_1 <- get_ylabel(dist_1)
  label_2 <- get_ylabel(dist_2)
  label_3 <- get_ylabel(dist_3)
  label_4 <- get_ylabel(dist_4)
  label_5 <- get_ylabel(dist_5)
  label_6 <- get_ylabel(dist_6)
  label_7 <- get_ylabel(dist_7)
  label_8_a <- get_ylabel(dist_8_a)
  label_8_b <- get_ylabel(dist_8_b)
  label_8_c <- get_ylabel(dist_8_c)
  label_8_d <- get_ylabel(dist_8_d)
  label_9 <- get_ylabel(dist_9)
  label_10 <- get_ylabel(dist_10)
  label_11 <- get_ylabel(dist_11)
  label_12 <- get_ylabel(dist_12)
  
  # Then get correct labels
  expect_equal(label_1, "AdaBoost exponential bound")
  expect_equal(label_2, "Bernoulli deviance")
  expect_equal(label_3,"Cox partial deviance")
  expect_equal(label_4, "Gamma deviance")
  expect_equal(label_5, "Squared error loss")
  expect_equal(label_6, "Hinged loss")
  expect_equal(label_7, "Absolute loss")
  expect_equal(label_8_a, "Fraction of concordant pairs")
  expect_equal(label_8_b, "Mean average precision")
  expect_equal(label_8_c, "Mean reciprocal rank")
  expect_equal(label_8_d, "Normalized discounted cumulative gain")
  expect_equal(label_9, "Poisson deviance")
  expect_equal(label_10, "Quantile loss")
  expect_equal(label_11, "t-distribution deviance")
  expect_equal(label_12, "Tweedie deviance")
})
test_that("get_ylabel throws error if passed unrecognised GBMDist class", {
  # Given a GBM dist
  dist <- gbm_dist()
  
  # When GBMDist not recognised
  class(dist) <- c("What is this?", "GBMDist")
  
  # Then error thrown with get_ylabel
  expect_error(get_ylabel(dist))
})

context("Testing plot method for GBMFit")
test_that("Error thrown if type of plot not 'link' or 'response' ", {
  # Given a GBM fit and type not 'link' or 'response'
  type_plot <- "stuff"
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit with unknown type
  # Then error thrown
  expect_error(plot(fit, type=type_plot))
})
test_that("Error thrown if var_index has variable outside range not used in fit", {
  # Given a GBM fit and var_index outside range of predictors
  # Used in fit
  var_ind <- 7
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit with out of range var index
  # Then error thrown
  expect_error(plot(fit, var_index = var_ind))
  
})
test_that("Response throws warning if not Bernoulli, Poisson, Gamma, Pairwise or Tweedie", {
  # Given distribution and response
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("CoxPH")
  dist_3 <- gbm_dist("Gaussian")
  dist_4 <- gbm_dist("Huberized")
  dist_5 <- gbm_dist("Laplace")
  dist_6 <- gbm_dist("Quantile")
  dist_7 <- gbm_dist("TDist")
  resp <- rep(1, 10)
  
  # When response S3 method called
  # Then warning thrown
  expect_warning(response(resp, dist_1))
  expect_warning(response(resp, dist_2))
  expect_warning(response(resp, dist_3))
  expect_warning(response(resp, dist_4))
  expect_warning(response(resp, dist_5))
  expect_warning(response(resp, dist_6))
  expect_warning(response(resp, dist_7))
  
})
test_that("Response returns NULL if not Bernoulli, Poisson, Gamma, Pairwise or Tweedie", {
  # Given distribution and response
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("CoxPH")
  dist_3 <- gbm_dist("Gaussian")
  dist_4 <- gbm_dist("Huberized")
  dist_5 <- gbm_dist("Laplace")
  dist_6 <- gbm_dist("Quantile")
  dist_7 <- gbm_dist("TDist")
  resp <- rep(1, 10)
  
  # When response S3 method called
  # Then returns NULL
  expect_null(response(resp, dist_1))
  expect_null(response(resp, dist_2))
  expect_null(response(resp, dist_3))
  expect_null(response(resp, dist_4))
  expect_null(response(resp, dist_5))
  expect_null(response(resp, dist_6))
  expect_null(response(resp, dist_7))
})
test_that("Response returns correct answer for Bernoulli, Pairwise, Poisson, Gamma and Tweedie", {
  # Given distributions which don't return NULL and response
  dist_1 <- gbm_dist("Bernoulli")
  dist_2 <- gbm_dist("Pairwise")
  dist_3 <- gbm_dist("Poisson")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Tweedie")
  
  resp <- rep(1, 10)
  
  # When response S3 method called
  # Then returns correct answer
  expect_equal(response(resp, dist_1), 1/(1+exp(-resp)))
  expect_equal(response(resp, dist_2), 1/(1+exp(-resp)))
  expect_equal(response(resp, dist_3), exp(resp))
  expect_equal(response(resp, dist_4), exp(resp))
  expect_equal(response(resp, dist_5), exp(resp))
})
test_that("get_default_grid_levels returns answer of correct type and size", {
  # Given a fitted gbm object, var_index and continuous resolution
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  var_index <- 1
  continuous_resolution <- 100
  
  # When default grids generated
  # Then output is of correct type and size
  expect_true(is.list(get_default_grid_levels(fit, var_index, continuous_resolution)))
  expect_equal(length(get_default_grid_levels(fit, var_index, continuous_resolution)), var_index)
})
test_that("generate_grid_levels throws an error if length of grid_levels not same length as var_index", {
  # Given a fitted gbm object, var_index, grid_levels and continuous resolution
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  var_index <- 1
  continuous_resolution <- 100
  grid_levels <- get_default_grid_levels(fit, c(1, 2), continuous_resolution)
  
  # When generate_grid_levels called
  # Then error is thrown
  expect_error(generate_grid_levels(grid_levels, fit, var_index))
})
test_that("warning thrown if num_trees exceeds those in fit and then uses number of trees in fit to plot", {
  # Given a GBM fit and num_trees exceeding those in fit
  num_trees_plot <- 100000 
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=200, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit with too many trees
  # Then warning thrown and uses number of trees in fit
  expect_warning(plot(fit, num_trees=num_trees_plot))
})
test_that("warning thrown if number var indices > 3", {
  # Given a GBM fit and more than 3 var indices
  var_ind <- c(1, 2, 3, 4)
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit with too indices
  # Then warning thrown 
  expect_warning(plot(fit, var_index = var_ind))
})
test_that("return_grid=TRUE returns the grid", {
  # Given a GBM fit and return_grid=TRUE
  grid_return <- TRUE
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit with grid_return returns grid
  # Then output is same
  expect_true(is.list(plot(fit, return_grid=grid_return)))
})
test_that("number of var indices >3 sets return_grid to TRUE", {
  # Given a GBM fit and more than 3 var indices
  var_ind <- c(1, 2, 3, 4)
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit with too indices and return_grid = TRUE
  # Then output is same
  expect_warning(plot(fit, var_index=var_ind, return_grid=FALSE))
  expect_equal(plot(fit, var_index = var_ind, return_grid=FALSE),  plot(fit, var_index = var_ind, return_grid=TRUE))
})
test_that("can plot with one variable selected", {
  # Given a GBM fit 
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit  - 1 variable
  # Then no Error thrown 
  expect_error(plot(fit), NA)
})
test_that("can correctly get one variable y-label", {
  # Given an object for each distribution
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Pairwise")
  dist_9 <- gbm_dist("Poisson")
  dist_10 <- gbm_dist("Quantile")
  dist_11 <- gbm_dist("TDist")
  dist_12 <- gbm_dist("Tweedie")
  
  # When y-label retreived
  # Then correct
  expect_equal(get_ylabel_one_var(dist_1), "")
  expect_equal(get_ylabel_one_var(dist_2), "Predicted Probability")
  expect_equal(get_ylabel_one_var(dist_3), "")
  expect_equal(get_ylabel_one_var(dist_4), "")
  expect_equal(get_ylabel_one_var(dist_5), "")
  expect_equal(get_ylabel_one_var(dist_6), "")
  expect_equal(get_ylabel_one_var(dist_7), "")
  expect_equal(get_ylabel_one_var(dist_8), "Predicted Probability")
  expect_equal(get_ylabel_one_var(dist_9), "Predicted Count")
  expect_equal(get_ylabel_one_var(dist_10), "")
  expect_equal(get_ylabel_one_var(dist_11), "")
  expect_equal(get_ylabel_one_var(dist_12), "")
})
test_that("can plot with two variables", {
  # Given a GBM fit and two variables selected - all combinations
  var_1 <- c(1, 2)
  var_2 <- c(1, 3)
  var_3 <- c(3, 5)
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit  - 2  variables
  # Then no Error thrown 
  expect_error(plot(fit, var_index = var_1), NA)
  expect_error(plot(fit, var_index = var_2), NA)
  expect_error(plot(fit, var_index = var_3), NA)
})
test_that("can plot with 3 variables selected", {
  # Given a GBM fit and 3 variables selected - all combinations
  var_1 <- c(1, 2, 6)
  var_2 <- c(1, 3, 2)
  var_3 <- c(4, 3, 1)
  var_4 <- c(3, 4, 5)
  
  ## test Gaussian distribution gbm model
  set.seed(1)
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=25)] <- NA
  X3[sample(1:N,size=25)] <- NA
  
  w <- rep(1,N)
  offset <- rep(0, N)
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  
  # Set up for new API
  params <- training_params(num_trees=20, interaction_depth=3, min_num_obs_in_node=10, 
                            shrinkage=0.005, bag_fraction=0.5, id=seq(nrow(data)), num_train=N/2, num_features=6)
  dist <- gbm_dist("Gaussian")
  
  fit <- gbmt(Y~X1+X2+X3+X4+X5+X6, data=data, distribution=dist, weights=w, offset=offset,
              train_params=params, var_monotone=c(0, 0, 0, 0, 0, 0), keep_gbm_data=TRUE, cv_folds=10, is_verbose=FALSE)
  
  # When plotting fit  - 3 variables
  # Then no Error thrown 
  expect_error(plot(fit, var_index = var_1), NA)
  expect_error(plot(fit, var_index = var_2), NA)
  expect_error(plot(fit, var_index = var_3), NA)
  expect_error(plot(fit, var_index = var_4), NA)
})

# Extract observations in fold
# 
# Extract the relevant observations to fit a CV fold - reorders appropriately.
# 
# @usage extract_obs_in_fold(gbm_data_obj, gbm_dist_obj, train_params, cv_groups, fold_num, is_verbose)
# 
# @param gbm_data_obj a GBMData object containing all of the data used to fit a gbm model. 
# 
# @param gbm_dist_obj a GBMDist object specifying the distribution and any additional parameters needed.
# 
# @param train_params a GBMTrainParams object containing generic parameters defining how the model should be
# trained.
# 
# @param cv_groups vector of integers specifying which folds each row of data belongs to.
# 
# @param fold_num integer (>=1) specifying which fold under consideration.
#
# @param is_verbose if TRUE, will print out progress and performance of the fitting.
# 
# @return list of input gbm objects updated so cross_validated data is ready for gbm_call
# 

extract_obs_in_fold <- function(gbm_data_obj, gbm_dist_obj, train_params, cv_groups, fold_num) {
  # Observations in the training set
  obs_in_training_set <- train_params$id %in% as.integer(names(train_params$num_rows_per_obs[seq_len(train_params$num_train)]))
  train_params$id <- train_params$id[obs_in_training_set]
  
  # Observations in cv_group
  obs_id_in_cv_group <- train_params$id %in% as.integer(names(train_params$num_rows_per_obs[seq_len(train_params$num_train)]))[(cv_groups == fold_num)]

  # Extract relevent data - split into training and validation sets 
  # Calculate new number of training rows and observations
  train_params$num_train <- length(unique(train_params$id[!obs_id_in_cv_group]))
  train_params$num_train_rows <- length(train_params$id[!obs_id_in_cv_group])

  gbm_data_obj <- split_and_join(gbm_data_obj, train_params, obs_in_training_set, obs_id_in_cv_group)
  gbm_dist_obj <- update_fold_dist_data(gbm_dist_obj, gbm_data_obj, train_params, obs_in_training_set, obs_id_in_cv_group)
  
  # NB: Update train_params id afterwards
  train_params$id <- train_params$id[!obs_id_in_cv_group]
  
  return(list("data"=gbm_data_obj, "dist"=gbm_dist_obj, "params"=train_params))
}

#' GBM Distribution
#' 
#' Generates distribution object for gbmt.
#' 
#' @usage gbm_dist(name="Gaussian", ...)
#' 
#' @param name The name (a string) of the distribution to be initialized and used
#' in fitting a gradient boosted model via gbmt.  The current distributions
#' available can be viewed using the function \code{available_distributions}.  If no 
#' distribution is specified this function constructs a Gaussian distribution by
#' default.
#' 
#' @param ...  Extra named parameters required for initializing certain distributions.  
#' If t-distribution is selected, an additional parameter (\code{df}) specifying the number of 
#' degrees of freedom can be given.  The default degrees of freedom is set to four.  
#' 
#' If quantile is selected then the quantile to estimate may be specified using the
#' named parameter \code{alpha}.  The default quantile to estimate is 0.25.
#' 
#' If the tweedie distribution is selected the power-law specifying the distribution
#' may be set via the named parameter: \code{power}.  This parameter defaults to unity.
#' 
#' If a Cox Partial Hazards model is selected a number of additional parameters are
#' required, these are:
#' 
#' \describe{
#'  \item{\code{strata}}{A vector of integers (or factors) specifying which strata each data-row belongs to, 
#'   if none is specified it is assumed all training data is in the same stratum.}
#'   \item{\code{ties}}{String specifying the method to be used when dealing with tied
#'   event times.  Currently only "breslow" and "efron" are available, with the latter
#'   being the default.}
#'   \item{\code{prior_node_coeff_var}}{It is a prior on the coefficient of variation associated with the hazard 
#' rate assigned to each terminal node when fitting a tree. Increasing its value emphasizes 
#' the importance of the training data in the node when assigning a prediction to said node. This defaults to 1000.}
#' }
#' 
#' Finally, if the pairwise distribution is selected a number of parameters also need to be
#' specified. These parameters are \code{group}, \code{metric} and \code{max_rank}.
#' The first is a character vector  with the column names of data that jointly indicate the group 
#' an instance belongs to (typically a query in Information Retrieval).  For
#' training, only pairs of instances from the same group and with different target
#' labels may be considered. \code{metric} is
#' the IR measure to use, one of
#' \describe{
#'   \item{list("conc")}{Fraction of concordant pairs; for binary labels,
#'    this is equivalent to the Area under the ROC Curve}
#'   \item{:}{Fraction of concordant pairs; for binary labels, this
#' is equivalent to the Area under the ROC Curve}
#'   \item{list("mrr")}{Mean reciprocal rank of the highest-ranked positive instance}
#'   \item{:}{Mean reciprocal rank of the highest-ranked positive instance}
#'   \item{list("map")}{Mean average precision, a generalization of \code{mrr}
#'   to multiple positive instances}
#'   \item{:}{Mean average precision, a generalization of \code{mrr} to multiple
#'    positive instances}
#'   \item{list("ndcg:")}{Normalized discounted cumulative gain.
#' The score is the weighted sum (DCG) of the user-supplied target values, weighted by
#' log(rank+1), and normalized to the maximum achievable value. This is the
#' default if the user did not specify a metric.}
#' }
#' 
#' \code{ndcg} and \code{conc} allow arbitrary target values, while binary
#' targets \{0,1\} are expected for \code{map} and \code{mrr}. For \code{ndcg}
#' and \code{mrr}, a cut-off can be chosen using a positive integer parameter
#' \code{max_rank}. If left unspecified, all ranks are taken into account.
#' 
#' @author James Hickey
#' 
#' @return returns a \code{GBMDist} object.
#' 
#' @export
#' 

gbm_dist <- function(name="Gaussian", ...) {
  # Check if name is available
  if(!(name %in% available_distributions())) {
    stop("The distribution ", name, " is not available in the gbm package.")
  }
  
  # Get a skeleton distribution object
  bare_distribution <- empty_distribution(name)
  
  # Create object and return
  return(create_dist(bare_distribution, ...))
}

####################
# Author: James Hickey
#
# Series of tests to check if prediction scales are adjusted correctly.
#
####################

context("Testing adjust_pred_scale")
test_that("AdaBoost predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("AdaBoost")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, 1/(1 + exp(-2 * preds)))
})
test_that("Bernoulli predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Bernoulli")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, 1/(1 + exp(-preds)))
})
test_that("CoxPH predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("CoxPH")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, preds)
})
test_that("Gamma predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Gamma")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, exp(preds))
})
test_that("Gaussian predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Gaussian")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, preds)
})
test_that("Huberized predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Huberized")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, preds)
})
test_that("Laplace predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Laplace")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, preds)
})
test_that("Pairwise predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Pairwise")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, 1/(1 + exp(-preds)))
})
test_that("Poisson predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Poisson")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, exp(preds))
})
test_that("Quantile predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Quantile")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, preds)
})
test_that("TDist predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("TDist")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds, preds)
})
test_that("Tweedie predictions are adjusted correctly", {
  # Given a set of predictions and the appropriate distribution obj
  N <- 100
  preds <- runif(N)
  dist <- gbm_dist("Tweedie")
  
  # When predictions are adjusted
  adj_preds <- adjust_pred_scale(preds, dist)
  
  # Then they are correctly adjusted
  expect_equal(adj_preds,  exp(preds))
})

####################
# Author: James Hickey
#
# Series of tests for get_misc functionality
#
####################

context("Testing ability to get misc for a distribution object")
test_that("Error thrown when passed object not GBMDist class", {
  # Given a distribution object
  dist <- gbm_dist()
  
  # When it has its class removed
  class(dist) <- ""
  
  # Then error is thrown on get_misc
  expect_error(get_misc(dist))
})
test_that("Error thrown for unrecognised distribution object", {
  # Given a distribution object
  dist <- gbm_dist()
  
  # When it has an unrecognised class
  class(dist) <- c("WeirdGBMDist", "GBMDist")
  
  # Then error is thrown on get_misc
  expect_error(get_misc(dist))
})
test_that("get_misc returns a list", {
  # Given a distribution object
  # for each distribution
  dist_1 <- gbm_dist("AdaBoost")
  dist_2 <- gbm_dist("Bernoulli")
  dist_3 <- gbm_dist("CoxPH")
  dist_4 <- gbm_dist("Gamma")
  dist_5 <- gbm_dist("Gaussian")
  dist_6 <- gbm_dist("Huberized")
  dist_7 <- gbm_dist("Laplace")
  dist_8 <- gbm_dist("Pairwise")
  dist_9 <- gbm_dist("Poisson")
  dist_10 <- gbm_dist("Quantile")
  dist_11 <- gbm_dist("TDist")
  dist_12 <- gbm_dist("Tweedie")
    
  # When it misc is gotten
  # Then returns a list
  expect_true(is.list(get_misc(dist_1)))
  expect_true(is.list(get_misc(dist_2)))
  expect_true(is.list(get_misc(dist_3)))
  expect_true(is.list(get_misc(dist_4)))
  expect_true(is.list(get_misc(dist_5)))
  expect_true(is.list(get_misc(dist_6)))
  expect_true(is.list(get_misc(dist_7)))
  expect_true(is.list(get_misc(dist_8)))
  expect_true(is.list(get_misc(dist_9)))
  expect_true(is.list(get_misc(dist_10)))
  expect_true(is.list(get_misc(dist_11)))
  expect_true(is.list(get_misc(dist_12)))
  
})
test_that("Can get misc for AdaBoost", {
  # Given an AdaBoost dist
  dist <- gbm_dist("AdaBoost")
  
  # Then get_misc finds NA
  expect_true(is.na(get_misc(dist)))
})
test_that("Can get misc for Bernoulli", {
  # Given an Bernoulli dist
  dist <- gbm_dist("Bernoulli")
  
  # Then get_misc finds NA
  expect_true(is.na(get_misc(dist)))
})
test_that("Can get misc for CoxPH", {
  # Given a CoXPH distribution
  dist <- gbm_dist("CoxPH", ties="breslow")
  
  # When get misc
  misc <- get_misc(dist)
  
  # Then it is the ties
  expect_equal(misc$ties, "breslow")
})
test_that("Can get misc for Gamma", {
  # Given an Gamma dist
  dist <- gbm_dist("Gamma")
  
  # Then get_misc finds NA
  expect_true(is.na(get_misc(dist)))
})
test_that("Can get misc for Gaussian", {
  # Given an Gaussian dist
  dist <- gbm_dist("Gaussian")
  
  # Then get_misc finds NA
  expect_true(is.na(get_misc(dist)))
})
test_that("Can get misc for Huberized", {
  # Given an Huberized dist
  dist <- gbm_dist("Huberized")
  
  # Then get_misc finds NA
  expect_true(is.na(get_misc(dist)))
})
test_that("Can get misc for Laplace", {
  # Given an Laplace dist
  dist <- gbm_dist("Laplace")
  
  # Then get_misc finds NA
  expect_true(is.na(get_misc(dist)))
})
test_that("Can get misc for Pairwise", {
  # Given a pairwise dist
  group <- "query"
  max_rank <- 2
  dist <- gbm_dist("Pairwise", group="query", max_rank=max_rank)
  
  # When misc is gotten
  misc <- get_misc(dist)
  
  # Then it is a c(group, max_rank) in a list
  expect_equal(misc$GroupsAndRanks, c(group, max_rank))
})
test_that("Can get misc for Poisson", {
  # Given an Poisson dist
  dist <- gbm_dist("Poisson")
  
  # Then get_misc finds NA
  expect_true(is.na(get_misc(dist)))
})
test_that("Can get misc for Quantile", {
  # Given a Quantile dist
  dist <- gbm_dist("Quantile", alpha=0.25)
  
  # When misc gotten
  misc <- get_misc(dist)
  
  # Then misc is alpha
  expect_equal(misc$alpha, 0.25)
  
})
test_that("Can get misc for TDist", {
  # Given a TDist dist
  df <- 4
  dist <- gbm_dist("TDist", df=df)
  
  # When misc is gotten
  misc <- get_misc(dist)
  
  # Then it is df
  expect_equal(misc$df, df)
})
test_that("Can get misc for Tweedie", {
  # Given a Tweedie distribution
  power <- 1.7
  dist <- gbm_dist("Tweedie", power=power)
  
  # When misc is gotten
  misc <- get_misc(dist)
  
  # Then it is the power of the dist
  expect_equal(misc$power, power)
})

#' Perform additional boosting
#' 
#' Method to perform additional boosting using a \code{GBMFit} object
#' - does not support further cross validation.
#' 
#' @param gbm_fit_obj a \code{GBMFit} object produced using
#' \code{\link{gbmt}}.  This object describes the boosted model on
#' which to perform additional boosting.
#' 
#' @param num_new_trees a positive integer specifying how many
#' additional iterations to perform.  This has a default value of
#' \code{100}.
#' 
#' @param data a \code{data.frame} or \code{matrix} containing the new
#' values for the predictor and response variables for the additional
#' iterations.  The names of the variables must match those appearing
#' in the original fit (as well as the number of rows), and this value
#' defaults to \code{NULL}.  With a value of \code{NULL} the original
#' data may be used for the additional boosting, if no original or new
#' data is specified an error will be thrown.
#' 
#' @param weights an atomic vector of doubles specifying the
#' importance of each row of the \code{data} in the additional
#' iterations. If the previous data used is kept within
#' \code{gbm_fit_obj}; then the weights are extracted from the stored
#' \code{GBMData} object.
#' 
#' @param offset an atomic vector of doubles specifying the offset for
#' each response value in the data used for additional boosting.
#' 
#' @param is_verbose a logical specifying whether or not the
#' additional fitting should run "noisely" with feedback on progress
#' provided to the user.
#' 
#' @return the input \code{GBMFit} object with additional iterations
#' provided for the fit.
#' 
#' @export 
gbm_more <- function(gbm_fit_obj, num_new_trees=100,
                     data=NULL, weights=NULL, offset=NULL,
                     is_verbose=FALSE){
  the_call <- match.call()
  
  # Check inputs
  check_if_gbm_fit(gbm_fit_obj)
  check_if_natural_number(num_new_trees)

  if(!is.logical(is_verbose) || (length(is_verbose) > 1) || is.na(is_verbose)) {
      stop("is_verbose must be a logical - not NA")
  }
  
  if(is.null(gbm_fit_obj$gbm_data_obj) && is.null(data)) {
      stop("keep_data was set to FALSE on original gbmt call and argument 'data' is NULL")
  }

  if (has_cross_validation(gbm_fit_obj)) {
      warning("gbm_more is incompatible with cross-validation; losing cv results.")
  }
  
  # If no data is stored create appropriate data objects 
  if(is.null(gbm_fit_obj$gbm_data_obj)) {
    m <- eval(gbm_fit_obj$model, parent.frame())
    Terms <- attr(m, "terms")
    a <- attributes(Terms)
    y <- as.vector(model.extract(m, "response"))
    offset <- model.extract(m,offset)
    x <- model.frame(delete.response(Terms),
                     data,
                     na.action=na.pass)
    w <- weights
    
    # If offset or weights NULL change to simple values
    if(length(weights)==0) w <- rep(1, nrow(x))
    if(length(offset)==0) offset <- rep(0, nrow(x))
    
    # Create gbm_data
    gbm_data_obj <- gbm_data(x, y, weights, offset)
    
    # Get distribution and set up groups
    distribution <- determine_groups(colnames(data), gbm_data_obj, gbm_fit_obj$distribution)
    
    # Create strata
    distribution <- create_strata(gbm_data_obj, gbm_fit_obj$params, distribution)
    
    # Process data obj and validate
    gbm_data_obj <- convert_factors(gbm_data_obj)
    gbm_data_obj <- validate_gbm_data(gbm_data_obj, distribution)
    
    # Order the data
    gbm_data_obj <- order_data(gbm_data_obj, distribution, gbm_fit_obj$params)
    
  } else {
    gbm_data_obj <- gbm_fit_obj$gbm_data_obj
    distribution <- gbm_fit_obj$distribution
  }
  
  # Reorder fit so as to be in order of data
  if (distribution_name(gbm_fit_obj) == "Pairwise") 
  {
    gbm_fit_obj$fit   <- gbm_fit_obj$fit[gbm_fit_obj$distribution$group_order] # object$fit is stored in the original order
  } else if (distribution_name(gbm_fit_obj) == "CoxPH") {
    gbm_fit_obj$fit <- gbm_fit_obj$fit[gbm_fit_obj$time_order]
  }

  # Call GBM package
  gbm_more_fit <-
      .Call("gbm",
            Y=as.matrix(as.data.frame(gbm_data_obj$y)),
            intResponse = as.matrix(cbind(distribution$strata, distribution$sorted)),
            Offset=as.double(gbm_data_obj$offset),
            X=as.matrix(as.data.frame(gbm_data_obj$x)),
            X.order=as.integer(gbm_data_obj$x_order),
            weights=as.double(gbm_data_obj$weights),
            Misc=get_misc(distribution),
            prior.node.coeff.var = ifelse(is.null(distribution$prior_node_coeff_var), as.double(0),
                as.double(distribution$prior_node_coeff_var)),
            id = as.integer(gbm_fit_obj$params$id),
            var.type=as.integer(gbm_fit_obj$variables$var_type),
            var.monotone=as.integer(gbm_fit_obj$variables$var_monotone),
            distribution=gbm_call_dist_name(distribution),
            n.trees=as.integer(num_new_trees),
            interaction.depth=as.integer(gbm_fit_obj$params$interaction_depth),
            n.minobsinnode=as.integer(gbm_fit_obj$params$min_num_obs_in_node),
            shrinkage=as.double(gbm_fit_obj$params$shrinkage),
            bag.fraction=as.double(gbm_fit_obj$params$bag_fraction),
            nTrainRows=as.integer(gbm_fit_obj$params$num_train_rows),
            nTrainObs = as.integer(gbm_fit_obj$params$num_train),
            mFeatures=as.integer(gbm_fit_obj$params$num_features),
            fit.old=as.double(gbm_fit_obj$fit),
            n.cat.splits.old=as.integer(length(gbm_fit_obj$c.splits)),
            n.trees.old=length(trees(gbm_fit_obj)),
            par_details=gbm_fit_obj$par_details,
            verbose=as.integer(is_verbose),
            PACKAGE = "gbm3")
  
  # Set class
  class(gbm_more_fit) <- "GBMFit"
  
  # Store correct parameters
  gbm_more_fit$distribution <- distribution
  gbm_more_fit$params <- gbm_fit_obj$params
  gbm_more_fit$variables <- gbm_fit_obj$variables
  gbm_more_fit$Terms     <- gbm_fit_obj$Terms
  
  # Transfer old results across
  gbm_more_fit$initF         <- gbm_fit_obj$initF
  gbm_more_fit$train.error   <- c(gbm_fit_obj$train.error,
                                  gbm_more_fit$train.error)
  gbm_more_fit$valid.error   <- c(gbm_fit_obj$valid.error,
                                  gbm_more_fit$valid.error)
  gbm_more_fit$oobag.improve <- c(gbm_fit_obj$oobag.improve,
                                  gbm_more_fit$oobag.improve)
  gbm_more_fit$trees         <- c(trees(gbm_fit_obj),
                                  gbm_more_fit$trees)
  gbm_more_fit$c.splits      <- c(gbm_fit_obj$c.splits,
                                  gbm_more_fit$c.splits)
  gbm_more_fit$params$num_trees <- length(gbm_more_fit$trees)
    
  # Compatibility with 1.6
  gbm_more_fit$num.classes <- gbm_fit_obj$num.classes

  # Reorder if necessary 
  gbm_more_fit <- reorder_fit(gbm_more_fit, distribution)

  # Store data 
  if(!is.null(gbm_fit_obj$gbm_data_obj)) {
    gbm_more_fit$gbm_data_obj <- gbm_fit_obj$gbm_data_obj
  }
  gbm_more_fit$model <- gbm_fit_obj$model
  gbm_more_fit$par_details <- gbm_fit_obj$par_details
  gbm_more_fit$call <- the_call
  
  return(gbm_more_fit)
  
}

#' Training parameters
#' 
#' Class that contains the training parameters for the gbm model
#' 
#' @param num_trees Number of trees used in the fit.
#' 
#' @param interaction_depth Maximum depth of each tree
#' 
#' @param min_num_obs_in_node Minimum number of observations each node
#' in a tree must have.
#' 
#' @param shrinkage shrinkage parameter applied to each tree in the
#' expansion.  Also known as the learning rate or step-size reduction.
#' 
#' @param bag_fraction fraction of independent training observations
#' selected to create the next tree in the expansion.  Introduces
#' randomness in the model fit; if bag_fraction < 1 then running the
#' same model twice will result in similar but different fits.
#' 
#' @param num_train number of obs of data used in training the model.
#' This defaults to the minimum number of  observations allowed - 
#' \code{(2*min_num_obs_in_node + 1)/bag_fraction + 1}.
#' 
#' @param id optional vector of integers, specifying which rows in the
#' data correspond to which observations. Individual observations may
#' have many rows of data associated with them. This defaults to
#' \code{seq_len(num_train)}.  NB: When calling \code{\link{gbmt}} or
#' \code{\link{gbmt_fit}} the id should be the default.
#' 
#' @param num_features number of random features/columns to use in
#' training model.  This defaults to \code{1}.
#' 
#' @return training parameters object
#' 
#' @author James Hickey
#' 
#' @export
training_params <-
    function(num_trees=100, interaction_depth=1,
             min_num_obs_in_node=10, shrinkage=0.001,
             bag_fraction=0.5,
             num_train=(2*min_num_obs_in_node+1)/bag_fraction + 1, 
             id=seq_len(num_train),
             num_features=1) {
  # Check the parameters
  check_if_natural_number(num_trees, "number of trees")
  check_if_natural_number(interaction_depth, "interaction depth")
  check_if_natural_number(min_num_obs_in_node, "minimum number of node obs")
  check_if_natural_number(num_train, "number of training observations")
  check_if_natural_number(num_features, "number of features")
  check_interaction_depth(interaction_depth)
  
  if(!is.atomic(id) || !all(id == as.integer(id))) {
    stop("Observation ids must be a vector of integers")
  }
  
  if(!is.double(shrinkage) || (shrinkage > 1.0) 
     || is.infinite(shrinkage) || (length(shrinkage) > 1)) {
    stop("Shrinkage should be a double < 1.0")
  }
  if(!is.double(bag_fraction) || (bag_fraction > 1.0)
     || (length(bag_fraction) > 1) || (bag_fraction < 0.0)) {
    stop("Bag fraction should be a double between 0.0 and 1.0")
  }
  
  # Order the ids 
  id_order <- order(id)
  num_rows_per_obs <- table(id[order(id)])
  num_train_rows <-  sum(num_rows_per_obs[seq_len(min(num_train, length(num_rows_per_obs)))])

  if(num_train * bag_fraction <= 2*min_num_obs_in_node+1) {
    stop("The dataset size is too small or subsampling rate is too large: num_obs*bag.fraction <= 2*min_num_obs_in_node+1")
  }
  
  if(num_train_rows > sum(num_rows_per_obs)) {
    stop("Number of training rows selected exceeds number available")
  }
  
  if(num_train > num_train_rows) {
    warning("Number of training observations EXCEEDS number of training rows!")
  }
  
  structure(list("num_trees"=num_trees,
                 "interaction_depth"=interaction_depth,
                 "min_num_obs_in_node"=min_num_obs_in_node,
                 "shrinkage"=shrinkage,
                 "bag_fraction"=bag_fraction,
                 "id"= id,
                 "id_order"=id_order,
                 "num_train"=num_train,
                 "num_train_rows"=num_train_rows,
                 "num_features"=num_features,
                 "num_rows_per_obs"=num_rows_per_obs,
                 "train_fraction"= num_train/length(unique(id)) ),
            class="GBMTrainParams")
}

####################
# Author: James Hickey
#
# Series of test to validate the GBMTrainParams objects.
#
####################

##### Definition #####
context("Check the definition of the GBMTrainParams object")
test_that("Training parameters are of class GBMTrainParams", {
  expect_true("GBMTrainParams" %in% class(training_params(num_train=100, id=seq_len(100), num_features = 3)))
})

test_that("GBMTrainParams structure is correct", {
  expect_equal(names(training_params(num_train=100, id=seq_len(100), num_features=3)), 
               c("num_trees", "interaction_depth", "min_num_obs_in_node", "shrinkage",
                 "bag_fraction", "id", "id_order", "num_train", "num_train_rows", "num_features", "num_rows_per_obs", "train_fraction"))
})

test_that("Calculation of number of rows per observation is correct", {
  # Given training obs and ids
  train_no1 <- 100
  train_no2 <- 25
  id1 <- seq_len(100)
  id2 <- rep(1:25, 4)
  
  # Calculates number of rows per observation correctly on construction
  expect_equal(training_params(num_train = train_no1, id=id1, num_features = 3)$num_rows_per_obs, table(id1[order(id1)]))
  expect_equal(training_params(num_train = train_no2, id=id2, num_features = 3, min_num_obs_in_node = 1)$num_rows_per_obs, table(id2[order(id2)]))
})

test_that("Calculation of training fraction of observations is correct", {
  # Given training ids and num_train
  train_no <- 25
  id1 <- seq_len(4 * train_no)
  
  # When training_params called
  # Then correctly calculates train_fraction
  expect_equal(training_params(num_train = train_no, id=id1, bag_fraction=1, num_features = 3)$train_fraction, train_no/length(unique(id1)))
})

test_that("Id order is stored", {
  # Given training id
  id <- sample(1:10, 100, replace=TRUE)
  train_no <- 5
  
  # When passed to train_params
  # Then id order is correctly calculated
  expect_equal(training_params(num_train= train_no, id=id, bag_fraction = 1, num_trees = 3, min_num_obs_in_node = 1)$id_order, order(id))
})

##### Errors #####
context("Using invalid contruction parameters will thrown an error")
test_that("Number of trees must be a positive integer", {
  expect_error(training_params(num_trees=-0.1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(num_trees=-1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(num_trees=c(1, 2), num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(num_trees=Inf, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(num_trees=NULL, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(num_trees=NA, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(num_trees="Incorrect", num_train=100, id=seq_len(100), num_features=3))
})

test_that("Interaction depth must be a positive integer", {
  expect_error(training_params(interaction_depth=-0.1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(interaction_depth=-1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(interaction_depth=c(1, 2), num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(interaction_depth=Inf, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(interaction_depth=NULL, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(interaction_depth=NA, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(interaction_depth="Incorrect", num_train=100, id=seq_len(100), num_features=3))
})

test_that("Minimum number of node observations must be a positive integer", {
  expect_error(training_params(min_num_obs_in_node=-0.1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(min_num_obs_in_node=-1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(min_num_obs_in_node=c(1, 2), num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(min_num_obs_in_node=Inf, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(min_num_obs_in_node=NULL, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(min_num_obs_in_node=NA, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(min_num_obs_in_node="Incorrect", num_train=100, id=seq_len(100), num_features=3))
})

test_that("Number of training rows must be a positive integer", {
  expect_error(training_params(num_train=-0.1, id=seq_len(100), num_features=3))
  expect_error(training_params(num_train=-1,  id=seq_len(100), num_features=3))
  expect_error(training_params(num_train=c(1, 2), id=seq_len(100), num_features=3))
  expect_error(training_params(num_train=Inf,  id=seq_len(100), num_features=3))
  expect_error(training_params(num_train=NULL,  id=seq_len(100), num_features=3))
  expect_error(training_params(num_train=NA,  id=seq_len(100), num_features=3))
  expect_error(training_params(num_train="Incorrect",  id=seq_len(100), num_features=3))
})

test_that("Number of features for tree growing must be a positive integer", {
  expect_error(training_params(num_train=100, id=seq_len(100), num_features=-0.1))
  expect_error(training_params(num_train=100,  id=seq_len(100), num_features=-1))
  expect_error(training_params(num_train=100, id=seq_len(100), num_features=c(1,2)))
  expect_error(training_params(num_train=100,  id=seq_len(100), num_features=Inf))
  expect_error(training_params(num_train=100,  id=seq_len(100), num_features=NULL))
  expect_error(training_params(num_train=100,  id=seq_len(100), num_features=NA))
  expect_error(training_params(num_train=100,  id=seq_len(100), num_features="Incorrect"))
})

test_that("Shrinkage must be a double", {
  expect_error(training_params(shrinkage=c(1, 2), num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(shrinkage=Inf, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(shrinkage=NULL, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(shrinkage=NA, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(shrinkage="Incorrect", num_train=100, id=seq_len(100), num_features=3))
})

test_that("Bag fraction must be a double between 0.0 and 1.0", {
  expect_error(training_params(bag_fraction=-0.1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(bag_fraction=-1, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(bag_fraction=1.5, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(bag_fraction=c(1, 2), num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(bag_fraction=Inf, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(bag_fraction=NULL, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(bag_fraction=NA, num_train=100, id=seq_len(100), num_features=3))
  expect_error(training_params(bag_fraction="Incorrect", num_train=100, id=seq_len(100), num_features=3))
})

test_that("Id must be a vector of integers", {
  expect_error(training_params(num_train=100, id=-0.1, num_features=3))
  expect_error(training_params(num_train=100, id=c(0.1, 0.2, 3), num_features=3))
  expect_error(training_params(num_train=100, id=Inf, num_features=3))
  expect_error(training_params(num_train=100, id="Incorrect", num_features=3))
  expect_error(training_params(num_train=100, id=NA, num_features=3))
  expect_error(training_params(num_train=100, id=NULL, num_features=3))
})

test_that("Too few observations throws error", {
  expect_error(training_params(num_train=1, id=1, num_features=3))
})

test_that("Number of training observations must not exceed maximum amount id'ed", {
  expect_error(training_params(num_train=100, id=rep(1:25, 4), num_feactures=3))
})

# Get Misc Parameters
# 
# S3 method to get the "misc" objects from GBM distributions.
# This method (internal) is used with gbm_call to extract the appropriate "misc"
# for passing to the C++ layer.
# 
# @usage get_misc(distribution_obj)
# 
# @param distribution_obj a GBMDist object
# 
# @author James Hickey
#
# @return a list of additional parameters specific to the distribution
# 

get_misc <- function(distribution_obj) {
  check_if_gbm_dist(distribution_obj)
  UseMethod("get_misc", distribution_obj)
}

#' @export
get_misc.default <- function(distribution_obj) {
  stop("Distribution not recognised - can't get misc")
}

#' @export
get_misc.AdaBoostGBMDist <- function(distribution_obj) {
  return(as.list(NA))
}

#' @export
get_misc.BernoulliGBMDist <- function(distribution_obj) {
  return(as.list(NA))
}

#' @export
get_misc.CoxPHGBMDist <- function(distribution_obj) {
  return(list(ties=distribution_obj$ties))
}

#' @export
get_misc.GammaGBMDist <- function(distribution_obj) {
  return(as.list(NA))
}

#' @export
get_misc.GaussianGBMDist <- function(distribution_obj) {
  return(as.list(NA))
}

#' @export
get_misc.HuberizedGBMDist <- function(distribution_obj) {
  return(as.list(NA))
}

#' @export
get_misc.LaplaceGBMDist <- function(distribution_obj) {
  return(as.list(NA))
}

#' @export
get_misc.PairwiseGBMDist <- function(distribution_obj) {
  return(list("GroupsAndRanks"=c(distribution_obj$group, distribution_obj$max_rank)))
}

#' @export
get_misc.PoissonGBMDist <- function(distribution_obj) {
  return(as.list(NA))
}

#' @export
get_misc.QuantileGBMDist <- function(distribution_obj) {
  return(list(alpha=distribution_obj$alpha))
}

#' @export
get_misc.TDistGBMDist <- function(distribution_obj) {
  return(list(df=distribution_obj$df))
}

#' @export
get_misc.TweedieGBMDist <- function(distribution_obj) {
  return(list(power=distribution_obj$power))
}

#' Generalized Boosted Regression Modeling
#' 
#' Fits generalized boosted regression models.
#' 
#' See the gbm vignette for technical details.
#' 
#' This package implements the generalized boosted modeling
#' framework. Boosting is the process of iteratively adding basis
#' functions in a greedy fashion so that each additional basis
#' function further reduces the selected loss function. This
#' implementation closely follows Friedman's Gradient Boosting Machine
#' (Friedman, 2001).
#' 
#' In addition to many of the features documented in the Gradient
#' Boosting Machine, \code{gbm} offers additional features including
#' the out-of-bag estimator for the optimal number of iterations, the
#' ability to store and manipulate the resulting \code{GBMFit} object,
#' and a variety of other loss functions that had not previously had
#' associated boosting algorithms, including the Cox partial
#' likelihood for censored data, the poisson likelihood for count
#' outcomes, and a gradient boosting implementation to minimize the
#' AdaBoost exponential loss function.
#' 
#' \code{gbm} is a deprecated function that now acts as a front-end to
#' \code{gbmt_fit} that uses the familiar R modeling
#' formulas. However, \code{\link[stats]{model.frame}} is very slow if
#' there are many predictor variables. For power users with many
#' variables use \code{gbm.fit} over \code{gbm}; however \code{gbmt}
#' and \code{gbmt_fit} are now the current APIs.
#' 
#' @param formula a symbolic description of the model to be fit. The
#' formula may include an offset term (e.g. y~offset(n)+x). If
#' \code{keep.data=FALSE} in the initial call to \code{gbm} then it is
#' the user's responsibility to resupply the offset to
#' \code{\link{gbm_more}}.
#' 
#' @param distribution either a character string specifying the name
#' of the distribution to use or a list with a component \code{name}
#' specifying the distribution and any additional parameters
#' needed. If not specified, \code{gbm} will try to guess: if the
#' response has only two unique values, bernoulli is assumed 
#' (two-factor responses are converted to 0,1);
#' otherwise, if the response has class "Surv", coxph is assumed;
#' otherwise, gaussian is assumed.
#' 
#' Available distributions are "gaussian" (squared error), "laplace"
#' (absolute loss), "tdist" (t-distribution loss), "bernoulli"
#' (logistic regression for 0-1 outcomes), "huberized" (Huberized
#' hinge loss for 0-1 outcomes), "adaboost" (the AdaBoost exponential
#' loss for 0-1 outcomes), "poisson" (count outcomes), "coxph" (right
#' censored observations), "quantile", or "pairwise" (ranking measure
#' using the LambdaMART algorithm).
#' 
#' If quantile regression is specified, \code{distribution} must be a
#' list of the form \code{list(name="quantile",alpha=0.25)} where
#' \code{alpha} is the quantile to estimate. Non-constant weights are
#' unsupported.
#' 
#' If "tdist" is specified, the default degrees of freedom is four and
#' this can be controlled by specifying
#' \code{distribution=list(name="tdist", df=DF)} where \code{DF} is
#' your chosen degrees of freedom.
#' 
#' If "pairwise" regression is specified, \code{distribution} must be a list of
#' the form \code{list(name="pairwise",group=...,metric=...,max.rank=...)}
#' (\code{metric} and \code{max.rank} are optional, see below). \code{group} is
#' a character vector with the column names of \code{data} that jointly
#' indicate the group an instance belongs to (typically a query in Information
#' Retrieval applications). For training, only pairs of instances from the same
#' group and with different target labels can be considered. \code{metric} is
#' the IR measure to use, one of
#' \describe{
#'   \item{list("conc")}{Fraction of concordant pairs; for binary labels, this is equivalent to the Area under the ROC Curve}
#'   \item{:}{Fraction of concordant pairs; for binary labels, this
#' is equivalent to the Area under the ROC Curve}
#'   \item{list("mrr")}{Mean reciprocal rank of the highest-ranked positive instance}
#'   \item{:}{Mean reciprocal rank of the highest-ranked positive instance}
#'   \item{list("map")}{Mean average precision, a generalization of \code{mrr}
#'   to multiple positive instances}
#'   \item{:}{Mean average precision, a generalization of \code{mrr} to multiple positive instances}
#'   \item{list("ndcg:")}{Normalized discounted cumulative gain. The score is the weighted sum (DCG) of the user-supplied target values, weighted by
#' log(rank+1), and normalized to the maximum achievable value. This is the
#' default if the user did not specify a metric.}
#' }
#' 
#' \code{ndcg} and \code{conc} allow arbitrary target values, while binary
#' targets \{0,1\} are expected for \code{map} and \code{mrr}. For \code{ndcg}
#' and \code{mrr}, a cut-off can be chosen using a positive integer parameter
#' \code{max.rank}. If left unspecified, all ranks are taken into account.
#' 
#' Note that splitting of instances into training and validation sets follows
#' group boundaries and therefore only approximates the specified
#' \code{train.fraction} ratio (the same applies to cross-validation folds).
#' Internally queries are randomly shuffled before training to avoid bias.
#' 
#' Weights can be used in conjunction with pairwise metrics, however it is
#' assumed that they are constant for instances from the same group.
#' 
#' For details and background on the algorithm, see e.g. Burges (2010).
#'
#' @param data an optional data frame containing the variables in the
#' model. By default the variables are taken from
#' \code{environment(formula)}, typically the environment from which
#' \code{gbm} is called. If \code{keep.data=TRUE} in the initial call
#' to \code{gbm} then \code{gbm} stores a copy with the object. If
#' \code{keep.data=FALSE} then subsequent calls to
#' \code{\link{gbm_more}} must resupply the same dataset. It becomes
#' the user's responsibility to resupply the same data at this point.
#'
#' @param weights an optional vector of weights to be used in the
#' fitting process. The weights must be positive but do not need to be
#' normalized. If \code{keep.data=FALSE} in the initial call to
#' \code{gbm}, then it is the user's responsibility to resupply the
#' weights to \code{\link{gbm_more}}.
#'
#' @param subset an optional vector defining a subset of the data to be used
#'
#' @param offset an optional model offset
#'
#' @param var.monotone an optional vector, the same length as the
#' number of predictors, indicating which variables have a monotone
#' increasing (+1), decreasing (-1), or arbitrary (0) relationship
#' with the outcome.
#'
#' @param n.trees the total number of trees to fit. This is equivalent
#' to the number of iterations and the number of basis functions in
#' the additive expansion.
#'
#' @param cv.folds Number of cross-validation folds to perform. If
#' \code{cv.folds}>1 then \code{gbm}, in addition to the usual fit,
#' will perform a cross-validation and calculate an estimate of
#' generalization error returned in \code{cv_error}.
#'
#' @param interaction.depth The maximum depth of variable
#' interactions: 1 builds an additive model, 2 builds a model with up
#' to two-way interactions, etc.
#'
#' @param n.minobsinnode minimum number of observations (not total
#' weights) in the terminal nodes of the trees.
#'
#' @param shrinkage a shrinkage parameter applied to each tree in the
#' expansion. Also known as the learning rate or step-size reduction.
#'
#' @param bag.fraction the fraction of independent training
#' observations (or patients) randomly selected to propose the next
#' tree in the expansion, depending on the obs.id vector multiple
#' training data rows may belong to a single 'patient'. This
#' introduces randomness into the model fit.  If \code{bag.fraction}<1
#' then running the same model twice will result in similar but
#' different fits. \code{gbm} uses the R random number generator, so
#' \code{set.seed} ensures the same model can be
#' reconstructed. Preferably, the user can save the returned
#' \code{GBMFit} object using \code{\link{save}}.
#'
#' @param train.fraction The first \code{train.fraction * nrows(data)}
#' observations are used to fit the \code{gbm} and the remainder are used for
#' computing out-of-sample estimates of the loss function.
#'
#' @param nTrain An integer representing the number of unique patients,
#' each patient could have multiple rows associated with them, on which
#' to train.  This is the preferred way of specification for
#' \code{gbm.fit}; The option \code{train.fraction} in \code{gbm.fit}
#' is deprecated and only maintained for backward compatibility. These
#' two parameters are mutually exclusive. If both are unspecified, all
#' data is used for training.
#'
#' @param mFeatures Each node will be trained on a random subset of
#' \code{mFeatures} number of features. Each node will consider a new
#' random subset of features, adding variability to tree growth and
#' reducing computation time. \code{mFeatures} will be bounded between
#' 1 and \code{nCols}. Values outside of this bound will be set to the
#' lower or upper limits.
#'
#' @param keep.data a logical variable indicating whether to keep the
#' data and an index of the data stored with the object. Keeping the
#' data and index makes subsequent calls to \code{\link{gbm_more}}
#' faster at the cost of storing an extra copy of the dataset.
#'
#' @param verbose If TRUE, gbm will print out progress and performance
#' indicators. If this option is left unspecified for gbm_more then it uses
#' \code{verbose} from \code{object}.
#' 
#' @param class.stratify.cv whether the cross-validation should be
#' stratified by class. Is only implemented for
#' \code{bernoulli}. The purpose of stratifying
#' the cross-validation is to help avoiding situations in which
#' training sets do not contain all classes.
#'
#' @param x,y For \code{gbm.fit}: \code{x} is a data frame or data
#' matrix containing the predictor variables and \code{y} is a matrix
#' of outcomes.  Excluding \code{coxph} this matrix of outcomes
#' collapses to a vector, in the case of \code{coxph} it is a survival
#' object where the event times fill the first one (or two columns)
#' and the status fills the final column.  The number of rows in
#' \code{x} must be the same as the length of the 1st dimension of
#' \code{y}.
#'
#' @param w For \code{gbm.fit}: \code{w} is a vector of weights of the same
#' length as the 1st dimension of \code{y}.
#' 
#' @param var.names For \code{gbm.fit}: A vector of strings of length
#' equal to the number of columns of \code{x} containing the names of
#' the predictor variables.
#'
#' @param response.name For \code{gbm.fit}: A character string label
#' for the response variable.
#'
#' @param group \code{group} used when \code{distribution =
#' 'pairwise'.}
#' 
#' @param tied.times.method For \code{gbm} and \code{gbm.fit}: This is
#' an optional string used with \code{CoxPH} distribution specifying
#' what method to employ when dealing with tied times.  Currently only
#' "efron" and "breslow" are available; the default value is "efron".
#' Setting the string to any other value reverts the method to the
#' original CoxPH model implementation where ties are not explicitly
#' dealt with.
#' 
#' @param prior.node.coeff.var Optional double only used with the
#' \code{coxph} distribution.  It is a prior on the coefficient of
#' variation associated with the hazard rate assigned to each terminal
#' node when fitting a tree.Increasing its value emphasises the
#' importance of the training data in the node when assigning a
#' prediction to said node.
#'
#' @param strata Optional vector of integers (or factors) only used
#' with the \code{coxph} distributions.  Each integer in this vector
#' represents the stratum the corresponding row in the data belongs
#' to, e. g. if the 10th element is 3 then the 10th data row belongs
#' to the 3rd strata.
#' 
#' @param obs.id Optional vector of integers used to specify which
#' rows of data belong to individual patients.  Data is then bagged by
#' patient id; the default sets each row of the data to belong to an
#' individual patient.
#'
#' @param n.cores number of cores to use for parallelization.  Please
#' use \code{par.details} instead; this argument is only provided for
#' convenience.
#' 
#' @param par.details Details of the parallelization to use in the
#' core algorithm.
#' 
#' @param fold.id An optional vector of values identifying what fold
#' each observation is in. If supplied, cv.folds can be missing. Note:
#' Multiple observations to the same patient must have the same fold
#' id.
#' @return \code{gbm} and \code{gbm.fit} return a \code{GBMFit}
#' object.
#' @author James Hickey, Greg Ridgeway \email{gregridgeway@@gmail.com}
#' 
#' Quantile regression code developed by Brian Kriegler
#' \email{bk@@stat.ucla.edu}
#' 
#' t-distribution code developed by Harry Southworth and
#' Daniel Edwards
#' 
#' Pairwise code developed by Stefan Schroedl \email{schroedl@@a9.com}
#' 
#' @seealso \code{\link{gbmt}}, \code{\link{gbmt_fit}} \code{\link{gbmt_performance}},
#' \code{\link{plot}}, \code{\link{predict.GBMFit}},
#' \code{\link{summary.GBMFit}}, \code{\link{pretty_gbm_tree}}, \code{\link{gbmParallel}}.
#' @references Y. Freund and R.E. Schapire (1997) \dQuote{A decision-theoretic
#' generalization of on-line learning and an application to boosting,}
#' \emph{Journal of Computer and System Sciences,} 55(1):119-139.
#' 
#' G. Ridgeway (1999). \dQuote{The state of boosting,} \emph{Computing Science
#' and Statistics} 31:172-181.
#' 
#' J.H. Friedman, T. Hastie, R. Tibshirani (2000). \dQuote{Additive Logistic
#' Regression: a Statistical View of Boosting,} \emph{Annals of Statistics}
#' 28(2):337-374.
#' 
#' J.H. Friedman (2001). \dQuote{Greedy Function Approximation: A Gradient
#' Boosting Machine,} \emph{Annals of Statistics} 29(5):1189-1232.
#' 
#' J.H. Friedman (2002). \dQuote{Stochastic Gradient Boosting,}
#' \emph{Computational Statistics and Data Analysis} 38(4):367-378.
#' 
#' B. Kriegler (2007). \emph{Cost-Sensitive Stochastic Gradient Boosting
#' Within a Quantitative Regression Framework}. PhD dissertation, UCLA
#' Statistics.
#' 
#' C. Burges (2010). \dQuote{From RankNet to LambdaRank to LambdaMART: An
#' Overview,} Microsoft Research Technical Report MSR-TR-2010-82.
#' 
#' \href{https://gregridgeway.github.io/}{Greg Ridgeway's site}.
#' @keywords models nonlinear survival nonparametric tree
#' @examples
#'  # A least squares regression example # create some data
#' 
#' N <- 1000
#' X1 <- runif(N)
#' X2 <- 2*runif(N)
#' X3 <- ordered(sample(letters[1:4],N,replace=TRUE),levels=letters[4:1])
#' X4 <- factor(sample(letters[1:6],N,replace=TRUE))
#' X5 <- factor(sample(letters[1:3],N,replace=TRUE))
#' X6 <- 3*runif(N) 
#' mu <- c(-1,0,1,2)[as.numeric(X3)]
#' 
#' SNR <- 10 # signal-to-noise ratio
#' Y <- X1**1.5 + 2 * (X2**.5) + mu
#' sigma <- sqrt(var(Y)/SNR)
#' Y <- Y + rnorm(N,0,sigma)
#' 
#' # introduce some missing values
#' X1[sample(1:N,size=500)] <- NA
#' X4[sample(1:N,size=300)] <- NA
#' 
#' data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
#' 
#' # fit initial model
#' gbm1 <-
#' gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
#'     data=data,                   # dataset
#'     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease,
#'                                  # +1: monotone increase,
#'                                  #  0: no monotone restrictions
#'     distribution="gaussian",     # see the help for other choices
#'     n.trees=1000,                # number of trees
#'     shrinkage=0.05,              # shrinkage or learning rate,
#'                                  # 0.001 to 0.1 usually work
#'     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
#'     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
#'     train.fraction = 0.5,        # fraction of data for training,
#'                                  # first train.fraction*N used for training
#'     mFeatures = 3,               # half of the features are considered at each node
#'     n.minobsinnode = 10,         # minimum total weight needed in each node
#'     cv.folds = 3,                # do 3-fold cross-validation
#'     keep.data=TRUE,              # keep a copy of the dataset with the object
#'     verbose=FALSE                # don't print out progress
#' # , par.details=gbmParallel(num_threads=15) # option for gbm3 to parallelize
#'     )                   
#'
#' # check performance using an out-of-bag estimator
#' # OOB underestimates the optimal number of iterations
#' best_iter <- gbmt_performance(gbm1,method="OOB")
#' print(best_iter)
#' 
#' # check performance using a 50% heldout test set
#' best_iter <- gbmt_performance(gbm1,method="test")
#' print(best_iter)
#' 
#' # check performance using 3-fold cross-validation
#' best_iter <- gbmt_performance(gbm1,method="cv")
#' print(best_iter)
#' 
#' # plot the performance # plot variable influence
#' summary(gbm1, num_trees=1)         # based on the first tree
#' summary(gbm1, num_trees=best_iter) # based on the estimated best number of trees
#' 
#' # compactly print the first and last trees for curiosity
#' print(pretty_gbm_tree(gbm1,1))
#' print(pretty_gbm_tree(gbm1,gbm1$params$num_trees))
#' 
#' # make some new data
#' N <- 1000
#' X1 <- runif(N)
#' X2 <- 2*runif(N)
#' X3 <- ordered(sample(letters[1:4],N,replace=TRUE))
#' X4 <- factor(sample(letters[1:6],N,replace=TRUE))
#' X5 <- factor(sample(letters[1:3],N,replace=TRUE))
#' X6 <- 3*runif(N) 
#' mu <- c(-1,0,1,2)[as.numeric(X3)]
#' 
#' Y <- X1**1.5 + 2 * (X2**.5) + mu + rnorm(N,0,sigma)
#' 
#' data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
#' 
#' # predict on the new data using "best" number of trees
#' # f.predict generally will be on the canonical scale (logit,log,etc.)
#' f.predict <- predict(gbm1,data2,best_iter)
#' 
#' # least squares error
#' print(sum((data2$Y-f.predict)^2))
#' 
#' # create marginal plots
#' # plot variable X1,X2,X3 after "best" iterations
#' oldpar <- par(no.readonly = TRUE)
#' par(mfrow=c(1,3))
#' plot(gbm1,1,best_iter)
#' plot(gbm1,2,best_iter)
#' plot(gbm1,3,best_iter)
#' par(mfrow=c(1,1))
#' # contour plot of variables 1 and 2 after "best" iterations
#' plot(gbm1,1:2,best_iter)
#' # lattice plot of variables 2 and 3
#' plot(gbm1,2:3,best_iter)
#' # lattice plot of variables 3 and 4
#' plot(gbm1,3:4,best_iter)
#' 
#' # 3-way plots
#' plot(gbm1,c(1,2,6),best_iter,cont=20)
#' plot(gbm1,1:3,best_iter)
#' plot(gbm1,2:4,best_iter)
#' plot(gbm1,3:5,best_iter)
#' par(oldpar) # reset graphics options to previous settings
#' 
#' # do another 100 iterations
#' gbm2 <- gbm_more(gbm1,100,
#'                  is_verbose=FALSE) # stop printing detailed progress
#' @export
gbm <- function(formula = formula(data),
                distribution = "bernoulli",
                data = list(),
                weights,
                subset = NULL,
                offset = NULL,
                var.monotone = NULL,
                n.trees = 100,
                interaction.depth = 1,
                n.minobsinnode = 10,
                shrinkage = 0.001,
                bag.fraction = 0.5,
                train.fraction = 1.0,
                mFeatures = NULL,
                cv.folds=0,
                keep.data = TRUE,
                verbose = FALSE,
                class.stratify.cv=NULL,
                n.cores=NULL,
                par.details=getOption('gbm.parallel'),
                fold.id = NULL,
                tied.times.method="efron",
                prior.node.coeff.var=1000,
                strata=NA, obs.id=1:nrow(data)) {
  # Extract the model
  the_call <- match.call()
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("formula", "data", "weights", "offset"), names(mf), 0)
  mf <- mf[c(1, m)]
  mf$drop.unused.levels <- TRUE
  mf$na.action <- na.pass
  mf[[1]] <- as.name("model.frame")
  m <- mf
  mf <- eval(mf, parent.frame())
  Terms <- attr(mf, "terms")
  y <- model.response(mf)
  w <- model.weights(mf)
  offset_mf <- model.offset(mf)

  ## allow n.cores for historical reasons
  if (!missing(n.cores)) {
      if (!missing(par.details)) {
          stop("You have provided both n.cores and par.details")
      }
      message("n.cores is deprecated, please use par.details")
      par.details <- gbmParallel(num_threads=n.cores)
  }
  
  # Set stratification to FALSE if NULL
  if(is.null(class.stratify.cv)) class.stratify.cv <- FALSE
  
  # get the character name of the response variable
  var.names <- attributes(Terms)$term.labels
  x <- model.frame(terms(reformulate(var.names)),
                   data,
                   na.action=na.pass)
  
  # Set offset/ weights if not specified
  if(!is.null(w)) {
    weights <- w
  } else {
    weights <- rep(1, length(obs.id))
  } 
  if(!is.null(offset_mf)){
    offset <- offset_mf
  } else {
    offset <- rep(0, length(obs.id))
  }
  
  # Change cv.folds to correct default
  if(cv.folds == 0) cv.folds <- 1
  
  # Set distribution object
  #   put in all possible additional parameters (this will generate warnings)
  if(missing(distribution)) {distribution <- guess_distribution(y)}
  if (is.character(distribution)){ distribution <- list(name=distribution)}
  
  # Extract and call correct gbm_distribution object
  dist_obj <- create_dist_obj_for_gbmt_fit(distribution, tied.times.method, 
                                           strata, prior.node.coeff.var)
  
  # Set up training parameters
  if(is.null(mFeatures)) mFeatures <- ncol(x) 
  params <- training_params(num_trees=n.trees, 
                            interaction_depth=interaction.depth, 
                            min_num_obs_in_node=n.minobsinnode, 
                            shrinkage=shrinkage, 
                            bag_fraction=bag.fraction, 
                            id=obs.id,
                            num_train=round(train.fraction*length(unique(obs.id))),
                            num_features=mFeatures)
  
  # Check and infer folds if necessary
  check_cv_parameters(cv.folds, class.stratify.cv, fold.id, params)
  if (!is.null(fold.id)) {
    if (length(fold.id) != nrow(x)){
      stop("fold.id inequal to number of rows.")
    }

    num_inferred_folds <- length(unique(fold.id))
    if (cv.folds != num_inferred_folds) {
      # Warn if cv.folds and fold.id disagree, but take fold.id.
      warning("CV folds changed from ", cv.folds, " to ", num_inferred_folds,
              " because of levels in fold.id.")
    } 
    cv.folds <- num_inferred_folds
    
    # Set fold_id from whatever it is to an integer ascending from 1. Lazy way.
    fold.id <- as.numeric(as.factor(fold.id))
  }
  
  # Update distribution according to groups
  dist_obj <- determine_groups(data, y, dist_obj)
  
  # Update weights according to groupings
  weights <- weight_group_consistency(dist_obj, weights)
  
  # Update number of training rows based off of groups
  params <- update_num_train_groups(params, dist_obj)
  
  # Call gbmt_fit 
  gbm_fit_obj <- gbmt_fit(x, y, dist_obj, weights, offset,
                          params, as.character(formula[[2]]),
                          var.monotone, var.names,
                          keep.data, cv.folds,
                          class.stratify.cv, fold.id, par.details, verbose)
  
  # Wrap up extra pieces 
  gbm_fit_obj$model <- m
  gbm_fit_obj$Terms <- Terms
  gbm_fit_obj$call <- the_call
  gbm_fit_obj$is_verbose <- verbose
  
  return(gbm_fit_obj)
}

##' Control parallelization options
##'
##' \code{gbmt} uses openmp to parallelize its core algorithm, and the
##' details are controlled by this object.  As guidance, set
##' \code{num_threads} to the number of cores on your computer, and set
##' \code{array_chunk_size} to a reasonable - not necessarily small -
##' size.
##' 
##' @param num_threads the number of threads to use (a positive
##'     integer).  The number of cores on your computer is a
##'     reasonable default.
##' @param array_chunk_size the size of chunks to use in array scans.
##'     Values that are too small result in a great deal of overhead;
##'     The default of 1024 appears reasonable, but do experiment.
##' @return an object of type \code{gbmParallel}
##' @export
gbmParallel <- function(num_threads=1, array_chunk_size=1024) {
    res <- list(num_threads=num_threads,
                array_chunk_size=array_chunk_size)
    class(res) <- "gbmParallel"
    res
}

##' @export
print.gbmParallel <- function(x, ...) {
    cat("GBM parallelization\n\n",
        "number of threads: ", x$num_threads, "\n",
        "array chunk size : ", x$array_chunk_size, "\n",
        sep="")
    invisible(x)
}

## Accessor methods for gbm fit objects

##' Extract trees from GBMFit objects
##'
##' @param obj a GBMFit object
##' @return a vector containing the trees generated in the fit
##' @export
trees <- function(obj) {
    check_if_gbm_fit(obj)
    obj$trees
}

##' Extract errors from GBMFit objects
##'
##' @param obj a GBMFit object
##' @param which which error measure to extract
##' @return a vector giving the error by tree
##' @export
iteration_error <- function(obj, which=c('train', 'valid', 'cv')) {
    check_if_gbm_fit(obj)
    switch(match.arg(which),
           train=obj$train.error,
           valid=obj$valid.error,
           cv=obj$cv_error,
           stop("Unknown error measure"))
}

##' What is the distribution name used here?
##'
##' @param obj the object for which the distribution name is needed
##' @param ... other parameters
##' @return a string identifying the distribution
##' @export
distribution_name <- function(obj, ...) {
    UseMethod("distribution_name")
}

##' @export
distribution_name.default <- function(obj, ...) {
    stop("I don't know how to handle this")
}

##' @export
distribution_name.GBMFit <- function(obj, ...) {
    distribution_name(obj$distribution)
}

##' @export
distribution_name.GBMDist <- function(obj, ...) {
    obj$name
}

#' Methods for estimating relative influence
#' 
#' This function offers a method for computing the relative influence in
#' \code{\link{summary.GBMFit}}, and is not intended to be called directly.
#'
#' @param gbm_fit_obj a \code{GBMFit} object created from an initial
#' call to \code{\link{gbmt}}.
#' 
#' @param num_trees the number of trees to use for computations. If
#' not provided, the function will guess: if a test set was used in
#' fitting, the number of trees resulting in lowest test set error
#' will be used; otherwise, if cross-validation was performed, the
#' number of trees resulting in lowest cross-validation error will be
#' used; otherwise, all trees will be used.
#' 
#' @param rescale whether or not the result should be scaled. Defaults
#' to \code{FALSE}.
#' 
#' @param sort_it whether or not the results should be (reverse)
#' sorted.  Defaults to \code{FALSE}.
#' 
#' @return By default, returns an unprocessed vector of estimated
#' relative influences. If the \code{rescale} and \code{sort}
#' arguments are used, returns a processed version of the same.
#' 
#' @author James Hickey, Greg Ridgeway \email{gregridgeway@@gmail.com}
#' @details \code{\link{relative_influence}} is the same as that
#' described in Friedman (2001).
#' \code{\link{permutation_relative_influence}} randomly permutes each
#' predictor variable at a time and computes the associated reduction in
#' predictive performance. This is similar to the variable importance measures
#' Breiman uses for random forests, but \code{gbmt} currently computes using the
#' entire training dataset (not the out-of-bag observations).
#' @seealso \code{\link{summary.GBMFit}}
#' @references J.H. Friedman (2001). "Greedy Function Approximation: A Gradient
#' Boosting Machine," Annals of Statistics 29(5):1189-1232.
#' 
#' L. Breiman (2001).
#' \href{https://www.stat.berkeley.edu/~breiman/randomforest2001.pdf}{Random
#' Forests}.
#' @keywords hplot
#' @export
relative_influence <- function(gbm_fit_obj, num_trees,
                               rescale = FALSE, sort_it = FALSE){
  # Initial checks
  check_if_gbm_fit(gbm_fit_obj)
  if(!is.logical(rescale) || (length(rescale) > 1))
    stop("rescale argument must be a logical")
  if(!is.logical(sort_it) || (length(sort_it) > 1))
    stop("sort_it must be a logical")
  
  # Fill in missing values
  if( missing( num_trees ) ){
    if ( gbm_fit_obj$params$train_fraction < 1 ){
      num_trees <- gbmt_performance(gbm_fit_obj, method="test")
    }
    else if ( has_cross_validation(gbm_fit_obj) ) {
      num_trees <- gbmt_performance(gbm_fit_obj, method="cv")
    }
    else{
      num_trees <- gbm_fit_obj$params$num_trees
    }
    message("num_trees not given. Using ", num_trees, " trees.")
    
  } else if (num_trees > length(trees(gbm_fit_obj))) {
    stop("num_trees exceeds number in fit")
  }

  # Create relative influence for every variable
  rel_inf_verbose <- unlist(lapply(trees(gbm_fit_obj)[seq_len(num_trees)],
                                   get_rel_inf_of_vars))
  
  # Sum across trees and remove unused variables (names are "-1")
  rel_inf_compact <- unlist(lapply(split(rel_inf_verbose, names(rel_inf_verbose)), sum))
  rel_inf_compact <- rel_inf_compact[names(rel_inf_compact) != "-1"]
  
  # rel_inf_compact excludes those variable that never entered the model
  # insert 0's for the excluded variables
  rel_inf <- rep(0, length(gbm_fit_obj$variables$var_names))
  i <- as.numeric(names(rel_inf_compact))+1
  rel_inf[i] <- rel_inf_compact
  names(rel_inf) <- gbm_fit_obj$variables$var_names
  
  # Rescale and sort
  if (rescale) rel_inf <- rel_inf / max(rel_inf)
  if (sort_it) rel_inf <- rev(sort(rel_inf))
  
  return(rel_inf)
}

#### Helper function ####
get_rel_inf_of_vars <- function(obj) {
  lapply(split(obj[[6]], obj[[1]]), sum) # 6 - Improvement, 1 - var name
}

#' Baseline hazard function
#' 
#' Computes the Breslow estimator of the baseline hazard function for a
#' proportional hazard regression model - only for censored survival data.
#' 
#' The proportional hazard model assumes h(t|x)=lambda(t)*exp(f(x)).
#' \code{\link{gbmt}} can estimate the f(x) component via partial likelihood.
#' After estimating f(x), \code{baseline_hazard} can compute a nonparametric
#' estimate of lambda(t).
#' 
#' @usage baseline_hazard(surv_times, delta, coxph_preds, eval_times=NULL, smooth=FALSE,
#' cumulative=TRUE)
#' 
#' @param surv_times the survival times - an atomic vector of doubles
#' 
#' @param delta the censoring indicator - a vector same length as surv_times
#' 
#' @param coxph_preds the predicted values of the regression model on the log hazard
#' scale
#' 
#' @param eval_times values at which the baseline hazard will be evaluated
#' 
#' @param smooth if \code{TRUE} \code{baseline_hazard} will smooth the estimated
#' baseline hazard using Friedman's super smoother \code{\link{supsmu}}
#' 
#' @param cumulative if \code{TRUE} the cumulative survival function will be
#' computed
#' 
#' @return a vector of length equal to the length of surv_times (or of length
#' \code{eval_times} if \code{eval_times} is not \code{NULL}) containing the baseline
#' hazard evaluated at t (or at \code{eval_times} if \code{eval_times} is not
#' \code{NULL}). If \code{cumulative} is set to \code{TRUE} then the returned
#' vector evaluates the cumulative hazard function at those values.
#' 
#' @author James Hickey, Greg Ridgeway \email{gregridgeway@@gmail.com}
#' 
#' @seealso \code{\link[survival]{survfit}}, \code{\link{gbmt}}
#' 
#' @references N. Breslow (1972). "Discussion of `Regression Models and
#' Life-Tables' by D.R. Cox," Journal of the Royal Statistical Society, Series
#' B, 34(2):216-217.
#' 
#' N. Breslow (1974). "Covariance analysis of censored survival data,"
#' Biometrics 30:89-99.
#' 
#' @keywords methods survival
#' 
#' @export

baseline_hazard <- function(surv_times, delta, coxph_preds,
                            eval_times=NULL,
                            smooth=FALSE,
                            cumulative=TRUE) {
  # Initial checks
  if(!is.logical(cumulative) || (length(cumulative) > 1) || is.na(cumulative) || is.nan(cumulative)) {
    stop("cumulative must be a logical")
  }
  if(!is.logical(smooth) || (length(smooth) > 1) || is.na(smooth) || is.nan(smooth)) {
    stop("smooth must be a logical")
  }
  if(!is.atomic(surv_times) || any(surv_times != as.double(surv_times))) {
    stop("surv_times must be a vector of doubles")
  }
  
  if(!is.atomic(delta) || any(!(delta %in% c(0, 1))) ) {
    stop("delta must be a vector with elements eiter 0 or 1")
  }
  
  if(!is.null(eval_times) && 
     (!is.atomic(eval_times) || any(eval_times != as.double(eval_times))) ) {
    stop("eval_times must be a vector of doubles if not NULL")
  }
  
  if(!smooth && !cumulative)
  {
    if(!is.null(eval_times))
    {
      stop("Cannot evaluate unsmoothed baseline hazard at eval_times.")
    }
  }
  
  # Calculate survival function - alpha
  unique_death_times <- sort(unique(surv_times[delta==1]))
  alpha <- length(unique_death_times)
  for(i in seq_along(unique_death_times)) {
    alpha[i] <- sum(surv_times[delta==1]==unique_death_times[i])/
      sum(exp(coxph_preds[surv_times >= unique_death_times[i]]))
  }
  
  # Check if need cumulative survival function
  if(cumulative) alpha <- cumsum(alpha)
  
  # Set evaluation times
  if(is.null(eval_times)) eval_times <- surv_times
  
  # Use super smoother and evaluate 
  lambda <- list(x=unique_death_times, y=alpha)
  if(smooth) {
    lambda <- supsmu(unique_death_times, alpha)
  } 

  return(approx(lambda$x, lambda$y, xout=eval_times)$y)
}

####################
# Author: James Hickey
#
# Series of tests to test baseline hazard function
#
####################

context("Testing error checking - baseline_hazard")
test_that("error thrown if cumulative is not logical", {
  # Given inputs  - MADE UP
  N<-1
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # When cumulative is not logical 
  # Then error is thrown
  expect_error(baseline_hazard(surv_time, delta, cox_preds, cumulative = NA))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, cumulative = NaN))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, cumulative = "Wrong"))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, cumulative = Inf))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, cumulative = 1))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, cumulative = c(TRUE, FALSE)))
})
test_that("error thrown if smooth is not logical", {
  # Given inputs  - MADE UP
  N<-1
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # When smooth is not logical 
  # Then error is thrown
  expect_error(baseline_hazard(surv_time, delta, cox_preds, smooth = NA))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, smooth = NaN))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, smooth = "Wrong"))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, smooth = Inf))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, smooth = 1))
  expect_error(baseline_hazard(surv_time, delta, cox_preds, smooth = c(TRUE, FALSE)))
})
test_that("error thrown if delta not in {0, 1}", {
  # Given inputs  - MADE UP
  N<-1
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # When all delta not in {0, 1}
  delta[1] <- 1.1
  # Then error is thrown
  expect_error(baseline_hazard(surv_time, delta, cox_preds))
})
test_that("error thrown if delta not an atomic", {
  # Given inputs  - MADE UP
  N<-1
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # When all delta not an atomic
  # Then error is thrown
  expect_error(baseline_hazard(surv_time, as.list(delta), cox_preds))
})
test_that("error thrown when smooth=FALSE, cumulative=FALSE and eval_times are not NULL", {
  # Given inputs  - MADE UP
  N<-1
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # When smooth=FALSE, cumulative=FALSE and eval_times not NULL 
  # Then error is thrown
  expect_error(baseline_hazard(surv_time, delta, cox_preds, eval_times = runif(N), 
                               cumulative = FALSE, smooth=FALSE))
})
test_that("error thrown when survival times are not atomic of doubles", {
  # Given inputs  - MADE UP
  N<-1
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # When survival_times not an atomic of doubls
  # Then error is thrown
  expect_error(baseline_hazard(as.list(surv_time), delta, cox_preds, eval_times = runif(N)))
  expect_error(baseline_hazard(Inf, delta, cox_preds, eval_times = runif(N)))
})

context("Testing it can run - baseline hazard")
test_that("can run cumulative=TRUE", {
  # Given inputs  - MADE UP
  N<-100
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # Whenrun cumulative=TRUE
  # Then no error is thrown
  expect_error(baseline_hazard(surv_time, delta, cox_preds, cumulative=TRUE), NA)
})

test_that("can run smooth=TRUE", {
  # Given inputs  - MADE UP
  N<-100
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # Whenrun cumulative=TRUE
  # Then no error is thrown
  expect_error(baseline_hazard(surv_time, delta, cox_preds, smooth=TRUE), NA)
})

test_that("can run with eval_times", {
  # Given inputs  - MADE UP
  N<-100
  surv_time <- runif(N)
  delta <- sample(0:1, N, replace=TRUE)
  cox_preds <- abs(runif(N))
  cox_preds <- cox_preds/(max(cox_preds) - min(cox_preds))
  
  # Whenrun cumulative=TRUE
  # Then no error is thrown
  expect_error(baseline_hazard(surv_time, delta, cox_preds, eval_times=surv_time+1), NA)
})

test_that("eval_times controls output locations", {
  surv_time <- c(1, 2, 3, 4)
  delta <- c(1, 1, 0, 1)
  cox_preds <- rep(0, length(surv_time))
  eval_times <- c(1.5, 3.5)

  hazard <- baseline_hazard(surv_time, delta, cox_preds,
                            eval_times=eval_times)

  expect_length(hazard, length(eval_times))
  expect_equal(hazard, c(5 / 12, 4 / 3))
})

#' GBMT fit
#' 
#' Fits a generalized boosting model.  This is for "power" users who
#' have a large number of variables who wish to avoid calling
#' \code{model.frame} which can be slow in this instance.
#' 
#' @param x a data frame or data matrix containing the predictor
#' variables.
#' 
#' @param y is a matrix of outcomes. Excluding CoxPH this matrix of
#' outcomes collapses to a vector; in the case of CoxPH it is a
#' survival object where the event times fill the first one (or two
#' columns) and the status fills the final column.  The length of the
#' 1st dimension of y must match the number of rows in x.
#' 
#' @param distribution a \code{GBMDist} object specifying the
#' distribution and any additional parameters needed.
#' 
#' @param weights optional vector of weights used in the fitting
#' process.  These weights must be positive but need not be
#' normalized. By default they are set to 1 for each data row.
#' 
#' @param offset optional vector specifying the model offset; must be
#' positive.  This defaults to a vector of 0's, the length of which is
#' equal to the rows of x.
#' 
#' @param train_params a GBMTrainParams object which specifies the
#' parameters used in growing decision trees.
#' 
#' @param response_name a string specifying the name of the response -
#' defaults to "y".
#' 
#' @param var_monotone optional vector, the same length as the number
#' of predictors, indicating the relationship each variable has with
#' the outcome.  It have a monotone increasing (+1) or decreasing (-1)
#' or an arbitrary relationship.
#' 
#' @param var_names a vector of strings of containing the names of the
#' predictor variables.
#' 
#' @param keep_gbm_data a bool specifying whether or not the gbm_data
#' object created in this method should be stored in the results.
#' 
#' @param cv_folds a positive integer specifying the number of folds
#' to be used in cross-validation of the gbm fit. If cv_folds > 1 then
#' cross-validation is performed; the default of cv_folds is 1.
#' 
#' @param cv_class_stratify a bool specifying whether or not to
#' stratify via response outcome. Currently only applies to
#' "Bernoulli" distribution and defaults to false.
#' 
#' @param fold_id An optional vector of values identifying what fold
#' each observation is in. If supplied, cv_folds can be missing. Note:
#' Multiple rows of the same observation must have the same fold_id.
#' 
#' @param par_details Details of the parallelization to use in the
#' core algorithm.
#' 
#' @param is_verbose if TRUE, gbmt will print out progress and
#' performance of the fit.
#' 
#' @return a \code{GBMFit} object.
#' 
#' @export
gbmt_fit <- function(x, y, distribution=gbm_dist("Gaussian"),
                     weights=rep(1, nrow(x)), offset=rep(0, nrow(x)),
                     train_params=training_params(num_trees=100,
                         interaction_depth=3,
                         min_num_obs_in_node=10,
                         shrinkage=0.001, bag_fraction=0.5,
                         id=seq_len(nrow(x)), num_train=round(0.5 * nrow(x)),
                         num_features=ncol(x)), response_name="y",
                     var_monotone=NULL,
                     var_names=NULL, keep_gbm_data=FALSE, cv_folds=1,
                     cv_class_stratify=FALSE, fold_id=NULL,
                     par_details=getOption('gbm.parallel'),
                     is_verbose=FALSE) {
  
  # Check num_features makes sense
  if(train_params$num_features > ncol(x)) {
    warning("Number of features exceeds the number of predictor variables - 
            setting number of features to number of predictors")
    train_params$num_features <- ncol(x)
  }
  
  # Create gbm_data_obj
  gbm_data_obj <- gbm_data(x, y, weights, offset)
  
  # Set-up variable containers
  variables <- var_container(gbm_data_obj, var_monotone, var_names)
  
  # Create strata
  distribution <- create_strata(gbm_data_obj, train_params, distribution)
  
  # Process data obj and validate
  gbm_data_obj <- convert_factors(gbm_data_obj)
  gbm_data_obj <- validate_gbm_data(gbm_data_obj, distribution)
  
  # Order the data
  gbm_data_obj <- order_data(gbm_data_obj, distribution, train_params)
  
  # Order ids according to group_order if necessary
  if(!is.null(distribution$group_order)) {
    train_params$id <- train_params$id[distribution$group_order]
  } 
  train_params$id <- train_params$id[train_params$id_order]
  
  # Get CV groups
  cv_groups <- create_cv_groups(gbm_data_obj, distribution,
                                train_params, cv_folds,
                                cv_class_stratify, fold_id)
  # Create fitted object
  gbm_fit_obj <- gbm_cross_val(gbm_data_obj, distribution,
                               train_params, variables,
                               cv_folds, cv_groups,
                               par_details, is_verbose)
  
  # Keep original data
  if(keep_gbm_data) {
    gbm_fit_obj$gbm_data_obj <- gbm_data_obj
  }
  # Reorder if necessary
  gbm_fit_obj <- reorder_fit(gbm_fit_obj, distribution)
  gbm_fit_obj$fold_ids <- fold_id
  gbm_fit_obj$par_details <- par_details
  gbm_fit_obj$response_name <- response_name
  
  return(gbm_fit_obj)
}

## accessor functions
has_train_test_split <- function(gbm_fit_obj) {
    gbm_fit_obj$params$train_fraction < 1
}

has_cross_validation <- function(gbm_fit_obj) {
    !is.null(gbm_fit_obj$cv_error)
}

# Create CV Groups
# 
# Methods that create cv groups for gbm2 fitting.
# Used internally, within \code{\link{gbm_cross_val}}, to split data and distribution
# specific parameters into train and test folds.
# 
# @usage create_cv_groups(gbm_data_obj, gbm_dist_obj, train_params, cv_folds,
#  cv_class_stratify, fold_id)
# 
# @param gbm_data_obj a GBMData object containing all of the data used to fit
# a gbm model. 
# 
# @param gbm_dist_obj a GBMDist object specifying the distribution and
# any additional parameters needed.
# 
# @param train_params a GBMTrainParams object containing generic parameters
# defining how the model should be trained.
# 
# @param cv_folds a positive integer specifying the number of folds to be used
# in cross-validation of the gbm fit. If cv_folds > 1 then cross-validation is
# performed; the default of cv_folds is 1.
# 
# @param cv_class_stratify a bool specifying whether or not to stratify via
# response outcome. Currently only applies to "Bernoulli" distribution and
# defaults to false.
# 
# @param fold_id An optional vector of values identifying what fold
# each observation is in.
# 
# @return a vector of numbers mapping each row of data to a cv fold.
#  
# @author James Hickey
#  
create_cv_groups <- function(gbm_data_obj, gbm_dist_obj, train_params, cv_folds,
                             cv_class_stratify, fold_id) {
  # Initial checks
  check_if_gbm_data(gbm_data_obj)
  check_if_gbm_dist(gbm_dist_obj)
  check_if_gbm_train_params(train_params)
  check_if_natural_number(cv_folds)
  if(!is.logical(cv_class_stratify) || (length(cv_class_stratify) > 1)) {
    stop("cv_class_stratify must be a logical")
  }
  
  UseMethod("create_cv_groups", gbm_dist_obj)
}

#' @export
create_cv_groups.default<- function(gbm_data_obj, gbm_dist_obj, train_params, cv_folds,
                                             cv_class_stratify, fold_id) {
  if(!is.null(fold_id)) {
    return(fold_id)
  } else {
    return( sample(rep(seq_len(cv_folds), length=train_params$num_train)) )
  }
}

#' @export
create_cv_groups.BernoulliGBMDist <- function(gbm_data_obj, gbm_dist_obj, train_params, cv_folds,
                                             cv_class_stratify, fold_id) {
  if(cv_class_stratify) {
    # Number in each class
    Ones <- tabulate(gbm_data_obj$y[seq_len(train_params$num_train)])
    Zeros <- length(gbm_data_obj$y[seq_len(train_params$num_train)])-Ones
    
    smallGroup <- min(c(Ones, Zeros))
    
    if(smallGroup < cv_folds){
      stop(
        paste("The smallest class has only"
              ,smallGroup
              ,"objects in the training set. Can't do"
              ,cv_folds, "fold cross-validation.")
      )
    }
    
    cv_group <- vector(length=train_params$num_train)
    cv_group[gbm_data_obj$y[seq_len(train_params$num_train)] == 0] <- sample(rep(seq_len(cv_folds), length=Zeros))
    cv_group[gbm_data_obj$y[seq_len(train_params$num_train)] == 1] <- sample(rep(seq_len(cv_folds), length=Ones))
    return(cv_group)
    
  } else if(!is.null(fold_id)) {
    return(fold_id)
  } else {
    return( sample(rep(seq_len(cv_folds), length=train_params$num_train)) )
  }
}

#' @export
create_cv_groups.PairwiseGBMDist <- function(gbm_data_obj, gbm_dist_obj, train_params, cv_folds,
                                             cv_class_stratify, fold_id) {
  # Split into CV folds at group boundaries
  samp <- sample(rep(seq_len(cv_folds), length=nlevels(gbm_dist_obj$group)))
  return(samp[as.integer(gbm_dist_obj$group[seq_len(train_params$num_train)])])
}

#' Estimate the strength of interaction effects
#' 
#' Computes Friedman's H-statistic to assess the strength of variable
#' interactions.
#' 
#' \code{interact.GBMFit} computes Friedman's H-statistic to assess the relative
#' strength of interaction effects in non-linear models. H is on the scale of
#' [0-1] with higher values indicating larger interaction effects. To connect
#' to a more familiar measure, if \eqn{x_1} and \eqn{x_2} are uncorrelated
#' covariates with mean 0 and variance 1 and the model is of the form
#' \deqn{y=\beta_0+\beta_1x_1+\beta_2x_2+\beta_3x_3} then
#' \deqn{H=\frac{\beta_3}{\sqrt{\beta_1^2+\beta_2^2+\beta_3^2}}}
#' 
#' Note that if the main effects are weak, the estimated H will be unstable.
#' For example, if (in the case of a two-way interaction) neither main effect
#' is in the selected model (relative influence is zero), the result will be
#' 0/0. Also, with weak main effects, rounding errors can result in values of H
#' > 1 which are not possible.
#' 
#' @usage interact(gbm_fit_obj, data, var_indices=1, num_trees=gbm_fit_obj$params$num_trees)
#' 
#' @param gbm_fit_obj a \code{GBMFit} object fitted using a call to \code{\link{gbmt}}.
#' 
#' @param data the dataset used to construct \code{gbm_fit_obj}. If the original dataset
#' is large, a random subsample may be used to accelerate the computation.
#' 
#' @param var_indices a vector of indices or the names of the variables for compute
#' the interaction effect. If using indices, the variables are indexed in the
#' same order that they appear in the initial \code{gbmt} formula.
#' 
#' @param num_trees the number of trees used to generate the plot. Only the first
#' \code{num_trees} trees will be used.
#' 
#' @return Returns the value of \code{H}.
#' 
#' @author Greg Ridgeway \email{gregridgeway@@gmail.com}
#' @seealso \code{\link{gbmt}}
#' @references J.H. Friedman and B.E. Popescu (2005). \dQuote{Predictive
#' Learning via Rule Ensembles.} Section 8.1
#' @keywords methods
#' @export 
#'

interact <- function(gbm_fit_obj, data, var_indices=1, num_trees=gbm_fit_obj$params$num_trees) {
  UseMethod("interact", gbm_fit_obj)
}

#' @name interact
#' @export
interact.GBMFit <- function(gbm_fit_obj, data, var_indices=1, num_trees = gbm_fit_obj$params$num_trees){
  # Initial input checks and set up
  if(!is.data.frame(data) && !is.matrix(data)) {
    stop("data argument should be a data.frame or matrix")  
  }
  if(!is.atomic(var_indices) ||
     any(is.infinite(var_indices)) || any(is.na(var_indices)) || any(is.nan(var_indices))|| 
     !(all(var_indices == as.integer(var_indices)) || all(var_indices == as.character(var_indices)))
     || is.na(all(var_indices == as.integer(var_indices))) || 
          is.na(all(var_indices == as.character(var_indices)))) {
    stop("Variables indices must be a vector of integers or characters")
  }
  if (gbm_fit_obj$params$interaction_depth < length(var_indices)){
    stop("interaction_depth < length(variables_indices): too low in model call")
  }
  var_indices <- check_and_set_variables_indices(gbm_fit_obj, var_indices)
  num_trees <- check_and_set_num_trees(gbm_fit_obj, num_trees)

  # Convert factors to appropriate numerics
  for(var in var_indices) {
    if(is.factor(data[, gbm_fit_obj$variables$var_names[var]]))
      data[, gbm_fit_obj$variables$var_names[var]] <-
        as.numeric(data[, gbm_fit_obj$variables$var_names[var]])-1
  }
  
  # Generate a list with all combinations of variables
  all_combinations_vars <- apply(expand.grid(rep(list(c(FALSE,TRUE)), length(var_indices)))[-1,], 1,
             function(x) as.numeric(which(x)))
  
  # Compute predictions and "parity" for all variable combinations  
  preds_for_comb_vars <- compute_preds_for_all_var_combinations(data, gbm_fit_obj, all_combinations_vars, var_indices, num_trees)
  
  # Compute H-statistic
  # Set to prediction with all variables
  H_stat_squared <- preds_for_comb_vars[[length(all_combinations_vars)]]$preds
  
  # Loop over other combinations and see what variables have been excluded
  # Add to predictions for all variables with correct sign
  for(vars in seq_len((length(all_combinations_vars)-1))){
    i1 <- apply(preds_for_comb_vars[[length(all_combinations_vars)]]$data[, all_combinations_vars[[vars]], drop=FALSE],
                1, paste, collapse="\r")
    i2 <- apply(preds_for_comb_vars[[vars]]$data, 1, paste,collapse="\r")
    i <- match(i1, i2)
    
    H_stat_squared <- H_stat_squared + with(preds_for_comb_vars[[vars]], sign*preds[i,])
  }
  
  # The H-statistic squared is given by sum over variables predictions with all included 
  # minus the partial dependence on a variable + partial dependence excluding a variable
  # This sum is normalized by the sum of the prediction with no variables excluded
  weights <- matrix(preds_for_comb_vars[[length(all_combinations_vars)]]$num_levels_factors, ncol=1)
  sum_preds_no_exclusion <- matrix(preds_for_comb_vars[[length(all_combinations_vars)]]$preds^2, ncol=1, byrow=FALSE)
  
  numerator <- apply(H_stat_squared^2, 2, weighted.mean, w = weights, na.rm = TRUE)
  denominator <- apply(sum_preds_no_exclusion, 2, weighted.mean, w = weights, na.rm = TRUE)
  H_stat_squared <- numerator / denominator
  
  # If H > 1, rounding and tiny main effects have messed things up
  H_stat_squared[H_stat_squared > 1] <- NaN
  
  return(sqrt(H_stat_squared))
}


#### Helper Functions - Not to be used outside of this interact function ####
check_and_set_num_trees <- function(gbm_fit_obj, num_trees) {
  if(length(num_trees) > 1) {
    warning("length num_trees > 1: using first element")
    num_trees <- num_trees[1]
  }
  check_if_natural_number(num_trees)
  if (num_trees > gbm_fit_obj$params$num_trees) {
    warning(paste("num_trees exceeds the number of trees in the model, ",
                  gbm_fit_obj$params$num_trees,". Using ", gbm_fit_obj$params$num_trees, " trees.", sep = ""))
    num_trees <- gbm_fit_obj$params$num_trees
  }
  
  return(num_trees)
}

check_and_set_variables_indices <- function(gbm_fit_obj, variables_indices) {
  # Match up variable_indices to var_names - convert characters
  if (all(is.character(variables_indices))){
    i <- match(variables_indices, gbm_fit_obj$variables$var_names)
    if (any(is.na(i))) {
      stop("Variables given are not used in gbm model fit: ", variables_indices[is.na(i)])
    }
    else {
      variables_indices <- i
    }
  }
  
  if ((min(variables_indices) < 1) || (max(variables_indices) > length(gbm_fit_obj$variables$var_names))) {
    warning("variables_indices must be between 1 and ", length(gbm_fit_obj$variables$var_names))
  }
  
  return(variables_indices)
}

table_of_unique_values <- function(data, variables_indices) {
  unique_vars <- unique(data[, variables_indices,drop=FALSE])
  # the additional unique() in levels should not be necessary,
  #   but on fedora build getting duplicate factor levels (2024-01-14)
  unique_vars$num_levels_factors <- table(factor(apply(data[, variables_indices,drop=FALSE],1,paste,collapse="\r"),
                      levels=unique(apply(unique_vars, 1,paste,collapse="\r"))))
  return(unique_vars)
}

compute_preds_for_all_var_combinations <- function(data, gbm_fit_obj, all_combinations_vars, variables_indices, num_trees) {
  preds_for_comb_vars <- vector("list", length(all_combinations_vars))
  for(vars in seq_along(all_combinations_vars)) {
    # Get data for combination
    preds_for_comb_vars[[vars]]$data <- 
      data.frame(table_of_unique_values(data,
                                        gbm_fit_obj$variables$var_names[variables_indices[all_combinations_vars[[vars]]]]))
    
    preds_for_comb_vars[[vars]]$num_levels_factors <- as.numeric(preds_for_comb_vars[[vars]]$data$num_levels_factors)
    preds_for_comb_vars[[vars]]$data$num_levels_factors <- NULL
    
    # Make predictions using the current combination of variables
    preds_for_comb_vars[[vars]]$preds <- .Call("gbm_plot",
                                               X = data.matrix(preds_for_comb_vars[[vars]]$data),
                                               i.var = as.integer(variables_indices[all_combinations_vars[[vars]]] - 1),
                                               n.trees = as.integer(num_trees),
                                               initF = as.double(gbm_fit_obj$initF),
                                               trees = trees(gbm_fit_obj),
                                               c.splits = gbm_fit_obj$c.splits,
                                               var.type = as.integer(gbm_fit_obj$variables$var_type),
                                               PACKAGE = "gbm3")
    
    # Convert predictions to flat matrix
    preds_for_comb_vars[[vars]]$preds <- matrix(preds_for_comb_vars[[vars]]$preds, ncol=1, byrow=FALSE)
    
    # Centre the predictions
    preds_for_comb_vars[[vars]]$preds <- apply(preds_for_comb_vars[[vars]]$preds, 2, function(x, w){
      x - weighted.mean(x, w, na.rm=TRUE)
    }, w=preds_for_comb_vars[[vars]]$num_levels_factors)
    
    # precompute the sign of these terms to appear in H - statistic
    # if same "parity" return 1, else -1
    preds_for_comb_vars[[vars]]$sign <- ifelse((length(all_combinations_vars[[vars]]) %% 2) == 
                                                        (length(variables_indices) %% 2), 1, -1)
  }
  
  return(preds_for_comb_vars)
}

####################
# Author: James Hickey
#
# Series of tests to check the calculation of the variable's 
# interaction strength
#
####################

context("Testing help functions - Interact")
test_that("check_and_set_num_trees throws a warning if given a vector", {
  # Given a vector for num_trees and a fake fit object
  gbm_fake_fit <- list()
  gbm_fake_fit$params$num_trees <- 2000
  num_trees <- c(200, 500)
  
  # When checking and setting the number of trees
  # Then a warning is thrown
  expect_warning(check_and_set_num_trees(gbm_fake_fit, num_trees), 
                 "length num_trees > 1: using first element")
})
test_that("check_and_set_num_trees throws an error if not given natural number for num_trees", {
  # Given a fake fit object and a num_trees which isn't a positive integer
  gbm_fake_fit <- list()
  gbm_fake_fit$params$num_trees <- 2000
  num_trees <- "wrong"
  
  # When checking and setting the number of trees
  # Then an error is thrown
  expect_error(check_and_set_num_trees(gbm_fake_fit, num_trees))
})
test_that("check_and_set_num_trees throws a warning if number of trees exceeds those in original fit", {
  # Given a fake fit object and num_trees > those fitted
  gbm_fake_fit <- list()
  gbm_fake_fit$params$num_trees <- 2000
  num_trees <- 5000
  
  # When checking and setting the number of trees
  # Then a warning is thrown
  expect_warning(check_and_set_num_trees(gbm_fake_fit, num_trees), 
                 paste("num_trees exceeds the number of trees in the model, ",
                       gbm_fake_fit$params$num_trees,". Using ", gbm_fake_fit$params$num_trees, " trees.", 
                       sep = ""))
  
})
test_that("check_and_set_num_trees returns 1st element of vector is < those in original fit", {
  # Given a vector for num_trees and a fake fit object
  gbm_fake_fit <- list()
  gbm_fake_fit$params$num_trees <- 2000
  num_trees <- c(200, 500)
  
  # When checking and setting the number of trees
  # Then returns first element of vector
  expect_equal(suppressWarnings(check_and_set_num_trees(gbm_fake_fit, num_trees)),
               num_trees[1])
})
test_that("check_and_set_num_trees returns those in original fit if number specified exceeds total in fit", {
  # Given a fake fit object and num_trees > those fitted
  gbm_fake_fit <- list()
  gbm_fake_fit$params$num_trees <- 2000
  num_trees <- 5000
  
  # When checking and setting the number of trees
  # Then returns number of trees in fit
  expect_equal(suppressWarnings(check_and_set_num_trees(gbm_fake_fit, num_trees)),
               gbm_fake_fit$params$num_trees)
})
test_that("check_and_set_num_trees returns argument num_trees if it is a positive integer <= the total num_trees in original fit", {
  # Given a fake fit object and num_trees < those fitted
  gbm_fake_fit <- list()
  gbm_fake_fit$params$num_trees <- 2000
  num_trees <- 100
  
  # When checking and setting the number of trees
  # Then returns the num_trees passed in
  expect_equal(check_and_set_num_trees(gbm_fake_fit, num_trees), num_trees)
})
test_that("check_and_set_variables_indices throws an error if all strings but not all in fit's variable names", {
  # Given a fake fit object and a character vector with unrecognised variables
  gbm_fake_fit <- list()
  gbm_fake_fit$variables$var_names <- c("Var1", "Var2", "Var3")
  var_indices <- "Var4"
  
  # When checking and setting variable indices
  # Then an error is thrown
  expect_error(check_and_set_variables_indices(gbm_fake_fit, var_indices))
})
test_that("check_and_set_variables_indices throws a warning if indices not between 1 and the number of variables in fit", {
  # Given a fake fit object and index outside range
  gbm_fake_fit <- list()
  gbm_fake_fit$variables$var_names <- c("Var1", "Var2", "Var3")
  var_indices <- 4
  
  # When checking and setting variable indices
  # Then a warning will be thrown
  expect_warning(check_and_set_variables_indices(gbm_fake_fit, var_indices),
                 paste0("variables_indices must be between 1 and ", 
                        length(gbm_fake_fit$variables$var_names)))
})
test_that("check_and_set_variables_indices converts character vector input to integers", {
  # Given a fake fit object and variable indices that are strings
  gbm_fake_fit <- list()
  gbm_fake_fit$variables$var_names <- c("Var1", "Var2", "Var3")
  var_indices <- c("Var1", "Var2")
  
  # When checking and setting variables indices
  # Then the indices are converted to appropriate integers
  expect_equal(check_and_set_variables_indices(gbm_fake_fit, var_indices),
               c(1, 2))
})
test_that("check_and_set_variables_indices returns original vector if positive integers", {
  # Given a fake fit object and correct indices
  gbm_fake_fit <- list()
  gbm_fake_fit$variables$var_names <- c("Var1", "Var2", "Var3")
  var_indices <- c(2, 3)
  
  # When checking and setting variables indices
  # Then the original indices are returned
  expect_equal(check_and_set_variables_indices(gbm_fake_fit, var_indices),
               var_indices)
})
test_that("table_of_unique_values works correctly", {
  # Given data and correct variable indices
  N <- 100
  data <- data.frame("Var1"=rnorm(N), "Var2"=sample(1:5, N, replace=TRUE),
                     "Var3"=rnorm(N))
  var_indices <- c(1, 2)
  
  # When table_of_unique_values is called
  output <- table_of_unique_values(data, var_indices)
  
  # Then correct output produced
  corr_out <- unique(data[, var_indices, drop=FALSE])
  corr_out$num_levels_factors <- table(factor(apply(data[, var_indices,drop=FALSE],1,paste,collapse="\r"),
                                              levels=apply(corr_out, 1,paste,collapse="\r")))
  expect_equal(output, corr_out)
})
test_that("output of compute_preds_for_all_var_combinations is a list with the correct fields", {
  # Given data, a fit object, variable indices and all combinations of variables
  var_indices <- c(1, 2, 3)
  all_combinations_vars <- apply(expand.grid(rep(list(c(FALSE,TRUE)), length(var_indices)))[-1,], 1,
                                 function(x) as.numeric(which(x)))
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=10)] <- NA
  X3[sample(1:N,size=30)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm_fit_obj <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                     # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=20,                  # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     mFeatures = 3,               # Number of features to consider at each node.
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=2,                  # do 2-fold cross-validation
                     verbose = FALSE)             # don't print progress
  
  # When calculating predictions for all combinations of variables
  # Then has correct fields
  expect_equal(names(compute_preds_for_all_var_combinations(data, gbm_fit_obj,
                                                      all_combinations_vars, var_indices, 1)[[1]]),
  c("data", "num_levels_factors", "preds", "sign"))
})
test_that("output of compute_pred_for_all_var_combinations is of the correct size", {
  # Given data, a fit object, variable indices and all combinations of variables
  var_indices <- c(1, 2, 3)
  all_combinations_vars <- apply(expand.grid(rep(list(c(FALSE,TRUE)), length(var_indices)))[-1,], 1,
                                 function(x) as.numeric(which(x)))
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=10)] <- NA
  X3[sample(1:N,size=30)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm_fit_obj <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                     # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=20,                  # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     mFeatures = 3,               # Number of features to consider at each node.
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=2,                  # do 2-fold cross-validation
                     verbose = FALSE)             # don't print progress
  
  # When calculating predictions for all combinations of variables
  # Then has correct size
  expect_equal(length(compute_preds_for_all_var_combinations(data, 
                                                             gbm_fit_obj,
                                                             all_combinations_vars, 
                                                             var_indices, 
                                                             num_trees = 1)),
               length(all_combinations_vars))
})
test_that("sign field of output of compute_pred_for_all_var_combinations consists of +/-1s", {
  # Given data, a fit object, variable indices and all combinations of variables
  var_indices <- c(1, 2, 3)
  all_combinations_vars <- apply(expand.grid(rep(list(c(FALSE,TRUE)), length(var_indices)))[-1,], 1,
                                 function(x) as.numeric(which(x)))
  
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=10)] <- NA
  X3[sample(1:N,size=30)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm_fit_obj <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                     # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=20,                  # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     mFeatures = 3,               # Number of features to consider at each node.
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=2,                  # do 2-fold cross-validation
                     verbose = FALSE)             # don't print progress
  
  # When calculating predictions for all combinations of variables
  preds_for_combs <- compute_preds_for_all_var_combinations(data, 
                                                            gbm_fit_obj,
                                                            all_combinations_vars, 
                                                            var_indices, 
                                                            num_trees = 1)
  # Then sign field is +-1s
  expect_true(all(preds_for_combs[[1]]$sign %in% c(1, -1)))
})

context("testing interact.GBMFit")
test_that("if data not a data.frame or matrix then an error is thrown", {
  # Given data and a fit object 
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=10)] <- NA
  X3[sample(1:N,size=30)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm_fit_obj <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                     # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=20,                  # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     mFeatures = 3,               # Number of features to consider at each node.
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=2,                  # do 2-fold cross-validation
                     verbose = FALSE)             # don't print progress
  
  # When calculating the model interactions with data
  # not in data frame or matrix
  # Then an error is thrown
  expect_error(interact(gbm_fit_obj, as.list(data)), 
               "data argument should be a data.frame or matrix")
})
test_that("if var_indices not a vector of characters or integers then an error is thrown", {
  # Given data and a fit object 
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=10)] <- NA
  X3[sample(1:N,size=30)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm_fit_obj <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                     # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=20,                  # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     mFeatures = 3,               # Number of features to consider at each node.
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=2,                  # do 2-fold cross-validation
                     verbose = FALSE)             # don't print progress
  
  # When calculating interactions using var_indices that aren't a
  # vector of integers or character vectors
  var_ind1 <- Inf
  var_ind2 <- NA
  var_ind3 <- c(1, "Hello")
  
  # Then error is thrown
  expect_error(interact(gbm_fit_obj, data, var_ind1), 
               "Variables indices must be a vector of integers or characters")
  expect_error(interact(gbm_fit_obj, data, var_ind2), 
               "Variables indices must be a vector of integers or characters")
  expect_error(suppressWarnings(interact(gbm_fit_obj, data, var_ind3)),
               "Variables indices must be a vector of integers or characters")
})
test_that("Error thrown if length var_indices exceeds interaction depth of fit", {
  # Given data and a fit object 
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=10)] <- NA
  X3[sample(1:N,size=30)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm_fit_obj <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                     data=data,                   # dataset
                     var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
                     distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
                     # list(name="quantile",alpha=0.05) for quantile regression
                     n.trees=20,                  # number of trees
                     shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
                     interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc
                     bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
                     train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
                     mFeatures = 3,               # Number of features to consider at each node.
                     n.minobsinnode = 10,         # minimum number of obs needed in each node
                     keep.data=TRUE,
                     cv.folds=2,                  # do 2-fold cross-validation
                     verbose = FALSE)             # don't print progress
  
  # When evaluating the interactions for more than variables than interaction depth
  # Then an error is thrown
  expect_error(interact(gbm_fit_obj, data, 1:4))
})
test_that("Given correct inputs will return H-statistic < 1", {
  # Given data and a fit object 
  # create some data
  N <- 100
  X1 <- runif(N)
  X2 <- 2*runif(N)
  X3 <- factor(sample(letters[1:4],N,replace=T))
  X4 <- ordered(sample(letters[1:6],N,replace=T))
  X5 <- factor(sample(letters[1:3],N,replace=T))
  X6 <- 3*runif(N)
  mu <- c(-1,0,1,2)[as.numeric(X3)]
  
  SNR <- 10 # signal-to-noise ratio
  Y <- X1**1.5 + 2 * (X2**.5) + mu
  sigma <- sqrt(var(Y)/SNR)
  Y <- Y + rnorm(N,0,sigma)
  
  # create a bunch of missing values
  X1[sample(1:N,size=10)] <- NA
  X3[sample(1:N,size=30)] <- NA
  
  # random weights if you want to experiment with them
  w <- rep(1,N)
  
  data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)
  
  # fit initial model
  gbm_fit_obj <- gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
                    data=data,                   # dataset
              var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease, +1: monotone increase, 0: no monotone restrictions
              distribution="Gaussian",     # bernoulli, adaboost, gaussian, poisson, coxph, or
              # list(name="quantile",alpha=0.05) for quantile regression
              n.trees=20,                  # number of trees
              shrinkage=0.005,             # shrinkage or learning rate, 0.001 to 0.1 usually work
              interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc
              bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
              train.fraction = 0.5,        # fraction of data for training, first train.fraction*N used for training
              mFeatures = 3,               # Number of features to consider at each node.
              n.minobsinnode = 10,         # minimum number of obs needed in each node
              keep.data=TRUE,
              cv.folds=2,                  # do 2-fold cross-validation
              verbose = FALSE)             # don't print progress
  
  # When calculating the interactions 
  int_stats <- interact(gbm_fit_obj, data=data, var_indices=1:2)
  
  # Then H-statistic is correct and < 1
  expect_true(is.double(int_stats))
  expect_true(int_stats < 1)
})

# Rearrange training set and distribution data for CV
# 
# Methods converts the gbm data and distribution parameters for CV fitting
# 
# @usage split_and_join(gbm_data_obj, train_params, rows_in_training, rows_in_fold)
# 
# @param gbm_data_obj  a GBMData  containing correctly ordered and validated data.
# 
# @param train_params a validated GBMTrainParams object
# 
# @param rows_in_training vector of logicals that determine what data rows are in the training set.
# 
# @param rows_in_fold vector of logicals indicating whether a row of training data is in the fold or not.
# 
# @return gbm_data_obj with validation data moved to end of fields and recalculation of predictor ordering
# 
# @author James Hickey

split_and_join <- function(gbm_data_obj, train_params, rows_in_training, rows_in_fold) {
  
  # Initial checks
  check_if_gbm_data(gbm_data_obj)
  check_if_gbm_train_params(train_params)
  if(!is.atomic(rows_in_training) || any(!is.logical(rows_in_training))) {
    stop("rows_in_training must be a vector of logicals")
  }
  if(!is.atomic(rows_in_fold) || any(!is.logical(rows_in_fold)) ||
     (length(rows_in_fold[rows_in_fold==FALSE]) != train_params$num_train_rows)) {
    stop("rows_in_fold must be a vector of logicals of length the number of training rows")
  }
  
  # Get Validation and training folds
  gbm_data_obj_train <- gbm_data_obj
  
  # Predictors
  x_valid <- as.data.frame(gbm_data_obj$x[rows_in_training, ,drop=FALSE][rows_in_fold, ,drop=FALSE])
  gbm_data_obj_train$x <- as.data.frame(gbm_data_obj$x[rows_in_training, ,drop=FALSE][!rows_in_fold, ,drop=FALSE])
  
  # Responses 
  y_valid <- as.data.frame(as.matrix(gbm_data_obj$y)[rows_in_training, ,drop=FALSE][rows_in_fold, ,drop=FALSE])
  gbm_data_obj_train$y <- as.data.frame(as.matrix(gbm_data_obj$y)[rows_in_training, ,drop=FALSE][!rows_in_fold, ,drop=FALSE])
  
  # Offset, weights and predictor order for validation
  offset_valid <- gbm_data_obj$offset[rows_in_training][rows_in_fold]
  weights_valid <- gbm_data_obj$weights[rows_in_training][rows_in_fold]
  x_order_valid <- as.data.frame(subset(gbm_data_obj$x_order, rows_in_fold, drop=FALSE))
  gbm_data_obj_train$offset <- gbm_data_obj$offset[rows_in_training][!rows_in_fold]
  gbm_data_obj_train$weights <- gbm_data_obj$weights[rows_in_training][!rows_in_fold]

  # Reorder predictors for fitting
  gbm_data_obj_train <- predictor_order(gbm_data_obj_train, train_params)
  
  # Copy column names - required for recombination
  colnames(y_valid) <- colnames(gbm_data_obj_train$y)
  
  # Recombine - and convert to correct format
  gbm_data_obj$x <- rbind(gbm_data_obj_train$x, x_valid)
  gbm_data_obj$y <- rbind(gbm_data_obj_train$y, y_valid)
  gbm_data_obj$offset <- c(gbm_data_obj_train$offset, offset_valid)
  gbm_data_obj$weights <- c(gbm_data_obj_train$weights, weights_valid)
  gbm_data_obj$x_order <- as.matrix(gbm_data_obj_train$x_order)
  
  return(gbm_data_obj)
}

#' GBMT Performance
#' 
#' Estimates optimal number of boosting iterations given a
#' \code{GBMFit} object and optionally plots various performance
#' measures.
#'
#' @param object a \code{GBMFit} object created from an initial
#' call to \code{\link{gbmt}} or \code{\link{gbm}}.
#'
#' @param plot.it an indicator of whether or not to plot the
#' performance measures. Setting \code{plot.it=TRUE} creates two
#' plots. The first plot plots the train error (in black)
#' and the validation error (in red) versus the iteration
#' number. The scale of the error measurement, shown on the left
#' vertical axis, depends on the \code{distribution} argument used in
#' the initial call.
#'
#' @param oobag.curve indicates whether to plot the out-of-bag
#' performance measures in a second plot.
#'
#' @param overlay if TRUE and oobag.curve=TRUE then a right y-axis is
#' added to the training and test error plot and the estimated
#' cumulative improvement in the loss function is plotted versus the
#' iteration number.
#' 
#' @param method indicate the method used to estimate the optimal
#' number of boosting iterations. \code{method="OOB"} computes the
#' out-of-bag estimate and \code{method="test"} uses the test (or
#' validation) dataset to compute an out-of-sample
#' estimate. \code{method="cv"} extracts the optimal number of
#' iterations using cross-validation if \code{gbm} was called with
#' \code{cv.folds}>1.
#' 
#' @param main the main title for the plot. Defaults to \code{main =
#' ""}.
#'
#' @return \code{gbm.perf} returns the estimated optimal number of iterations.
#' The method of computation depends on the \code{method} argument.
#' 
#' @seealso \code{\link{gbmt}} \code{\link{gbmt_performance}}
#' \code{\link{plot.GBMTPerformance}}
#'
#' @keywords nonlinear survival nonparametric tree
#'
#' @export
gbm.perf <- function(object,
                     plot.it=TRUE,
                     oobag.curve=FALSE,
                     overlay=TRUE,
                     method,
                     main="") {
    if(!is.logical(plot.it) || (length(plot.it)) > 1 || is.na(plot.it))
        stop("plot.it must be a logical - excluding NA")

    performance <- gbmt_performance(object, method)
    if (plot.it) {
        plot(performance,
             out_of_bag_curve=oobag.curve,
             overlay=overlay,
             main=main)
    }

    as.numeric(performance)
}

##' Get performance details for gbm fit
##'
##' \code{gbmt_performance} estimates the optimal number of boosting
##' iterations from a model fit by \code{\link{gbmt}}.  The precise
##' method used depends on the \code{method} parameter.
##' 
##' @param gbm_fit_obj a \code{GBMFit} created from an initial call to
##' \code{\link{gbmt}}.
##'
##' @param method indicate the method used to estimate the optimal
##' number of boosting iterations. \code{method="OOB"} computes the
##' out-of-bag estimate and \code{method="test"} uses the test (or
##' validation) dataset to compute an out-of-sample
##' estimate. \code{method="cv"} extracts the optimal number of
##' iterations using cross-validation if \code{gbmt} was called with
##' \code{cv_folds}>1.
##'
##' @return a GBMTPerformance object, which is a number - the optimal
##' iteration number - with various attributes.
##' @export
gbmt_performance <- function(gbm_fit_obj, method) {
    check_if_gbm_fit(gbm_fit_obj)

    ## guess the method

    if (missing(method)) {
        method <- guess_error_method(gbm_fit_obj)
        message("Using ", method, " method...")
    }
    
    result <-
        switch(method,
               OOB=best_iter_out_of_bag(gbm_fit_obj),
               cv=best_iter_cv(gbm_fit_obj),
               test=best_iter_test(gbm_fit_obj),
               stop("method must be cv, test, or OOB"))

    attr(result, 'decoration') <-
        list(method=method,
             gbm_fit_obj=gbm_fit_obj)
    class(result) <- "GBMTPerformance"
    result
}


##' @export
as.double.GBMTPerformance <- function(x, ...) {
    as.double(unclass(x))
}

##' @export
print.GBMTPerformance <- function(x, ...) {
    decoration <- attr(x, 'decoration')
    method_descriptor <-
        switch(decoration$method,
               cv="cross-validation",
               test="test-set",
               OOB="out-of-bag",
               stop("Unknown method."))
    
    cat("The best ", method_descriptor, " iteration was ", x, ".\n",
        sep="")
    invisible(x)
}

##' Plot GBM performance details
##'
##' The train and validation error (in black and red respectively) are
##' plotted against the iteration number.  If the initial model was
##' built with cross-validation, the cross-validation error is shown
##' in green.
##'
##' The scale of the error measurement, shown on the
##' left vertical axis, depends on the \code{distribution} argument
##' used in the initial call to \code{\link{gbmt}}.
##'
##' @param x a \code{GBMTPerformance} object (created by
##' \code{\link{gbmt_performance}})
##' 
##' @param out_of_bag_curve indicates whether to plot the out-of-bag
##' performance measures in a second plot.
##' 
##' @param overlay if TRUE and out_of_bag_curve=TRUE then a right
##' y-axis is added to the training and test error plot and the
##' estimated cumulative improvement in the loss function is plotted
##' versus the iteration number.
##'
##' @param main the main title for the plot.
##'
##' @param \dots currently ignored
##' 
##' @return No return value, only plot generation
##'
##' @export
plot.GBMTPerformance <- function(x,
                                 out_of_bag_curve=FALSE,
                                 overlay=TRUE,
                                 main="", ...) {
    decoration <- attr(x, 'decoration')
    perf_plot(decoration$gbm_fit_obj, x,
              out_of_bag_curve, overlay,
              decoration$method,
              main)
}

#### Helper functions ####
best_iter_test <- function(gbm_fit_obj) {
  check_if_gbm_fit(gbm_fit_obj)
  best_iter_test <- which.min(iteration_error(gbm_fit_obj, 'valid'))
  return(best_iter_test)
}

best_iter_cv <- function(gbm_fit_obj) {
  check_if_gbm_fit(gbm_fit_obj)

  if(!has_cross_validation(gbm_fit_obj)) {
      stop('In order to use method="cv" gbm must be called with cv_folds>1.')
  }
  
  best_iter_cv <- which.min(iteration_error(gbm_fit_obj, 'cv'))
  return(best_iter_cv)
}

best_iter_out_of_bag <- function(gbm_fit_obj) {
  check_if_gbm_fit(gbm_fit_obj)
  if(gbm_fit_obj$params$bag_fraction==1)
    stop("Cannot compute OOB estimate or the OOB curve when bag_fraction=1")
  if(all(!is.finite(gbm_fit_obj$oobag.improve)))
    stop("Cannot compute OOB estimate or the OOB curve. No finite OOB estimates of improvement")
  
  warning("OOB generally underestimates the optimal number of iterations although predictive performance is reasonably competitive. Using cv_folds>1 when calling gbm usually results in improved predictive performance.")
  smoother <- generate_smoother_oobag(gbm_fit_obj)
  best_iter_oob <- smoother$x[which.min(-cumsum(smoother$y))]
  return(best_iter_oob)
}

generate_smoother_oobag <- function(gbm_fit_obj) {
  check_if_gbm_fit(gbm_fit_obj)
  smoother <- NULL
  x <- seq_len(gbm_fit_obj$params$num_trees)
  smoother <- loess(gbm_fit_obj$oobag.improve~x,
                    enp.target=min(max(4,length(x)/10),50))
  smoother$y <- smoother$fitted
  smoother$x <- x
  return(smoother)
}

## What error method should we use?
guess_error_method <- function(gbm_fit_obj) {
    if (has_train_test_split(gbm_fit_obj)) {
        "test"
    } else if (has_cross_validation(gbm_fit_obj)) {
        "cv"
    } else {
        "OOB"
    }
}

#' @importFrom Rcpp sourceCpp
#' @useDynLib gbm3
#' @importFrom survival Surv
#' @import lattice
#' @importFrom graphics abline axis barplot lines mtext par plot polygon rug
#' @importFrom grDevices rainbow
#' @importFrom stats approx binomial delete.response gaussian glm loess
#' @importFrom stats mad median model.extract model.frame model.offset
#' @importFrom stats model.response model.weights na.pass poisson
#' @importFrom stats predict quantile reformulate runif supsmu
#' @importFrom stats terms weighted.mean
NULL

## on package load set default options
.onLoad <- function(libname, pkgname) {
    already.set <- options()

    if (!('gbm.parallel' %in% names(already.set))) {
        options(gbm.parallel=gbmParallel())
    }

    invisible()
}



#' Generalized Boosted Regression Model Object
#' 
#' These are objects representing fitted \code{gbm}s.
#' 
#' 
#' @return \item{initF}{the "intercept" term, the initial predicted value to
#' which trees make adjustments} 
#' \item{fit}{a vector containing the fitted
#' values on the scale of regression function (e.g. log-odds scale for
#' bernoulli, log scale for poisson)} 
#' \item{train.error}{a vector of length
#' equal to the number of fitted trees containing the value of the loss
#' function for each boosting iteration evaluated on the training data}
#' \item{valid.error}{a vector of length equal to the number of fitted trees
#' containing the value of the loss function for each boosting iteration
#' evaluated on the validation data}
#' \item{cv_error}{if \code{cv_folds}<2 this
#' component is NULL. Otherwise, this component is a vector of length equal to
#' the number of fitted trees containing a cross-validated estimate of the loss
#' function for each boosting iteration}
#' \item{oobag.improve}{a vector of
#' length equal to the number of fitted trees containing an out-of-bag estimate
#' of the marginal reduction in the expected value of the loss function. The
#' out-of-bag estimate uses only the training data and is useful for estimating
#' the optimal number of boosting iterations. See \code{\link{gbmt_performance}}}
#' \item{trees}{a list containing the tree structures. The components are best
#' viewed using \code{\link{pretty_gbm_tree}}}
#' \item{c.splits}{a list of all
#' the categorical splits in the collection of trees. If the \code{trees[[i]]}
#' component of a \code{gbm} object describes a categorical split then the
#' splitting value will refer to a component of \code{c.splits}. That component
#' of \code{c.splits} will be a vector of length equal to the number of levels
#' in the categorical split variable. -1 indicates left, +1 indicates right,
#' and 0 indicates that the level was not present in the training data}
#' \item{cv_fitted}{If cross-validation was performed, the cross-validation
#' predicted values on the scale of the linear predictor. That is, the fitted
#' values from the ith CV-fold, for the model having been trained on the data
#' in all other folds.}
#' @section Structure: The following components must be included in a
#' legitimate \code{GBMFit} object.
#' 
#' @author Greg Ridgeway \email{gregridgeway@@gmail.com}
#' @seealso \code{\link{gbmt}}
#' @keywords methods
#' @name gbm_object
NULL


#' Generalized Boosted Regression Models
#' 
#' This package implements extensions to Freund and Schapire's AdaBoost
#' algorithm and J. Friedman's gradient boosting machine. Includes regression
#' methods for least squares, absolute loss, logistic, Poisson, Cox
#' proportional hazards partial likelihood, t-distribution,
#' AdaBoost exponential loss, Learning to Rank, and Huberized hinge loss.
#' 
#' Further information is available in the following vignettes: \tabular{ll}{
#' \code{gbm} \tab Generalized Boosted Models: A guide to the gbm package
#' (source, pdf)\cr}
#' 
#' @name gbm3-package
#' @aliases gbm_package
#' @author James Hickey, Greg Ridgeway \email{gregridgeway@@gmail.com} with contributions by
#' Daniel Edwards, Brian Kriegler, Stefan Schroedl and Harry Southworth.
#' @references Y. Freund and R.E. Schapire (1997) \dQuote{A decision-theoretic
#' generalization of on-line learning and an application to boosting,}
#' \emph{Journal of Computer and System Sciences,} 55(1):119-139.
#' 
#' G. Ridgeway (1999). \dQuote{The state of boosting,} \emph{Computing Science
#' and Statistics} 31:172-181.
#' 
#' J.H. Friedman, T. Hastie, R. Tibshirani (2000). \dQuote{Additive Logistic
#' Regression: a Statistical View of Boosting,} \emph{Annals of Statistics}
#' 28(2):337-374.
#' 
#' J.H. Friedman (2001). \dQuote{Greedy Function Approximation: A Gradient
#' Boosting Machine,} \emph{Annals of Statistics} 29(5):1189-1232.
#' 
#' J.H. Friedman (2002). \dQuote{Stochastic Gradient Boosting,}
#' \emph{Computational Statistics and Data Analysis} 38(4):367-378.
#' 
#' @keywords internal
"_PACKAGE"

# Convert factors
# 
# Function that converts factors or ordered predictors
# to an appropriate numeric type. 
# 
# @usage convert_factor(gbm_data_obj)
#
# @param gbm_data_obj a gbm_data object
# 
# @author James Hickey
#
# @return gbm_data object with ordered/factor predictor variables
# appropriately converted
# 

convert_factors <- function(gbm_data_obj) {
  check_if_gbm_data(gbm_data_obj)
  gbm_data_obj$x <- sapply(gbm_data_obj$x, function(col) if(is.ordered(col) || is.factor(col)){
                                                         col <- as.numeric(factor(col))-1 } else { col <- col} )
  return(gbm_data_obj)
}

# Functions to compute IR measures for pairwise loss for
# a single group
# Notes:
# * Inputs are passed as a 2-element (y,f) list, to
#   facilitate the 'by' iteration
# * Return the respective metric, or a negative value if
#   it is undefined for the given group
# * For simplicity, we have no special handling for ties;
#   instead, we break ties randomly. This is slightly
#   inaccurate for individual groups, but should have
#   a small effect on the overall measure.


# Area under ROC curve = ratio of correctly ranking pairs


#' Compute Information Retrieval measures.
#' 
#' Functions to compute Information Retrieval measures for pairwise loss for a
#' single group. The function returns the respective metric, or a negative
#' value if it is undefined for the given group.
#' 
#' For simplicity, we have no special handling for ties; instead, we break ties
#' randomly. This is slightly inaccurate for individual groups, but should have
#' only a small effect on the overall measure.
#' 
#' \code{gbm_conc} computes the concordance index: Fraction of all pairs (i,j)
#' with i<j, x[i] != x[j], such that x[j] < x[i]
#' 
#' If \code{obs} is binary, then \code{gbm_roc_area(obs, pred) =
#' gbm.conc(obs[order(-pred)])}.
#' 
#' \code{gbm_conc} is more general as it allows non-binary targets, but is
#' significantly slower.
#' 
#' @aliases gbm_roc_area gbm_conc
#' @param obs Observed value
#' @param pred Predicted value
#' @return The requested performance measure.
#' @author Stefan Schroedl
#' @seealso \code{\link{gbm}}
#' @references C. Burges (2010). "From RankNet to LambdaRank to LambdaMART: An
#' Overview", Microsoft Research Technical Report MSR-TR-2010-82.
#' @keywords models
#' @examples
#' 
#' ##---- Should be DIRECTLY executable !! ----
#' ##-- ==>  Define data, use random,
#' ##--	or do  help(data=index)  for the standard data sets.
#' 
#' @export
gbm_roc_area <- function(obs, pred) {
   n1 <- sum(obs)
   n <- length(obs)
   if (n==n1) { return(1) }
   # Fraction of concordant pairs
   # = sum_{pos}(rank-1) / #pairs with different labels
   # #pairs = n1 * (n-n1)
   return ((mean(rank(pred)[obs > 0]) - (n1 + 1)/2)/(n - n1))
}

# Concordance Index:
# Fraction of all pairs (i,j) with i<j, x[i] != x[j], such that x[j] < x[i]
# Invariant: if obs is binary, then
#      gbm_roc_area(obs, pred) = gbm_conc(obs[order(-pred)])
# gbm_conc is more general as it allows non-binary targets,
# but is significantly slower
gbm_conc <- function(x) {
   lx <- length(x)
   return (sum(mapply(function(r) { sum(x[(r+1):lx]<x[r]) }, 1:(lx-1))))
}

#' Quantile rug plot
#' 
#' Marks the quantiles on the axes of the current plot.
#' 
#' 
#' @param x a numeric vector.
#' @param prob the quantiles of x to mark on the x-axis.
#' @param ... additional graphics parameters currently ignored.
#' @return No return values
#' @author Greg Ridgeway \email{gregridgeway@@gmail.com}
#' @seealso \code{\link[graphics]{plot}}, \code{\link[stats]{quantile}},
#' \code{\link[base]{jitter}}, \code{\link[graphics]{rug}}.
#' @keywords aplot
#' @examples
#' 
#' x <- rnorm(100)
#' y <- rnorm(100)
#' plot(x,y)
#' quantile_rug(x)
#' @export 
#' 

quantile_rug <- function(x, prob=(0:10)/10, ...) {
  quants <- quantile(x[!is.na(x)], prob=prob)
  if(length(unique(quants)) < length(prob)) {
    quants <- jitter(quants)
  }
  rug(quants, ...)
}

####################
# Author: James Hickey
#
# Series of tests to check if quantile rug plot is running correctly
#
####################

context("Testing quantile_rug plot")
test_that("quantile_rug runs without error", {
  probs <- rnorm(100)
  probs <- abs(probs)/sum(abs(probs))
  
  # Add simple plot - for rug
  plot(probs)
  
  expect_error(quantile_rug(rnorm(100)), NA)
  expect_error(quantile_rug(rnorm(100), probs), NA)
})
test_that("quantile_rug throws error if probabilities outsider [0, 1]", {
  probs <- rnorm(100)
  probs[1] <- 5
  
  # Add simple plot for rug
  plot(probs)
  
  expect_error(quantile_rug(rnorm(100), probs))
})
test_that("output from quantile_rug is correct - no jittering", {
  probs <- (0:10)/10
  x <- rnorm(100)
  
  # Add simple plot -for rug
  plot(probs)
  
  output <- quantile_rug(x, probs)
  expect_equal(output, rug(quantile(x[!is.na(x)], probs)))
})
test_that("quantile_rug jitters the inputs if quantiles < length(probabilities)", {
  set.seed(1)
  x <- rep(1:5, 2)
  probs <- (0:100)/100
  output <- quantile_rug(x, probs)
  
  # Add simple plot - for rug
  plot(probs)
  
  set.seed(1)
  true_output <- quantile(x, probs)
  true_output <- jitter(true_output)
  
  expect_equal(output, rug(true_output))
})

# Set Up Groups
# 
# Internal S3 method to set up groups - currently this only
# applies to the pairwise distribution. 
# 
# @usage determine_groups(original_data, response, distribution_obj)
# 
# @param original_data the original data used on the initial call to \code{\link{gbm2}}.
# 
# @param response the response data provided on the initial call to \code{\link{gbm2}}.
# 
# @param distribution_obj a GBMDist object created using gbm_dist.
# 
# @author James Hickey 
#
# @return an updated distribution_object with the group, group_order and group_index updated
# 

determine_groups <- function(original_data, response, distribution_obj) {
  check_if_gbm_dist(distribution_obj)
  UseMethod("determine_groups", distribution_obj) 
}

#' @export
determine_groups.default <- function(original_data, response, distribution_obj) {
  return(distribution_obj)
}

#' @export
determine_groups.PairwiseGBMDist <- function(original_data, response, distribution_obj) {
  if (is.null(distribution_obj$group)) {
    stop("For pairwise regression, the distribution parameter must be a list with a parameter 'group' for 
         the list of the column names indicating groups, for example group=c(\"date\",\"session\",\"category\",\"keywords\").")
  }
  
  # Check if grouping is specified in data
  i <- match(distribution_obj$group, colnames(original_data))
  if (any(is.na(i))) {
    stop("Group column does not occur in data: ", distribution_obj$group[is.na(i)])
  }
  
  # Construct group index
  distribution_obj$group_index <- factor(do.call(paste, c(original_data[,distribution_obj$group, drop=FALSE], sep=":")))

  # Shuffle groups, to remove bias when splitting into train/test set and/or CV folds
  perm_levels  <- levels(distribution_obj$group_index)[sample(1:nlevels(distribution_obj$group_index))]
  distribution_obj$group        <- factor(distribution_obj$group_index, levels=perm_levels)
  
  # The C++ function expects instances to be sorted by group and descending by target
  distribution_obj$group_order    <- order(distribution_obj$group, -response)
  distribution_obj$group  <- distribution_obj$group[distribution_obj$group_order]

  return(distribution_obj)
}

# Verify Weights Group Consistency
# Function that updates the weights in gbm_data 
# so they are consistent with the groupings.

weight_group_consistency <- function(distribution_obj, weights) {
    UseMethod("weight_group_consistency")
}

#' @export
weight_group_consistency.default <- function(distribution_obj, weights) {
    stop("The impossible has happened")
}

#' @export
weight_group_consistency.GBMDist <- function(distribution_obj, weights) {
    weights
}

#' @export
weight_group_consistency.PairwiseGBMDist <-
    function(distribution_obj, weights) {
        if(is.null(distribution_obj$group_index)) {
            stop("Pairwise distribution object has no group defined.")
        }
        
        ## Check that weights are constant across groups
        if ((!missing(weights)) && (!is.null(weights))) {
            w_min <- tapply(weights,
                            INDEX=distribution_obj$group_index, FUN=min)
            w_max <- tapply(weights,
                            INDEX=distribution_obj$group_index, FUN=max)
      
            if (any(w_min != w_max)) {
                stop("For distribution 'pairwise', all instances for the same group must have the same weight")
            }
      
      # Normalize across groups
            weights <- weights * length(w_min) / sum(w_min)
        }

        weights
    }

# Recreate distribution data for CV folds
# 
# Modify distribution object data for CV gbm fit.
# 
# @usage update_fold_dist_data(gbm_dist_obj, gbm_data_obj, train_params, rows_in_training, rows_in_fold)
#  
# @param gbm_dist_obj a GBMDist object properly formatted for gbmt_fit.
#
# @param gbm_data_obj  a GBMData object containing correctly ordered and validated data.
# 
# @param train_params a validated GBMTrainParams object
#
# @param rows_in_training vector of logicals that determine what data rows are in the training set.
# 
# @param rows_in_fold vector of logicals indicating whether a row of training data is in the fold or not.
# 
# @author James Hickey
#
# @return modified \code{GBMDist} object for CV fit - strata and groupings updated if needed.
#


update_fold_dist_data <- function(gbm_dist_obj, gbm_data_obj, train_params, rows_in_training, rows_in_fold) {
  check_if_gbm_dist(gbm_dist_obj)
  check_if_gbm_data(gbm_data_obj)
  check_if_gbm_train_params(train_params)
  if(!is.atomic(rows_in_fold) || any(!is.logical(rows_in_fold)) ||
     (length(rows_in_fold[rows_in_fold == FALSE]) != train_params$num_train_rows)) {
    stop("rows_in_fold must be a vector of logicals of length the number of training rows")
  }
  UseMethod("update_fold_dist_data", gbm_dist_obj)
}

#' @export
update_fold_dist_data.default <- function(gbm_dist_obj, gbm_data_obj, train_params, rows_in_training, rows_in_fold) {
  return(gbm_dist_obj)
}

#' @export
update_fold_dist_data.CoxPHGBMDist <- function(gbm_dist_obj, gbm_data_obj, train_params, rows_in_training, rows_in_fold) {
  # Reset strata using folds
  gbm_dist_obj$original_strata_id <- gbm_dist_obj$original_strata_id[train_params$id_order]
  gbm_dist_obj$original_strata_id <- c(gbm_dist_obj$original_strata_id[rows_in_training][!rows_in_fold], gbm_dist_obj$original_strata_id[rows_in_training][rows_in_fold])
  gbm_dist_obj$strata <- NULL
  gbm_dist_obj$sorted <- NULL
  gbm_dist_obj$time_order <- NULL
  gbm_dist_obj <- create_strata(gbm_data_obj, train_params, gbm_dist_obj, order_strata_by_id=FALSE)
  return(gbm_dist_obj)
}

#' @export
update_fold_dist_data.PairwiseGBMDist <- function(gbm_dist_obj, gbm_data_obj, train_params, rows_in_training, rows_in_fold) {
  gbm_dist_obj$group <- c(gbm_dist_obj$group[rows_in_training][!rows_in_fold],
                          gbm_dist_obj$group[rows_in_training][rows_in_fold])
  return(gbm_dist_obj)
}

#' Relative influence via permutation
#'
#' This function offers a method for computing the relative influence in
#' \code{\link{summary.GBMFit}}, and is not intended to be called directly.
#'
#' Calculates the relative influence of predictors via random
#' permutation of each predictor one at a time and calculating the
#' associated reduction in predictive performance.  This experimental
#' measure is similar to the variable importance measures Breiman uses
#' for random forests, but \code{\link{gbmt}} currently computes using
#' the entire training dataset (not the out-of-bag observations).
#' 
#' @param gbm_fit_obj a \code{GBMFit} object from an initial call to
#' \code{\link{gbmt}}.
#' 
#' @param num_trees the number of trees to use for computations. If
#' not provided, the function will guess: if a test set was used in
#' fitting, the number of trees resulting in lowest test set error
#' will be used; otherwise, if cross-validation was performed, the
#' number of trees resulting in lowest cross-validation error will be
#' used; otherwise, all trees will be used.
#' 
#' @param rescale whether or not the result should be scaled. Defaults
#' to \code{FALSE}.
#' 
#' @param sort_it whether or not the results should be (reverse)
#' sorted.  Defaults to \code{FALSE}.
#' 
#' @return By default, returns an unprocessed vector of estimated
#' relative influences. If the \code{rescale} and \code{sort}
#' arguments are used, returns a processed version of the same.
#' 
#' @author Greg Ridgeway \email{gregridgeway@@gmail.com}
#'
#' @seealso \code{\link{summary.GBMFit}}
#' \href{https://www.stat.berkeley.edu/~breiman/randomforest2001.pdf}{Random
#' Forests}.
#' @keywords hplot
#' @export
permutation_relative_influence <- function(gbm_fit_obj,
                                           num_trees,
                                           rescale=FALSE,
                                           sort_it=FALSE) {
  # Checks initial inputs
  check_if_gbm_fit(gbm_fit_obj)
  check_if_natural_number(num_trees)
  if(num_trees > length(trees(gbm_fit_obj)))
    stop("num_trees exceeds maximum")
  if(is.na(rescale) || !is.logical(rescale) || (length(rescale) > 1))
    stop("rescale argument must be a logical")
  if(is.na(sort_it) || !is.logical(sort_it) || (length(sort_it) > 1))
    stop("sort_it must be a logical")
  if(is.null(gbm_fit_obj$gbm_data_obj))
    stop("Model was fit with keep_data=FALSE: permutation_relative_influence has not been implemented for that case.")
  
  # Get variables used in the model
  variables_indices <- sort(unique(unlist(lapply(trees(gbm_fit_obj)[seq_len(num_trees)],
                                      function(x){unique(x[[1]])}))))
  
  # Remove unused variables (those = "-1") and adjust indices from C convention 
  variables_indices <- variables_indices[variables_indices != -1] + 1
  
  # Set up rel_inf results
  rel_inf <- rep(0, length(gbm_fit_obj$variables$var_names))
  
  # Extract data for loss calculation
  y            <- gbm_fit_obj$gbm_data_obj$y
  offset       <- gbm_fit_obj$gbm_data_obj$offset
  weights      <- gbm_fit_obj$gbm_data_obj$weights
  x            <- matrix(gbm_fit_obj$gbm_data_obj$x, ncol=length(gbm_fit_obj$variables$var_names))
  gbm_fit_obj$Terms <- NULL # this makes predict.GBMFit take x as it is
    
  # Shuffle rows - indices
  shuffled_rows <- sample(1:nrow(x))
  for(i in seq_len(length(variables_indices))) {
    # Shuffle  predictor variables
    x[ ,variables_indices[i]]  <- x[shuffled_rows, variables_indices[i]]
    new_preds <- predict(gbm_fit_obj,
                         newdata=as.data.frame(x),
                         n.trees=num_trees)
    
    # Calculate loss with shuffled data
    rel_inf[variables_indices[i]] <-
        loss(y, new_preds, weights, offset,
             gbm_fit_obj$distribution,
             baseline=rep(iteration_error(gbm_fit_obj, 'train')[num_trees],
                 length(weights)))
    
    # Unshuffle variable - only permute variables one at a time
    x[shuffled_rows, variables_indices[i]] <- x[ ,variables_indices[i]]
  }
  
  # Scale and sort relative influence
  if (rescale) rel_inf <- rel_inf / max(rel_inf)
  if (sort_it) rel_inf <- rev(sort(rel_inf))
  
  return(rel_inf)
}

# GBMFit loss
# 
# S3 method for calculating the loss given data and a GBMDist object
# 
# @param y a vector of responses.
# 
# @param predictions vector of predicted responses.
# 
# @param weights weightings of each point in loss calculation.
# 
# @param offset offset for each prediction.
# 
# @param distribution_obj a GBMDist object which determines how the loss will
# be calculated.
# 
# @param baseline a vector of doubles specifying the baseline from which the
# loss is calculated. This defaults to 0.
# 
# @author James Hickey
# 
# @return the loss associated with the fit and distribution - vector of doubles.
# 
# @export 
loss <- function(y, predictions, weights, offset, distribution_obj, baseline=rep(0, length(y))) {
  # Check inputs
  check_if_gbm_dist(distribution_obj)
  if(is.logical(y) || 
     is.character(y) || 
     any(y != as.double(y)) || 
     any(is.na(y))) {
    stop("Responses must be doubles")
  }
  if(is.logical(predictions) || 
     is.character(predictions) || 
     any(predictions != as.double(predictions)) || 
     any(is.na(predictions))) {
    stop("Predictions must be doubles")
  }
  if(is.logical(weights) || 
     is.character(weights) || 
     any(weights != as.double(weights)) || 
     any(is.na(weights))) {
    stop("Weights must be doubles")
  }
  if(is.logical(baseline) || 
     is.character(baseline) || 
     any(baseline != as.double(baseline)) || 
     any(is.na(baseline))) {
    stop("Baseline must consist of doubles")
  }
  
  if(is.logical(offset) || 
     is.character(offset) || 
     any(offset != as.double(offset)) || 
     any(is.na(offset))) {
    stop("Offset must consist of doubles")
  }
  
  if((length(y) != length(predictions)) || 
     (length(predictions) != length(weights)) || 
     (length(y) != length(baseline))) {
    stop("Predictions, responses, weights and baseline all must have the same number
         of elements")
  }
  
  if((length(offset) != length(predictions))) {
    stop("Offset must be the same length as the prediction ")
  }
   
  UseMethod("loss", distribution_obj)
}

# @name loss
# @export
#' @export
loss.default <- function(y, predictions, weights, offset, distribution_obj, 
                         baseline=rep(0, length(y))) {
  stop("loss function not specified for distribution object provided.")
}

# @name loss
# @export
#' @export
loss.AdaBoostGBMDist <- function(y, predictions, weights, offset, 
                                 distribution_obj, baseline=rep(0, length(y))) {
  return(weighted.mean(exp(-(2*y-1)*(predictions+offset)), weights) - baseline)
}

# @name loss
# @export
#' @export
loss.BernoulliGBMDist <- function(y, predictions, weights, offset, 
                                  distribution_obj, baseline=rep(0, length(y))) {
  eta <- predictions + offset
  softplus <- numeric(length(eta))
  positive_eta <- eta > 0
  softplus[positive_eta] <- eta[positive_eta] + log1p(exp(-eta[positive_eta]))
  softplus[!positive_eta] <- log1p(exp(eta[!positive_eta]))
  return(-2*weighted.mean(y*eta - softplus, weights) - baseline)
}

# @name loss
# @export
#' @export
loss.CoxPHGBMDist <- function(y, predictions, weights, offset, 
                              distribution_obj, baseline=rep(0, length(y))) {
  stop("Loss method for ",  class(distribution_obj)[1]," not yet supported.")
}

# @name loss
# @export
#' @export
loss.GammaGBMDist <- function(y, predictions, weights, offset, 
                              distribution_obj, baseline=rep(0, length(y))) {
  stop("Loss method for ",  class(distribution_obj)[1]," not yet supported.")
}

# @name loss
# @export
#' @export
loss.GaussianGBMDist <- function(y, predictions, weights, offset, 
                                 distribution_obj, baseline=rep(0, length(y))) {
  return(weighted.mean((y - predictions - offset)^2, weights) - baseline)
}

# @name loss
# @export
#' @export
loss.HuberizedGBMDist <- function(y, predictions, weights, offset, 
                                  distribution_obj, baseline=rep(0, length(y))) {
  stop("Loss method for ",  class(distribution_obj)[1]," not yet supported.")
}

# @name loss
# @export
#' @export
loss.LaplaceGBMDist <- function(y, predictions, weights, offset, 
                                distribution_obj, baseline=rep(0, length(y))) {
  return(weighted.mean(abs(y-predictions - offset), weights) - baseline)
}

# @name loss
# @export
#' @export
loss.PairwiseGBMDist <- function(y, predictions, weights, offset, 
                                 distribution_obj, baseline=rep(0, length(y))) {
  if(is.null(distribution_obj$group_index)) stop("loss for pairwise requires group_index field to be specified")
  return((1 - perf_pairwise(y, predictions+offset, distribution_obj$group_index,
                            distribution_obj$metric, 
                            weights, distribution_obj$max_rank)) - baseline)
}

# @name loss
# @export
#' @export
loss.PoissonGBMDist <- function(y, predictions, weights, offset, 
                                distribution_obj, baseline=rep(0, length(y))) {
  return(-2*weighted.mean(y*(predictions+offset)-exp(predictions+offset), weights) - baseline)
}

# @name loss
# @export
#' @export
loss.QuantileGBMDist <- function(y, predictions, weights, offset, 
                                 distribution_obj, baseline=rep(0, length(y))) {
  stop("Loss method for ",  class(distribution_obj)[1]," not yet supported.")
}

# @name loss
# @export
#' @export
loss.TDistGBMDist <- function(y, predictions, weights, offset, 
                              distribution_obj, baseline=rep(0, length(y))) {
  stop("Loss method for ",  class(distribution_obj)[1]," not yet supported.")
}

# @name loss
# @export
#' @export
loss.TweedieGBMDist <- function(y, predictions, weights, offset, 
                                distribution_obj, baseline=rep(0, length(y))) {
  stop("Loss method for ",  class(distribution_obj)[1]," not yet supported.")
}

#' Summary of a GBMFit object
#' 
#' Computes the relative influence of each variable in the
#' \code{GBMFit} object.
#' 
#' For \code{GBMGaussianDist} this returns exactly the reduction of
#' squared error attributable to each variable. For other loss
#' functions this returns the reduction attributable to each variable
#' in sum of squared error in predicting the gradient on each
#' iteration. It describes the relative influence of each variable in
#' reducing the loss function. See the references below for exact
#' details on the computation.
#' 
#' @param object a \code{GBMFit} object created from an initial call
#' to \code{\link{gbmt}}.
#' 
#' @param cBars the number of bars to plot. If \code{order_it=TRUE}
#' then only the \code{cBars} variables with the largest relative
#' influence will appear in the barplot. If \code{order_it=FALSE} then
#' the first \code{cBars} variables will appear in the plot. In either
#' case, the function will return the relative influence of all of the
#' variables.
#' 
#' @param num_trees the number of trees used to generate the
#' plot. Only the first \code{num_trees} trees will be used.
#' 
#' @param plot_it an indicator as to whether the plot is generated.
#' 
#' @param order_it an indicator as to whether the plotted and/or
#' returned relative influences are sorted.
#' 
#' @param method The function used to compute the relative influence.
#' \code{\link{relative_influence}} is the default and is the same as
#' that described in Friedman (2001). The other current (and
#' experimental) choice is
#' \code{\link{permutation_relative_influence}}. This method randomly
#' permutes each predictor variable at a time and computes the
#' associated reduction in predictive performance. This is similar to
#' the variable importance measures Breiman uses for random forests,
#' but \code{gbm} currently computes using the entire training dataset
#' (not the out-of-bag observations).
#' 
#' @param normalize if \code{FALSE} then \code{summary.gbm} returns
#' the unnormalized influence.
#' 
#' @param ...  other arguments passed to the plot function.
#' 
#' @return Returns a data frame where the first component is the
#' variable name and the second is the computed relative influence,
#' normalized to sum to 100.
#' @author James Hickey, Greg Ridgeway \email{gregridgeway@@gmail.com}
#' @seealso \code{\link{gbmt}}
#' @references J.H. Friedman (2001). "Greedy Function Approximation: A
#' Gradient Boosting Machine," Annals of Statistics 29(5):1189-1232.
#' 
#' L. Breiman (2001). \href{https://www.stat.berkeley.edu/~breiman/randomforest2001.pdf}{Random Forests}.
#' @keywords hplot
#' @export
summary.GBMFit <- function(object,
                        cBars=length(object$variables$var_names),
                        num_trees=length(trees(object)),
                        plot_it=TRUE,
                        order_it=TRUE,
                        method=relative_influence,
                        normalize=TRUE,
                        ...)
{
  # Initial checks
  check_if_natural_number(num_trees)
  check_if_natural_number(cBars)
  check_if_gbm_fit(object)
  if(!is.logical(plot_it) || (length(plot_it) > 1) || is.na(plot_it)) {
    stop("argument plot_it must be a logical - excluding NA")
  }  
  if(!is.logical(order_it) || (length(order_it) > 1) || is.na(order_it)) {
    stop("argument order_it must be a logical - excluding NA")
  }  
  if(!is.logical(normalize) || (length(normalize) > 1) || is.na(normalize)) {
    stop("argument normalize must be a logical - excluding NA")
  }  
  
  # Set inputs (if required)
  if(cBars==0) cBars <- min(10, length(object$variables$var_names))
  if(cBars>length(object$variables$var_names)) cBars <- length(object$variables$var_names)
  if(num_trees > object$params$num_trees)
    warning("Exceeded total number of GBM terms. Results use num_trees=", object$params$num_trees," terms.\n")
  num_trees <- min(num_trees, object$params$num_trees)
  
  # Calculate relative influence and order/normalize
  rel_inf <- method(object, num_trees=num_trees)
  rel_inf[rel_inf<0] <- 0
  if(normalize) rel_inf <- 100*rel_inf/sum(rel_inf)
  
  ordering <- seq_len(length(rel_inf))
  if(order_it) {
    ordering <- order(-rel_inf)
  }
  
  # Bar plot of relative influence
  if(plot_it) {
    barplot(rel_inf[ordering[cBars:1]],
            horiz=TRUE,
            col=rainbow(cBars,start=3/6,end=4/6),
            names=object$variables$var_names[ordering[cBars:1]],
            xlab="Relative influence",
            las=1,...)
  }
  return(data.frame(var=object$variables$var_names[ordering],
                    rel_inf=rel_inf[ordering]))
}

#' @describeIn gbm Core fitting code, for experts only.
#' @export 
gbm.fit <- function(x, y,
                    offset = NULL,
                    distribution = "bernoulli",
                    w = NULL,
                    var.monotone = NULL,
                    n.trees = 100,
                    interaction.depth = 1,
                    n.minobsinnode = 10,
                    shrinkage = 0.001,
                    bag.fraction = 0.5,
                    nTrain = NULL,
                    train.fraction = NULL,
                    mFeatures = NULL,
                    keep.data = TRUE,
                    verbose = TRUE,
                    var.names = NULL,
                    response.name = "y",
                    group = NULL,
                    tied.times.method="efron",
                    prior.node.coeff.var = 1000,
                    strata = NA, obs.id = 1:nrow(x)) {
  # Reconstruct data
  if(is.null(var.names)) {
    var.names <- get_var_names(x)
  }
 
  # Calculate nTrain if necessary 
  if(!is.null(nTrain) && !is.null(train.fraction)) {
    stop("Parameters 'nTrain' and 'train.fraction' cannot both be specified")
    
  } else if(!is.null(train.fraction)) {
    warning("Parameter 'train.fraction' of gbm.fit is deprecated, please specify 'nTrain' instead")
    unique_obs <- unique(obs.id)
    nTrain <- floor(train.fraction*length(unique_obs))
  } else if(is.null(nTrain)) {
    # both undefined, use all training data
    nTrain <- length(unique(obs.id))
  }
  
  # Set offset/ weights if not specified
  if(is.null(w)) {
    weights <- rep(1, length(obs.id))
  } 
  if(is.null(offset)){
    offset <- rep(0, length(obs.id))
  }
  
  # Set distribution object - put in all possible additional parameters (this will generate warnings)
  if (is.character(distribution)){ distribution <- list(name=distribution) }
  dist_obj <- create_dist_obj_for_gbmt_fit(distribution, tied.times.method, strata, prior.node.coeff.var)
  
  # Set up training parameters
  if(is.null(mFeatures)) mFeatures <- ncol(x)
  params <- training_params(num_trees=n.trees, interaction_depth=interaction.depth, min_num_obs_in_node=n.minobsinnode, 
                            shrinkage=shrinkage, bag_fraction=bag.fraction, id=obs.id,
                            num_train=nTrain, num_features=mFeatures)
  
  # Call gbm2 - formula defined from the data
  gbm_fit_obj <- gbmt_fit(x, y, distribution=dist_obj, weights=w, offset=offset,
                      train_params= params, var_monotone=var.monotone, var_names=var.names,
                      keep_gbm_data=keep.data, is_verbose=verbose)
  return(gbm_fit_obj)
}

# Create Strata 
# 
# Function used to create to appropriate strata vectors -
# currently only implemented for the CoxPH distribution.

# 
# @param gbm_data_obj gbm data object
# 
# @param train_params gbm training parameters object
# 
# @param distribution_obj a gbm distribution object - the strata in this object will be
# updated if they exist - this is only the case for the CoxPH model at this moment.
# 
# @param order_strata_by_id indicates whether or not to order the entire strata
# vector by observation id. This defaults to \code{TRUE} but should be set to \code{FALSE}
# for cross-validation strata creation. 
# 
# @author James Hickey
# 
# @return updated distribution_obj
# 

create_strata <- function(gbm_data_obj, train_params, distribution_obj, order_strata_by_id=TRUE) {
  check_if_gbm_dist(distribution_obj)
  check_if_gbm_data(gbm_data_obj)
  
  # Put in defaults
  if(is.null(distribution_obj$sorted)) distribution_obj$sorted <- NA
  if(is.null(distribution_obj$strata)) distribution_obj$strata <- NA
  
  if(distribution_name(distribution_obj) == "CoxPH") {
    num_train_rows <- train_params$num_train_rows
    num_test_rows <- nrow(gbm_data_obj$x) - num_train_rows
    
    # Determine test indices
    if(num_test_rows == 0) {
      test_indices <- 0
    } else {
      test_indices <- (num_train_rows+1):nrow(gbm_data_obj$x)
    }
      
    # Set up strata 
    if(!is.na(distribution_obj$original_strata_id[1])) {
      # Sort strata according to patient ID
      distribution_obj$strata <- distribution_obj$original_strata_id
      if(order_strata_by_id) {
        distribution_obj$strata <- distribution_obj$strata[order(train_params$id)]
      }
      
      # Order strata and split into train/test
      strataVecTrain <- distribution_obj$strata[seq_len(num_train_rows)]
      strataVecTest <- distribution_obj$strata[test_indices]
      
      # Cum sum the number in each stratum and pad with NAs
      # between train and test strata
      strataVecTrain <- as.vector(cumsum(table(strataVecTrain)))
      strataVecTest <- as.vector(cumsum(table(strataVecTest)))
      
      strataVecTrain <- c(strataVecTrain, rep(NA, num_train_rows-length(strataVecTrain)))
      
      # If no test set make empty
      if(num_test_rows == 0) {
        strataVecTest <- c() 
      } else {
        strataVecTest <- c(strataVecTest, rep(NA, max(num_test_rows-length(strataVecTest), 0)))  
      }
    
      # Recreate Strata Vec to Pass In
      nstrat <- c(strataVecTrain, strataVecTest)
      
    }
    else
    {
      # Put all the train and test data in a single stratum
      distribution_obj$strata <- rep(1, nrow(gbm_data_obj$x))
      trainStrat <- c(num_train_rows, rep(NA, num_train_rows-1))
      if(num_test_rows == 0) {
        testStrat <- c()
      } else {
        testStrat <- c(num_test_rows, rep(NA, max(num_test_rows-1, 0)))
      }
      nstrat <- c(trainStrat, testStrat)
    }
    
    # Sort response according to strata
    # i_order sets order of outputs
    if (ncol(gbm_data_obj$y)==2) {
      sorted <- c(order(distribution_obj$strata[seq_len(num_train_rows)], -gbm_data_obj$y[seq_len(num_train_rows), 1]),
                  order(distribution_obj$strata[test_indices],
                        -gbm_data_obj$y[test_indices, 1])) 
      i_order <- c(order(distribution_obj$strata[seq_len(num_train_rows)], -gbm_data_obj$y[1:num_train_rows, 1]),
                   order(distribution_obj$strata[test_indices],
                         -gbm_data_obj$y[test_indices, 1]) + num_train_rows)
    } else if (ncol(gbm_data_obj$y)==3) {
      sorted <- cbind(c(order(distribution_obj$strata[seq_len(num_train_rows)], -gbm_data_obj$y[seq_len(num_train_rows), 1]),
                        order(distribution_obj$strata[test_indices], -gbm_data_obj$y[test_indices, 1])),
                      c(order(distribution_obj$strata[seq_len(num_train_rows)], -gbm_data_obj$y[seq_len(num_train_rows), 2]),
                        order(distribution_obj$strata[test_indices], -gbm_data_obj$y[test_indices, 2])))
      i_order <- c(order(distribution_obj$strata[seq_len(num_train_rows)], -gbm_data_obj$y[seq_len(num_train_rows), 1]),
                   order(distribution_obj$strata[test_indices], -gbm_data_obj$y[test_indices, 1]) + num_train_rows)
    }
    else
    {
      stop("Survival object must be either right or counting type.")
    }
    
    
    # Add in sorted column and strata
    StrataVec <-  nstrat
    sortedVec <- sorted-1L
    
    distribution_obj$time_order <- i_order
    distribution_obj$sorted <- as.matrix(as.data.frame(sortedVec))
    distribution_obj$strata <- as.double(StrataVec)
    
  }
  
  return(distribution_obj)
}

# Predictor Order
# 
# Function that calculates the predictor variable order for GBM.
# This is used internally to create the order of variables when growing
# trees in the C++ layer.
# 
# @usage predictor_order(gbm_data_obj, train_params)
# 
# @param gbm_data_obj a \code{GBMData} object.
# 
# @param train_params a \code{GBMTrainParams} object.
#
# @return an updated gbm_data_object that now contains the predictor variable order.
#
# @author James Hickey

predictor_order <- function(gbm_data_obj, train_params) {
  gbm_data_obj$x_order <- apply(gbm_data_obj$x[seq_len(train_params$num_train_rows),,drop=FALSE],2,order,na.last=FALSE)-1
  return(gbm_data_obj)
}

# Check Distribution Parameters
# 
# Check if function parameters for creating distribution
# object are of correct form - internal function.
# 
# @usage check_dist_params(empty_obj, ...)
# 
# @param empty_obj A skeleton distribution object 
# (see \code{\link{empty_distribution()}}).
# 
# @param ... Parameters defining how to create a full distribution object,
#  see \code{\link{gbm_dist}}.
#  
# @author James Hickey
#
# @return warnings/Errors
# 


check_dist_params <- function(empty_obj, ...) {
  UseMethod("check_dist_params", empty_obj)
}

#' @export
check_dist_params.AdaBoostGBMDist <- function(empty_obj, ...) {
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class does not use any additional
            parameters in construction.")
  }
}

#' @export
check_dist_params.BernoulliGBMDist <- function(empty_obj, ...) {
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class does not use any additional
            parameters in construction.")
  }
}

#' @export
check_dist_params.CoxPHGBMDist <- function(empty_obj, strata, sorted, ties, prior_node_coeff, ...) {
  # Check if additional parameters specified
  if((length(list(...)) > 0)) {
    warning("The ", class(empty_obj)[1], "class only requires 4 additional
            parameters for construction - others provided are ignored.")
  }
  # Check ties
  if(!exists("ties")) {
    stop("Ties parameter not specified - distribution can't be constructed")
  } else if(!is.character(ties)) {
    stop("Ties parameter must be a character vector - distribution
         can't be constructed")
  } else if(!(ties %in% c("breslow", "efron"))) {
    stop("Ties parameter must be either 'breslow' or 'efron'")
  }
  
  # Check strata
  if(!exists("strata")) {
    stop("Strata not specified - distribution could not be constructed")
  } else if (is.null(strata)) {
    stop("Strata should not be NULL")
  } else if(!is.atomic(strata) || 
            any(is.infinite(strata)) || 
            (!is.factor(strata) &&
             any(!is.na(strata) & (strata!=as.integer(strata))))) {
    stop("Strata parameter must be an atomic of integers or factors")
  } 
  
  # Check sorted
  if(!exists("sorted")) {
    stop("Sorted not specified - distribution could not be constructed")
  } else if(is.null(sorted)) {
    stop("Sorted parameter cannot be NULL")
  } else if(!is.atomic(sorted) || 
            any(is.infinite(sorted)) || 
            any(!is.na(sorted) & (sorted!=as.integer(sorted)))) {
    stop("Sorted parameter must be an atomic of integers")
  } 
  
  # Check coeff
  if(!exists("prior_node_coeff")) {
    stop("Prior node coefficient of variation not specified - distribution could not be constructed")
  } else if(!is.double(prior_node_coeff) || is.infinite(prior_node_coeff) ||
            (length(prior_node_coeff) > 1) || prior_node_coeff <= 0.0) {
    stop("Prior node coefficient must be a positive finite double - distribution could not be constructed")
  } 
}

#' @export
check_dist_params.GammaGBMDist <- function(empty_obj, ...) {
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class does not use any additional parameters in construction.")
  }
}

#' @export
check_dist_params.GaussianGBMDist <- function(empty_obj, ...) {
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], " class does not use any additional
            parameters in construction.")
  }
}

#' @export
check_dist_params.HuberizedGBMDist <- function(empty_obj, ...) {
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class does not use any additional
            parameters in construction.")
  }
}

#' @export
check_dist_params.LaplaceGBMDist <- function(empty_obj, ...) {
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class does not use any additional
            parameters in construction.")
  }
}



#' @export
check_dist_params.PairwiseGBMDist <- function(empty_obj, group, metric,
                                              max_rank, group_index, ...) {
  # Check if parameters are specified
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class only requires 3 additional
            parameters for construction - others provided are ignored")
  }
  
  # Check group is specified correctly
  if(!exists("group")) {
    stop("Group not specified - ", class(empty_obj)[1], " cannot be initialized.")
  } else if(!is.atomic(group) || (length(group) > 1) || !is.character(group)) {
    stop("Group parameter isn't a character atomic - ", class(empty_obj)[1], 
         " cannot be constructed.")
  }
  
  # Check Metric
  if(!exists("metric")) {
    stop("Metric not specified - ", class(empty_obj)[1], " cannot be constructed.")
  } else if(!(metric %in% c("conc", "mrr", "map", "ndcg"))) {
    stop("Metric specified not supported by gbm package")
  }
    
  # Check max_rank is specified correctly
  if(!exists("max_rank")) {
    stop("Max rank not specified - check default settings")
    
  } else if(!is.double(max_rank) || is.infinite(max_rank) || (length(max_rank) > 1)
            || (max_rank < 0.0)) {
    stop("Max rank provided is not a finite double greater than zero - distribution cannot be constructed")
  } else if((max_rank != 0.0) && (metric %in% c("conc", "map"))){
    stop("Max rank cannot be specified for metrics - conc and map")
  }
  
  # Check group_index
  if(!is.null(group_index)) {
    message("group_index provided this will be updated according to group and data if calling gbmt.")
    if(!is.vector(group_index) || any(group_index != as.integer(group_index))) {
    stop("if group indices are provided it must be a vector of integers")
    }
  }
}

#' @export
check_dist_params.PoissonGBMDist <- function(empty_obj, ...) {
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class does not use any additional
            parameters in construction.")
  }
}

#' @export
check_dist_params.QuantileGBMDist <- function(empty_obj, alpha, ...) {
  # Check if parameters are specified
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class only requires one additional
            parameter for construction - others are ignored")
  }
  
  if(!exists("alpha")) {
    stop("Alpha is not specified - distribution cannot be specified")
  } else if (!is.double(alpha) || (length(alpha) > 1) || is.infinite((alpha))
             || (alpha < 0.0) || (alpha > 1.0)) {
    stop("Alpha provided is not a finite double between 0.0 and 1.0 - distribution 
         cannot be constructed")
  }
}

#' @export
check_dist_params.TDistGBMDist <- function(empty_obj, df, ...) {
  # Check if parameters are specified
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class only requires one additional
            parameter for construction - others are ignored")
  }

  if(!exists("df")) {
    stop("Degrees of freedom (df) is not specified - distribution cannot be specified")
  } else if (!is.double(df) || (length(df) > 1) 
             || is.infinite((df)) || df < 0.0) {
    stop("df provided is not a finite double bigger than 0.0 - distribution 
         cannot be constructed")
  }
  
}

#' @export
check_dist_params.TweedieGBMDist <- function(empty_obj, power, ...) {
  # Check if parameters are specified
  if(length(list(...)) > 0) {
    warning("The ", class(empty_obj)[1], "class only requires one additional
            parameters for construction - others provided are ignored")
  }
  
  if(!exists("power")) {
    stop("Power of distribution (power) is not specified - distribution cannot be specified")
  } else if (!(is.double(power)) || (length(power) > 1)
             || is.infinite((power)) || is.na(power)) {
    stop("Power provided is not a finite double  - distribution 
         cannot be constructed")
  } else if (power == 0.0) {
    stop("Tweedie power cannot be 0 in this implementation. Use the Gaussian distribution instead.")
  } else if (power == 1.0) {
    stop("Tweedie power cannot be 1 in this implementation. Use the Poisson distribution instead.")
  } else if (power == 2.0) {
    stop("Tweedie power cannot be 2 in this implementation. Use the Gamma distribution instead.")
  } else if (power > 0.0 && power < 1.0) {
    stop("Tweedie power must not be between 0 and 1.")
  }
}

#' Calibration plot
#' 
#' An experimental diagnostic tool that plots the fitted values versus the
#' actual average values. If \code{distribution} is "Bernoulli" or "Poisson", 
#' then the predictions are converted to the response scale (probability or
#' rate). For all other distributions, the calibration plot uses least squares
#' and predicts an expected value.
#' 
#' Uses natural splines to estimate E(y|p). Well-calibrated predictions imply
#' that E(y|p) = p. The plot also includes a pointwise 95% confidence band.
#' 
#' @param y the outcome 0-1 variable
#' @param p the predictions estimating E(y|x)
#' @param distribution the loss function used in creating \code{p}.
#' \code{Bernoulli} and \code{Poisson} are currently the only special options.
#' All others default to squared error assuming \code{Gaussian}
#' @param replace determines whether this plot will replace or overlay the
#' current plot.  \code{replace=FALSE} is useful for comparing the calibration
#' of several methods
#' @param line.par graphics parameters for the line
#' @param shade_col color for shading the 2 SE region. \code{shade_col=NA}
#' implies no 2 SE region
#' @param shade_density the \code{density} parameter for \code{\link{polygon}}
#' @param rug.par graphics parameters passed to \code{\link{rug}}
#' @param xlab x-axis label corresponding to the predicted values
#' @param ylab y-axis label corresponding to the observed average
#' @param xlim,ylim x and y-axis limits. If not specified the function will
#' select limits
#' @param knots,df these parameters are passed directly to
#' \code{\link[splines]{ns}} for constructing a natural spline smoother for the
#' calibration curve
#' @param ...  other graphics parameters passed on to the plot function
#' @return \code{calibrate.plot} returns no values.
#' @author James Hickey, Greg Ridgeway \email{gregridgeway@@gmail.com}
#' @references J.F. Yates (1982). "External correspondence: decomposition of
#' the mean probability score," Organisational Behaviour and Human Performance
#' 30:132-156.
#' 
#' D.J. Spiegelhalter (1986). "Probabilistic Prediction in Patient Management
#' and Clinical Trials," Statistics in Medicine 5:421-433.
#' @keywords hplot
#' @examples
#' 
#' dataSim <- data.frame(x=rnorm(1000))
#' dataSim$y <- with(dataSim, rbinom(1000, 1, 1/(1+exp(-(x-0.5*x^2)))))
#' 
#' # showing poor calibration of a linear model
#' glm1 <- glm(y~x, data=dataSim, family=binomial)
#' p <- predict(glm1, type="response")
#' calibrate_plot(dataSim$y, p, xlim=c(0,1), ylim=c(0,1))
#' 
#' # showing better calibration with quadratic
#' glm1 <- glm(y~poly(x,2), data=dataSim, family=binomial)
#' p <- predict(glm1,type="response")
#' calibrate_plot(dataSim$y, p, xlim=c(0,1), ylim=c(0,1))
#' 
#' @importFrom splines ns
#' @export 
calibrate_plot <- function(y, p,
                           distribution="Bernoulli",
                           replace=TRUE,
                           line.par=list(col="black"),
                           shade_col="lightyellow",
                           shade_density=NULL,
                           rug.par=list(side=1),
                           xlab="Predicted value",
                           ylab="Observed average",
                           xlim=NULL,ylim=NULL,
                           knots=NULL,df=6,
                           ...) {
  # Some initial checks
  if(is.null(knots) && is.null(df))
    stop("Either knots or df must be specified")
  if(!is.null(knots) && !is.null(df))
  {
    warning("knots and df are both supplied. Using only knots and ignoring df.")
    df <- NULL
  }
  if(!is.null(df))
  {
    if(length(df)>1)
    {
      warning("Multiple values of df are given. Using only the first one.")
      df <- df[1]
    }
    if((df != round(df)) || (df<1))
      stop("df must be a positive integer.")
  }
  
  # Set up data and family
  data <- data.frame(y=y,p=p)
  family1 <- switch(tolower(distribution),
                    bernoulli=binomial,
                    poisson=poisson,
                    gaussian)
  
  # Fit a generalized linear model using splined predictions
  # as predictor variables
  gam1 <- glm(y~ns(p, df=df, knots=knots), data=data, family=family1)
  
  # Set up x and y values (new pre)
  x <- seq(min(p),max(p),length=200)
  glm_preds <- predict(gam1, newdata=data.frame(p=x), se.fit=TRUE, type="response")
  
  # Remove NAs
  x <- x[!is.na(glm_preds$fit)]
  glm_preds$se.fit <- glm_preds$se.fit[!is.na(glm_preds$fit)]
  glm_preds$fit <- glm_preds$fit[!is.na(glm_preds$fit)]
  
  # Set shading limits and 
  if(!is.na(shade_col)) {
    se.lower <- glm_preds$fit-2*glm_preds$se.fit
    se.upper <- glm_preds$fit+2*glm_preds$se.fit
    
    # Truncate upper and lower values - so they
    # make sense for the distribution under consideration
    if(tolower(distribution) %in% c("bernoulli", "poisson", "gamma", "tweedie"))
      se.lower[se.lower < 0] <- 0
    if(tolower(distribution)=="bernoulli") se.upper[se.upper > 1] <- 1
    
    # Set x and y limits
    if(is.null(xlim)) xlim <- range(se.lower,se.upper,x)
    if(is.null(ylim)) ylim <- range(se.lower,se.upper,x)
    
  } else {
    if(is.null(xlim)) xlim <- range(glm_preds$fit,x)
    if(is.null(ylim)) ylim <- range(glm_preds$fit,x)
  }
  
  
  # Plot
  if(replace) {
    plot(0,0,
         type="n",
         xlab=xlab,ylab=ylab,
         xlim=xlim,ylim=ylim,
         ...)
  }
  
  # Add shaded
  if(!is.na(shade_col)) {
    polygon(c(x,rev(x),x[1]),
            c(se.lower,rev(se.upper),se.lower[1]),
            col=shade_col,
            border=NA,
            density=shade_density)
  }
  lines(x, glm_preds$fit,col=line.par$col)
  quantile_rug(p,side=rug.par$side)
  abline(0, 1, col="red")
}

####################
# Author: James Hickey
#
# Series of tests to check if calibration plots working
#
####################


context("Testing calibration plot")
test_that("Error thrown if neither the knots or df specified - both NULL", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  
  # Then error thrown when both knots and df are NULL
  expect_error(calibrate_plot(y, p, df=NULL, knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
})
test_that("Error thrown if df is not a positive integer (if vector first element must be a positive integer)", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  
  # Then error thrown when df Not a positive integer
  expect_error(calibrate_plot(y, p, df=0, knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
  expect_error(calibrate_plot(y, p, df=1.4, knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
  expect_error(calibrate_plot(y, p, df="Wrong", knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
  expect_warning(calibrate_plot(y, p, df=c(1, 2), knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
  expect_error(calibrate_plot(y, p, df=c(1.4, 2), knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
  expect_error(calibrate_plot(y, p, df=NaN, knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
  expect_error(calibrate_plot(y, p, df=NA, knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
  expect_error(calibrate_plot(y, p, df=Inf, knots=NULL, xlim=c(0,0.6), ylim=c(0,0.6)))
})
test_that("Error thrown if y and p not same length", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  p <- p[seq(length(p)-1)]
  
  # Then error thrown
  expect_error(calibrate_plot(y, p))
})
test_that("Can run with defaults", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  
  
  # Then no error thrown
  expect_error(calibrate_plot(y, p), NA)
})
test_that("Can run with shade_col not NA", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  
  
  # Then no error thrown
  expect_error(calibrate_plot(y, p, shade_col=1), NA)
})
test_that("Can run with replace = FALSE", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  
  
  # Then no error thrown
  expect_error(calibrate_plot(y, p, replace=FALSE), NA)
})
test_that("Can run with shade_density != NULL", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  
  
  # Then no error thrown
  expect_error(calibrate_plot(y, p, shade_density=2.0), NA)
})
test_that("Can run  all distributions", {
  # Given data - based on example - but y and p not same length now
  kyph_dat <- data.frame(Kyphosis=sample(as.factor(c("absent", "present")), 100, replace=TRUE),
                         Age = sample(seq_len(175), 100, replace=TRUE), ncol=2)
  y <- as.numeric(kyph_dat$Kyphosis)-1
  x <- kyph_dat$Age
  glm1 <- glm(y~poly(x,2),family=binomial)
  p <- predict(glm1, type="response")
  
  
  # Then no error thrown
  expect_error(calibrate_plot(y, p, distribution = "AdaBoost"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Bernoulli"), NA)
  expect_error(calibrate_plot(y, p, distribution = "CoxPH"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Gamma"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Gaussian"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Laplace"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Huberized"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Pairwise"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Poisson"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Quantile"), NA)
  expect_error(calibrate_plot(y, p, distribution = "TDist"), NA)
  expect_error(calibrate_plot(y, p, distribution = "Tweedie"), NA)
})

# Check Response Values
#
# check responses are consistent with the distribution specified.
# 
# @usage check_response_values(y, distribution_obj)
# 
# @param y a matrix or vector of outcomes
# 
# @param distribution_obj a GBMDist object
# 
# @author James Hickey
#
# @return Warnings/Errors
# 

check_response_values <- function(distribution_obj, y) {
  # Check inputs
  if( !inherits(distribution_obj,
        paste0(available_distributions(),"GBMDist")) ) {
    stop("Distribution object not recognised - use gbm_dist to create a valid object")
  }
  
  if(!is.matrix(y) && !is.data.frame(y) &&
     !is.atomic(y)) {
    stop("Responses must be in a dataframe, matrix, or vector")
  } 
  
  # Call correct method
  UseMethod("check_response_values", distribution_obj)
  
}

#' @export
check_response_values.AdaBoostGBMDist <-function(distribution_obj, y) {
  if(!all(is.element(y,0:1))) {
    stop("This version of AdaBoost requires the response to be in {0,1}")
  }
}

#' @export
check_response_values.BernoulliGBMDist <-function(distribution_obj, y) {
  if(!all(is.element(y, 0:1))) {
    stop("Bernoulli requires the response to be in {0,1}")
  }
}

#' @export
check_response_values.CoxPHGBMDist <-function(distribution_obj, y) {
  if(!inherits(y, "Surv")) {
    stop("Outcome must be a survival object Surv(time1, failure) 
         or Surv(time1, time2, failure)")
  }
  
  # Check length if not default
  if(!all(is.na(distribution_obj$original_strata_id)) && 
     (length(distribution_obj$original_strata_id) != nrow(y)) ){
    stop("Strata indices must be provided for every data point")
  }
}

#' @export
check_response_values.GammaGBMDist <-function(distribution_obj, y) {
  if(any(y<=0)) {
    stop("Gamma requires the response to be positive")
  }
}

#' @export
check_response_values.GaussianGBMDist <-function(distribution_obj, y) {}

#' @export
check_response_values.HuberizedGBMDist <-function(distribution_obj, y) {
  if(!all(is.element(y,0:1))) {
    stop("Huberized square hinged loss requires the response to be in {0,1}")
  }
}

#' @export
check_response_values.LaplaceGBMDist <-function(distribution_obj, y) {}

#' @export
check_response_values.PairwiseGBMDist <-function(distribution_obj, y) {
  if (any(y<0)) {
    stop("targets for 'pairwise' should be non-negative")
  }
  
  if (is.element(distribution_obj$metric, c("mrr", "map")) && 
      !all(is.element(y, 0:1))) {
    stop("Metrics 'map' and 'mrr' require the response to be in {0,1}")
  }
}

#' @export
check_response_values.PoissonGBMDist <-function(distribution_obj, y) {
  if(any(y != trunc(y)) || any(y<0)) {
    stop("Poisson requires the response to be a non-negative integer")
  }
}

#' @export
check_response_values.QuantileGBMDist <-function(distribution_obj, y) {}

#' @export
check_response_values.TDistGBMDist <-function(distribution_obj, y) {}

#' @export
check_response_values.TweedieGBMDist <-function(distribution_obj, y) {
  if(any(y<0)) {
    stop("Tweedie requires the response to be non-negative")
  }
}

# LOGISTIC REGRESSION EXAMPLE

cat("Running logistic regression example.\n")
# create some data
N <- 1000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
mu <- c(-1,0,1,2)[as.numeric(X3)]

p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
Y <- rbinom(N,1,p)

# random weights if you want to experiment with them
w <- rexp(N)
w <- N*w/sum(w)

data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)

train_params <- training_params(num_trees = 3000, 
                                shrinkage = 0.001, 
                                bag_fraction = 0.5,
                                num_train = N/2, 
                                id=seq_len(nrow(data)), 
                                min_num_obs_in_node = 10,
                                interaction_depth = 3, 
                                num_features = 3)

# fit initial model
gbm1 <- gbmt(Y~X1+X2+X3,                # formula
            data=data,                  # dataset
            weights=w,
            var_monotone=c(0,0,0),      # -1: monotone decrease, 
                                        # +1: monotone increase, 
                                        # 0: no monotone restrictions
            distribution=gbm_dist("Bernoulli"),
            train_params = train_params,
            cv_folds=5,                 # do 5-fold cross-validation
            is_verbose = FALSE)         # don't print progress

# plot the performance
# returns out-of-bag estimated best number of trees
best.iter.oob <- gbmt_performance(gbm1,method="OOB")  
print(best.iter.oob)
# returns 5-fold cv estimate of best number of trees
best.iter.cv <- gbmt_performance(gbm1,method="cv")   
print(best.iter.cv)
# returns test set estimate of best number of trees
best.iter.test <- gbmt_performance(gbm1,method="test") 
print(best.iter.test)

best.iter <- best.iter.test

# plot variable influence
summary(gbm1,num_trees=1)         # based on the first tree
summary(gbm1,num_trees=best.iter) # based on the estimated best number of trees

# create marginal plots
# plot variable X1,X2,X3 after "best" iterations
par.old <- par(mfrow=c(1,3))
plot(gbm1,1,best.iter)
plot(gbm1,2,best.iter)
plot(gbm1,3,best.iter)
par(mfrow=c(1,1))
plot(gbm1,1:2,best.iter) # contour plot of variables 1 and 2 after "best" number iterations
plot(gbm1,2:3,best.iter) # lattice plot of variables 2 and 3 after "best" number iterations

# 3-way plot
plot(gbm1,1:3,best.iter)

# print the first and last trees
print(pretty_gbm_tree(gbm1,1))
print(pretty_gbm_tree(gbm1, gbm1$params$num_trees))

# make some new data
N <- 1000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
mu <- c(-1,0,1,2)[as.numeric(X3)]

p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
Y <- rbinom(N,1,p)
data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)

# predict on the new data using "best" number of trees
# f.predict will be on the canonical scale (logit,log,etc.)
f.predict <- predict(gbm1,data2,
                     num_trees=c(best.iter.oob,best.iter.cv,best.iter.test))
# transform to probability scale for logistic regression
p.pred <- 1/(1+exp(-f.predict))

# calibration plot for logistic regression - well calibrated means a 45 degree line
par(mfrow=c(1,1))
calibrate_plot(Y,p.pred[,3])
par(par.old) # reset graphics options to previous settings

# logistic error
sum(data2$Y*f.predict[,1] - log(1+exp(f.predict[,1])))
sum(data2$Y*f.predict[,2] - log(1+exp(f.predict[,2])))
sum(data2$Y*f.predict[,3] - log(1+exp(f.predict[,3])))

# Adjust scales of prediction
# 
# S3 method (used internally within predict.GBMFit) that adjusts the scales of the predictor variable.
# 
# @usage adjust_pred_scale(gbm_predictions, distribution_obj)
# 
# @param gbm_predictions a matrix containing predictions generated from the C++ function \code{\link{gbm_pred}}.
# 
# @param distribution_obj a GBMDist object used in the fitting a gbm model and generating the gbm_predictions.
# 
# @author James Hickey
#
# @return gbm_predictions scaled to the response of the distribution
#
adjust_pred_scale <- function(gbm_predictions, distribution_obj) {
  UseMethod("adjust_pred_scale", distribution_obj)
}

#' @export
adjust_pred_scale.AdaBoostGBMDist <- function(gbm_predictions, distribution_obj) {
  gbm_predictions <- 1/(1 + exp(-2*gbm_predictions))
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.BernoulliGBMDist <- function(gbm_predictions, distribution_obj) {
  gbm_predictions <- 1/(1+exp(-gbm_predictions))
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.CoxPHGBMDist <- function(gbm_predictions, distribution_obj) {
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.GammaGBMDist <- function(gbm_predictions, distribution_obj) {
  gbm_predictions <- exp(gbm_predictions)
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.GaussianGBMDist <- function(gbm_predictions, distribution_obj) {
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.HuberizedGBMDist <- function(gbm_predictions, distribution_obj) {
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.LaplaceGBMDist <- function(gbm_predictions, distribution_obj) {
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.PairwiseGBMDist <- function(gbm_predictions, distribution_obj) {
  gbm_predictions <- 1/(1+exp(-gbm_predictions))
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.PoissonGBMDist <- function(gbm_predictions, distribution_obj) {
  gbm_predictions <- exp(gbm_predictions)
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.QuantileGBMDist <- function(gbm_predictions, distribution_obj) {
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.TDistGBMDist <- function(gbm_predictions, distribution_obj) {
  return(gbm_predictions)
}

#' @export
adjust_pred_scale.TweedieGBMDist <- function(gbm_predictions, distribution_obj) {
  gbm_predictions <- exp(gbm_predictions)
  return(gbm_predictions)
}

# Empty Distribution
#  
# Creates an empty distribution object
# 
# @usage empty_distribution(name)
# 
# @param name A string specifying which empty distribution object to build.
# 
# @author James Hickey
#
# @return An empty distribution class - only the name defined.
#

empty_distribution <- function(name) {
  return(structure(list(name=name, reorder=FALSE),
                   class=c(paste0(name, "GBMDist"), "GBMDist")))
}

#' Predictions for CV fitted GBM models
#' 
#' Calculate predictions from cross validated generalized boosting
#' model from gbm2.
#' 
#' @param object a GBMCVFit object containing CV gbm models
#' 
#' @param gbm_data_obj a GBMData object containing all of the data
#' used to fit a gbm model.
#' 
#' @param cv_folds a positive integer specifying the number of folds
#' to be used in cross-validation of the gbm fit.
#' 
#' @param cv_group vector of integers specifying which row of data
#' belongs to which cv_fold.
#' 
#' @param best_iter_cv number of trees with the smallest cv error
#'
#' @param \dots not currently used
#' 
#' @author James Hickey
#'
#' @return a matrix of predictions for each cv fold.
#' @importFrom stats predict
#' @export
predict.GBMCVFit <- function(object, gbm_data_obj, cv_folds,
                             cv_group, best_iter_cv, ...) {

  # Extract data
  data <- gbm_data_obj$original_data[seq_len(object[[1]]$params$num_train), ,drop=FALSE]
  
  ## test cv_group and data match
  if (nrow(data) != length(cv_group)) {
    stop("mismatch between data and cv_group")
  }
  
  #Flatten data for prediction
  num_cols <- 1
  result <- matrix(nrow=nrow(data), ncol=num_cols)
  data_names <- names(data)
  
  for (ind in seq_len(cv_folds)) {
    flag <- cv_group == ind
    model <- object[[ind+1]]
    my_data  <- data[flag, model$variables$var_names, drop=FALSE]
    predictions <- predict(model, newdata=my_data, n.trees=best_iter_cv)
    predictions <- matrix(predictions, ncol=num_cols)
    result[flag,] <- predictions
  }
  
  return(as.numeric(result))
}

---
title: "Guide to the Cox Proportional Hazards model"
author: "James Hickey"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{CoxPH}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, message=FALSE, echo=FALSE}
library("gbm3")
```

`gbm` facilitates the creation of boosted Cox proportional hazards models, a particularly useful feature when dealing with survival data.  The package can handle two types of survival object as the response, namely a right censored or counting survival object.  Both of these objects can be created using the `Surv` command found in the `survival` package. 

# Set-up of data and distribution object 
Right censored survival data consists of a time to event number and the event indicator - 0 if no event has taken place and 1 if the event has happened. On the other hand, counting survival data contains start and stop times along with an event indicator again indicating if an event has taken place in that period.  The data may be organised into strata and this should be passed to `gbm_dist` on creation of the `CoxPHGBMDist` object - see  the "Model Specific Parameters" vignette for more details.  The dataset considered here is provided by the `survival` package.

```{r, message=FALSE}
## Install package
require(survival)

# get datasets
right_cens <- cgd[cgd$enum==1, ]
start_stop <- cgd

# Set up GBMDist objects
right_cens_dist <- gbm_dist("CoxPH", strata=right_cens$hos.cat)
start_stop_dist <- gbm_dist("CoxPH", strata=start_stop$hos.cat)
```

# Creating a boosted model
Now to create the underlying boosted model the training parameters need to be defined and `gbmt` called. In this instance the data has observation ids associated with it and so it is necessary to create specific `GBMTrainParams` objects rather than relying on the defaults.

```{r, messagee=FALSE}
# Set-up training parameters
params_right_cens <- training_params(num_trees = 2000, interaction_depth = 3, 
                                     id=right_cens$id,
                                     num_train=round(0.5 * length(unique(right_cens$id))) )
params_start_stop <- training_params(num_trees = 2000, interaction_depth = 3, 
                                     id=start_stop$id,
                                     num_train=round(0.5 * length(unique(start_stop$id))) )

# Call to gbmt
fit_right_cens <- gbmt(Surv(tstop, status)~ age + sex + inherit +
                     steroids + propylac, data=right_cens, 
                     distribution = right_cens_dist,
                     train_params = params_right_cens, cv_folds=10,
                     keep_gbm_data = TRUE)
fit_start_stop <- gbmt(Surv(tstart, tstop, status)~ age + sex + inherit +
                     steroids + propylac, data=start_stop, 
                     distribution = start_stop_dist,
                     train_params = params_start_stop, cv_folds=10, 
                     keep_gbm_data = TRUE)

# Plot performance
best_iter_right <- gbmt_performance(fit_right_cens, method='test')
best_iter_stop_start <- gbmt_performance(fit_start_stop, method='test')
```
# Strata Updates
During the fitting process the original strata vector is updated in the following way.
When the data is split into a training and validation set the strata vector is also split.  The strata vector is then updated so as represent the cumulative count of the number of observations in each stratum in the training and validation sets.  The vector is padded with `NAs` so it is of the same length as the original strata vector provided and such that the validation set cumulative strata sums are separated from the training set strata counts by the appropriate amount.  

The original strata vector is stored within the `GBMFit` object and can be accessed as follows: `fit$distribution$original_strata_id`. The data in the `original_strata_id` field 
is used to recreate the correct strata when performing additional iterations using `gbm_more`.


# Role of additional parameters in GBMDist
## ties and prior_node_coeff_var
The `ties` and `prior_node_coeff_var` parameters may also be specified on construction of the `CoxPHGBMDist` object.  The former is a string specifying the method by which the algorithm deals with tied event times.  This may be set to either "breslow" or "efron" depending on your preference, with the latter being the default.  The role of the `prior_node_coeff_var` parameter is slightly more subtle and complex.  When fitting a boosted tree, the optimal predictions of the terminal nodes must be set.  These predictions determine the predictions made by the `GBMFit` object.  The role of `prior_node_coeff_var` is to ensure that the predictions are finite and it does this by acting as a regularization for the terminal node predictions.  It should be a finite positive double and is by default set to a 1000.  An exact description of its role in the underlying algorithm is described in the next section.

```{r, message=FALSE, fig.height=6, fig.width=6}
# Example using Breslow and Efron tie-breaking

# Create data
require(survival)

set.seed(1)
N <- 3000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=T))
mu <- c(-1,0,1,2)[as.numeric(X3)]

f <- 0.5*sin(3*X1 + 5*X2^2 + mu/10)
tt.surv <- rexp(N,exp(f))
tt.cens <- rexp(N,0.5)
delta <- as.numeric(tt.surv <= tt.cens)
tt <- apply(cbind(tt.surv,tt.cens),1,min)

# throw in some missing values
X1[sample(1:N,size=100)] <- NA
X3[sample(1:N,size=300)] <- NA

# random weights if you want to experiment with them
w <- rep(1,N)
data <- data.frame(tt=tt,delta=delta,X1=X1,X2=X2,X3=X3)

# Set up distribution objects
cox_breslow <- gbm_dist("CoxPH", ties="breslow", prior_node_coeff_var=100)
cox_efron <- gbm_dist("CoxPH", ties="efron", prior_node_coeff_var=100)

# Define training parameters
params <- training_params(num_trees=3000, interaction_depth=3, min_num_obs_in_node=10, 
                          shrinkage=0.001, bag_fraction=0.5, id=seq(nrow(data)), 
                          num_train=N/2, num_features=3)

# Fit gbm 
fit_breslow <- gbmt(Surv(tt, delta)~X1+X2+X3, data=data, distribution=cox_breslow, 
                    weights=w, train_params=params, var_monotone=c(0, 0, 0), 
                    keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)

fit_efron <- gbmt(Surv(tt, delta)~X1+X2+X3, data=data, distribution=cox_efron,
                  weights=w, train_params=params, var_monotone=c(0, 0, 0), 
                  keep_gbm_data=TRUE, cv_folds=5, is_verbose = FALSE)


# Evaluate fit 
plot(gbmt_performance(fit_breslow, method='test'))
legend("topleft", c("training error", "test error", "optimal iteration"),
       lty=c(1, 1, 2), col=c("black", "red", "blue"))
plot(gbmt_performance(fit_efron, method='test'))
legend("topleft", c("training error", "test error", "optimal iteration"),
       lty=c(1, 1, 2), col=c("black", "red", "blue"))
```


# Description of the underlying algorithm - specifically for CoxPH
The `gbm` algorithm works to estimate, via tree boosting, the function $f(\textbf{x})$, which maps covariates ($\textbf{x}$) to the response variable *y* - in this case the event indicator.
For `CoxPH`, the algorithm calculates both the partial log likelihood and martingale residuals ($\textbf{m}$) using the following approach.  The algorithm walks backwards in time until it encounters the "stop" time of an observation.  When this happens the weighted risk associated with that observation, $\omega_i e^{f(\textbf{x}_i)}$, is added to the total cumulative hazard: $S = \sum \omega_j e^{f(\textbf{x}_j)}$, which is initialized at $0$.   Continuing backwards in time when we reach a time before an observation was in the study, that is the algorithm leaves the associated time segment (start, stop], the observation's contribution to the cumulative hazard is subtracted off.  The algorithm is robust to overflow/underflows occurring in $e^{f(\textbf{x}_i)}$ by subtracting a constant off of the risk score.  This constant drifts to ensure overflow does not occur.

This algorithm deals with tied event times using either the Breslow or Efron approximations.  The method used is specified by the user but in the event of tied deaths, it defaults to the Efron approximation.  It also allows for the introduction of strata and start as well as stop times for each observation, see the previous Sections.  

As well as calculating the partial log likelihood the algorithm also calculates the martingale residuals.  The risk scores are related to the covariate matrix, $\mathbb{X}$, via:
$$ f(\textbf{x}_i) = (\mathbb{X}\boldsymbol{\beta})_i.   \qquad (1) $$
The derivative of the partial log likelihood, $l(\boldsymbol{\beta})$, with respect to the parameter vector $\boldsymbol{\beta}$ is related to the martingale residuals through:
$$\frac{\partial}{\partial \boldsymbol{\beta}} l(\boldsymbol{\beta}) = \mathbb{X}^{T} \textbf{m}.  \qquad (2) $$
Defining the loss function as the negative of the partial log likelihood then using the chain rule in combination with Equation (1) the residuals are given by:
$$ z_i = -\frac{\partial}{\partial f(\textbf{x}_{\textit{i}})}\Psi(\textit{y}_{\textit{i}},f(\textbf{x}_\textit{i})) = (\mathbb{X}\mathbb{X}^{T}\textbf{m})_i.  \qquad (3)$$

At this point the covariate matrix should only decide what splits the tree will make thus covariate matrix in Equation (3) is free to be set to the identity matrix and so:
$$ z_i = \textbf{m}_i. \qquad (4)$$

Finally, the updated implementation calculates the optimal terminal node predictions in the following way. Looping over the bagged observations  in the terminal node of interest the expected number of events is given by: $\sum_i \max(0.0, y_i - \textbf{m}_i) + 1/c$.  The constant $c$ represents the prior on the baseline number of events that occur within a given terminal node; it can be set on construction of the `CoxPHGBMDist` through the `prior_node_coeff_var` parameter.  From this the terminal node prediction is given by:
$$ \log(\frac{\sum_i y_i + 1/c}{\sum_i \max(0.0, y_i - \textbf{m}_i) + 1/c}). \qquad (5)$$

If this `prior_node_coeff_var` is set incorrectly, i. e. to not a finite positive double, then the predictions of the fitted model can become non-sensical.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relative-influence.r
\name{relative_influence}
\alias{relative_influence}
\title{Methods for estimating relative influence}
\usage{
relative_influence(gbm_fit_obj, num_trees, rescale = FALSE, sort_it = FALSE)
}
\arguments{
\item{gbm_fit_obj}{a \code{GBMFit} object created from an initial
call to \code{\link{gbmt}}.}

\item{num_trees}{the number of trees to use for computations. If
not provided, the function will guess: if a test set was used in
fitting, the number of trees resulting in lowest test set error
will be used; otherwise, if cross-validation was performed, the
number of trees resulting in lowest cross-validation error will be
used; otherwise, all trees will be used.}

\item{rescale}{whether or not the result should be scaled. Defaults
to \code{FALSE}.}

\item{sort_it}{whether or not the results should be (reverse)
sorted.  Defaults to \code{FALSE}.}
}
\value{
By default, returns an unprocessed vector of estimated
relative influences. If the \code{rescale} and \code{sort}
arguments are used, returns a processed version of the same.
}
\description{
This function offers a method for computing the relative influence in
\code{\link{summary.GBMFit}}, and is not intended to be called directly.
}
\details{
\code{\link{relative_influence}} is the same as that
described in Friedman (2001).
\code{\link{permutation_relative_influence}} randomly permutes each
predictor variable at a time and computes the associated reduction in
predictive performance. This is similar to the variable importance measures
Breiman uses for random forests, but \code{gbmt} currently computes using the
entire training dataset (not the out-of-bag observations).
}
\references{
J.H. Friedman (2001). "Greedy Function Approximation: A Gradient
Boosting Machine," Annals of Statistics 29(5):1189-1232.

L. Breiman (2001).
\href{https://www.stat.berkeley.edu/~breiman/randomforest2001.pdf}{Random
Forests}.
}
\seealso{
\code{\link{summary.GBMFit}}
}
\author{
James Hickey, Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-print.r
\name{print.GBMFit}
\alias{print.GBMFit}
\alias{print}
\title{Print model summary}
\usage{
\method{print}{GBMFit}(x, ...)
}
\arguments{
\item{x}{is a fitted generalized boosting object of class \code{GBMFit}.}

\item{\dots}{arguments passed to \code{print.default}.}
}
\value{
Returns \code{x} invisibly
}
\description{
Display basic information about a \code{GBMFit} object.
}
\details{
Prints some information about the model object. In particular, this method
prints the call to \code{gbmt}, the type of loss function that was used,
and the total number of iterations.

If cross-validation was performed, the 'best' number of trees as estimated
by cross-validation error is displayed. If a test set was used, the 'best'
number of trees as estimated by the test set error is displayed.

The number of available predictors, and the number of those having non-zero
influence on predictions is given (which might be interesting in data mining
applications).

If bernoulli or adaboost was used, the confusion matrix and
prediction accuracy are printed (objects being allocated to the class with
highest probability for bernoulli). These classifications
are performed using the cross-validation fitted values.

If the 'distribution' was specified as gaussian, laplace, quantile or
t-distribution, a summary of the residuals is displayed.  The residuals are
the cross-validation residuals. Also, a pseudo R-squared value is displayed.
For Gaussian response, this is 1 - sum(r*r) / sum(z*z) where z = y -
mean(y). For the other distributions, this is 1 - (median(abs(r)) /
mad(y))^2, following the suggestion of Rousseeuw and Leroy (equation 3.11).
Note that this definition of a robust R-squared is contentious.
}
\references{
P. J. Rousseeuw and A. M. Leroy, Robust Regression and Outlier
Detection, Wiley, 1987 (2003).
}
\seealso{
\code{\link{gbmt}}
}
\author{
James Hickey, Harry Southworth, Daniel Edwards
}
\keyword{models}
\keyword{nonlinear}
\keyword{nonparametric}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-more.r
\name{gbm_more}
\alias{gbm_more}
\title{Perform additional boosting}
\usage{
gbm_more(
  gbm_fit_obj,
  num_new_trees = 100,
  data = NULL,
  weights = NULL,
  offset = NULL,
  is_verbose = FALSE
)
}
\arguments{
\item{gbm_fit_obj}{a \code{GBMFit} object produced using
\code{\link{gbmt}}.  This object describes the boosted model on
which to perform additional boosting.}

\item{num_new_trees}{a positive integer specifying how many
additional iterations to perform.  This has a default value of
\code{100}.}

\item{data}{a \code{data.frame} or \code{matrix} containing the new
values for the predictor and response variables for the additional
iterations.  The names of the variables must match those appearing
in the original fit (as well as the number of rows), and this value
defaults to \code{NULL}.  With a value of \code{NULL} the original
data may be used for the additional boosting, if no original or new
data is specified an error will be thrown.}

\item{weights}{an atomic vector of doubles specifying the
importance of each row of the \code{data} in the additional
iterations. If the previous data used is kept within
\code{gbm_fit_obj}; then the weights are extracted from the stored
\code{GBMData} object.}

\item{offset}{an atomic vector of doubles specifying the offset for
each response value in the data used for additional boosting.}

\item{is_verbose}{a logical specifying whether or not the
additional fitting should run "noisely" with feedback on progress
provided to the user.}
}
\value{
the input \code{GBMFit} object with additional iterations
provided for the fit.
}
\description{
Method to perform additional boosting using a \code{GBMFit} object
- does not support further cross validation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-perf.r
\name{gbm.perf}
\alias{gbm.perf}
\title{GBMT Performance}
\usage{
gbm.perf(
  object,
  plot.it = TRUE,
  oobag.curve = FALSE,
  overlay = TRUE,
  method,
  main = ""
)
}
\arguments{
\item{object}{a \code{GBMFit} object created from an initial
call to \code{\link{gbmt}} or \code{\link{gbm}}.}

\item{plot.it}{an indicator of whether or not to plot the
performance measures. Setting \code{plot.it=TRUE} creates two
plots. The first plot plots the train error (in black)
and the validation error (in red) versus the iteration
number. The scale of the error measurement, shown on the left
vertical axis, depends on the \code{distribution} argument used in
the initial call.}

\item{oobag.curve}{indicates whether to plot the out-of-bag
performance measures in a second plot.}

\item{overlay}{if TRUE and oobag.curve=TRUE then a right y-axis is
added to the training and test error plot and the estimated
cumulative improvement in the loss function is plotted versus the
iteration number.}

\item{method}{indicate the method used to estimate the optimal
number of boosting iterations. \code{method="OOB"} computes the
out-of-bag estimate and \code{method="test"} uses the test (or
validation) dataset to compute an out-of-sample
estimate. \code{method="cv"} extracts the optimal number of
iterations using cross-validation if \code{gbm} was called with
\code{cv.folds}>1.}

\item{main}{the main title for the plot. Defaults to \code{main =
""}.}
}
\value{
\code{gbm.perf} returns the estimated optimal number of iterations.
The method of computation depends on the \code{method} argument.
}
\description{
Estimates optimal number of boosting iterations given a
\code{GBMFit} object and optionally plots various performance
measures.
}
\seealso{
\code{\link{gbmt}} \code{\link{gbmt_performance}}
\code{\link{plot.GBMTPerformance}}
}
\keyword{nonlinear}
\keyword{nonparametric}
\keyword{survival}
\keyword{tree}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-accessors.r
\name{iteration_error}
\alias{iteration_error}
\title{Extract errors from GBMFit objects}
\usage{
iteration_error(obj, which = c("train", "valid", "cv"))
}
\arguments{
\item{obj}{a GBMFit object}

\item{which}{which error measure to extract}
}
\value{
a vector giving the error by tree
}
\description{
Extract errors from GBMFit objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-predict.r
\name{predict.GBMFit}
\alias{predict.GBMFit}
\title{Predict method for GBM Model Fits}
\usage{
\method{predict}{GBMFit}(object, newdata, n.trees, type = "link", single.tree = FALSE, ...)
}
\arguments{
\item{object}{Object of class inheriting from \code{GBMFit}.}

\item{newdata}{Data frame of observations for which to make
predictions}

\item{n.trees}{Number of trees used in the prediction. If
\code{n.trees} is a vector, predictions are returned for each
iteration specified.}

\item{type}{The scale on which gbm makes the predictions}

\item{single.tree}{If \code{single.tree=TRUE} then \code{gbm_predict}
returns only the predictions from tree(s) \code{n.trees}}

\item{\dots}{further arguments passed to or from other methods}
}
\value{
Returns a vector of predictions. By default the predictions
are on the scale of f(x). For example, for the Bernoulli loss the
returned value is on the log odds scale, poisson loss on the log
scale, and coxph is on the log hazard scale.

If \code{type="response"} then \code{gbmt} converts back to the same scale as
the outcome. Currently the only effect this will have is returning
probabilities for bernoulli and expected counts for poisson. For the other
distributions "response" and "link" return the same.
}
\description{
Predicted values based on a generalized boosted model object - from
gbmt
}
\details{
\code{predict.GBMFit} produces predicted values for each
observation in a new dataset \code{newdata} using the first
\code{n.trees} iterations of the boosting sequence. If
\code{n.trees} is a vector than the result is a matrix with each
column representing the predictions from gbm models with
\code{n.trees[1]} iterations, \code{n.trees[2]} iterations, and
so on.

The predictions from \code{gbmt} do not include the offset
term. The user may add the value of the offset to the predicted
value if desired.

If \code{gbm_fit_obj} was fit using \code{\link{gbmt}}, there will
be no \code{Terms} component. Therefore, the user has greater
responsibility to make sure that \code{newdata} is of the same
format (order and number of variables) as the one originally used
to fit the model.
}
\seealso{
\code{\link{gbmt}}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pretty-gbm-tree.r
\name{pretty_gbm_tree}
\alias{pretty_gbm_tree}
\title{Print gbm tree components}
\usage{
pretty_gbm_tree(gbm_fit_obj, tree_index = 1)
}
\arguments{
\item{gbm_fit_obj}{a \code{GBMFit} object initially fit using
\code{\link{gbmt}}.}

\item{tree_index}{the index of the tree component to extract from \code{gbm_fit_obj}
and display.}
}
\value{
\code{pretty_gbm_tree} returns a data frame. Each row corresponds to
a node in the tree. Columns indicate \item{SplitVar}{index of which variable
is used to split. -1 indicates a terminal node.} \item{SplitCodePred}{if the
split variable is continuous then this component is the split point. If the
split variable is categorical then this component contains the index of
\code{gbm_fit_obj$c.split} that describes the categorical split. If the node is a
terminal node then this is the prediction.} \item{LeftNode}{the index of the
row corresponding to the left node.} \item{RightNode}{the index of the row
corresponding to the right node.} \item{ErrorReduction}{the reduction in the
loss function as a result of splitting this node.} \item{Weight}{the total
weight of observations in the node. If weights are all equal to 1 then this
is the number of observations in the node.}
}
\description{
\code{GBMFit} stores the collection of trees used to construct the model in a
compact matrix structure. This function extracts the information from a
single tree and displays it in a slightly more readable form. This function
is mostly for debugging purposes and to satisfy some users' curiosity.
}
\seealso{
\code{\link{gbm}}, \code{\link{gbmt}}
}
\author{
James Hickey, Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm.fit.r, R/gbm.r
\name{gbm.fit}
\alias{gbm.fit}
\alias{gbm}
\title{Generalized Boosted Regression Modeling}
\usage{
gbm.fit(
  x,
  y,
  offset = NULL,
  distribution = "bernoulli",
  w = NULL,
  var.monotone = NULL,
  n.trees = 100,
  interaction.depth = 1,
  n.minobsinnode = 10,
  shrinkage = 0.001,
  bag.fraction = 0.5,
  nTrain = NULL,
  train.fraction = NULL,
  mFeatures = NULL,
  keep.data = TRUE,
  verbose = TRUE,
  var.names = NULL,
  response.name = "y",
  group = NULL,
  tied.times.method = "efron",
  prior.node.coeff.var = 1000,
  strata = NA,
  obs.id = 1:nrow(x)
)

gbm(
  formula = formula(data),
  distribution = "bernoulli",
  data = list(),
  weights,
  subset = NULL,
  offset = NULL,
  var.monotone = NULL,
  n.trees = 100,
  interaction.depth = 1,
  n.minobsinnode = 10,
  shrinkage = 0.001,
  bag.fraction = 0.5,
  train.fraction = 1,
  mFeatures = NULL,
  cv.folds = 0,
  keep.data = TRUE,
  verbose = FALSE,
  class.stratify.cv = NULL,
  n.cores = NULL,
  par.details = getOption("gbm.parallel"),
  fold.id = NULL,
  tied.times.method = "efron",
  prior.node.coeff.var = 1000,
  strata = NA,
  obs.id = 1:nrow(data)
)
}
\arguments{
\item{x, y}{For \code{gbm.fit}: \code{x} is a data frame or data
matrix containing the predictor variables and \code{y} is a matrix
of outcomes.  Excluding \code{coxph} this matrix of outcomes
collapses to a vector, in the case of \code{coxph} it is a survival
object where the event times fill the first one (or two columns)
and the status fills the final column.  The number of rows in
\code{x} must be the same as the length of the 1st dimension of
\code{y}.}

\item{offset}{an optional model offset}

\item{distribution}{either a character string specifying the name
of the distribution to use or a list with a component \code{name}
specifying the distribution and any additional parameters
needed. If not specified, \code{gbm} will try to guess: if the
response has only two unique values, bernoulli is assumed 
(two-factor responses are converted to 0,1);
otherwise, if the response has class "Surv", coxph is assumed;
otherwise, gaussian is assumed.

Available distributions are "gaussian" (squared error), "laplace"
(absolute loss), "tdist" (t-distribution loss), "bernoulli"
(logistic regression for 0-1 outcomes), "huberized" (Huberized
hinge loss for 0-1 outcomes), "adaboost" (the AdaBoost exponential
loss for 0-1 outcomes), "poisson" (count outcomes), "coxph" (right
censored observations), "quantile", or "pairwise" (ranking measure
using the LambdaMART algorithm).

If quantile regression is specified, \code{distribution} must be a
list of the form \code{list(name="quantile",alpha=0.25)} where
\code{alpha} is the quantile to estimate. Non-constant weights are
unsupported.

If "tdist" is specified, the default degrees of freedom is four and
this can be controlled by specifying
\code{distribution=list(name="tdist", df=DF)} where \code{DF} is
your chosen degrees of freedom.

If "pairwise" regression is specified, \code{distribution} must be a list of
the form \code{list(name="pairwise",group=...,metric=...,max.rank=...)}
(\code{metric} and \code{max.rank} are optional, see below). \code{group} is
a character vector with the column names of \code{data} that jointly
indicate the group an instance belongs to (typically a query in Information
Retrieval applications). For training, only pairs of instances from the same
group and with different target labels can be considered. \code{metric} is
the IR measure to use, one of
\describe{
  \item{list("conc")}{Fraction of concordant pairs; for binary labels, this is equivalent to the Area under the ROC Curve}
  \item{:}{Fraction of concordant pairs; for binary labels, this
is equivalent to the Area under the ROC Curve}
  \item{list("mrr")}{Mean reciprocal rank of the highest-ranked positive instance}
  \item{:}{Mean reciprocal rank of the highest-ranked positive instance}
  \item{list("map")}{Mean average precision, a generalization of \code{mrr}
  to multiple positive instances}
  \item{:}{Mean average precision, a generalization of \code{mrr} to multiple positive instances}
  \item{list("ndcg:")}{Normalized discounted cumulative gain. The score is the weighted sum (DCG) of the user-supplied target values, weighted by
log(rank+1), and normalized to the maximum achievable value. This is the
default if the user did not specify a metric.}
}

\code{ndcg} and \code{conc} allow arbitrary target values, while binary
targets \{0,1\} are expected for \code{map} and \code{mrr}. For \code{ndcg}
and \code{mrr}, a cut-off can be chosen using a positive integer parameter
\code{max.rank}. If left unspecified, all ranks are taken into account.

Note that splitting of instances into training and validation sets follows
group boundaries and therefore only approximates the specified
\code{train.fraction} ratio (the same applies to cross-validation folds).
Internally queries are randomly shuffled before training to avoid bias.

Weights can be used in conjunction with pairwise metrics, however it is
assumed that they are constant for instances from the same group.

For details and background on the algorithm, see e.g. Burges (2010).}

\item{w}{For \code{gbm.fit}: \code{w} is a vector of weights of the same
length as the 1st dimension of \code{y}.}

\item{var.monotone}{an optional vector, the same length as the
number of predictors, indicating which variables have a monotone
increasing (+1), decreasing (-1), or arbitrary (0) relationship
with the outcome.}

\item{n.trees}{the total number of trees to fit. This is equivalent
to the number of iterations and the number of basis functions in
the additive expansion.}

\item{interaction.depth}{The maximum depth of variable
interactions: 1 builds an additive model, 2 builds a model with up
to two-way interactions, etc.}

\item{n.minobsinnode}{minimum number of observations (not total
weights) in the terminal nodes of the trees.}

\item{shrinkage}{a shrinkage parameter applied to each tree in the
expansion. Also known as the learning rate or step-size reduction.}

\item{bag.fraction}{the fraction of independent training
observations (or patients) randomly selected to propose the next
tree in the expansion, depending on the obs.id vector multiple
training data rows may belong to a single 'patient'. This
introduces randomness into the model fit.  If \code{bag.fraction}<1
then running the same model twice will result in similar but
different fits. \code{gbm} uses the R random number generator, so
\code{set.seed} ensures the same model can be
reconstructed. Preferably, the user can save the returned
\code{GBMFit} object using \code{\link{save}}.}

\item{nTrain}{An integer representing the number of unique patients,
each patient could have multiple rows associated with them, on which
to train.  This is the preferred way of specification for
\code{gbm.fit}; The option \code{train.fraction} in \code{gbm.fit}
is deprecated and only maintained for backward compatibility. These
two parameters are mutually exclusive. If both are unspecified, all
data is used for training.}

\item{train.fraction}{The first \code{train.fraction * nrows(data)}
observations are used to fit the \code{gbm} and the remainder are used for
computing out-of-sample estimates of the loss function.}

\item{mFeatures}{Each node will be trained on a random subset of
\code{mFeatures} number of features. Each node will consider a new
random subset of features, adding variability to tree growth and
reducing computation time. \code{mFeatures} will be bounded between
1 and \code{nCols}. Values outside of this bound will be set to the
lower or upper limits.}

\item{keep.data}{a logical variable indicating whether to keep the
data and an index of the data stored with the object. Keeping the
data and index makes subsequent calls to \code{\link{gbm_more}}
faster at the cost of storing an extra copy of the dataset.}

\item{verbose}{If TRUE, gbm will print out progress and performance
indicators. If this option is left unspecified for gbm_more then it uses
\code{verbose} from \code{object}.}

\item{var.names}{For \code{gbm.fit}: A vector of strings of length
equal to the number of columns of \code{x} containing the names of
the predictor variables.}

\item{response.name}{For \code{gbm.fit}: A character string label
for the response variable.}

\item{group}{\code{group} used when \code{distribution =
'pairwise'.}}

\item{tied.times.method}{For \code{gbm} and \code{gbm.fit}: This is
an optional string used with \code{CoxPH} distribution specifying
what method to employ when dealing with tied times.  Currently only
"efron" and "breslow" are available; the default value is "efron".
Setting the string to any other value reverts the method to the
original CoxPH model implementation where ties are not explicitly
dealt with.}

\item{prior.node.coeff.var}{Optional double only used with the
\code{coxph} distribution.  It is a prior on the coefficient of
variation associated with the hazard rate assigned to each terminal
node when fitting a tree.Increasing its value emphasises the
importance of the training data in the node when assigning a
prediction to said node.}

\item{strata}{Optional vector of integers (or factors) only used
with the \code{coxph} distributions.  Each integer in this vector
represents the stratum the corresponding row in the data belongs
to, e. g. if the 10th element is 3 then the 10th data row belongs
to the 3rd strata.}

\item{obs.id}{Optional vector of integers used to specify which
rows of data belong to individual patients.  Data is then bagged by
patient id; the default sets each row of the data to belong to an
individual patient.}

\item{formula}{a symbolic description of the model to be fit. The
formula may include an offset term (e.g. y~offset(n)+x). If
\code{keep.data=FALSE} in the initial call to \code{gbm} then it is
the user's responsibility to resupply the offset to
\code{\link{gbm_more}}.}

\item{data}{an optional data frame containing the variables in the
model. By default the variables are taken from
\code{environment(formula)}, typically the environment from which
\code{gbm} is called. If \code{keep.data=TRUE} in the initial call
to \code{gbm} then \code{gbm} stores a copy with the object. If
\code{keep.data=FALSE} then subsequent calls to
\code{\link{gbm_more}} must resupply the same dataset. It becomes
the user's responsibility to resupply the same data at this point.}

\item{weights}{an optional vector of weights to be used in the
fitting process. The weights must be positive but do not need to be
normalized. If \code{keep.data=FALSE} in the initial call to
\code{gbm}, then it is the user's responsibility to resupply the
weights to \code{\link{gbm_more}}.}

\item{subset}{an optional vector defining a subset of the data to be used}

\item{cv.folds}{Number of cross-validation folds to perform. If
\code{cv.folds}>1 then \code{gbm}, in addition to the usual fit,
will perform a cross-validation and calculate an estimate of
generalization error returned in \code{cv_error}.}

\item{class.stratify.cv}{whether the cross-validation should be
stratified by class. Is only implemented for
\code{bernoulli}. The purpose of stratifying
the cross-validation is to help avoiding situations in which
training sets do not contain all classes.}

\item{n.cores}{number of cores to use for parallelization.  Please
use \code{par.details} instead; this argument is only provided for
convenience.}

\item{par.details}{Details of the parallelization to use in the
core algorithm.}

\item{fold.id}{An optional vector of values identifying what fold
each observation is in. If supplied, cv.folds can be missing. Note:
Multiple observations to the same patient must have the same fold
id.}
}
\value{
\code{gbm} and \code{gbm.fit} return a \code{GBMFit}
object.
}
\description{
Fits generalized boosted regression models.
}
\details{
See the gbm vignette for technical details.

This package implements the generalized boosted modeling
framework. Boosting is the process of iteratively adding basis
functions in a greedy fashion so that each additional basis
function further reduces the selected loss function. This
implementation closely follows Friedman's Gradient Boosting Machine
(Friedman, 2001).

In addition to many of the features documented in the Gradient
Boosting Machine, \code{gbm} offers additional features including
the out-of-bag estimator for the optimal number of iterations, the
ability to store and manipulate the resulting \code{GBMFit} object,
and a variety of other loss functions that had not previously had
associated boosting algorithms, including the Cox partial
likelihood for censored data, the poisson likelihood for count
outcomes, and a gradient boosting implementation to minimize the
AdaBoost exponential loss function.

\code{gbm} is a deprecated function that now acts as a front-end to
\code{gbmt_fit} that uses the familiar R modeling
formulas. However, \code{\link[stats]{model.frame}} is very slow if
there are many predictor variables. For power users with many
variables use \code{gbm.fit} over \code{gbm}; however \code{gbmt}
and \code{gbmt_fit} are now the current APIs.
}
\section{Functions}{
\itemize{
\item \code{gbm.fit()}: Core fitting code, for experts only.

}}
\examples{
 # A least squares regression example # create some data

N <- 1000
X1 <- runif(N)
X2 <- 2*runif(N)
X3 <- ordered(sample(letters[1:4],N,replace=TRUE),levels=letters[4:1])
X4 <- factor(sample(letters[1:6],N,replace=TRUE))
X5 <- factor(sample(letters[1:3],N,replace=TRUE))
X6 <- 3*runif(N) 
mu <- c(-1,0,1,2)[as.numeric(X3)]

SNR <- 10 # signal-to-noise ratio
Y <- X1**1.5 + 2 * (X2**.5) + mu
sigma <- sqrt(var(Y)/SNR)
Y <- Y + rnorm(N,0,sigma)

# introduce some missing values
X1[sample(1:N,size=500)] <- NA
X4[sample(1:N,size=300)] <- NA

data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

# fit initial model
gbm1 <-
gbm(Y~X1+X2+X3+X4+X5+X6,         # formula
    data=data,                   # dataset
    var.monotone=c(0,0,0,0,0,0), # -1: monotone decrease,
                                 # +1: monotone increase,
                                 #  0: no monotone restrictions
    distribution="gaussian",     # see the help for other choices
    n.trees=1000,                # number of trees
    shrinkage=0.05,              # shrinkage or learning rate,
                                 # 0.001 to 0.1 usually work
    interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
    bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
    train.fraction = 0.5,        # fraction of data for training,
                                 # first train.fraction*N used for training
    mFeatures = 3,               # half of the features are considered at each node
    n.minobsinnode = 10,         # minimum total weight needed in each node
    cv.folds = 3,                # do 3-fold cross-validation
    keep.data=TRUE,              # keep a copy of the dataset with the object
    verbose=FALSE                # don't print out progress
# , par.details=gbmParallel(num_threads=15) # option for gbm3 to parallelize
    )                   

# check performance using an out-of-bag estimator
# OOB underestimates the optimal number of iterations
best_iter <- gbmt_performance(gbm1,method="OOB")
print(best_iter)

# check performance using a 50\% heldout test set
best_iter <- gbmt_performance(gbm1,method="test")
print(best_iter)

# check performance using 3-fold cross-validation
best_iter <- gbmt_performance(gbm1,method="cv")
print(best_iter)

# plot the performance # plot variable influence
summary(gbm1, num_trees=1)         # based on the first tree
summary(gbm1, num_trees=best_iter) # based on the estimated best number of trees

# compactly print the first and last trees for curiosity
print(pretty_gbm_tree(gbm1,1))
print(pretty_gbm_tree(gbm1,gbm1$params$num_trees))

# make some new data
N <- 1000
X1 <- runif(N)
X2 <- 2*runif(N)
X3 <- ordered(sample(letters[1:4],N,replace=TRUE))
X4 <- factor(sample(letters[1:6],N,replace=TRUE))
X5 <- factor(sample(letters[1:3],N,replace=TRUE))
X6 <- 3*runif(N) 
mu <- c(-1,0,1,2)[as.numeric(X3)]

Y <- X1**1.5 + 2 * (X2**.5) + mu + rnorm(N,0,sigma)

data2 <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3,X4=X4,X5=X5,X6=X6)

# predict on the new data using "best" number of trees
# f.predict generally will be on the canonical scale (logit,log,etc.)
f.predict <- predict(gbm1,data2,best_iter)

# least squares error
print(sum((data2$Y-f.predict)^2))

# create marginal plots
# plot variable X1,X2,X3 after "best" iterations
oldpar <- par(no.readonly = TRUE)
par(mfrow=c(1,3))
plot(gbm1,1,best_iter)
plot(gbm1,2,best_iter)
plot(gbm1,3,best_iter)
par(mfrow=c(1,1))
# contour plot of variables 1 and 2 after "best" iterations
plot(gbm1,1:2,best_iter)
# lattice plot of variables 2 and 3
plot(gbm1,2:3,best_iter)
# lattice plot of variables 3 and 4
plot(gbm1,3:4,best_iter)

# 3-way plots
plot(gbm1,c(1,2,6),best_iter,cont=20)
plot(gbm1,1:3,best_iter)
plot(gbm1,2:4,best_iter)
plot(gbm1,3:5,best_iter)
par(oldpar) # reset graphics options to previous settings

# do another 100 iterations
gbm2 <- gbm_more(gbm1,100,
                 is_verbose=FALSE) # stop printing detailed progress
}
\references{
Y. Freund and R.E. Schapire (1997) \dQuote{A decision-theoretic
generalization of on-line learning and an application to boosting,}
\emph{Journal of Computer and System Sciences,} 55(1):119-139.

G. Ridgeway (1999). \dQuote{The state of boosting,} \emph{Computing Science
and Statistics} 31:172-181.

J.H. Friedman, T. Hastie, R. Tibshirani (2000). \dQuote{Additive Logistic
Regression: a Statistical View of Boosting,} \emph{Annals of Statistics}
28(2):337-374.

J.H. Friedman (2001). \dQuote{Greedy Function Approximation: A Gradient
Boosting Machine,} \emph{Annals of Statistics} 29(5):1189-1232.

J.H. Friedman (2002). \dQuote{Stochastic Gradient Boosting,}
\emph{Computational Statistics and Data Analysis} 38(4):367-378.

B. Kriegler (2007). \emph{Cost-Sensitive Stochastic Gradient Boosting
Within a Quantitative Regression Framework}. PhD dissertation, UCLA
Statistics.

C. Burges (2010). \dQuote{From RankNet to LambdaRank to LambdaMART: An
Overview,} Microsoft Research Technical Report MSR-TR-2010-82.

\href{https://gregridgeway.github.io/}{Greg Ridgeway's site}.
}
\seealso{
\code{\link{gbmt}}, \code{\link{gbmt_fit}} \code{\link{gbmt_performance}},
\code{\link{plot}}, \code{\link{predict.GBMFit}},
\code{\link{summary.GBMFit}}, \code{\link{pretty_gbm_tree}}, \code{\link{gbmParallel}}.
}
\author{
James Hickey, Greg Ridgeway \email{gregridgeway@gmail.com}

Quantile regression code developed by Brian Kriegler
\email{bk@stat.ucla.edu}

t-distribution code developed by Harry Southworth and
Daniel Edwards

Pairwise code developed by Stefan Schroedl \email{schroedl@a9.com}
}
\keyword{models}
\keyword{nonlinear}
\keyword{nonparametric}
\keyword{survival}
\keyword{tree}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-perf.r
\name{gbmt_performance}
\alias{gbmt_performance}
\title{Get performance details for gbm fit}
\usage{
gbmt_performance(gbm_fit_obj, method)
}
\arguments{
\item{gbm_fit_obj}{a \code{GBMFit} created from an initial call to
\code{\link{gbmt}}.}

\item{method}{indicate the method used to estimate the optimal
number of boosting iterations. \code{method="OOB"} computes the
out-of-bag estimate and \code{method="test"} uses the test (or
validation) dataset to compute an out-of-sample
estimate. \code{method="cv"} extracts the optimal number of
iterations using cross-validation if \code{gbmt} was called with
\code{cv_folds}>1.}
}
\value{
a GBMTPerformance object, which is a number - the optimal
iteration number - with various attributes.
}
\description{
\code{gbmt_performance} estimates the optimal number of boosting
iterations from a model fit by \code{\link{gbmt}}.  The precise
method used depends on the \code{method} parameter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbmt-fit.r
\name{gbmt_fit}
\alias{gbmt_fit}
\title{GBMT fit}
\usage{
gbmt_fit(
  x,
  y,
  distribution = gbm_dist("Gaussian"),
  weights = rep(1, nrow(x)),
  offset = rep(0, nrow(x)),
  train_params = training_params(num_trees = 100, interaction_depth = 3,
    min_num_obs_in_node = 10, shrinkage = 0.001, bag_fraction = 0.5, id =
    seq_len(nrow(x)), num_train = round(0.5 * nrow(x)), num_features = ncol(x)),
  response_name = "y",
  var_monotone = NULL,
  var_names = NULL,
  keep_gbm_data = FALSE,
  cv_folds = 1,
  cv_class_stratify = FALSE,
  fold_id = NULL,
  par_details = getOption("gbm.parallel"),
  is_verbose = FALSE
)
}
\arguments{
\item{x}{a data frame or data matrix containing the predictor
variables.}

\item{y}{is a matrix of outcomes. Excluding CoxPH this matrix of
outcomes collapses to a vector; in the case of CoxPH it is a
survival object where the event times fill the first one (or two
columns) and the status fills the final column.  The length of the
1st dimension of y must match the number of rows in x.}

\item{distribution}{a \code{GBMDist} object specifying the
distribution and any additional parameters needed.}

\item{weights}{optional vector of weights used in the fitting
process.  These weights must be positive but need not be
normalized. By default they are set to 1 for each data row.}

\item{offset}{optional vector specifying the model offset; must be
positive.  This defaults to a vector of 0's, the length of which is
equal to the rows of x.}

\item{train_params}{a GBMTrainParams object which specifies the
parameters used in growing decision trees.}

\item{response_name}{a string specifying the name of the response -
defaults to "y".}

\item{var_monotone}{optional vector, the same length as the number
of predictors, indicating the relationship each variable has with
the outcome.  It have a monotone increasing (+1) or decreasing (-1)
or an arbitrary relationship.}

\item{var_names}{a vector of strings of containing the names of the
predictor variables.}

\item{keep_gbm_data}{a bool specifying whether or not the gbm_data
object created in this method should be stored in the results.}

\item{cv_folds}{a positive integer specifying the number of folds
to be used in cross-validation of the gbm fit. If cv_folds > 1 then
cross-validation is performed; the default of cv_folds is 1.}

\item{cv_class_stratify}{a bool specifying whether or not to
stratify via response outcome. Currently only applies to
"Bernoulli" distribution and defaults to false.}

\item{fold_id}{An optional vector of values identifying what fold
each observation is in. If supplied, cv_folds can be missing. Note:
Multiple rows of the same observation must have the same fold_id.}

\item{par_details}{Details of the parallelization to use in the
core algorithm.}

\item{is_verbose}{if TRUE, gbmt will print out progress and
performance of the fit.}
}
\value{
a \code{GBMFit} object.
}
\description{
Fits a generalized boosting model.  This is for "power" users who
have a large number of variables who wish to avoid calling
\code{model.frame} which can be slow in this instance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create-distributions.r
\name{create_dist}
\alias{create_dist}
\title{Create distributions}
\usage{
create_dist(empty_obj, ...)
}
\arguments{
\item{empty_obj}{An empty distribution object of the correct class.}

\item{...}{extra parameters used to define the distribution object,
see \code{\link{gbm_dist}}.}
}
\value{
an appropriated gbm distribution object
}
\description{
Creation methods that take an empty distribution object
and builds on top of the skeleton.
}
\author{
James Hickey
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-cv-predict.r
\name{predict.GBMCVFit}
\alias{predict.GBMCVFit}
\title{Predictions for CV fitted GBM models}
\usage{
\method{predict}{GBMCVFit}(object, gbm_data_obj, cv_folds, cv_group, best_iter_cv, ...)
}
\arguments{
\item{object}{a GBMCVFit object containing CV gbm models}

\item{gbm_data_obj}{a GBMData object containing all of the data
used to fit a gbm model.}

\item{cv_folds}{a positive integer specifying the number of folds
to be used in cross-validation of the gbm fit.}

\item{cv_group}{vector of integers specifying which row of data
belongs to which cv_fold.}

\item{best_iter_cv}{number of trees with the smallest cv error}

\item{\dots}{not currently used}
}
\value{
a matrix of predictions for each cv fold.
}
\description{
Calculate predictions from cross validated generalized boosting
model from gbm2.
}
\author{
James Hickey
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-package.r
\name{gbm_object}
\alias{gbm_object}
\title{Generalized Boosted Regression Model Object}
\value{
\item{initF}{the "intercept" term, the initial predicted value to
which trees make adjustments} 
\item{fit}{a vector containing the fitted
values on the scale of regression function (e.g. log-odds scale for
bernoulli, log scale for poisson)} 
\item{train.error}{a vector of length
equal to the number of fitted trees containing the value of the loss
function for each boosting iteration evaluated on the training data}
\item{valid.error}{a vector of length equal to the number of fitted trees
containing the value of the loss function for each boosting iteration
evaluated on the validation data}
\item{cv_error}{if \code{cv_folds}<2 this
component is NULL. Otherwise, this component is a vector of length equal to
the number of fitted trees containing a cross-validated estimate of the loss
function for each boosting iteration}
\item{oobag.improve}{a vector of
length equal to the number of fitted trees containing an out-of-bag estimate
of the marginal reduction in the expected value of the loss function. The
out-of-bag estimate uses only the training data and is useful for estimating
the optimal number of boosting iterations. See \code{\link{gbmt_performance}}}
\item{trees}{a list containing the tree structures. The components are best
viewed using \code{\link{pretty_gbm_tree}}}
\item{c.splits}{a list of all
the categorical splits in the collection of trees. If the \code{trees[[i]]}
component of a \code{gbm} object describes a categorical split then the
splitting value will refer to a component of \code{c.splits}. That component
of \code{c.splits} will be a vector of length equal to the number of levels
in the categorical split variable. -1 indicates left, +1 indicates right,
and 0 indicates that the level was not present in the training data}
\item{cv_fitted}{If cross-validation was performed, the cross-validation
predicted values on the scale of the linear predictor. That is, the fitted
values from the ith CV-fold, for the model having been trained on the data
in all other folds.}
}
\description{
These are objects representing fitted \code{gbm}s.
}
\section{Structure}{
 The following components must be included in a
legitimate \code{GBMFit} object.
}

\seealso{
\code{\link{gbmt}}
}
\author{
Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ir-measures.r
\name{gbm_roc_area}
\alias{gbm_roc_area}
\alias{gbm_conc}
\title{Compute Information Retrieval measures.}
\usage{
gbm_roc_area(obs, pred)
}
\arguments{
\item{obs}{Observed value}

\item{pred}{Predicted value}
}
\value{
The requested performance measure.
}
\description{
Functions to compute Information Retrieval measures for pairwise loss for a
single group. The function returns the respective metric, or a negative
value if it is undefined for the given group.
}
\details{
For simplicity, we have no special handling for ties; instead, we break ties
randomly. This is slightly inaccurate for individual groups, but should have
only a small effect on the overall measure.

\code{gbm_conc} computes the concordance index: Fraction of all pairs (i,j)
with i<j, x[i] != x[j], such that x[j] < x[i]

If \code{obs} is binary, then \code{gbm_roc_area(obs, pred) =
gbm.conc(obs[order(-pred)])}.

\code{gbm_conc} is more general as it allows non-binary targets, but is
significantly slower.
}
\examples{

##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.

}
\references{
C. Burges (2010). "From RankNet to LambdaRank to LambdaMART: An
Overview", Microsoft Research Technical Report MSR-TR-2010-82.
}
\seealso{
\code{\link{gbm}}
}
\author{
Stefan Schroedl
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-perf.r
\name{plot.GBMTPerformance}
\alias{plot.GBMTPerformance}
\title{Plot GBM performance details}
\usage{
\method{plot}{GBMTPerformance}(x, out_of_bag_curve = FALSE, overlay = TRUE, main = "", ...)
}
\arguments{
\item{x}{a \code{GBMTPerformance} object (created by
\code{\link{gbmt_performance}})}

\item{out_of_bag_curve}{indicates whether to plot the out-of-bag
performance measures in a second plot.}

\item{overlay}{if TRUE and out_of_bag_curve=TRUE then a right
y-axis is added to the training and test error plot and the
estimated cumulative improvement in the loss function is plotted
versus the iteration number.}

\item{main}{the main title for the plot.}

\item{\dots}{currently ignored}
}
\value{
No return value, only plot generation
}
\description{
The train and validation error (in black and red respectively) are
plotted against the iteration number.  If the initial model was
built with cross-validation, the cross-validation error is shown
in green.
}
\details{
The scale of the error measurement, shown on the
left vertical axis, depends on the \code{distribution} argument
used in the initial call to \code{\link{gbmt}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-summary.r
\name{summary.GBMFit}
\alias{summary.GBMFit}
\title{Summary of a GBMFit object}
\usage{
\method{summary}{GBMFit}(
  object,
  cBars = length(object$variables$var_names),
  num_trees = length(trees(object)),
  plot_it = TRUE,
  order_it = TRUE,
  method = relative_influence,
  normalize = TRUE,
  ...
)
}
\arguments{
\item{object}{a \code{GBMFit} object created from an initial call
to \code{\link{gbmt}}.}

\item{cBars}{the number of bars to plot. If \code{order_it=TRUE}
then only the \code{cBars} variables with the largest relative
influence will appear in the barplot. If \code{order_it=FALSE} then
the first \code{cBars} variables will appear in the plot. In either
case, the function will return the relative influence of all of the
variables.}

\item{num_trees}{the number of trees used to generate the
plot. Only the first \code{num_trees} trees will be used.}

\item{plot_it}{an indicator as to whether the plot is generated.}

\item{order_it}{an indicator as to whether the plotted and/or
returned relative influences are sorted.}

\item{method}{The function used to compute the relative influence.
\code{\link{relative_influence}} is the default and is the same as
that described in Friedman (2001). The other current (and
experimental) choice is
\code{\link{permutation_relative_influence}}. This method randomly
permutes each predictor variable at a time and computes the
associated reduction in predictive performance. This is similar to
the variable importance measures Breiman uses for random forests,
but \code{gbm} currently computes using the entire training dataset
(not the out-of-bag observations).}

\item{normalize}{if \code{FALSE} then \code{summary.gbm} returns
the unnormalized influence.}

\item{...}{other arguments passed to the plot function.}
}
\value{
Returns a data frame where the first component is the
variable name and the second is the computed relative influence,
normalized to sum to 100.
}
\description{
Computes the relative influence of each variable in the
\code{GBMFit} object.
}
\details{
For \code{GBMGaussianDist} this returns exactly the reduction of
squared error attributable to each variable. For other loss
functions this returns the reduction attributable to each variable
in sum of squared error in predicting the gradient on each
iteration. It describes the relative influence of each variable in
reducing the loss function. See the references below for exact
details on the computation.
}
\references{
J.H. Friedman (2001). "Greedy Function Approximation: A
Gradient Boosting Machine," Annals of Statistics 29(5):1189-1232.

L. Breiman (2001). \href{https://www.stat.berkeley.edu/~breiman/randomforest2001.pdf}{Random Forests}.
}
\seealso{
\code{\link{gbmt}}
}
\author{
James Hickey, Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/available-distributions.r
\name{available_distributions}
\alias{available_distributions}
\title{Available Distributions}
\usage{
available_distributions()
}
\value{
a vector containing the names of the available distributions in the package.
}
\description{
Returns a vector of the distributions implemented in the package.
}
\author{
James Hickey
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbmt.r
\name{gbmt}
\alias{gbmt}
\title{GBMT}
\usage{
gbmt(
  formula,
  distribution = gbm_dist("Gaussian"),
  data,
  weights = rep(1, nrow(data)),
  offset = rep(0, nrow(data)),
  train_params = training_params(num_trees = 2000, interaction_depth = 3,
    min_num_obs_in_node = 10, shrinkage = 0.001, bag_fraction = 0.5, id =
    seq_len(nrow(data)), num_train = round(0.5 * nrow(data)), num_features = ncol(data) -
    1),
  var_monotone = NULL,
  var_names = NULL,
  cv_folds = 1,
  cv_class_stratify = FALSE,
  fold_id = NULL,
  keep_gbm_data = FALSE,
  par_details = getOption("gbm.parallel"),
  is_verbose = FALSE
)
}
\arguments{
\item{formula}{a symbolic description of the model to be fit.  The
formula may include an offset term (e.g. y~offset(n) + x).}

\item{distribution}{a \code{GBMDist} object specifying the
distribution and any additional parameters needed. If not
specified then the distribution will be guessed.}

\item{data}{a data frame containing the variables in the model.
By default, the variables are taken from the environment.}

\item{weights}{optional vector of weights used in the fitting
process.  These weights must be positive but need not be
normalized. By default they are set to 1 for each data row.}

\item{offset}{optional vector specifying the model offset; must be
positive.  This defaults to a vector of 0's, the length of which
is equal to the number rows of data.}

\item{train_params}{a GBMTrainParams object which specifies the
parameters used in growing decision trees.}

\item{var_monotone}{optional vector, the same length as the number
of predictors, indicating the relationship each variable has with
the outcome.  It have a monotone increasing (+1) or decreasing
(-1) or an arbitrary relationship.}

\item{var_names}{a vector of strings of containing the names of
the predictor variables.}

\item{cv_folds}{a positive integer specifying the number of folds
to be used in cross-validation of the gbm fit. If cv_folds > 1
then cross-validation is performed; the default of cv_folds is 1.}

\item{cv_class_stratify}{a bool specifying whether or not to
stratify via response outcome. Currently only applies to
"Bernoulli" distribution and defaults to false.}

\item{fold_id}{An optional vector of values identifying what fold
each observation is in. If supplied, cv_folds can be
missing. Note: Multiple rows of the same observation must have the
same fold_id.}

\item{keep_gbm_data}{a bool specifying whether or not the gbm_data
object created in this method should be stored in the results.}

\item{par_details}{Details of the parallelization to use in the
core algorithm (\code{\link{gbmParallel}}).}

\item{is_verbose}{if TRUE, gbmt will print out progress and
performance of the fit.}
}
\value{
a \code{GBMFit} object.
}
\description{
Fits generalized boosted regression models - new API. This
prepares the inputs, performing tasks such as creating cv folds,
before calling \code{gbmt_fit} to call the underlying C++ and fit
a generalized boosting model.
}
\examples{
## create some data
N <- 1000
X1 <- runif(N)
X2 <- runif(N)
X3 <- factor(sample(letters[1:4],N,replace=TRUE))
mu <- c(-1,0,1,2)[as.numeric(X3)]

p <- 1/(1+exp(-(sin(3*X1) - 4*X2 + mu)))
Y <- rbinom(N,1,p)

# random weights if you want to experiment with them
w <- rexp(N)
w <- N*w/sum(w)

data <- data.frame(Y=Y,X1=X1,X2=X2,X3=X3)

\donttest{
# takes longer, but num_trees=3000 preferable
train_params <-
     training_params(num_trees = 3000,
                     shrinkage = 0.001,
                     bag_fraction = 0.5,
                     num_train = N/2,
                     id=seq_len(nrow(data)),
                     min_num_obs_in_node = 10,
                     interaction_depth = 3,
                     num_features = 3)
}

# for the example to run quickly, num_trees=100
train_params <-
     training_params(num_trees = 100,
                     shrinkage = 0.001,
                     bag_fraction = 0.5,
                     num_train = N/2,
                     id=seq_len(nrow(data)),
                     min_num_obs_in_node = 10,
                     interaction_depth = 3,
                     num_features = 3)
 
# fit initial model
gbm1 <- gbmt(Y~X1+X2+X3,                # formula
             data=data,                 # dataset
             weights=w,
             var_monotone=c(0,0,0),     # -1: monotone decrease,
                                        # +1: monotone increase, 
                                        #  0: no monotone restrictions
             distribution=gbm_dist("Bernoulli"),
             train_params = train_params,
             cv_folds=5,                # do 5-fold cross-validation
             is_verbose = FALSE)           # don't print progress

# plot the performance
#   returns out-of-bag estimated best number of trees
best.iter.oob <- gbmt_performance(gbm1,method="OOB")  
plot(best.iter.oob)
print(best.iter.oob)

# returns 5-fold cv estimate of best number of trees
best.iter.cv <- gbmt_performance(gbm1,method="cv")   
plot(best.iter.cv)
print(best.iter.cv)

# returns test set estimate of best number of trees
best.iter.test <- gbmt_performance(gbm1,method="test") 
plot(best.iter.cv)
print(best.iter.test)

best.iter <- best.iter.test

# plot variable influence
summary(gbm1,num_trees=1)         # based on first tree
summary(gbm1,num_trees=best.iter) # based on  estimated best number of trees

# create marginal plots
# plot variable X1,X2,X3 after "best" iterations
oldpar <- par(no.readonly = TRUE)
par(mfrow=c(1,3))
plot(gbm1,1,best.iter)
plot(gbm1,2,best.iter)
plot(gbm1,3,best.iter)
par(mfrow=c(1,1))
plot(gbm1,1:2,best.iter) # contour plot vars 1 & 2 after "best" num iterations
plot(gbm1,2:3,best.iter) # lattice plot vars 2 & 3 after "best" num iterations

# 3-way plot
plot(gbm1,1:3,best.iter)

# print the first and last trees
print(pretty_gbm_tree(gbm1,1))
print(pretty_gbm_tree(gbm1, gbm1$params$num_trees))
par(oldpar) # reset graphics options to previous settings
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calibrate-plot.r
\name{calibrate_plot}
\alias{calibrate_plot}
\title{Calibration plot}
\usage{
calibrate_plot(
  y,
  p,
  distribution = "Bernoulli",
  replace = TRUE,
  line.par = list(col = "black"),
  shade_col = "lightyellow",
  shade_density = NULL,
  rug.par = list(side = 1),
  xlab = "Predicted value",
  ylab = "Observed average",
  xlim = NULL,
  ylim = NULL,
  knots = NULL,
  df = 6,
  ...
)
}
\arguments{
\item{y}{the outcome 0-1 variable}

\item{p}{the predictions estimating E(y|x)}

\item{distribution}{the loss function used in creating \code{p}.
\code{Bernoulli} and \code{Poisson} are currently the only special options.
All others default to squared error assuming \code{Gaussian}}

\item{replace}{determines whether this plot will replace or overlay the
current plot.  \code{replace=FALSE} is useful for comparing the calibration
of several methods}

\item{line.par}{graphics parameters for the line}

\item{shade_col}{color for shading the 2 SE region. \code{shade_col=NA}
implies no 2 SE region}

\item{shade_density}{the \code{density} parameter for \code{\link{polygon}}}

\item{rug.par}{graphics parameters passed to \code{\link{rug}}}

\item{xlab}{x-axis label corresponding to the predicted values}

\item{ylab}{y-axis label corresponding to the observed average}

\item{xlim, ylim}{x and y-axis limits. If not specified the function will
select limits}

\item{knots, df}{these parameters are passed directly to
\code{\link[splines]{ns}} for constructing a natural spline smoother for the
calibration curve}

\item{...}{other graphics parameters passed on to the plot function}
}
\value{
\code{calibrate.plot} returns no values.
}
\description{
An experimental diagnostic tool that plots the fitted values versus the
actual average values. If \code{distribution} is "Bernoulli" or "Poisson", 
then the predictions are converted to the response scale (probability or
rate). For all other distributions, the calibration plot uses least squares
and predicts an expected value.
}
\details{
Uses natural splines to estimate E(y|p). Well-calibrated predictions imply
that E(y|p) = p. The plot also includes a pointwise 95% confidence band.
}
\examples{

dataSim <- data.frame(x=rnorm(1000))
dataSim$y <- with(dataSim, rbinom(1000, 1, 1/(1+exp(-(x-0.5*x^2)))))

# showing poor calibration of a linear model
glm1 <- glm(y~x, data=dataSim, family=binomial)
p <- predict(glm1, type="response")
calibrate_plot(dataSim$y, p, xlim=c(0,1), ylim=c(0,1))

# showing better calibration with quadratic
glm1 <- glm(y~poly(x,2), data=dataSim, family=binomial)
p <- predict(glm1,type="response")
calibrate_plot(dataSim$y, p, xlim=c(0,1), ylim=c(0,1))

}
\references{
J.F. Yates (1982). "External correspondence: decomposition of
the mean probability score," Organisational Behaviour and Human Performance
30:132-156.

D.J. Spiegelhalter (1986). "Probabilistic Prediction in Patient Management
and Clinical Trials," Statistics in Medicine 5:421-433.
}
\author{
James Hickey, Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbmt-plot.r
\name{plot.GBMFit}
\alias{plot.GBMFit}
\title{Marginal plots of fitted gbm objects}
\usage{
\method{plot}{GBMFit}(
  x,
  var_index = 1,
  num_trees = gbm_fit_obj$params$num_trees,
  continuous_resolution = 100,
  grid_levels = NULL,
  return_grid = FALSE,
  type = "link",
  ...
)
}
\arguments{
\item{x}{a \code{GBMFit} object fitted using a call to
\code{\link{gbmt}}}

\item{var_index}{a vector of indices or the names of the variables
to plot. If using indices, the variables are indexed in the same
order that they appear in the initial \code{gbmt} formula.  If
\code{length(var_index)} is between 1 and 3 then \code{plot.GBMFit}
produces the plots. Otherwise, \code{plot.GBMFit} returns only the
grid of evaluation points and their average predictions}

\item{num_trees}{the number of trees used to generate the
plot. Only the first \code{num_trees} trees will be used}

\item{continuous_resolution}{The number of equally space points at
which to evaluate continuous predictors}

\item{grid_levels}{A list containing the points at which to
evaluate each predictor in \code{var_index} (in the same order as
\code{var_index}). For continuous predictors this is usually a
regular sequence of values within the range of the variable. For
categorical predictors, the points are the levels of the
factor. When \code{length(var_index)} is one, the values can be
provided directly, outside a list.  This is NULL by default and
generated automatically from the data, using
\code{continuous_resolution} for continuous predictors. Forcing the
values can be useful to evaluate two models on the same exact range}

\item{return_grid}{if \code{TRUE} then \code{plot.GBMFit} produces no
graphics and only returns the grid of evaluation points and their
average predictions. This is useful for customizing the graphics
for special variable types or for dimensions greater than 3}

\item{type}{the type of prediction to plot on the vertical
axis. See \code{predict_gmt}}

\item{\dots}{other arguments passed to the plot function}
}
\value{
Nothing unless \code{return_grid} is true then
\code{plot.GBMFit} produces no graphics and only returns the grid of
evaluation points and their average predictions.
}
\description{
Plots the marginal effect of the selected variables by
"integrating" out the other variables.
}
\details{
\code{plot.GBMFit} produces low dimensional projections of the
\code{GBMFit} object, see \code{\link{gbmt}}, by integrating out
the variables not included in the \code{var_index} argument. The
function selects a grid of points and uses the weighted tree
traversal method described in Friedman (2001) to do the
integration. Based on the variable types included in the
projection, \code{plot_gbmt} selects an appropriate display
choosing amongst line plots, contour plots, and
\code{\link[lattice]{lattice}} plots. If the default graphics are
not sufficient the user may set \code{return.grid=TRUE}, store the
result of the function, and develop another graphic display more
appropriate to the particular example.
}
\references{
J.H. Friedman (2001). "Greedy Function Approximation: A Gradient
Boosting Machine," Annals of Statistics 29(4).
}
\seealso{
\code{\link{gbmt}}, \code{\link[graphics]{plot}}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-accessors.r
\name{distribution_name}
\alias{distribution_name}
\title{What is the distribution name used here?}
\usage{
distribution_name(obj, ...)
}
\arguments{
\item{obj}{the object for which the distribution name is needed}

\item{...}{other parameters}
}
\value{
a string identifying the distribution
}
\description{
What is the distribution name used here?
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/to-old-gbm.r
\name{to_old_gbm}
\alias{to_old_gbm}
\title{Convert GBMFit to previous gbm object}
\usage{
to_old_gbm(gbm_fit_obj)
}
\arguments{
\item{gbm_fit_obj}{a \code{GBMFit} object produced by a call to \code{\link{gbmt}}.}
}
\value{
a \code{gbm} object of the form from Version 2.1 of the package.
}
\description{
Function that takes a \code{GBMFit} object produced from a call
to \code{\link{gbmt}} and converts it to the form of a gbm object from
Version 2.1 of the package.
}
\author{
James Hickey
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-distribution.r
\name{gbm_dist}
\alias{gbm_dist}
\title{GBM Distribution}
\usage{
gbm_dist(name="Gaussian", ...)
}
\arguments{
\item{name}{The name (a string) of the distribution to be initialized and used
in fitting a gradient boosted model via gbmt.  The current distributions
available can be viewed using the function \code{available_distributions}.  If no 
distribution is specified this function constructs a Gaussian distribution by
default.}

\item{...}{Extra named parameters required for initializing certain distributions.
If t-distribution is selected, an additional parameter (\code{df}) specifying the number of 
degrees of freedom can be given.  The default degrees of freedom is set to four.  

If quantile is selected then the quantile to estimate may be specified using the
named parameter \code{alpha}.  The default quantile to estimate is 0.25.

If the tweedie distribution is selected the power-law specifying the distribution
may be set via the named parameter: \code{power}.  This parameter defaults to unity.

If a Cox Partial Hazards model is selected a number of additional parameters are
required, these are:

\describe{
 \item{\code{strata}}{A vector of integers (or factors) specifying which strata each data-row belongs to, 
  if none is specified it is assumed all training data is in the same stratum.}
  \item{\code{ties}}{String specifying the method to be used when dealing with tied
  event times.  Currently only "breslow" and "efron" are available, with the latter
  being the default.}
  \item{\code{prior_node_coeff_var}}{It is a prior on the coefficient of variation associated with the hazard 
rate assigned to each terminal node when fitting a tree. Increasing its value emphasizes 
the importance of the training data in the node when assigning a prediction to said node. This defaults to 1000.}
}

Finally, if the pairwise distribution is selected a number of parameters also need to be
specified. These parameters are \code{group}, \code{metric} and \code{max_rank}.
The first is a character vector  with the column names of data that jointly indicate the group 
an instance belongs to (typically a query in Information Retrieval).  For
training, only pairs of instances from the same group and with different target
labels may be considered. \code{metric} is
the IR measure to use, one of
\describe{
  \item{list("conc")}{Fraction of concordant pairs; for binary labels,
   this is equivalent to the Area under the ROC Curve}
  \item{:}{Fraction of concordant pairs; for binary labels, this
is equivalent to the Area under the ROC Curve}
  \item{list("mrr")}{Mean reciprocal rank of the highest-ranked positive instance}
  \item{:}{Mean reciprocal rank of the highest-ranked positive instance}
  \item{list("map")}{Mean average precision, a generalization of \code{mrr}
  to multiple positive instances}
  \item{:}{Mean average precision, a generalization of \code{mrr} to multiple
   positive instances}
  \item{list("ndcg:")}{Normalized discounted cumulative gain.
The score is the weighted sum (DCG) of the user-supplied target values, weighted by
log(rank+1), and normalized to the maximum achievable value. This is the
default if the user did not specify a metric.}
}

\code{ndcg} and \code{conc} allow arbitrary target values, while binary
targets \{0,1\} are expected for \code{map} and \code{mrr}. For \code{ndcg}
and \code{mrr}, a cut-off can be chosen using a positive integer parameter
\code{max_rank}. If left unspecified, all ranks are taken into account.}
}
\value{
returns a \code{GBMDist} object.
}
\description{
Generates distribution object for gbmt.
}
\author{
James Hickey
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-baseline-hazard.r
\name{baseline_hazard}
\alias{baseline_hazard}
\title{Baseline hazard function}
\usage{
baseline_hazard(surv_times, delta, coxph_preds, eval_times=NULL, smooth=FALSE,
cumulative=TRUE)
}
\arguments{
\item{surv_times}{the survival times - an atomic vector of doubles}

\item{delta}{the censoring indicator - a vector same length as surv_times}

\item{coxph_preds}{the predicted values of the regression model on the log hazard
scale}

\item{eval_times}{values at which the baseline hazard will be evaluated}

\item{smooth}{if \code{TRUE} \code{baseline_hazard} will smooth the estimated
baseline hazard using Friedman's super smoother \code{\link{supsmu}}}

\item{cumulative}{if \code{TRUE} the cumulative survival function will be
computed}
}
\value{
a vector of length equal to the length of surv_times (or of length
\code{eval_times} if \code{eval_times} is not \code{NULL}) containing the baseline
hazard evaluated at t (or at \code{eval_times} if \code{eval_times} is not
\code{NULL}). If \code{cumulative} is set to \code{TRUE} then the returned
vector evaluates the cumulative hazard function at those values.
}
\description{
Computes the Breslow estimator of the baseline hazard function for a
proportional hazard regression model - only for censored survival data.
}
\details{
The proportional hazard model assumes h(t|x)=lambda(t)*exp(f(x)).
\code{\link{gbmt}} can estimate the f(x) component via partial likelihood.
After estimating f(x), \code{baseline_hazard} can compute a nonparametric
estimate of lambda(t).
}
\references{
N. Breslow (1972). "Discussion of `Regression Models and
Life-Tables' by D.R. Cox," Journal of the Royal Statistical Society, Series
B, 34(2):216-217.

N. Breslow (1974). "Covariance analysis of censored survival data,"
Biometrics 30:89-99.
}
\seealso{
\code{\link[survival]{survfit}}, \code{\link{gbmt}}
}
\author{
James Hickey, Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{methods}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-interactions.r
\name{interact}
\alias{interact}
\alias{interact.GBMFit}
\title{Estimate the strength of interaction effects}
\usage{
interact(gbm_fit_obj, data, var_indices=1, num_trees=gbm_fit_obj$params$num_trees)

\method{interact}{GBMFit}(
  gbm_fit_obj,
  data,
  var_indices = 1,
  num_trees = gbm_fit_obj$params$num_trees
)
}
\arguments{
\item{gbm_fit_obj}{a \code{GBMFit} object fitted using a call to \code{\link{gbmt}}.}

\item{data}{the dataset used to construct \code{gbm_fit_obj}. If the original dataset
is large, a random subsample may be used to accelerate the computation.}

\item{var_indices}{a vector of indices or the names of the variables for compute
the interaction effect. If using indices, the variables are indexed in the
same order that they appear in the initial \code{gbmt} formula.}

\item{num_trees}{the number of trees used to generate the plot. Only the first
\code{num_trees} trees will be used.}
}
\value{
Returns the value of \code{H}.
}
\description{
Computes Friedman's H-statistic to assess the strength of variable
interactions.
}
\details{
\code{interact.GBMFit} computes Friedman's H-statistic to assess the relative
strength of interaction effects in non-linear models. H is on the scale of
[0-1] with higher values indicating larger interaction effects. To connect
to a more familiar measure, if \eqn{x_1} and \eqn{x_2} are uncorrelated
covariates with mean 0 and variance 1 and the model is of the form
\deqn{y=\beta_0+\beta_1x_1+\beta_2x_2+\beta_3x_3} then
\deqn{H=\frac{\beta_3}{\sqrt{\beta_1^2+\beta_2^2+\beta_3^2}}}

Note that if the main effects are weak, the estimated H will be unstable.
For example, if (in the case of a two-way interaction) neither main effect
is in the selected model (relative influence is zero), the result will be
0/0. Also, with weak main effects, rounding errors can result in values of H
> 1 which are not possible.
}
\references{
J.H. Friedman and B.E. Popescu (2005). \dQuote{Predictive
Learning via Rule Ensembles.} Section 8.1
}
\seealso{
\code{\link{gbmt}}
}
\author{
Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-parallel.r
\name{gbmParallel}
\alias{gbmParallel}
\title{Control parallelization options}
\usage{
gbmParallel(num_threads = 1, array_chunk_size = 1024)
}
\arguments{
\item{num_threads}{the number of threads to use (a positive
integer).  The number of cores on your computer is a
reasonable default.}

\item{array_chunk_size}{the size of chunks to use in array scans.
Values that are too small result in a great deal of overhead;
The default of 1024 appears reasonable, but do experiment.}
}
\value{
an object of type \code{gbmParallel}
}
\description{
\code{gbmt} uses openmp to parallelize its core algorithm, and the
details are controlled by this object.  As guidance, set
\code{num_threads} to the number of cores on your computer, and set
\code{array_chunk_size} to a reasonable - not necessarily small -
size.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/training-params.r
\name{training_params}
\alias{training_params}
\title{Training parameters}
\usage{
training_params(
  num_trees = 100,
  interaction_depth = 1,
  min_num_obs_in_node = 10,
  shrinkage = 0.001,
  bag_fraction = 0.5,
  num_train = (2 * min_num_obs_in_node + 1)/bag_fraction + 1,
  id = seq_len(num_train),
  num_features = 1
)
}
\arguments{
\item{num_trees}{Number of trees used in the fit.}

\item{interaction_depth}{Maximum depth of each tree}

\item{min_num_obs_in_node}{Minimum number of observations each node
in a tree must have.}

\item{shrinkage}{shrinkage parameter applied to each tree in the
expansion.  Also known as the learning rate or step-size reduction.}

\item{bag_fraction}{fraction of independent training observations
selected to create the next tree in the expansion.  Introduces
randomness in the model fit; if bag_fraction < 1 then running the
same model twice will result in similar but different fits.}

\item{num_train}{number of obs of data used in training the model.
This defaults to the minimum number of  observations allowed - 
\code{(2*min_num_obs_in_node + 1)/bag_fraction + 1}.}

\item{id}{optional vector of integers, specifying which rows in the
data correspond to which observations. Individual observations may
have many rows of data associated with them. This defaults to
\code{seq_len(num_train)}.  NB: When calling \code{\link{gbmt}} or
\code{\link{gbmt_fit}} the id should be the default.}

\item{num_features}{number of random features/columns to use in
training model.  This defaults to \code{1}.}
}
\value{
training parameters object
}
\description{
Class that contains the training parameters for the gbm model
}
\author{
James Hickey
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perf-pairwise.r
\name{perf_pairwise}
\alias{perf_pairwise}
\title{Performance for pairwise}
\usage{
perf_pairwise(y, f, group, metric = "ndcg", w = NULL, max_rank = 0)
}
\arguments{
\item{y}{responses used for fit.}

\item{f}{the predicted responses.}

\item{group}{the indices specifiying which response variables
belong to which groups.}

\item{metric}{What type of performance measure to compute in
\code{perf_pairwise}.  Can take values "ir_measure_conc",
"ir_measure_mrr", "ir_measure_map" or "ir_measure_ndgc".}

\item{w}{is the weights of each observation.}

\item{max_rank}{the maximum rank considered in the groups measure.}
}
\value{
returns a numeric value of the selected performance metric
}
\description{
Additional performance using appropriate metric for pairwise
distribution fit.
}
\seealso{
\code{\link{gbmt}}
}
\author{
Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{nonlinear}
\keyword{nonparametric}
\keyword{survival}
\keyword{tree}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permutation-relative-influence.r
\name{permutation_relative_influence}
\alias{permutation_relative_influence}
\title{Relative influence via permutation}
\usage{
permutation_relative_influence(
  gbm_fit_obj,
  num_trees,
  rescale = FALSE,
  sort_it = FALSE
)
}
\arguments{
\item{gbm_fit_obj}{a \code{GBMFit} object from an initial call to
\code{\link{gbmt}}.}

\item{num_trees}{the number of trees to use for computations. If
not provided, the function will guess: if a test set was used in
fitting, the number of trees resulting in lowest test set error
will be used; otherwise, if cross-validation was performed, the
number of trees resulting in lowest cross-validation error will be
used; otherwise, all trees will be used.}

\item{rescale}{whether or not the result should be scaled. Defaults
to \code{FALSE}.}

\item{sort_it}{whether or not the results should be (reverse)
sorted.  Defaults to \code{FALSE}.}
}
\value{
By default, returns an unprocessed vector of estimated
relative influences. If the \code{rescale} and \code{sort}
arguments are used, returns a processed version of the same.
}
\description{
This function offers a method for computing the relative influence in
\code{\link{summary.GBMFit}}, and is not intended to be called directly.
}
\details{
Calculates the relative influence of predictors via random
permutation of each predictor one at a time and calculating the
associated reduction in predictive performance.  This experimental
measure is similar to the variable importance measures Breiman uses
for random forests, but \code{\link{gbmt}} currently computes using
the entire training dataset (not the out-of-bag observations).
}
\seealso{
\code{\link{summary.GBMFit}}
\href{https://www.stat.berkeley.edu/~breiman/randomforest2001.pdf}{Random
Forests}.
}
\author{
Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantile-rug.r
\name{quantile_rug}
\alias{quantile_rug}
\title{Quantile rug plot}
\usage{
quantile_rug(x, prob = (0:10)/10, ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{prob}{the quantiles of x to mark on the x-axis.}

\item{...}{additional graphics parameters currently ignored.}
}
\value{
No return values
}
\description{
Marks the quantiles on the axes of the current plot.
}
\examples{

x <- rnorm(100)
y <- rnorm(100)
plot(x,y)
quantile_rug(x)
}
\seealso{
\code{\link[graphics]{plot}}, \code{\link[stats]{quantile}},
\code{\link[base]{jitter}}, \code{\link[graphics]{rug}}.
}
\author{
Greg Ridgeway \email{gregridgeway@gmail.com}
}
\keyword{aplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-package.r
\docType{package}
\name{gbm3-package}
\alias{gbm3}
\alias{gbm3-package}
\alias{gbm_package}
\title{Generalized Boosted Regression Models}
\description{
This package implements extensions to Freund and Schapire's AdaBoost
algorithm and J. Friedman's gradient boosting machine. Includes regression
methods for least squares, absolute loss, logistic, Poisson, Cox
proportional hazards partial likelihood, t-distribution,
AdaBoost exponential loss, Learning to Rank, and Huberized hinge loss.
}
\details{
Further information is available in the following vignettes: \tabular{ll}{
\code{gbm} \tab Generalized Boosted Models: A guide to the gbm package
(source, pdf)\cr}
}
\references{
Y. Freund and R.E. Schapire (1997) \dQuote{A decision-theoretic
generalization of on-line learning and an application to boosting,}
\emph{Journal of Computer and System Sciences,} 55(1):119-139.

G. Ridgeway (1999). \dQuote{The state of boosting,} \emph{Computing Science
and Statistics} 31:172-181.

J.H. Friedman, T. Hastie, R. Tibshirani (2000). \dQuote{Additive Logistic
Regression: a Statistical View of Boosting,} \emph{Annals of Statistics}
28(2):337-374.

J.H. Friedman (2001). \dQuote{Greedy Function Approximation: A Gradient
Boosting Machine,} \emph{Annals of Statistics} 29(5):1189-1232.

J.H. Friedman (2002). \dQuote{Stochastic Gradient Boosting,}
\emph{Computational Statistics and Data Analysis} 38(4):367-378.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/gbm-developers/gbm3}
}

}
\author{
James Hickey, Greg Ridgeway \email{gregridgeway@gmail.com} with contributions by
Daniel Edwards, Brian Kriegler, Stefan Schroedl and Harry Southworth.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbm-accessors.r
\name{trees}
\alias{trees}
\title{Extract trees from GBMFit objects}
\usage{
trees(obj)
}
\arguments{
\item{obj}{a GBMFit object}
}
\value{
a vector containing the trees generated in the fit
}
\description{
Extract trees from GBMFit objects
}
